#!/bin/sh
#
#
# Copy this script to your megaraid driver source subdir.
# Example - /lsi/megaraid_sas-v06.505.02.00
#
# This script will attempt to build the driver source in its native format as well as each patch version.
# If any version of the driver builds it will be listed at the end of the script.
#
#
#
# 02/21/13 - Initial release
# 02/21/13 - Cleaned up output and report all errors/warnings.
# 02/23/13 - Added notes
# 12/20/15 - Took out check for dkms.conf, not in current driver source package.
# 
#
#
#
#
#set -x
#
#
#Generic MegaRAID Linux Driver Install Notes
#
#Use ONE of the three methods to install the driver. These are examples,
#the file names may vary from driver version to driver version.
#
########################################################################
########################################################################
#Create a floppy Image with the appropriate .img file
########################################################################
########################################################################
#
#dd if=bootfloppy.iso of=/dev/fd0
#
#I.E.
#
#tar -zxvf megaraid_sas-v06.505.02.00-1-rhel5.8.x86_64.iso.gz
#dd if=dd.iso of=/dev/fd0
#
#When installing the OS use this as the driver diskette.
#
#
########################################################################
########################################################################
#Manually Compile LSI Driver Source Code 
########################################################################
########################################################################
#
#Extract Source - 
#
# tar -zxvf megaraid_sas-v06.505.02.00-src.tar.gz
#
#Patches are distribution dependent, not kernel version. If the Kernel 
#version matches the patch it may work with a different distribution.
#
########################################################################
#cd /usr/src/megaraid_sas/ (Directory where driver source is located)
#cp ./patches/rhel5.patch . (Patch for the distribution you have, release dependent.)
#patch -p1 < rhel5.patch
########################################################################
#
#At this point you should have cleanly patched .c and .h files
#
#Now run the command.
#
#make -C /lib/modules/<version>/build M=<location of driver source>
#
#I.E. 
#
#Note: The kernel version referenced below is an example only

#make -C /lib/modules/2.6.18-274.18.1.el5/build M=/lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch
#
#make: Entering directory `/usr/src/kernels/2.6.18-274.18.1.el5-x86_64'
#  LD      /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/built-in.o
#  CC [M]  /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas_base.o
#  CC [M]  /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas_fusion.o
#  CC [M]  /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas_fp.o
#  LD [M]  /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas.o
#  Building modules, stage 2.
#  MODPOST
#  CC      /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas.mod.o
#  LD [M]  /lsi/megaraid_sas-v06.505.02.00/build/rhel5u8.patch/megaraid_sas.ko
#make: Leaving directory `/usr/src/kernels/2.6.18-274.18.1.el5-x86_64'
#
#You can manually load the driver, i.e.;
#
#insmod megaraid_sas.ko
#
#This is will only stay loaded until reboot, to load the driver automatically copy it to the location where the current driver is located. 
#There may be numerous locations with the megaraid_sas.ko driver.
#
#This is distribution dependent, you will then need to run mkinitrd to create a boot image file.
#
#
#
########################################################################
########################################################################
#DKMS Install - NOT for manually compiled source!
#Install RPMs, driver binaries (IF applicable) 
########################################################################
########################################################################
#
#[root@localhost dkms-1]# rpm -ihv dkms-2.0.21.1-1.noarch.rpm
#Preparing...                ########################################### [100%]
#  1:dkms                   ########################################### [100%]
#[root@localhost dkms-1]#
#[root@localhost dkms-1]# rpm -ihv megaraid_sas-v06.505.02.00-1dkms.noarch.rpm
#Preparing...                ########################################### [100%]
#   1:megaraid_sas           ########################################### [100%]
#
#Creating symlink /var/lib/dkms/megaraid_sas/v06.505.02.00/source ->
#                 /usr/src/megaraid_sas-v06.505.02.00
#
#DKMS: add Completed.
#
#Kernel preparation unnecessary for this kernel.  Skipping...
#applying patch rhel5u7.patch...patching file megaraid_sas.h
#patching file megaraid_sas_base.c
#patching file megaraid_sas_fusion.c
#patching file megaraid_sas_fusion.h
#
#
#Building module:
#cleaning build area....
#make KERNELRELEASE=2.6.18-274.18.1.el5 -C /lib/modules/2.6.18-274.18.1.el5/build SUBDIRS=/var/lib/dkms/megaraid_sas/v06.505.02.00/build modules....
#cleaning build area....
#
#DKMS: build Completed.
#
#megaraid_sas.ko:
#Running module version sanity check.
#Warning! Cannot do version sanity checking because multiple megaraid_sas.ko
#modules were found in kernel 2.6.18-274.18.1.el5.
# - Original module
# - Found /lib/modules/2.6.18-274.18.1.el5/kernel/drivers/scsi/megaraid/megaraid_sas.ko
# - Storing in /var/lib/dkms/megaraid_sas/original_module/2.6.18-274.18.1.el5/x86_64/
# - Archiving for uninstallation purposes
# - Installation
# - Installing to /lib/modules/2.6.18-274.18.1.el5/extra/
#Adding any weak-modules
#WARNING: Can't read module /lib/modules/2.6.18-274.18.1.el5/weak-updates/megaraid_sas.ko: No such file or directory
#WARNING: Can't read module /lib/modules/2.6.18-274.18.1.el5/weak-updates/mpt2sas.ko: No such file or directory
#WARNING: Can't read module /lib/modules/2.6.18-274.18.1.el5/weak-updates/megaraid_sas.ko: No such file or directory
#WARNING: Can't read module /lib/modules/2.6.18-274.18.1.el5/weak-updates/mpt2sas.ko: No such file or directory
#
#depmod....
#
#Saving old initrd as /boot/initrd-2.6.18-274.18.1.el5_old.img
#Making new initrd as /boot/initrd-2.6.18-274.18.1.el5.img
#(If next boot fails, revert to the _old initrd image)
#mkinitrd.....
#
#DKMS: install Completed.
#[root@localhost dkms-1]#
#

kernel=`uname -r`

###########################################################################################################################
# Verify in correct subdir
###########################################################################################################################

for i in Makefile  Makefile.standalone  megaraid_sas_base.c  megaraid_sas_fp.c  megaraid_sas_fusion.c  megaraid_sas_fusion.h  megaraid_sas.h  ; do
	if [ ! -f $i ] ; then 
		
		echo "$i missing"

		#cho ".................................................||................................................."
		#echo "" 
		echo "" 
		echo "This script attempts to compile the standard driver source as well all patched versions."
		echo "" 
		echo "You are not in the proper directory or are missing files or your driver source or this script is too"
		echo "old. It has been tested against driver versions 5.3, 6.12, 6.18 & 6.505.02.00." 
		echo "" 
		exit
	fi
done	

###########################################################################################################################
# Verify in correct subdir
###########################################################################################################################

for i in patches ; do
	if [ ! -d $i ] ; then 

		echo "$i missing"

		#cho ".................................................||................................................."
		#echo "" 
		echo "" 
		echo "This script attempts to compile the standard driver source as well all patched versions."
		echo "" 
		echo "You are not in the proper directory or are missing files or your driver source or this script is too"
		echo "old. It has been tested against driver versions 5.3, 6.12, 6.18 & 6.505.02.00." 
		echo "" 
		exit
	fi
done	



if [ -e ./build ] ; then 
	rm -rf ./build  
fi


mkdir ./build


for i in $(ls ./patches); do
	mkdir ./build/$i
done

for i in $(ls ./build); do
	cp ./patches/$i ./build/$i > /dev/null 2>&1

	for j in Makefile  Makefile.standalone  megaraid_sas_base.c  megaraid_sas_fp.c  megaraid_sas_fusion.c  megaraid_sas_fusion.h  megaraid_sas.h ; do
		cp $j ./build/$i 
	done
done

for i in $(ls ./build); do
	cd ./build/$i
	patch -p1 < $i  > /dev/null 2>&1
	cd ../..
done


mkdir ./build/standard

for j in Makefile  Makefile.standalone  megaraid_sas_base.c  megaraid_sas_fp.c  megaraid_sas_fusion.c  megaraid_sas_fusion.h  megaraid_sas.h ; do
		cp $j ./build/standard
done

echo "" 
echo "" 

for i in $(ls ./build); do

	cd ./build/$i

	path_to_source=`pwd`
	
	make -C /lib/modules/$kernel/build M=$path_to_source > make_out.txt 2>&1
	echo "Attempting to build the driver based on $i Driver Source"
	cd ../..
done

echo ""
echo ""
echo ""
#cho ".................................................||................................................."
echo "These are the possible valid drivers for your distro/kernel. These built with your kernel source."
echo ""
echo "Official support is limited to our tested and supported distributions. The native driver source was" 
echo "built against as well as each patch file. The patch files signify a supported distribution."
echo ""
echo "Any errors or warnings logged during the build will be reported under the driver name."
echo ""
for i in $(ls ./build); do
	if [ -f ./build/$i/megaraid_sas.ko ] ; then
		ls ./build/$i/megaraid_sas.ko
		for j in $(grep -i -c error ./build/$i/make_out.txt) ; do
			if [ "$?" -eq "0" ] ; then
				echo "Built but with $j Errors" 
			fi
		done
		for j in $(grep -i -c warning ./build/$i/make_out.txt) ; do
			if [ "$?" -eq "0" ] ; then
				echo "Built but with $j Warnings" 
			fi
		done
	fi
done
echo ""
echo ""
echo "If there are no drivers listed above then there were no successful builds."
echo ""
echo "Below is a list of all warnings and errors for all build attempts."
echo ""



for i in $(ls ./build); do
	for j in $(grep -i -c error ./build/$i/make_out.txt) ; do
		if [ "$j" -ne "0" ] ; then
			echo "$i Build generated $j Errors"
		fi
	done

	for j in $(grep -i -c warning ./build/$i/make_out.txt) ; do
		if [ "$j" -ne "0" ] ; then
			echo "$i Build generated $j Warnings" 
		fi
	done
done
echo "" 