#!/bin/sh
#set -x
#
# This script is to be run in its own window or as a background process, i.e. "trigger_ext.sh".
# The function is to grep the /var/log/messages file in a tight loop and start data collection 
# when a specified error (Trigger Phrase) is found.
#
# 08/18/15 - Initial Release


if [ "$1" != "-run" ] ; then
	echo -e "\n"
	echo "Do not run this script unless you have reviewed it and updated it for your system."
	echo "To be used at FAE/TS direction only, improper use can have serious system"
	echo "stability and performance consequences."
	echo "Script needs the -run command line parameter to execute."
	echo -e "\n"
	exit 1
fi


#############################################################################
###CONFIGURABLE_VARIABLES - Must Configure
#############################################################################
###Can Add Additional Systems to Skip, uncomment SKIP_SYSTEM_SECTION section, for PXE booted systems.
###Expects unique string in "uname -a", i.e. server name/serial number/etc...
#SystemSkip=206341720008
#SystemSkip1=206341720002
###Subdir test is run from, normally the lsiget dir.
TestDir=.
###Network path tools/scripts are copied from, for PXE booted systems.
#ToolsTestDir=/mnt/pxe/tools/Avago_Debug/scripts_tools
###Network path logs are copied to, for PXE booted systems.
#LogsTestDir=/avago/logs
###Initial script to run?
#InitialScriptRun=installXYZ.sh
###IOC/HBA # under test
IocHbaNum=1


tw_host=$HOSTNAME
tw_hostname=`echo $tw_host | cut -d. -f1`
todayDate=`date '+DATE:%m%d%y' | cut -d: -f2`
currentTime=`date '+TIME:%H%M%S' | cut -d: -f2`
currentYear=`date | cut -d' ' -f 6`
currentDayMonth=`date '+DATE:%m%d' | cut -d: -f 2`
currentDate=${currentYear}${currentDayMonth}${currentTime}

###Primarily for PXE booted test systems
#IPadd=`ifconfig | grep -m 1 inet | cut -d. -f4 | cut -d' ' -f1`
#
#ipmitool lan print > /dev/null 2>&1
#if [ "$?" = "0" ] ; then 
#	IPMIadd=`ipmitool lan print | grep "IP Address              :" | cut -d. -f4` 
#	else
#	IPMIadd=NoIpmi
#fi
#subdirName=${tw_hostname}-${IPadd}-${IPMIadd}-${todayDate}-${currentTime}

subdirName=${tw_hostname}-${todayDate}-${currentTime}

###files needed in same subdir, normally the lsiget dir.
#lsigetlunix.sh 		= Main data collection script - included in lsiget
#all_cli 			= Tools - included in lsiget
#trigger_ext.sh			= Run in seperate window or as a background process, tails the /messages file for a "Trigger Phrase" - included in lsiget
#trigger_test.sh 		= Script to test trigger script - included in lsiget
#s2sll.sh			= Can set driver logging level for SAS2 HBAs - included in lsiget
#s3sll.sh			= Can set driver logging level for SAS3 HBAs - included in lsiget

#sg_raw ------------------------> Download from WEB "yum install sg3_utils", needed for SAS Analyzer trigger only. Must be in ${TestDir}

#############################################################################
###SKIP_SYSTEM_SECTION - Must Configure
#############################################################################
###Can Add Additional Systems to Skip, for PXE booted systems
#uname -a | grep -e ${SystemSkip} -e ${SystemSkip1} 
#if [ "$?" = "0" ] ; then
#	exit 1
#fi

###To ensure there isnt an old trigger phrase in the file
cat /dev/null > /var/log/messages


###Enables a 3MB RB and sets driver logging at 3F8
###For SAS 2 HBAs
./lsigetlunix.sh -e_sas2log1_rb
###For SAS 3 HBAs
#./lsigetlunix.sh -e_sas3log1_rb

while [ 1 = 1 ] ; do
	###Modify for the specific trigger condition you need. For testing the 
	tail -n 5000 /var/log/messages | grep -i -e "attempting task abort"  > trigger.out
	if [ "$?" = "0" ] ; then


#############################################################################
###TRIGGER_SAS_ANALYZER - Must Configure
#############################################################################
		###Analyzer Trigger - This will send the ATA Command Security Erase Prepare 0xF3h to the specified SG device
		###Make sure this is a valid SATA SG device #
		###This can be any unique, benign command.
		#${TestDir}/sg_raw /dev/sg3 85 06 00 00 00 00 00 00 00 00 00 00 00 40 f3 00
		###Gets Invalid command, 0xA3h is SCSI Report Priority, VERIFY this works as a trigger first - havent tested this as a trigger yet.
		#${TestDir}/sg_raw /dev/sg0 a3 0e 00 00 00 00 ff 00 00	

		###Kill all tests - Used to queice the system, these are just examples. Done twice just in case...
		#for i in $(  ps -ea | grep -i -e perl -e handoff -e fio | awk '{print $1}' ); do
		#echo "kill -9 $i"
		#ps -p $i -o comm=
		#kill -9 $i
		#done
		#for i in $(  ps -ea | grep -i -e perl -e handoff -e fio | awk '{print $1}' ); do
		#	echo "kill -9 $i"
		#	ps -p $i -o comm=
		#	kill -9 $i
		#done

		###Set mpt2sas driver logging level back to 0
		./s2sll.sh 0
		###Set mpt3sas driver logging level back to 0
		#./s3sll.sh 0

		###Grab system Capture
		./lsigetlunix.sh -b "Trigger = attempting task abort"

		###Used to copy captured data to a network location, normally for PXE booted tests systems.
		#if [ ! -d ${LogsTestDir} ] ; then mkdir ${LogsTestDir} ; fi
		#mkdir ${LogsTestDir}/${subdirName}

		###Gather debug data including RB/lsiget/inbandcli and scripts run
		#for i in trigger_ext.sh lsi.*.tar.gz trigger.out ; do
		#	if [ -f ${i} ] ; then
		#		cp ./${i} ${LogsTestDir}/${subdirName}/.
		#	fi
		#done
		
		exit 1
	fi
done

exit 0