#!/bin/sh
#
# File:  lsigetlunix.sh (Don't change this line, used in Solaris grep test)
#
# 
# See Readme.txt (Embedded further down)
#
#  How to run it:
#    1) Copy the included files to a file system with available space. 
#       In general, the size of the data collected is at least as big as the total size of
#       all log files.
#    3) Run it:   ./lsigetlunix.sh -H <enter> for help screen
#
# 
#  What to do when the script ends:
#    1) Send the .tgz file to Technical Support or to the FAE that you are in contact with.
# 
#  What to do when receiving the .tgz file:
#    1) Extract
#    3) Verify files are not empty & Controller_Cx.txt and/or Adapter_Ax.txt files are present.
#    4) Analyze the collected data 
#
#
#
# 02/20/10 - To many major changes have occurred to document them all. Revamped capture script that supports 3ware, MegaRAID
#	     & HBA's. All changes from here on out to be documented. 
# 02/21/10 - Grep'ed disk specific parameters together for easier troubleshooting for 3ware, already on MegaRAID. Added modpgctl.
# 02/22/10 - Fixed HBA version reporting.
# 02/24/10 - Changing export to x=y for FreeBSD, may break MacOS or Solaris, need to verify. Changed -? to -H for Help, 
#            reserved on FreeBSD. Grep MegaRAID specific info in FreeBSD messages file. 
# 02/25/10 - Fixed grepped disk specific parameters for multiple controllers on 3ware. Added FreeBSD Support for MegaRAID & HBA.
# 	     Cleaned up CLI naming/arch. Excluded all MegaRAID/HBA sections if MacOS. Changed to gunzip instead of tar -zxvf for 
#            Solaris. Added MegaRAID specific logging for Solaris. Added additional files to cleanup. 
# 04/10/10 - Added check for "OS_LSI"="" (Ubuntu others?), added check for iuCRC on MegaRAID. 
# 06/19/10 - Rearranged data for easier troubleshooting, less use of MegaCli, more grepping of master file for better performance.
#	     Created separate files for pdlist/adpalilog/adpallinfo/fwtermlog & added master file name to all as well as 
#	     adapter_ax.txt file (fwtermlog and all relevant data still in adapter_ax.txt file. Created separate files for grepped
#            fwtermlog errors to make it easier to read fwtermlog (instead of appending to end).
#
# 06/20/10 - GCA Release - Updated MegaCli to 10M05, updated help, fwtermlog errors in separate subdir.
#
# 09/14/10 - Added workaround for customer distribution. Updated MegaCli to 10M08. Updated tw_cli to 10.2 (esx35/40 still at 9.5.3)
# 09/15/10 - Replaced bad tw_cli images, readme update.
# 10/11/10 - Added beta 64bit MegaCli for FreeBSD
# 10/13/10 - Added 10M10/8.00.39 32/64bit MegaCli for FreeBSD, updated obsolete 3ware links to lsi.
# 10/25/10 - Changed uname -i to uname -s on custom OS check. Added 10M09/8.00.40 32bit MegaCli as 32/64bit for FreeBSD
# 11/24/10 - Added RC 10M09 8.00.36/8.00.39 32/64bit FreeBSD MegaCli's
# 02/16/11 - Added vars to internal util. Grabbing contents of /boot and menu.lst, updated cversions for 3w & MR.
# 03/11/11 - Updated MegaCli to 10M09P3/8.00.046 (10M10), Internal util to 1.66
# 03/17/11 - Added mrmonitor and vivaldiframeworkd status check.
# 03/25/11 - Fixed "mrmonitord: error while loading shared libraries: libxerces-c.so.28: cannot open shared object file: No such 
#            file or directory" error, now check "/etc/init.d/mrmonitor -v" instead of "mrmonitord -v". Updating ELF query, DS 3,
#            misc command additions and formatting changes. Updated MegaCli to 10M12/8.01.06 for all OSs but FreeBSD. Updated 
#            HBA section to better support Warp, still need separate cli. Added  tip for Solaris script issue.
#	     Started to use new adpalilog driver version feature, doesn't rely on modinfo which can provide false info as to whats 
#	     loaded.	
# 03/28/11 - Replaced bad copy of tw_cli for Solaris. Solaris old default shell workaround.		
# 03/28/11 - Additional hostname variable support.		
# 03/31/11 - GCA Release - Added -i for OSTYPE grep for all OSs, FreeBSD 6.1 uses lowercase. Replaced bad copy of tw_cli for FreeBSD.
# 06/17/11 - Added BETA MegaCli(64) from 11M06
# 09/28/11 - Added tw_cli from 10.2.1 code set, fixed issue with show diag not working with older fw.
# 09/30/11 - Moved /conf create and some file copies to main subdir creation section, cleaned up file cleanup logic on close/continue.
# 10/06/11 - Upgraded internal util to 1.67.
# 10/07/11 - Cleaned up grep of messages, for drivers, i.e. "kernel: driver" not valid in all distributions/versions.
# 10/09/11 - Added check for MR LD consistency, MR PCI Info, lsipci -t and grep for only 1st driver instance in Adpalilog.
# 01/27/12 - Updated MegaCli(64) to 8.02.21-1/11M08P3, added lsscsi support, perfmode.
# 02/02/12 - Added option 17 to internal util.
# 03/14/12 - Updated Linux MegaCli(64) to 8.03.07(Beta) to support the Linux 3.x Kernel.
# 05/24/12 - Added collection of WarpDrive info and ddoemcli exec files to 
#            support collection.  Added collection of Nytro XD info.
#            Updated MegaCli and MegaCli64 to 8.04.07 for Linux, FreeBSD, and
#            Solaris.
# 09/21/12 - Update collection of WarpDrive info to use lsi_diag.sh script if
#            available.  Added collection of Nytro MegaRAID info. 
#            Updated internal util to 1.69 and dd*cli to 01.250.41.10.
#            Updated MegaCli to 8.04.52 for Linux to pull in new commands for
#            Nytro MegaRAID.
# 11/15/12 - Added filtering for additional error messages. Updated 3ware errorcodes/aens to 9.5.5.1. 
#
# 01/23/13 - Cleaned up some old KB references. Changed clean up list/create script, got rid of old dependencies. Deleted all versions 
#            of MegaCli except for Solaris, still a dependency issue and replaced with Phase 3/1.02.08 storcli, still keeping MegaCli 
#            syntax for now and will add storcli syntax overtime. Updated 3ware & MR AEN grep list to 10.2.2.1 & 5.5 respectively.
#
# 02/16/13 - AEC and DPMSTAT added for MegaRAID. Added /MegaRAID/storcli, MegaCli and AENs/Info, Warning, Critical & Fatal subdirs. 
#            Added storcli output. Fixed indentation for if/for statements until end of storcli section. General cleanup and house keeping.
#
# 02/27/13 - Additionally separated AENs controller/type, compared pds by all variables, started to compare MR controllers by all variables.
#	     Pulled Sco cli's/build. Partial update of cversions_MR. Added driver build scripts and notes. Added vall show all for storcli.	
#	     Display all available temperature info. Support for udevadm & udevinfo. Fixed file clean up. Check if MR adapter exists before 
#	     capturing data with storcli.
#
# 03/02/13 - Added missing cx/vd/pd status/state checks with storcli.	
#
# 03/06/13 - Pulled out udevadm support - Hung OpenSuse 12.2 32bit and Centos 6.3 64bit. Fixed AEN separation, mrmonitord and MSM have different formatting of output.
#	
# 03/08/13 - Added /cx/eall show status, fixed comment when using lower case for variables, updated storcli to 1.03.11 (5.6).
#
# 04/26/13 - Added /cx show bios for storcli, Updated storcli to 1.04.07 (beta) except for Solaris. Issue with Solaris build currently. Beta drop of storcli adds NMR support.
#
# 05/29/13 - Added /cx/cv as /cx/bbu doesn't get cv info anymore with storcli. 1.04.07 storcli is now GCA and part of the 5.7 release.
#
# 08/02/13 - Added grep for "PCIE Link Status/Ctrl" in termlog for PCI-E link speed, in 2208 only. Need PR to include in "show pci" fro storcli.
#
# 11/26/13 - Added 25-2-0-0 dump on HBA. Updated storcli to 1.06.03 (5.9) for FreeBSD/Linux/Solaris, libstorelibir-2.so.14.07-0 included for Linux to support /XD.
#	     Updated internal util to 1.70. Updated dcli to 111.00.01.00. Added all /XD show commands. Added -power, -dump & -getpowerval to dcli output fro WD.
#
# 12/04/13 - Fixed solaris support, added "export LD_LIBRARY_PATH=/usr/sfw/lib" for libstdc++.so.6 & libgcc_s.so.1 for storcli. Solaris 10 does not have 
#            /usr/gnu/bin/grep  that supports the -A option so some comparisons of storcli output are not done. Fixed > null to >> ./misc_output.txt in two locations. 
#	     Fixed some exports for Solaris. Replaced all "if [ -e" options with -f. 2>&1 for whoami and groups output. 
#
# 01/07/14 - Replaced "if [ -f" with "if [ -e" on all checks for /sys/block/sdx. Increased 3ware smartctl support to 128 disks IF smartctl supports it. Moved creation of
# 	     sd_letter.txt before controller specific section to avoid duplication of sd entries. Fix smartctl capture on new 5.43 smartmontools, requires "sat+" for SATA disks.
#            Changed Readme to reflect MacOS is no longer being tested with this script but support was not explicitly removed.	
#
# 01/12/14 - Changed test for autobgi check to "No VDs have been configured". Fixed indentation formatting on "Solaris Work Around". Eliminated 2 "Solaris Work Around's"
#	     on the Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt file. Added show eghs and removed old syntax. Fixed MR DPM output file comments.
#
# 01/26/14 - Added Progress & Obsolete as AEN severity's to manage. Updated AENs to MR 6.2/storelib 4.14.
#
# 02/06/14 - Added lvm info capture, vgs, lvs, pvs, lvdisplay, pvdisplay & lvm.conf to /lvm
#
# 02/22/14 - Fixed Call_Eall_Sall_show_all-Compare-All-Parms.txt output, inquiry string going to Call_Eall_Sall_show_all-Compare-All-Parms_Cx.txt
#	     Added test to only do /cx/cv or bbu show/all if the device is present, storcli bug workaround. Removed duplicate file name in /call/eall show status output.
#	     Took PD wrcache out of /call/eall/sall output, set per VD, not PD.	Fixed Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt output, drives line had typo.
#            Added status to Call_show_all-Compare-All-Parms.txt
#
# 03/02/14 - Adding VMWare support, removed all tr commands on in common OS commands. Updated all storcli images to 6.2/1.07.07 except Solaris, need Solaris system to extract pkg file.
#            Updated all tw_cli images to 10.2.2.1/2.00.11.022. Updated all internal util images except 32bit Linux to 1.71, not available. Only including 32bit storcli/LSI WarpDrive Management Utility/internal util
#            binaries, tw_cli requires 32/64 bit versions. Don't do tw_cli /cx/px show smart/all in vmware, segfault, open defect, do each unique attribute - smart.
#	     Made date '+%H:%M:%S.%N' conditional on OS, vmware is date '+%H:%M:%S. Copying $MCLI_LOCATION$MCLI_NAME/* & /var/log/*. Replacing vmware tw_cli with  2.00.11.016esxi due to segfaults.
#	     Redirect stderr for internal util to "2>>./$fileName/script_workspace/lsiget_errorlog.txt" as its a linux util and produces the following errors on vmware "sh: /sbin/modprobe: not found"
#            and "mknod: /dev/mptctl: Function not implemented". Rearranged initial variables. Added esxcli output, combined common binaries under common OS section. Put all lsi product data
#            under /LSI_Products. Have only tested vmware 5.1.0, allow script to run on 5.x, 4.x & 3.x, BETA ONLY!
#            
# 03/11/14 - Added support for VMWare 5.5.0 & 4.1.0. Added back all 64bit binaries that I removed. If 64bit OS, try 64bit utils, if it doesn't work try the 32bit versions.
#            Changed some of the cli logic. 
#
# 03/17/14 - Made dmesg collection conditional. Added capture of /var/log/vmware conditionally. Piped standard out on a number of cli commands to ./$fileName/script_workspace/lsiget_errorlog.txt in 
#            order to avoid "/bin/sh: /usr/libexec/pci-info: not found" errors on VMWare 4.1. Segregated esxcli 4.x and 5.x commands.
#
# 03/21/14 - Added dmidecode/biosdecode & dvpdecode commands.
#
# 04/04/14 - Removed Linux only test for smartctl, should work in FreeBSD as well. Added EID 252 for Nytro pddffdiag  dump.
#
# 04/23/14 - Got rid of _all output for internal tool.
#
# 06/23/14 - Changed grep string for CV detection from SuperCaP to Cachevault_Info. Released version only includes Linux/FreeBSD/VmWare, Solaris cli's need to be added, script tested.
#
# 06/25/14 - Added "fdisk -lu" collection, updated storcli to (6.3) 1.09.08 on for linux/freebsd/vmware. Added SKIP_XD variable as storcli 1.08.09 segfaults on /xd show with Invader.
#
# 07/16/14 - Added LSI expander data collection, xu 16.00.00.00 and g3xu 5.00.00.00.
#
# 07/18/14 - Changed grep to account for G3 expander format.
#
# 07/24/14 - Modified xu/g3xu output.
#
# 07/28/14 - Added Perf_Summary_File.txt file, duplicate info from other files all in one.
#
# 07/29/14 - Fixed formatting. i.e. lined up if/fi's, while/done's, etc.
#
# 07/30/14 - Increased grepping of messages.x files from 20 to 200, changed Perf_Summary_File.txt file to Summary.txt
#
# 08/01/14 - Fixed e_dpmstat, output was being piped to non existent subdir.
#
# 08/28/14 - Added get attach to xu/g3xu capture.
#
# 08/29/14 - Fixed bug where Solaris X86 OS check was after exit of script for OS_LSI not being set.
#
# 09/02/14 - Commented out cleanup with re_execute_variable_shell.txt section. Added grepA=grep as default and grepA for g3xu/xu execution.
#
# 09/08/14 - Changed to grepA variable for -m parameter support in Solaris.
#
# 09/09/14 - Removed support for Nytro WD/XD due to sale of division to Seagate. Updated all storcli versions to 6.4/1.12.13.
#
# 09/26/14 - Added collection of ManPages 16-31 on HBAs. Updated internal util to 1.72. 
#
# 10/24/14 - Added additional data collection for LSI expanders and changed sub dir structure from /12Gb to /6-12Gb
#
# 10/27/14 - Added show output for MR/3W/HBA CLIs into script_workspace subdir. Updated Readme to reflect new code sets and VMWare support.
#
# 11/19/14 - Added collection of Region 0 on LSI expanders.
#
# 11/26/14 - Added checks if HBA's exist before running internal tool. Broke expander data collection into SAS 1/2/3, need to fine tune detection criteria.
#            
# 12/19/14 - Added check for LDBBM support, dump the bbmt for each VD, dump the enclosure phyerror counters, grep for persistent nvdata version.
#
# 02/25/15 - Added "2>>./$fileName/script_workspace/lsiget_errorlog.txt" to a few locations where it was missed on 3/2/14. Fixed create script to add internal util to vmware script.
#
# 03/17/15 - Added option 66 for internal util.
#
# 03/22/15 - Capturing additional expander info, cleaned up existing presentation. Added new data to Summary.txt file.
#
# 04/08/15 - Added objdump -ld for all mpt3 drivers.
#
# 04/10/15 - Added additional files to the exclude list for /etc.
#
# 05/18/15 - Added ls -latrR of /boot, /proc, /sys and /dev. Removed duplicate var/log/messages* collection.
#
# 05/19/15 - Added dump of LSI Expander enclLogicalId
#
# 06/06/15 - Added the ability to enable/disable linux HBA driver logging, register a Ring Buffer and to automatically read/release/unregister the RB is enabled.
#	     Updated the help for DPMSTAT/AEC to include MR as well as the new features.
#
# 06/07/15 - Added sas2ircu and sas3ircu output for linux/solaris and freebsd, vmware binaries are included but execution support not added yet.
#
# 06/08/15 - Allowing the script to run on VMWare 6.x for testing.
#
# 06/09/15 - Enabled VMWare 6x to run the same esxcli commands as 5x.
#
# 06/10/15 - Saved smartctl -h output to the script_workspace subdir for debugging.
#
# 06/11/15 - Added lsblk output, included set_logging_level_sas2/3.sh scripts, included info to allow HBA driver logging without losing /var/log/messages entries
#            as well as other info in /HBA/Notes_Scripts.
#
# 06/15/15 - Added cat /proc/cmdline capture.
#
# 06/16/15 - Added journalctl output and grab all .conf files in all primary child subdirs of /etc. Fixed RB enable message. 
#
# 07/08/15 - Added udev ata_id and scsi_id output. Fixed ls of /dev/disk/by-* data collection. Replaced all a-z sdX references with actual existing sdXYZ device nodes.
#	     Moved output of hba util to when the check takes place and tee the output of the command to a working subdir file.	
#
# 07/09/15 - Added strace to internal util output. Hard coded for linux, this  needs to be fixed for general distribution.
#
# 07/13/15 - Added ssptdebug and showlogs detail to expander data collection
#
# 07/14/15 - Commented out scsi_host capture, needs to be fixed. Also commented out g3xutil and xutil for debugging.
#
# 07/17/15 - Removed check for Readme.txt
#
# 07/18/15 - Added -SX/-sx command line switch to not run xutil and g3xutil for debugging SG driver related issues.
#
# 07/19/15 - Fixed check for print of /sys/block info to verify the device exists, i.e. not a sdX1 partition. Removed 3ware perf tuning section,
#            made generic perf section more generic.	
#
# 07/22/15 - Added cat of /sys and /proc/scsi, added ipmi lan print for local debugging that goes with ifconfig, can be disabled with -SN switch. 
#            Added switches for 3f8 driver logging.
#
# 08/04/15 - Added "grep -r . /proc/sys/kernel/" data collection, added avago in lspci grep for Controller_Disk_Association.txt. Replaced 1_8_16_17_25-2-0-0_42_47_21-1-2_HBA$i.txt
#	     with properties_hba$i.txt. Fixed some typo's. 
#
# 08/07/15 - Send stderror of udevadm --version to .txt file. Capture expander coredump region if [ -fs. Moved Generic_Perf_Tuning* into its own subdir.  
#
# 08/09/15 - Changed all expander output files to start with the SAS Address, moved all individual output files to the appropriate Exp dir under Details.
#	     Changed order of hba output to follow util option #'s numerically.
#
# 08/10/15 - Updated Sense-Key_ASC-ASCQ_Opcodes_SBC4R16.txt to include ATA Command/Set Features codes and changed the name to Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code.txt
#	     Updated HBA_Debug_Notes_Linux.txt
#
# 08/10/15 - Removed capture of showlogs for the expander, left "showlogs detail". Capture Gen3 expander coredump only if generated, removed download of region.
#
# 08/11/15 - Removed strace of internal util, added TWSETONLY variable to clean up logic and added CPU affinity capture, get_cpu_affinity.txt.
#
# 08/12/15 - Changed time in RB filename to reflect time when RB finished.	
#
# 08/17/15 - Updated syntax recommendation for irqbalance.
#
# 08/18/15 - Added trigger_ext.sh, trigger_test.sh, ssl2.sh and ssl3.sh to aid in automated data collection. Got rid of set_logging_level_sas2.sh
#      	     and set_logging_level_sas3.sh scripts in /HBA/Notes_Scripts subdir. Removed support for pathed MR cli and forced prompt for comment.
#
# 08/19/15 - Added HBA debugger output data collection.	
#
# 08/24/15 - Cleaned up /proc data collection section, added scsi_logging_level -g collection. Fixed bug in IOC/HBA iop/pl data collection logic.
#	     If a Ring Buffer (RB) exists in the Working Directory, mv it to the /HBA subdir and prefix it with In-WD_. Looking for RBs in the 
#            format of "IOC2_082415.122855.105962044.rb" as created in the trigger_ext.sh among other scripts.
#
# 08/27/15 - Added internal debug scripts, automatic collection of trigger output files and new -PT switch for lsiget output naming when used in a trigger script.
#
# 09/24/15 - Took out version test for smartmontools to run with MR.
#
# 12/04/15 - Took out erroneous OS_LSI=linux variable that was left over from debug.
#
# 12/20/15 - Generic housekeeping, fixed typo in external debug scripts readme, upgraded utility versions to latest. Added a number of new cx vx px show commands.
#
# 12/21/15 - Missed a utility update.
#
# 01/13/16 - Fixed -B switch, script exited if no Avago LSI products.
#
# 01/25/16 - Took out deletion of create.sh for internal use.
#
# 01/28/16 - Spell checked script. Put in -f check of /sys/block/sdX/queue data collection. Added additional lspci cmd line options.
#
# 02/12/16 - Uncommented coredump save for expander
#
# 02/25/16 - Added storcli /cx show termlog type=config
#
# 03/02/16 - Fixed error on Expander/12Gb/Details subdir create, introduced in 022516 version.
#
# 04/21/16 - Fixed MegaRAID smartctl output, worked at one point in time, not sure when the bug was introduced. Need to redo logic to make cleaner.
#
# 04/21/16 - Changed all " >> ./misc_output.txt 2>&1" instances to " >> ./misc_output.txt 2>&1". Changed all "2>>./$fileName/script_workspace/lsiget_errorlog.txt" instances
#	     to "2>> ./misc_output.txt". Changed all variations of piping to /dev/null to ./misc_output.txt.	
#
# 04/23/16 - Created seperate Internal and Internal with Scrutiny capture scripts. Deleted cversions_xxxx files and stopped separate version checking. 
#
# 04/25/16 - No longer sort MRMON AENs, now sort all controller AENs by type/controller. Added -UMS/-SMR/-S3W/-SHBA/-SEXP/-SC/-SRC/-SRS/-SDS/-F1/-F2/-F3/-F4/-EXT switches.
#            Still need -SCDA (Need to add CDA). Updated AEN list to 6.12.
#
# 05/18/16 - Delete lastlog from capture prior to compression.
#
# 05/19/16 - Dont copy /var/log/lastlog.
#
# 05/23/16 - Added PRERELEASE g3xu and scl for Cub support in the INTERNAL builds. Scrutiny GUI is bundled in the -IS build, doesnt currently work as is, need to investigate.
#
# 06/06/16 - Made check for arch before running g3xu.
#
# 06/07/16 - Added scli check for devices before exiting.
#
# 06/08/16 - Added if exist check for cub scli, only in internal version.
#
# 06/27/16 - Added additional serdes info to output file as well as ioc cli commands.
#
# 07/01/16 - Added back in a ls -latrR of /sys, it was commented out for some reason.
#
# 07/06/16 - Rebuilt on Centos, issues with building on OpenSuse
#
# 08/03/16 - Added 32bit version of lsut.
#
# 08/04/16 - Fixed -PT option, if -PT only get journalctl -a from time of test start. Added check for journalctl_time.txt file for only dumping the journal from the start of a test for triggered captured.
#
#set -x
# grep as variable, for Solaris - grep=/usr/xpg4/bin/grep for -xe and /usr/gnu/bin/grep for -A
#


todayDate=`date '+DATE:%m%d%y' | cut -d: -f2`
currentTime=`date '+TIME:%H%M%S' | cut -d: -f2`
currentYear=`date | cut -d' ' -f 6`
currentDayMonth=`date '+DATE:%m%d' | cut -d: -f 2`
currentDate=${currentYear}${currentDayMonth}${currentTime}

if [ -f ./misc_output.txt ] ; then mv -f ./misc_output.txt misc_output_${todayDate}.${currentTime}.txt > /dev/null 2>&1 ; fi

###########################################################################################################################
# Initialize Variables
###########################################################################################################################
Capture_Script_Version=080416
echo Capture_Script_Version_$Capture_Script_Version
TWGETLUNIXSTARTutc=`date -u`
TWGETLUNIXSTART=`date`
BASECMD=$0
OS_LSI=
grep=grep
CLEANED_UP=NO
VMWARE_SUPPORTED=NO
VMWARE_4x=
VMWARE_5x=
VMWARE_5x=
TWSKIPNETWORK=NO
TWGETSKIPCOMPRESSION=NO
TWGETSKIPCDA=NO

#3ware
TWPRINTFILENAME=
TWPRINTFILENAMETRIGGER=
TWGETDIRECTORYKEEP=
TWGETBATCHMODE=
TWGETPARTIALMODE=STANDARD
TWSETONLY=NO
TWGETMONITORMODE=
TW_Help_Screen=
TWPROMPTFORCOMMENT=NO
CLI_NAME=tw_cli
CLI_LOCATION=./
TWcomment=
tw_hostname=
tw_host=
OLD_EVT_CAP_EXIST=NO
NEW_EVT_CAP_EXIST=NO
TW_ECHO_AEC=
TW_ECHO_PMSTAT=
ex_enclosurenums=
TWPARTIALCAP=
TWGETSKIP3WARE=NO
NO_3Ware_Ctrls=YES

#MegaRAID
MCLI_NAME=
MCLI_LOCATION=./
num_mraid_adapters=
LimitMegaCliCMDs=
MR_ECHO_AEC=
MR_ECHO_PMSTAT=
MEGARAID_SASLOADED=NO
USEMEGACLISYNTAX=NO
TWGETSKIPMEGARAID=NO
TWGETSKIPRECORDCAPTURE=NO
TWGETSKIPRECORDSORT=NO
USINGSDSCLI=NO
SDSSTORCLI=NO
NO_MR_Adps=YES

#HBA
LSUT_NAME=
lsut_Bundled_work=NO
NO_scl_EXP_or_IOC=YES
NO_scl_EXP=YES
NO_scl_IOC=YES
NO_LSI_HBAs=YES
TWSKIPXUTILS=NO
MPT2SASLOADED=NO
MPT3SASLOADED=NO
rbcurrentTime=
TWGETSKIPHBA=NO
NO_lsut_LSI_HBAs=YES
NO_2ircu_HBAs=YES
NO_3ircu_HBAs=YES

#Expander
EXPANDER_INFO=NO
grepA=grep
TWGETSKIPEXPANDER=NO
NO_xu_GEN1_or_2_EXPs=YES
NO_g3xu_GEN3_EXPs=YES

###########################################################################################################################
# Set OS type - Need first, initially for correct grep version.
###########################################################################################################################

if [ `echo $OSTYPE | $grep -i linux` ] ; then
	OS_LSI=linux
fi

if [ `uname -s | grep VMkernel` ] ; then
	OS_LSI=vmware
fi

#Unique vmware versions



if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 6.0.0` ] ; then
		OS_LSI=vmware_6.0.0
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 6.` ] ; then
		OS_LSI=vmware_6.X_Not_Tested
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 5.5.0` ] ; then
		OS_LSI=vmware_5.5.0
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 5.1.0` ] ; then
		OS_LSI=vmware_5.1.0
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 4.1.0` ] ; then
		OS_LSI=vmware_4.1.0
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 5.` ] ; then
		OS_LSI=vmware_5.X_Not_Tested
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 4.` ] ; then
		OS_LSI=vmware_4.X_Not_Tested
	fi
fi

if [ "$OS_LSI" = "vmware" ] ; then
	if [ `uname -r | grep 3.` ] ; then
		OS_LSI=vmware_3.X_Not_Tested
	fi
fi

#Unique vmware options, ADD new supported versions
if [ "$OS_LSI" = "vmware_6.0.0" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_6.0.0" ] ; then VMWARE_6x=YES; fi
if [ "$OS_LSI" = "vmware_5.5.0" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_5.5.0" ] ; then VMWARE_5x=YES; fi
if [ "$OS_LSI" = "vmware_5.1.0" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_5.1.0" ] ; then VMWARE_5x=YES; fi
if [ "$OS_LSI" = "vmware_4.1.0" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_4.1.0" ] ; then VMWARE_4x=YES; fi
if [ "$OS_LSI" = "vmware_6.X_Not_Tested" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_6.X_Not_Tested" ] ; then VMWARE_6x=YES; fi
if [ "$OS_LSI" = "vmware_5.X_Not_Tested" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_5.X_Not_Tested" ] ; then VMWARE_5x=YES; fi
if [ "$OS_LSI" = "vmware_4.X_Not_Tested" ] ; then VMWARE_SUPPORTED=YES; fi
if [ "$OS_LSI" = "vmware_4.X_Not_Tested" ] ; then VMWARE_4x=YES; fi
if [ "$OS_LSI" = "vmware_3.X_Not_Tested" ] ; then VMWARE_SUPPORTED=YES; fi

if [ "$OS_LSI" = "vmware" ] ; then
	OS_LSI=vmware_unsupported_version
fi


if [ `echo $OSTYPE | $grep -i FreeBSD` ] ; then
	OS_LSI=freebsd
fi

if [ `echo $OSTYPE | $grep -i darwin` ] ; then
	OS_LSI=macos
fi

if [ "$OS_LSI" = "" ] ; then
	if [ `uname -s | $grep -i NIKOS` ] ; then
		OS_LSI=freebsd
		HOST=`uname -n`
	fi
fi

if [ "$OS_LSI" = "vmware_unsupported_version" ] ; then
	echo "Not a supported version of VMWare for lsiget yet, new feature"
	rm -f cleanup.txt
	exit 1
fi

if [ `uname | $grep -i SunOS` ] ; then
	OS_LSI=solaris

#Needed for -xe			
	if [ -f /usr/xpg4/bin/grep ] ; then 
		grep=/usr/xpg4/bin/grep
		else
		echo ""
		echo ""
		echo ""
		echo ""
		echo ""
		echo "This script requires /usr/xpg4/bin/grep that supports grep -xe to work correctly!"
		echo ""
		echo ""
		echo ""
		echo ""
		echo ""
		rm -f cleanup.txt
		exit 1
	fi
#Needed for -A or -o
	if [ -f /usr/gnu/bin/grep ] ; then 
		grepA=/usr/gnu/bin/grep
# This script should have /usr/gnu/bin/grep that supports grep -A but will run without it.
	fi
	if [ -f /usr/sfw/lib/libstdc++.so.6 ] ; then
		if [ -f /usr/sfw/lib/libgcc_s.so.1 ] ; then
			LD_LIBRARY_PATH=/usr/sfw/lib:$LD_LIBRARY_PATH
			export LD_LIBRARY_PATH
		fi	

	fi
	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		./storcli show >> ./misc_output.txt 2>&1
		if [ "$?" -eq "137" ] ; then
			echo ""
			echo ""
			echo ""
			echo ""
			echo ""
			echo "storcli requires libstdc++.so.6 & libgcc_s.so.1, they were not located in /usr/sfw/lib or in the LD_LIBRARY_PATH"
			echo ""
			echo ""
			echo ""
			echo ""
			echo ""
			rm -f cleanup.txt
			exit 1
		fi
	fi
fi

export OS_LSI

if [ "$OS_LSI" = "" ] ; then
	echo "OS_LSI is not set, either not a valid OS for this script or try running;"
	echo "sudo bash $BASECMD -D -Q"
	rm -f cleanup.txt
	exit 1
fi

#Cleanup
echo re_execute_variable_shell.txt > cleanup.txt
echo CtDbg.log >> cleanup.txt
echo MegaSAS.log >> cleanup.txt
echo CmdTool.log >> cleanup.txt
echo lsut >> cleanup.txt
echo MegaRAID_Terminology >> cleanup.txt
echo Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code >> cleanup.txt
echo freebsd_tw_cli.32 >> cleanup.txt
echo freebsd_tw_cli.64 >> cleanup.txt
echo lsut32 >> cleanup.txt
echo lsut64 >> cleanup.txt
echo linux_lsut.32 >> cleanup.txt
echo linux_lsut.64 >> cleanup.txt
echo linux_tw_cli.32 >> cleanup.txt
echo linux_tw_cli.64 >> cleanup.txt
echo macos_tw_cli.32 >> cleanup.txt
echo solaris_lsut.i386 >> cleanup.txt
echo solaris_storcli >> cleanup.txt
echo solaris_tw_cli.32 >> cleanup.txt
echo vmware_tw_cli.esxi >> cleanup.txt
echo linux_storcli64 >> cleanup.txt
echo linux_storcli >> cleanup.txt
echo sds_linux_storcli64 >> cleanup.txt
echo sds_linux_storcli >> cleanup.txt
echo linux_libstorelibir-2.so.14.07-0 >> cleanup.txt
echo solaris_storcli >> cleanup.txt
echo vmware_storcli >> cleanup.txt
echo vmware_libstorelib.so >> cleanup.txt
echo freebsd_storcli64 >> cleanup.txt
echo freebsd_storcli >> cleanup.txt
echo linux_xu >> cleanup.txt
echo linux_g3xu >> cleanup.txt
echo cub_linux_g3xu >> cleanup.txt
echo solaris_xu >> cleanup.txt
echo solaris_g3xu >> cleanup.txt
echo cub_solaris_g3xu >> cleanup.txt
echo freebsd_g3xu >> cleanup.txt
echo cub_freebsd_g3xu >> cleanup.txt
echo xu >> cleanup.txt
echo g3xu >> cleanup.txt
echo linux_mpt2sas_debug.h >> cleanup.txt
echo linux_mpt3sas_debug.h >> cleanup.txt
echo freebsd_sas2ircu.32 >> cleanup.txt
echo freebsd_sas2ircu.64 >> cleanup.txt
echo freebsd_sas3ircu.32 >> cleanup.txt
echo freebsd_sas3ircu.64 >> cleanup.txt
echo linux_sas2ircu.32 >> cleanup.txt
echo linux_sas3ircu.32 >> cleanup.txt
echo solaris_sas2ircu.32 >> cleanup.txt
echo solaris_sas3ircu.32 >> cleanup.txt
echo vmware-esx-sas2ircu.vib.esx50 >> cleanup.txt
echo vmware-esx-sas3ircu.vib.esx50 >> cleanup.txt
echo vmware-esx-sas3ircu.vib.esx55 >> cleanup.txt
echo create.sh >> cleanup.txt
echo precreate.sh >> cleanup.txt
echo Debug_Readme_int.txt >> cleanup.txt
echo Scrutiny.tar.gz >> cleanup.txt
echo linux_scl.64 >> cleanup.txt
echo linux_scl.32 >> cleanup.txt
echo cub_linux_scl >> cleanup.txt
echo cub_linux_scl.64 >> cleanup.txt
echo cub_linux_scl.32 >> cleanup.txt
echo scl >> cleanup.txt
echo freebsd_flashoem.64 >> cleanup.txt
echo freebsd_flashoem.32 >> cleanup.txt
echo linux_flashoem >> cleanup.txt
echo linux_check.sh >> cleanup.txt
echo linux_inbandcli.sh >> cleanup.txt
echo linux_kill_jobs.sh >> cleanup.txt
echo linux_reboot_loop_int.sh >> cleanup.txt
echo linux_reboot_loop_ext.sh >> cleanup.txt
echo linux_r.sh >> cleanup.txt
echo linux_rru.sh >> cleanup.txt
echo linux_trigger_int.sh >> cleanup.txt
echo linux_trigger_ext.sh >> cleanup.txt
echo linux_trigger_test.sh >> cleanup.txt
echo linux_s2sll.sh >> cleanup.txt
echo linux_s3sll.sh >> cleanup.txt
echo linux_Build_all_MR_driver_source.sh >> cleanup.txt
echo uart_login.ttl >> cleanup.txt
echo linux_HBA_Debug_Notes_int.txt >> cleanup.txt
echo linux_HBA_Debug_Notes_ext.txt >> cleanup.txt
echo tempfiles >> cleanup.txt
echo aens_mr.txt >> cleanup.txt

# 09/02/14 - Why is this here? After first iteration re_execute_variable_shell.txt wont exist anymore.... Don't remember use case.
#Shell work around for Solaris
#Checks in case script was exiting prematurely
#if [ "$OS_LSI" = "solaris" ] ; then
###########################################################################################################################
# Clean up
###########################################################################################################################
#
# Do NOT have re_execute_variable_shell.txt on this FIRST clean up function!
# MRR - Replace with new cleanup if ever reused.
#		if [ -f $i ] ; then
#			if [ -f re_execute_variable_shell.txt ] ; then  
#				rm -f re_execute_variable_shell.txt $i
#			fi
#		fi
#	done
#fi
#
#if [ "$OS_LSI" = "solaris" ] ; then
#	if [ ! -f re_execute_variable_shell.txt ] ; then  
#		if [ -f /usr/xpg4/bin/sh ] ; then 
#			date >> re_execute_variable_shell.txt
#			echo "exec /usr/xpg4/bin/sh $0 $*" >> re_execute_variable_shell.txt
#			exec /usr/xpg4/bin/sh $0 $* 
#		fi
#		if [ -f /bin/bash ] ; then 
#			date >> re_execute_variable_shell.txt
#			echo "exec /bin/bash $0 $*" >> re_execute_variable_shell.txt
#			exec /bin/bash $0 $*
#		fi
#	fi
#fi

###################################################
#Time Stamp
###################################################

if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
	date '+%H:%M:%S.%N'
fi
if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
	date '+%H:%M:%S'
fi

###################################################

GetKeystroke () { 
# no read -r in sh
trap "" 2 3 
oldSttySettings=`stty -g` 
stty -echo raw 
echo "`dd count=1 2>> ./misc_output.txt`" 
stty $oldSttySettings 
trap 2 3 
} 

WaitContinueOrQuit () { 
# no read -r in sh
keyStroke="" 
while [ "$keyStroke" != "C" ] && [ "$keyStroke" != "c" ]
do 
if [ "$keyStroke" = "Q" ] || [ "$keyStroke" = "q" ] ; then
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N'
	fi
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S'
	fi
	rm -f cleanup.txt
	exit 1
fi

echo "Type C to continue or Q to Quit." 
echo "...................................................................................................."
keyStroke=`GetKeystroke` 
done
}

WaitQuit () { 
# no read -r in sh
keyStroke="" 
while [ 1 ]
do 
if [ "$keyStroke" = "Q" ] || [ "$keyStroke" = "q" ] ; then
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N'
	fi
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S'
	fi
rm -f cleanup.txt
exit 1
fi

echo "Type Q to Quit." 
echo "...................................................................................................."
keyStroke=`GetKeystroke` 
done
}


###########################################################################################################################
# Set all_cli variable
###########################################################################################################################

all_cli=Missing

if [ -f all_cli ] ; then all_cli=all_cli ; fi
if [ "$all_cli" = "all_cli" ] ; then
	if [ -f all_clii ] ; then 
		echo "Both all_cli and all_clii exist..." >> ./misc_output.txt 2>&1 
		echo "Not a valid Capture Script..." >> ./misc_output.txt 2>&1 
		echo "exit" >> ./misc_output.txt 2>&1 
		echo "Both all_cli and all_clii exist..."
		echo "Not a valid Capture Script..."
		rm -f cleanup.txt
		exit 1
	fi
	if [ -f all_cliis ] ; then 
		echo "Both all_cli and all_cliis exist..." >> ./misc_output.txt 2>&1 
		echo "Not a valid Capture Script..." >> ./misc_output.txt 2>&1 
		echo "exit" >> ./misc_output.txt 2>&1 
		echo "Both all_cli and all_cliis exist..."
		echo "Not a valid Capture Script..."
		rm -f cleanup.txt
		exit 1
	fi
fi
if [ -f all_clii ] ; then all_cli=all_clii ; fi
if [ "$all_cli" = "all_clii" ] ; then
	if [ -f all_cliis ] ; then 
		echo "Both all_clii and all_cliis exist..." >> ./misc_output.txt 2>&1 
		echo "Not a valid Capture Script..." >> ./misc_output.txt 2>&1 
		echo "exit" >> ./misc_output.txt 2>&1 
		echo "Both all_clii and all_cliis exist..."
		echo "Not a valid Capture Script..."
		rm -f cleanup.txt
		exit 1
	fi
fi

if [ -f all_cli ] ; then all_cli=all_cli ; fi
if [ -f all_clii ] ; then all_cli=all_clii ; fi
if [ -f all_cliis ] ; then all_cli=all_cliis ; fi

 
###########################################################################################################################
# Verify all capture script files exist or exit.
###########################################################################################################################


if [ "$all_cli" = "Missing" ] ; then 

	echo "all_cli is missing"
	#cho ".................................................||................................................."
	#echo "" 
	echo "\"$BASECMD -H\" provides a help screen."
	echo "" 
	echo "This is not a valid lsigetlunix.sh installation. This script is available at;"
	echo "" 
	echo "http://mycusthelp.info/LSI/_cs/AnswerDetail.aspx?inc=8264"
	echo "" 
	echo "All files included in the original lsigetlunix_xxx_xxxxxx.tgz file MUST be kept"
	echo "in the same subdir as lsigetlunix.sh."
	echo "" 
	echo "" 
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N'
	fi
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S'
	fi
	rm -f cleanup.txt
	exit 1
fi
	     


###########################################################################################################################
# Command Line Options
# Looking for comments that are not command line options.
# Only 1 comment allowed, lowest variable # wins.
###########################################################################################################################

#-e vs -xe may not make a difference but need to verify on all other OS's, will verify eventually.
#Don't remember why I switched from -e to -xe, may be a corner case.

if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
	if [ "$6" != "" ] ; then 
		echo $6 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$6"
			fi
	fi
	if [ "$5" != "" ] ; then 
		echo $5 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$5"
			fi
	fi
	if [ "$4" != "" ] ; then 
		echo $4 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$4"
			fi
	fi
	if [ "$3" != "" ] ; then 
		echo $3 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$3"
			fi
	fi
	if [ "$2" != "" ] ; then 
		echo $2 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$2"
			fi
	fi
	if [ "$1" != "" ] ; then 
		echo $1 | $grep -i -xe -EXT -xe -M -xe -P -xe -PT -xe -D -xe -B -xe -Q -xe -F1 -xe -F2 -xe -F3 -xe -F4 -xe -UMS -xe -SRC -xe -SRS -xe -SCDA -xe -SMR -xe -S3W -xe -SHBA -xe -SEXP -xe -SC -xe -E_AEC -xe -E_DPMSTAT -xe -E_AEC_DPMSTAT -xe -G_AEC -xe -G_DPMSTAT -xe -G_AEC_DPMSTAT -xe -MRWA -xe -H -xe -E_SAS2LOG -xe -E_SAS2LOG1 -xe -D_SAS2LOG -xe -E_SAS2LOG_RB -xe -E_SAS2LOG1_RB -xe -E_SAS3LOG -xe -E_SAS3LOG1 -xe -D_SAS3LOG -xe -E_SAS3LOG_RB -xe -E_SAS3LOG1_RB -xe -E_RB -xe -SX -xe -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$1"
			fi
	fi
fi


if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
	if [ "$6" != "" ] ; then 
		echo $6 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$6"
			fi
	fi
	if [ "$5" != "" ] ; then 
		echo $5 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$5"
			fi
	fi
	if [ "$4" != "" ] ; then 
		echo $4 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$4"
			fi
	fi
	if [ "$3" != "" ] ; then 
		echo $3 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$3"
			fi
	fi
	if [ "$2" != "" ] ; then 
		echo $2 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$2"
			fi
	fi
	if [ "$1" != "" ] ; then 
		echo $1 | $grep -i -e -EXT -e -M -e -P -e -PT -e -D -e -B -e -Q -e -F1 -e -F2 -e -F3 -e -F4 -e -UMS -e -SRC -e -SRS -e -SCDA -e -SMR -e -S3W -e -SHBA -e -SEXP -e -SC -e -E_AEC -e -E_DPMSTAT -e -E_AEC_DPMSTAT -e -G_AEC -e -G_DPMSTAT -e -G_AEC_DPMSTAT -e -MRWA -e -H -e -E_SAS2LOG -e -E_SAS2LOG1 -e -D_SAS2LOG -e -E_SAS2LOG_RB -e -E_SAS2LOG1_RB -e -E_SAS3LOG -e -E_SAS3LOG1 -e -D_SAS3LOG -e -E_SAS3LOG_RB -e -E_SAS3LOG1_RB -e -E_RB -e -SX -e -SN >> ./misc_output.txt 2>&1
			if [ $? -ne 0 ] ; then TWcomment="$1"
			fi
	fi
fi


# Checking command line variables for options.
# Competing options are not allowed.

#Pulled out TWGETPARTIALMODE for x_SASxLOGx

for i in $* ; do
	  if [ "$i" = "-P" ] || [ "$i" = "-p" ] ; then TWPRINTFILENAME=YES ; fi
	  if [ "$i" = "-PT" ] || [ "$i" = "-pt" ] ; then TWPRINTFILENAMETRIGGER=YES ; fi
	  if [ "$i" = "-D" ] || [ "$i" = "-d" ] ; then TWGETDIRECTORYKEEP=YES ; fi
	  if [ "$i" = "-Q" ] || [ "$i" = "-q" ] ; then TWGETBATCHMODE=QUIET ; fi
	  if [ "$i" = "-B" ] || [ "$i" = "-b" ] ; then TWGETBATCHMODE=BATCH ; fi
	  if [ "$i" = "-UMS" ] || [ "$i" = "-ums" ] ; then USEMEGACLISYNTAX=YES ; fi
	  if [ "$i" = "-SMR" ] || [ "$i" = "-smr" ] ; then TWGETSKIPMEGARAID=YES ; fi
	  if [ "$i" = "-S3W" ] || [ "$i" = "-s3w" ] ; then TWGETSKIP3WARE=YES ; fi
	  if [ "$i" = "-SHBA" ] || [ "$i" = "-shba" ] ; then TWGETSKIPHBA=YES ; fi
	  if [ "$i" = "-SEXP" ] || [ "$i" = "-sexp" ] ; then TWGETSKIPEXPANDER=YES ; fi
	  if [ "$i" = "-SCDA" ] || [ "$i" = "-scda" ] ; then TWGETSKIPCDA=YES ; fi
	  if [ "$i" = "-SC" ] || [ "$i" = "-sc" ] ; then TWGETSKIPCOMPRESSION=YES ; fi
	  if [ "$i" = "-SRC" ] || [ "$i" = "-src" ] ; then TWGETSKIPRECORDCAPTURE=YES TWGETSKIPRECORDSORT=YES ; fi
	  if [ "$i" = "-SRS" ] || [ "$i" = "-srs" ] ; then TWGETSKIPRECORDSORT=YES ; fi
	  if [ "$i" = "-SDS" ] || [ "$i" = "-sds" ] ; then SDSSTORCLI=YES ; fi
	  if [ "$i" = "-F1" ] || [ "$i" = "-f1" ] ; then TWGETSKIPRECORDCAPTURE=YES TWGETSKIPRECORDSORT=YES ; fi
	  if [ "$i" = "-F2" ] || [ "$i" = "-f2" ] ; then TWGETSKIPRECORDCAPTURE=YES TWGETSKIPRECORDSORT=YES TWGETSKIPCDA=YES ; fi
	  if [ "$i" = "-F3" ] || [ "$i" = "-f3" ] ; then TWGETSKIPRECORDCAPTURE=YES TWGETSKIPRECORDSORT=YES TWGETSKIPCDA=YES ; fi
	  if [ "$i" = "-F4" ] || [ "$i" = "-f4" ] ; then TWGETSKIPRECORDCAPTURE=YES TWGETSKIPRECORDSORT=YES TWGETSKIPCDA=YES TWGETSKIPCOMPRESSION=YES ; fi
	  if [ "$i" = "-EXT" ] || [ "$i" = "-ext" ] ; then TWSETONLY=YES TWGETPARTIALMODE=EXTRACT EXTRACTUTILS=YES ; fi
	  if [ "$i" = "-E_AEC" ] || [ "$i" = "-e_aec" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_AEC ; fi
	  if [ "$i" = "-E_DPMSTAT" ] || [ "$i" = "-e_dpmstat" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_DPMSTAT ; fi
	  if [ "$i" = "-E_AEC_DPMSTAT" ] || [ "$i" = "-e_aec_dpmstat" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_AEC_DPMSTAT ; fi
	  if [ "$i" = "-E_SAS2LOG" ] || [ "$i" = "-e_sas2log" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS2LOG ; fi
	  if [ "$i" = "-E_SAS2LOG1" ] || [ "$i" = "-e_sas2log1" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS2LOG1 ; fi
	  if [ "$i" = "-D_SAS2LOG" ] || [ "$i" = "-d_sas2log" ] ; then TWSETONLY=YES TWGETPARTIALMODE=D_SAS2LOG ; fi
	  if [ "$i" = "-E_SAS2LOG_RB" ] || [ "$i" = "-e_sas2log_rb" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS2LOG_RB ; fi
	  if [ "$i" = "-E_SAS2LOG1_RB" ] || [ "$i" = "-e_sas2log1_rb" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS2LOG1_RB ; fi
	  if [ "$i" = "-E_SAS3LOG" ] || [ "$i" = "-e_sas3log" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS3LOG ; fi
	  if [ "$i" = "-E_SAS3LOG1" ] || [ "$i" = "-e_sas3log1" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS3LOG1 ; fi
	  if [ "$i" = "-D_SAS3LOG" ] || [ "$i" = "-d_sas3log" ] ; then TWSETONLY=YES TWGETPARTIALMODE=D_SAS3LOG ; fi
	  if [ "$i" = "-E_SAS3LOG_RB" ] || [ "$i" = "-e_sas3log_rb" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS3LOG_RB ; fi
	  if [ "$i" = "-E_SAS3LOG1_RB" ] || [ "$i" = "-e_sas3log1_rb" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_SAS3LOG1_RB ; fi
	  if [ "$i" = "-E_RB" ] || [ "$i" = "-e_rb" ] ; then TWSETONLY=YES TWGETPARTIALMODE=E_RB ; fi
	  if [ "$i" = "-G_AEC" ] || [ "$i" = "-g_aec" ] ; then TWGETPARTIALMODE=G_AEC ; fi
	  if [ "$i" = "-G_DPMSTAT" ] || [ "$i" = "-g_dpmstat" ] ; then TWGETPARTIALMODE=G_DPMSTAT ; fi
	  if [ "$i" = "-G_AEC_DPMSTAT" ] || [ "$i" = "-g_aec_dpmstat" ] ; then TWGETPARTIALMODE=G_AEC_DPMSTAT ; fi
	  if [ "$i" = "-M" ] || [ "$i" = "-m" ] ; then TWGETMONITORMODE=MONITOR ; fi
	  if [ "$i" = "-MRWA" ] || [ "$i" = "-mrwa" ] ; then LimitMegaCliCMDs=YES ; fi
	  if [ "$i" = "-SX" ] || [ "$i" = "-sx" ] ; then TWSKIPXUTILS=YES ; fi
	  if [ "$i" = "-SN" ] || [ "$i" = "-sn" ] ; then TWSKIPNETWORK=YES ; fi
	  if [ "$i" = "-H" ] || [ "$i" = "-h" ] ; then TW_Help_Screen=YES ; fi
done

export EXTRACTUTILS

###########################################################################################################################
# Option Override
###########################################################################################################################

if [ "$TWGETMONITORMODE" = "MONITOR" ] ; then TWGETPARTIALMODE=STANDARD ; fi


# MRR 06/03/15 - Commented out override for no comment, Don't see reason for exclusion, put in by someone else.
#
#if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
#	if [ "$TWGETBATCHMODE" = "" ] ; then
#	   # solaris 10 doesn't support -u option
#	   if [ `id | awk '{print $1}'` = "uid=0(root)" ] ; then
#	      TWGETBATCHMODE=QUIET
#	   fi
#	fi
#fi
###########################################################################################################################
# Bypass Comment prompt
###########################################################################################################################

if [ "$TWGETPARTIALMODE" = "EXTRACT" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_AEC" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_DPMSTAT" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_AEC_DPMSTAT" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS2LOG" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS2LOG1" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "D_SAS2LOG" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS2LOG_RB" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS2LOG1_RB" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS3LOG" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS3LOG1" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "D_SAS3LOG" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS3LOG_RB" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_SAS3LOG1_RB" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "E_RB" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETBATCHMODE" = "BATCH" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETBATCHMODE" = "QUIET" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWcomment" != "" ] ; then TWPROMPTFORCOMMENT=NO ; fi
if [ "$TWGETPARTIALMODE" = "G_AEC_DPMSTAT" ] ; then TWPARTIALCAP=YES ; fi
if [ "$TWGETPARTIALMODE" = "G_AEC" ] ; then TWPARTIALCAP=YES ; fi
if [ "$TWGETPARTIALMODE" = "G_DPMSTAT" ] ; then TWPARTIALCAP=YES ; fi

if [ "$TWGETDIRECTORYKEEP" != "YES" ] ; then
	if [ "$TWGETSKIPCOMPRESSION" = "YES" ] ; then
		echo .
		echo "You can't skip compression without keeping the directory..."
		echo "The -D parameter has been enabled..."
		TWGETDIRECTORYKEEP=YES
	fi
fi
		

###########################################################################################################################
# Help Screen
###########################################################################################################################

if [ "$TW_Help_Screen" = "YES" ] ; then

	#cho ".................................................||................................................."
	echo "" 
	echo "LSI HBA/MegaRAID/Expander/3ware Data collection script for Linux, FreeBSD, Solaris X86 and VMWare." 
	echo "This script will collect system logs and info as well as controller, disk and"
	echo "enclosure info for debugging purposes. All files included in the original" 
	echo "lsigetlunix_xxxxxx.tgz file MUST be kept in the same subdir as lsigetlunix.sh."
	echo "You MUST have root access rights to run this script, su/sudo/root$MCLI_LOCATION$MCLI_NAME. The"
	echo "latest version of this script as well as information on what data can be collected manually"
	echo "can be found at;"
	echo ""
	echo "http://www.avagotech.com/support/knowledgebase/1211161499563/lsiget-data-capture-script"
	echo ""
	echo "OR"
	echo ""
	echo "ftp0.lsil.com"
	echo "User:tsupport"
	echo "Password:tsupport"
	echo "/outgoing_perm/CaptureScripts  (Sometimes newer scripts than KB article)"
	echo "/outgoing_perm/CaptureScripts/BETA  (Latest and greatest)"
	echo ""
	echo "To automatically get the latest script you can download the following file & grep for the current"
	echo "latest file. This ensures support will always have access to the latest data to speed up the support"
	echo "process." 
	echo ""
	echo "Example;"
	echo ""
	echo "/outgoing_perm/CaptureScripts/Latest_Script_Versions.txt"
	echo "#Used for automated remote script updates"
	echo "LatestFreebsd#lsigetfreebsd_062012.tgz"
	echo "LatestLinux#lsigetlinux_062012.tgz"
	echo "LatestLunix#lsigetlunix_062012.tgz"
	echo "LatestMacOS#lsigetmacos_062012.tgz"
	echo "LatestSolaris#lsigetsolaris_062012.tgz"
	echo "LatestWin#lsigetwin_062012.tgz"
	echo ""
	echo "This script is being packaged for all supported linux/Unix based OS's"
	echo "together as well as individually for each OS with different bundled" 
	echo "utilities. The exact same script is used in all cases, this is being done" 
	echo "to cut down on the size of the full .tgz file."
	echo ""
	echo "	lsigetlunix_xxxxxx.tgz   - Linux/Unix - FreeBSD/Linux/Solaris/VMWare"
	echo "	lsigetfreebsd_xxxxxx.tgz - FreeBSD"
	echo "	lsigetlinux_xxxxxx.tgz   - Linux"   
	echo "	lsigetmacos_xxxxxx.tgz   - MacOS (MacOS - Not currently tested, code not removed though)"
	echo "	lsigetsolaris_xxxxxx.tgz - Solaris"
	echo "	lsigetvmware_xxxxxx.tgz  - VMWare"
	echo ""
	echo "Command Line Options:"
	echo "lsigetwin [Comment] [Option(s)]"		
	echo "Comment: Enclose noncontiguous strings in double quotes "My Comments"" 
	echo "Option:" 
	echo "-UMS		= Use MegaCli Syntax in addition to native storcli syntax."
	echo "-EXT		= EXTRACT - Extracts all available utilities for use."
	echo "-P		= PRINT filename in .\LSICAPTUREFILES.TXT for scripting."
	echo "-PT            	= Append the PRINTED filename of the lsiget output file with TRIGGERED, used to differentiate automated captures."
	echo "-D		= Working DIRECTORY is not deleted."
	echo "-Q		= QUIET Mode - No keystrokes required unless error."
	echo "-B		= BATCH Mode - No keystrokes required."
	echo "-E_RB          	= Clear and ENABLE 3MB Ring Buffer for all SAS HBA's. Captured automatically on next lsiget."
	echo "-E_SAS2LOG     	= ENABLE ALL SAS2 HBA Driver Logging, Option ffffffh. HUGE /var/log/messages file size! (Linux Only)"
	echo "-E_SAS2LOG1    	= ENABLE some SAS2 HBA Driver Logging, Option 3f8h. Very Large /var/log/messages file size! (Linux Only)"
	echo "-E_SAS3LOG     	= ENABLE ALL SAS3 HBA Driver Logging, Option ffffffh. HUGE /var/log/messages file size! (Linux Only)"
	echo "-E_SAS3LOG1    	= ENABLE some SAS3 HBA Driver Logging, Option 3f8h. Very Large /var/log/messages file size! (Linux Only)"
	echo "-E_SAS2LOG_RB  	= ENABLE ALL SAS2 HBA Driver Logging and Clear and ENABLE 3MB Ring Buffer for all SAS HBA's. (Linux Only)"
	echo "-E_SAS2LOG1_RB 	= ENABLE 3f8h SAS2 HBA Driver Logging and Clear and ENABLE 3MB Ring Buffer for all SAS HBA's. (Linux Only)"
	echo "-E_SAS3LOG_RB  	= ENABLE ALL SAS3 HBA Driver Logging and Clear and ENABLE 3MB Ring Buffer for all SAS HBA's. (Linux Only)"
	echo "-E_SAS3LOG1_RB 	= ENABLE 3f8h SAS3 HBA Driver Logging and Clear and ENABLE 3MB Ring Buffer for all SAS HBA's. (Linux Only)"
	echo "-D_SAS2LOG     	= Disables ALL SAS2 HBA Driver Logging, Option 000000h (Default). (Linux Only)"
	echo "-D_SAS3LOG     	= Disables ALL SAS3 HBA Driver Logging, Option 000000h (Default). (Linux Only)"
	echo "-E_AEC		= Clear and ENABLE AEC."
	echo "-E_DPM		= Clear and ENABLE DPMSTAT."
	echo "-E_AEC_DPM	= Clear and ENABLE AEC and DPMSTAT."
	echo "-G_AEC		= Disable and GET AEC Logs."
	echo "-G_DPM		= Disable and GET DPMSTAT Logs."
	echo "-G_AEC_DPM	= Disable and GET AEC and DPMSTAT Logs."
	echo "-M		= MONITOR Mode - Standard capture, daily/targeted logging."
	echo "-SDS		= Use the BETA storcli 2.03.00 for SDS support."
	echo "-MRWA          	= MegaRAID Work Around - Limit commands for compatibility issues with old code"
	echo "-H		= This Help Screen."
	echo ""
	echo "The following command line options are NOT standard and may be detrimental" 
	echo "to a root cause analysis of your issue. Used for repetitive data collection"
	echo "and script debugging, these are fluid and may change."
	echo ""
	echo "-SRS		= Skip MegaRAID AEN Record Sort, NOT standard."
	echo "-SRC		= Skip MegaRAID AEN Record Capture and Sort, NOT standard."
	echo "-SCDA (CDA not enabled yet) = Skip Controller Disk Association, NOT standard."
	echo "-SMR		= Skip MegaRAID DATA Capture, NOT standard."
	echo "-S3W		= Skip 3Ware DATA Capture, NOT standard."
	echo "-SHBA		= Skip HBA DATA Capture, NOT standard."
	echo "-SEXP		= Skip Expander DATA Capture, NOT standard."
	echo "-SC		= Skip Compression of output subdirectory, NOT standard."
	echo "-SX            	= Skip xutil and g3xutil for debugging purposes, dependent on the SG driver in linux." 	
	echo "-SN            	= Skip ifconfig and ipmitool lan print data capture, used for on site and MSM debugging but can be omitted." 
	echo "-F1		= FAST-1, -SRC, NOT standard (matches Windows, no -SWR switch)."
	echo "-F2		= FAST-2, combines -SRC/-SCDA, NOT standard."
	echo "-F3		= FAST-3, combines -SRC/-SCDA, NOT standard (matches Windows, no -SWR switch)."
	echo "-F4		= FAST-4, combines -SRC/-SCDA/-SC, NOT standard (matches Windows, no -SWR switch)."
	echo ""
	echo "Example $BASECMD -D -Q \"This is my comment\""
	echo "Runs the standard script leaving the working directory, without prompts" 
	echo "and leaves a comment."
	echo ""
	echo "Example $BASECMD -Q \"This is my comment\" -D -M"
	echo "Runs the standard script leaving the working directory, without prompts" 
	echo "and leaves a comment, once done the script stays resident in Monitor Mode."
	echo ""
	echo "Notes:" 
	echo "Send just the created .tar.gz file as is to your support rep."
	echo ""
	echo "AEC = Advanced Event Capture - Do NOT use AEC without being directed to by"
	echo "Technical Support or an FAE as it requires proprietary tools to decipher."
	echo ""
	echo "DPMSTAT = Disk Performance Monitoring Statistics - Captures performance"
	echo "related information on a controller/disk basis for the 3Ware 9650 or" 
	echo "the MegaRAID 9260/60 or later controllers."
	echo ""
	echo "-E_SASxLOG = This sets the HBA Driver logging level, -E_SAS2LOG is for all"
	echo "SAS2 HBAs and -E_SAS3LOG is for all SAS3 HBAs. This is VERY VERBOSE!"
	echo ""
	echo "All of the -E/D/G options can only be used alone."
	echo ""
	echo "All of the -G_* "GET" Options are done automatically if AEC or DPMSTAT was"
	echo 'enabled previously with the -E_* "ENABLE" options by default. The -G_*' 
	echo "options are meant for quick repetitive results without getting other system"
	echo "information. Normally you should run the standard $BASECMD file without"
	echo "a -G_* option to provide as much info as possible."
	echo ""
	echo "If there are competing comments the lowest variable number wins."
	echo "If there are contradictory options the lowest variable number with the"
	echo "option order listed in the help wins. Valid combinations would be;"
	echo "-D or -D with -B or -Q any -E_* or -G_* option by itself or in conjunction"
	echo "with a -D and -B/-Q option. -E_* is allowed with -D but has no effect as" 
	echo "there is no working directory created."
	echo ""
	echo "Monitor Mode = Runs the standard capture script and then remains resident"
	echo 'logging "show diag" approx. every 24 hours and also monitors for three' 
	echo "specific AEN's (Controller Reset/Degraded Array/Rebuild Started). If any"
	echo 'of these are encountered a final "show diag" will be done and the script' 
	echo "will finish normally. Use to capture the internal printlog/buf prior to"
	echo "the buffer being overwritten. Run a standard capture after Monitor Mode completes. (3ware Only)"
	echo ""
	echo "MRWA = MegaRAID Work Around - Limits the MegaCli(64) commands that are run."
	echo "MegaCli has been seen to hang in some cases when running the 92xx controllers with"
	echo "pre 4.1.1 FW and/or driver versions. Currently this switch bypasses the encinfo & adplilog"
	echo "parameters. Instead of using this switch it is recommended to upgrade your code as this"
	echo "work around is not always 100% effective. See the troubleshooting section for more information."
	echo ""
	echo "Trouble Shooting Script Issues -"
	echo ""
	echo "I. Ubuntu 9.04"
	echo "sudo $BASECMD -D -Q"
	echo "Tue Sep 22 17:06:32 PDT 2009"
	echo "export: 3: 22: bad variable name"
	echo ""
	echo "Run;"
	echo "sudo bash $BASECMD -D -Q"
	echo ""
	echo "II. Script hangs with MegaRAID Controller"
	echo ""
	echo "If you are positive the script is hung, CTRL-C the process, wait 3 minutes."
	echo "If the prompt doesn't come back kill the term window, do a ps -ea, note the"
	echo "# of any lsigetlunix.sh or MegaCli(64) processes. Do a kill -9 process-number"
	echo "for each process. If any can't be killed, wait 3 minutes, there is a 180 second"
	echo "timeout on MegaCli. Upgrade your driver/fw/capture script to the latest version and"
	echo "try again. If you cant upgrade or if you still have problems try the -MRWA switch."
	echo "If you still have problems manually zip the subdirectory structure and "
	echo "email it to your support rep."
	echo ""
	echo "III. Fails to run on Solaris - Error is "$BASECMD: test: argument expected""
	echo "Old version of Bourne shell is loaded by default, the following two shells were tried automatically"
	echo "/bin/sh was changed to /usr/xpg4/bin/sh and then /bin/bash"
	echo "depending on what is installed on the system, you can try others, i.e. csh/ksh or install a" 
	echo "supported shell..."
	echo ""
	echo "Recommended Code Set/Release Versions"
	echo ""
	echo "3ware-"
	echo ""
	echo "This script should run with any release after 7.6.0 on the 7k, 8k" 
	echo "and 9k family of controllers." 
	echo ""
	echo "MegaRAID -"
	echo ""
	echo "This script should run with any release after 3.1 on the 82xx, 83xx, 84xx, 87xx," 
	echo "88xx, 92xx and 93xx family of controllers." 
	echo ""
	echo "HBA -"
	echo ""
	echo "This script should run with any LSI HBA using an mpt based driver."
	echo ""
	echo "Capture Script Version: $Capture_Script_Version"
	echo ""
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N'
	fi
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S'
	fi
	rm -f cleanup.txt
	exit 0
fi
	

###########################################################################################################################
# Comment check
###########################################################################################################################

if [ "$TWPROMPTFORCOMMENT" != "NO" ] ; then

	#Update on Code Set Change
	#cho ".................................................||................................................."
	echo ""
	echo "\"$BASECMD -H\" provides a help screen."
	echo ""
	echo "To associate a user comment with this data capture session pass the comment as a variable."
	#echo "comment as a variable." 
	echo ""						  
	echo "Example;"
	echo "$BASECMD \"Case 123456-123456789 – performance problems after upgrade\""
	echo ""
	echo 'NOTE: Noncontiguous strings must be in double quotes "My Comments".' 	  
	echo ""
	echo ""
	 
	WaitContinueOrQuit

fi


###########################################################################################################################
# Set architecture, i.e. check to see if 64-bit or 32 bit
###########################################################################################################################

if [ `uname -m | $grep 64` ] ; then
	Arch32or64=64
	else
	Arch32or64=32
fi

###########################################################################################################################
# Clean up
###########################################################################################################################


for i in $( cat cleanup.txt ); do

	if [ -f ./$i ] ; then
		rm -f ./$i
	fi
done


CLEANED_UP=YES

###########################################################################################################################
# Unpack files - Gunzip used as tar -zxvf not supported on Solaris
###########################################################################################################################

if [ -f ./$all_cli ] ; then
	gunzip < $all_cli | tar xvf - >> ./misc_output.txt 2>&1
fi

CLEANED_UP=NO

if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	###########################################################################################################################
	# Rename appropriate 3ware cli based on OS type
	###########################################################################################################################
	
	if [ "$OS_LSI" = "linux" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f linux_tw_cli.64 tw_cli
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f linux_tw_cli.32 tw_cli
		fi
	fi
	
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f vmware_tw_cli.esxi tw_cli
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f vmware_tw_cli.esxi tw_cli
		fi
	fi
	
	if [ "$OS_LSI" = "freebsd" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f freebsd_tw_cli.64 tw_cli
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f freebsd_tw_cli.32 tw_cli
		fi
	fi
	
	# Only 32bit CLI as of 10.2.2.1
	if [ "$OS_LSI" = "macos" ] ; then
		mv -f macos_tw_cli.32 tw_cli
	fi
	
	if [ "$OS_LSI" = "solaris" ] ; then
		mv -f solaris_tw_cli.32 tw_cli
	fi
	
	###########################################################################################################################
	# Use the bundled tw_cli, if it doesn't work try the existing tw_cli.
	###########################################################################################################################
	
	./tw_cli help >> ./misc_output.txt 2>&1
	if [ "$?" -ne "0" ] ; then
		#Bundled tw_cli didn't execute or driver not loaded
		tw_cli_Bundled_work=NO
		else
		#Bundled tw_cli executed
		tw_cli_Bundled_work=YES
	fi
	
	if [ "$tw_cli_Bundled_work" = "YES" ] ; then
		./tw_cli show | $grep "No controller found" >> ./misc_output.txt 2>&1
		if [ "$?" -eq "0" ] ; then
			#Bundled tw_cli didn't work or driver not loaded
			tw_cli_Bundled_work=NO
			else
			#Bundled tw_cli worked
			tw_cli_Bundled_work=YES
			NO_3Ware_Ctrls=NO
		fi
	fi
	
	###########################################################################################################################
	# Check tw_cli existing version 
	###########################################################################################################################
	
	{ tw_cli help >> ./misc_output.txt 2>&1; } 2>> ./misc_output.txt
	if [ "$?" -eq "0" ] ; then
		#Existing tw_cli present
		tw_cli_Existing=YES
		else
		#Existing tw_cli not present
		tw_cli_Existing=NO
	fi
	
	if [ "$tw_cli_Existing" = "YES" ] ; then
		tw_cli show 2>&1 | $grep "No controller found" >> ./misc_output.txt 2>&1
		if [ "$?" -eq "0" ] ; then
			#Existing tw_cli didn't work or driver not loaded
			tw_cli_Existing_work=NO
			else
			#Existing tw_cli worked
			tw_cli_Existing_work=YES
		fi
	fi
	
	if [ "$tw_cli_Bundled_work" = "YES" ] ; then 
		CLI_LOCATION=./
		tw_cli_Functional=YES
		else
		if [ "$tw_cli_Existing_work" = "YES" ] ; then 
			CLI_LOCATION=
			tw_cli_Functional=YES
			else
			tw_cli_Functional=NO 
		fi
	fi
fi

if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	###########################################################################################################################
	# Rename appropriate MegaRAID cli based on OS type
	###########################################################################################################################

	#Temporary SDS BETA Cli - Linux Only
	if [ "$OS_LSI" = "linux" ] ; then
		mv -f linux_libstorelibir-2.so.14.07-0 libstorelibir-2.so.14.07-0
		if [ "$Arch32or64" = "64" ] ; then
			MCLI_NAME64=sds_linux_storcli64
			MCLI_NAME32=sds_linux_storcli 
			./$MCLI_NAME64 show | $grep SDS >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f sds_linux_storcli64 storcli64
				MCLI_NAME=storcli64
				#Bundled cli executed
				mcli_Bundled_work=YES
				else
				./$MCLI_NAME32 show | $grep SDS >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f sds_linux_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		if [ "$mcli_Bundled_work" = "YES" ] ; then USINGSDSCLI=YES ; fi
		fi
		if [ "$Arch32or64" = "32" ] ; then
			MCLI_NAME32=sds_linux_storcli 
			./$MCLI_NAME32 show | $grep SDS >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f sds_linux_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi
		if [ "$mcli_Bundled_work" = "YES" ] ; then USINGSDSCLI=YES ; fi			
		fi
	fi

	if [ "$USINGSDSCLI" = "YES" ] ; then
		echo .
		echo "SDS Beta storcli is being used..."
		echo "SDS Beta storcli is being used..." >> ./misc_output.txt 2>&1 
	fi
	
	if [ "$USINGSDSCLI" != "YES" ] ; then
		if [ "$OS_LSI" = "linux" ] ; then
			if [ -f linux_libstorelibir-2.so.14.07-0 ] ; then  mv -f linux_libstorelibir-2.so.14.07-0 libstorelibir-2.so.14.07-0 ; fi
			if [ "$Arch32or64" = "64" ] ; then
				MCLI_NAME64=linux_storcli64
				MCLI_NAME32=linux_storcli 
				./$MCLI_NAME64 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f linux_storcli64 storcli64
					MCLI_NAME=storcli64
					#Bundled cli executed
					mcli_Bundled_work=YES
					else
					./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
					if [ "$?" = "0" ] ; then
						mv -f linux_storcli storcli
						MCLI_NAME=storcli
						#Bundled cli executed
						mcli_Bundled_work=YES
					fi
				fi
				if [ "$MCLI_NAME" = " " ] ; then 
					#Bundled cli did not execute
					mcli_Bundled_work=NO
				fi			
			fi
			if [ "$Arch32or64" = "32" ] ; then
				MCLI_NAME32=linux_storcli 
				./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f linux_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
				if [ "$MCLI_NAME" = " " ] ; then 
					#Bundled cli did not execute
					mcli_Bundled_work=NO
				fi			
			fi
		fi
	fi

	#Temporary SDS BETA Cli - Linux Only
	if [ "$SDSSTORCLI" = "YES" ] ; then
		if [ "$OS_LSI" = "linux" ] ; then
			mv -f linux_libstorelibir-2.so.14.07-0 libstorelibir-2.so.14.07-0
			if [ "$Arch32or64" = "64" ] ; then
				MCLI_NAME64=sds_linux_storcli64
				MCLI_NAME32=sds_linux_storcli 
				./$MCLI_NAME64 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f sds_linux_storcli64 storcli64
					MCLI_NAME=storcli64
					#Bundled cli executed
					mcli_Bundled_work=YES
					else
					./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
					if [ "$?" = "0" ] ; then
						mv -f sds_linux_storcli storcli
						MCLI_NAME=storcli
						#Bundled cli executed
						mcli_Bundled_work=YES
					fi
				fi
				if [ "$MCLI_NAME" = " " ] ; then 
					#Bundled cli did not execute
					mcli_Bundled_work=NO
				fi			
			fi
			if [ "$Arch32or64" = "32" ] ; then
				MCLI_NAME32=sds_linux_storcli 
				./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f sds_linux_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
				if [ "$MCLI_NAME" = " " ] ; then 
					#Bundled cli did not execute
					mcli_Bundled_work=NO
				fi			
			fi
		fi
	fi
	
	if [ "$OS_LSI" = "freebsd" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			MCLI_NAME64=freebsd_storcli64
			MCLI_NAME32=freebsd_storcli 
			./$MCLI_NAME64 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f freebsd_storcli64 storcli64
				MCLI_NAME=storcli64
				#Bundled cli executed
				mcli_Bundled_work=YES
				else
				./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f freebsd_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
		if [ "$Arch32or64" = "32" ] ; then
			MCLI_NAME32=freebsd_storcli 
			./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f freebsd_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
	fi
	
	
	#Only 32bit currently
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		mv -f vmware_libstorelib.so libstorelib.so
		if [ "$Arch32or64" = "64" ] ; then
			MCLI_NAME64=vmware_storcli
			MCLI_NAME32=vmware_storcli 
			./$MCLI_NAME64 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f vmware_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
				else
				./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f vmware_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
		if [ "$Arch32or64" = "32" ] ; then
			MCLI_NAME32=vmware_storcli 
			./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f vmware_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
	fi
	
	#Only 32bit currently
	if [ "$OS_LSI" = "solaris" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			MCLI_NAME64=solaris_storcli
			MCLI_NAME32=solaris_storcli 
			./$MCLI_NAME64 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f solaris_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
				else
				./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
				if [ "$?" = "0" ] ; then
					mv -f solaris_storcli storcli
					MCLI_NAME=storcli
					#Bundled cli executed
					mcli_Bundled_work=YES
				fi
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
		if [ "$Arch32or64" = "32" ] ; then
			MCLI_NAME32=solaris_storcli 
			./$MCLI_NAME32 adpcount nolog | $grep "Controller Count:" >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mv -f solaris_storcli storcli
				MCLI_NAME=storcli
				#Bundled cli executed
				mcli_Bundled_work=YES
			fi
			if [ "$MCLI_NAME" = " " ] ; then 
				#Bundled cli did not execute
				mcli_Bundled_work=NO
			fi			
		fi
	fi
	
	###########################################################################################################################
	# Use the bundled MegaCli
	###########################################################################################################################
	if [ "$OS_LSI" != "macos" ] ; then
	
	
		if [ "$mcli_Bundled_work" = "YES" ] ; then
			./$MCLI_NAME adpcount nolog | $grep "Controller Count: 0." >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				mcli_Functional=NO
				else
				mcli_Functional=YES
				NO_MR_Adps=NO
			fi
		fi
	fi
fi


###########################################################################################################################
# Extract appropriate lsut util based on OS type
###########################################################################################################################

if [ "$OS_LSI" = "linux" ] ; then
	if [ "$Arch32or64" = "64" ] ; then
		LSUT_NAME64=linux_lsut.64
		LSUT_NAME32=linux_lsut.32
		./$LSUT_NAME64 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f linux_lsut.64 lsut64
			LSUT_NAME=lsut64
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
			else
			./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
			if [ "$?" = "0" ] ; then
				mv -f linux_lsut.32 lsut32
				LSUT_NAME=lsut32
				#Bundled cli executed
				lsut_Bundled_work=YES
				NO_LSI_HBAs=NO
				NO_lsut_LSI_HBAs=NO
			fi
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
	if [ "$Arch32or64" = "32" ] ; then
		LSUT_NAME32=linux_lsut.32
		./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f linux_lsut.32 lsut32
			LSUT_NAME=lsut32
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
fi

#Using 32bit linux lsut
if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
	if [ "$Arch32or64" = "64" ] ; then
		LSUT_NAME64=linux_lsut.32
		LSUT_NAME32=linux_lsut.32
		./$LSUT_NAME64 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f linux_lsut.32 lsut32
			LSUT_NAME=lsut32
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
			else
			./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
			if [ "$?" = "0" ] ; then
				mv -f linux_lsut.32 lsut32
				LSUT_NAME=lsut32
				#Bundled cli executed
				lsut_Bundled_work=YES
				NO_LSI_HBAs=NO
				NO_lsut_LSI_HBAs=NO
			fi
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
	if [ "$Arch32or64" = "32" ] ; then
		LSUT_NAME32=linux_lsut.32
		./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f linux_lsut.32 lsut32
			LSUT_NAME=lsut32
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
fi


if [ "$OS_LSI" = "solaris" ] ; then
	if [ "$Arch32or64" = "64" ] ; then
		LSUT_NAME64=solaris_lsut.i386
		LSUT_NAME32=solaris_lsut.i386
		./$LSUT_NAME64 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f solaris_lsut.i386 lsut32
			LSUT_NAME=lsut32
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
			else
			./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
			if [ "$?" = "0" ] ; then
				mv -f solaris_lsut.i386 lsut32
				LSUT_NAME=lsut32
				#Bundled cli executed
				lsut_Bundled_work=YES
				NO_LSI_HBAs=NO
				NO_lsut_LSI_HBAs=NO
			fi
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
	if [ "$Arch32or64" = "32" ] ; then
		LSUT_NAME32=solaris_lsut.i386
		./$LSUT_NAME32 0 2>> ./misc_output.txt | $grep "Chip Vendor" 1>> ./misc_output.txt 
		if [ "$?" = "0" ] ; then
			mv -f solaris_lsut.i386 lsut32
			LSUT_NAME=lsut32
			#Bundled cli executed
			lsut_Bundled_work=YES
			NO_LSI_HBAs=NO
			NO_lsut_LSI_HBAs=NO
		fi
		if [ "$LSUT_NAME" = " " ] ; then 
			#Bundled cli did not execute
			lsut_Bundled_work=NO
			NO_LSI_HBAs=YES
		fi			
	fi
fi


###########################################################################################################################
# Cub Support - Extract appropriate scl util based on OS type
###########################################################################################################################

if [ "$OS_LSI" = "linux" ] ; then
	if [ "$Arch32or64" = "64" ] ; then
		if [ -f cub_linux_scl.64 ] ; then cp cub_linux_scl.64 cub_linux_scl ; fi
		if [ -f cub_linux_scl ] ; then
			./cub_linux_scl --list | $grep SAS35x >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				#Bundled cli executed
				NO_scl_EXP_or_IOC=NO
				rm -f cub_linux_scl
				cp -f cub_linux_scl.64 linux_scl.64  >> ./misc_output.txt 2>&1
				echo .
				echo "Beta Cub scli is being used..."
				echo "Beta Cub scli is being used..." >> ./misc_output.txt 2>&1 
			fi
		fi
	fi
	if [ "$Arch32or64" = "32" ] ; then
		if [ -f cub_linux_scl.32 ] ; then cp cub_linux_scl.32 cub_linux_scl ; fi
		./cub_linux_scl --list | $grep SAS35x >> ./misc_output.txt 2>&1
		if [ -f cub_linux_scl ] ; then
			if [ "$?" = "0" ] ; then
				#Bundled cli executed
				NO_scl_EXP_or_IOC=NO
				rm -f cub_linux_scl
				cp -f cub_linux_scl.32 linux_scl.32  >> ./misc_output.txt 2>&1
				echo .
				echo "Beta Cub scli is being used..."
				echo "Beta Cub scli is being used..." >> ./misc_output.txt 2>&1 
			fi
		fi
	fi
fi



###########################################################################################################################
# Extract appropriate scl util based on OS type
###########################################################################################################################

if [ "$OS_LSI" = "linux" ] ; then
	if [ "$Arch32or64" = "64" ] ; then
		mv -f linux_scl.64 scl
		./scl --list >> ./misc_output.txt 2>&1
		if [ "$?" = "0" ] ; then
			#Bundled cli executed
			NO_scl_EXP_or_IOC=NO
		fi
	fi
	if [ "$Arch32or64" = "32" ] ; then
		mv -f linux_scl.32 scl
		./scl --list >> ./misc_output.txt 2>&1
		if [ "$?" = "0" ] ; then
			#Bundled cli executed
			NO_scl_EXP_or_IOC=NO
		fi
	fi
fi



###########################################################################################################################
# Cub Support - Extract appropriate xu/g3xu based on OS type
###########################################################################################################################

if [ "$OS_LSI" = "linux" ] ; then
	mv -f linux_xu xu
	if [ "$Arch32or64" = "64" ] ; then
		if [ -f cub_linux_g3xu ] ;  then 
			./cub_linux_g3xu -i get avail | $grep SAS_35X >> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then
				cp -f cub_linux_g3xu linux_g3xu >> ./misc_output.txt 2>&1
				echo .
				echo "Beta Cub g3xutil is being used..."
				echo "Beta Cub g3xutil is being used..." >> ./misc_output.txt 2>&1 
			fi
		fi
	fi
cp -f linux_g3xu g3xu >> ./misc_output.txt 2>&1 
fi

if [ "$OS_LSI" = "solaris" ] ; then
	mv -f solaris_xu xu
	if [ -f cub_solaris_g3xu ] ;  then 
		./cub_solaris_g3xu -i get avail | $grep SAS_35X >> ./misc_output.txt 2>&1
		if [ "$?" = "0" ] ; then
			cp -f cub_solaris_g3xu solaris_g3xu >> ./misc_output.txt 2>&1
			echo .
			echo "Beta Cub g3xutil is being used..."
			echo "Beta Cub g3xutil is being used..." >> ./misc_output.txt 2>&1 
		fi
	fi
cp -f solaris_g3xu g3xu >> ./misc_output.txt 2>&1 
fi

if [ "$OS_LSI" = "freebsd" ] ; then
	if [ -f cub_freebsd_g3xu ] ;  then 
		./cub_freebsd_g3xu -i get avail | $grep SAS_35X >> ./misc_output.txt 2>&1
		if [ "$?" = "0" ] ; then
			cp -f cub_freebsd_g3xu freebsd_g3xu >> ./misc_output.txt 2>&1
			echo .
			echo "Beta Cub g3xutil is being used..."
			echo "Beta Cub g3xutil is being used..." >> ./misc_output.txt 2>&1 
		fi
	fi
cp -f freebsd_g3xu g3xu >> ./misc_output.txt 2>&1 
fi



if [ "$TWGETSKIPHBA" != "YES" ] ; then
	###########################################################################################################################
	# Rename appropriate sas2ircu and sas3ircu based on OS type
	###########################################################################################################################
	
	if [ "$OS_LSI" = "linux" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f linux_sas2ircu.32 sas2ircu
			mv -f linux_sas3ircu.32 sas3ircu
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f linux_sas2ircu.32 sas2ircu
			mv -f linux_sas3ircu.32 sas3ircu
		fi
	fi
	
	if [ "$OS_LSI" = "freebsd" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f freebsd_sas2ircu.64 sas2ircu
			mv -f freebsd_sas3ircu.64 sas3ircu
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f freebsd_sas2ircu.32 sas2ircu
			mv -f freebsd_sas3ircu.32 sas3ircu
		fi
	fi
	
	if [ "$OS_LSI" = "solaris" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			mv -f solaris_sas2ircu.32 sas2ircu
			mv -f solaris_sas3ircu.32 sas3ircu
		fi
		if [ "$Arch32or64" = "32" ] ; then
			mv -f solaris_sas2ircu.32 sas2ircu
			mv -f solaris_sas3ircu.32 sas3ircu
		fi
	fi
	
	#Need to add VMWare support, vibs are part of the package.
fi

###########################################################################################################################
# IF lsut, tw_cli and MegaCli not functional but NOT batchmode
###########################################################################################################################

if [ "$TWGETBATCHMODE" != "BATCH" ] ; then	
	if [ "$tw_cli_Functional" = "NO" ] ; then
		if [ "$mcli_Functional" = "NO" ] ; then
			if [ "$NO_LSI_HBAs" = "YES" ] ; then
				if [ "$NO_scl_EXP_or_IOC" = "YES" ] ; then				
					### 
					#cho ".................................................||................................................."
					echo "...................................................................................................."
					echo "################## CLI incompatible or No 3ware/MegaRAID or HBAs in the system #####################"
					echo "################################################ OR ################################################"
					echo "################# You do not have root privileges which are required to run the CLI ################"
					echo ""
					echo "\"$BASECMD -H\" provides a help screen."
					echo ""
		
					###########################################################################################################################
					# Clean up
					###########################################################################################################################
		
					for i in $( cat cleanup.txt ); do
		
						if [ -f ./$i ] ; then
							rm -f ./$i
						fi
					done
					CLEANED_UP=YES
					rm -f cleanup.txt
					exit 1
				fi
			fi
		fi
	fi
fi


###########################################################################################################################
# Unpack Debug scripts - OS Dependent
###########################################################################################################################

if [ ! -f tempfiles ] ; then 
	if [ "$EXTRACTUTILS" = "YES" ]	; then 
		echo ""
		echo "Utilities extracted..."
		echo ""
	fi
fi	

if [ -f tempfiles ] ; then 
	mv -f tempfiles tempfiles.sh >> ./misc_output.txt 2>&1
	./tempfiles.sh
	rm -rf tempfiles.sh >> ./misc_output.txt 2>&1
fi	
	


if [ "$OS_LSI" = "linux" ] ; then
	if [ -f all_cli ] ; then
		echo "" > Debug_Readme_ext.txt
		echo "8/27/15" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "These scripts/Notes/Utilities are intended for generic distribution and are available via the -ext parameter." >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "WARNING - Most of these debug scripts need to be reviewed/modified to be fine tuned for your specific environment, USE AT YOUR OWN RISK!" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "s2sll.sh - " >> Debug_Readme_ext.txt
		echo "Set the mpt2sas (6Gb SAS 2) driver logging level - see mpt2sas_debug.h" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "s3sll.sh - " >> Debug_Readme_ext.txt
		echo "Set the mpt3sas (12Gb SAS 3) driver logging level - see mpt3sas_debug.h" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "linux_HBA_Debug_Notes_ext.txt - " >> Debug_Readme_ext.txt
		echo "Various Linux HBA debug tips and tricks" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code.txt - " >> Debug_Readme_ext.txt
		echo "Decode SCSI and ATA command and error codes" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "MegaRAID_Terminology.txt - " >> Debug_Readme_ext.txt
		echo "MegaRAID Terminology decoded/defined" >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "Build_all_MegaRAID_driver_source.sh - " >> Debug_Readme_ext.txt
		echo "Script to build all MegaRAID Linux driver variants as well as Tutorial on the different methods." >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
		echo "trigger_test.sh - " >> Debug_Readme_ext.txt
		echo "A script to test either the trigger_ext.sh or trigger_int.sh files." >> Debug_Readme_ext.txt
		echo "" >> Debug_Readme_ext.txt
	fi
fi

#For Notes/Scripts/Utils
for i in Notes Scripts Utils ; do
	if [ ! -d $i ] ; then mkdir $i >> ./misc_output.txt 2>&1 ; fi
done

if [ "$OS_LSI" = "linux" ] ; then
	#External		
	if [ "$all_cli" = "all_cli" ] ; then
		cp ./linux_HBA_Debug_Notes_ext.txt ./Notes/HBA_Debug_Notes_ext.txt >> ./misc_output.txt 2>&1
		cp ./linux_mpt2sas_debug.h ./Notes/mpt2sas_debug.h >> ./misc_output.txt 2>&1
		cp ./linux_mpt3sas_debug.h ./Notes/mpt3sas_debug.h >> ./misc_output.txt 2>&1
		mv -f ./Debug_Readme_ext.txt ./Scripts/Debug_Readme_ext.txt >> ./misc_output.txt 2>&1
		cp ./uart_login.ttl ./Scripts/uart_login.ttl >> ./misc_output.txt 2>&1

		for i in reboot_loop_ext.sh trigger_ext.sh trigger_test.sh s2sll.sh s3sll.sh Build_all_MR_driver_source.sh ; do
			cp ./linux_$i ./Scripts/$i >> ./misc_output.txt 2>&1
		done
	fi
	#Internal
	if [ "$all_cli" != "all_cli" ] ; then
		cp ./linux_HBA_Debug_Notes_int.txt ./Notes/HBA_Debug_Notes_int.txt >> ./misc_output.txt 2>&1
		cp ./linux_mpt2sas_debug.h ./Notes/mpt2sas_debug.h >> ./misc_output.txt 2>&1
		cp ./linux_mpt3sas_debug.h ./Notes/mpt3sas_debug.h >> ./misc_output.txt 2>&1
		cp ./uart_login.ttl ./Scripts/uart_login.ttl >> ./misc_output.txt 2>&1

		for i in check.sh inbandcli.sh kill_jobs.sh reboot_loop_int.sh r.sh rru.sh trigger_int.sh trigger_test.sh s2sll.sh s3sll.sh Build_all_MR_driver_source.sh ; do
			cp ./linux_$i ./Scripts/$i >> ./misc_output.txt 2>&1
		done
	fi
fi


if [ "$EXTRACTUTILS" = "YES" ] ; then
	if [ -f all_cli ] ; then
		echo ""
		echo "All Utilities extracted..."
		echo ""
	fi
###########################################################################################################################
# Clean up
###########################################################################################################################


	for i in $( cat cleanup.txt ); do

		if [ -f ./$i ] ; then
			rm -f ./$i
		fi
	done
	
	CLEANED_UP=YES
	rm -f cleanup.txt	
	exit 0
fi


###########################################################################################################################
# Create unique file/subdirectory name
###########################################################################################################################


if [ "$OS_LSI" = "linux" ] ; then tw_host=$HOSTNAME; fi
if [ "$VMWARE_SUPPORTED" = "YES" ] ; then tw_host=`hostname`; fi
if [ "$OS_LSI" = "freebsd" ] ; then tw_host=$HOST; fi
if [ "$OS_LSI" = "macos" ] ; then tw_host=$HOSTNAME; fi
if [ "$OS_LSI" = "solaris" ] ; then tw_host=`hostname`; fi

if [ "$TWGETPARTIALMODE" = "STANDARD" ] ; then TWdescriptor=lsi; fi
if [ "$TWGETPARTIALMODE" = "G_AEC" ] ; then TWdescriptor=AEC; fi
if [ "$TWGETPARTIALMODE" = "G_DPMSTAT" ] ; then TWdescriptor=DPMSTAT; fi
if [ "$TWGETPARTIALMODE" = "G_AEC_DPMSTAT" ] ; then TWdescriptor=AEC_DPMSTAT; fi
if [ "$TWGETMONITORMODE" = "MONITOR" ] ; then TWdescriptor=MONITOR; fi
if [ "$TWPRINTFILENAMETRIGGER" = "YES" ] ; then TWdescriptor=TRIGGERED; fi

tw_hostname=`echo $tw_host | cut -d. -f1`

fileName=$TWdescriptor.$OS_LSI.$tw_hostname.$todayDate.$currentTime

###########################################
# 3W/MR/HBA - Not required for Enable Feature switches
###########################################
if [ "$TWSETONLY" = "NO" ] ; then
	mkdir $fileName
	mkdir $fileName/script_workspace
	mkdir $fileName/LSI_Products
	mkdir $fileName/LSI_Products/3ware
	mkdir $fileName/LSI_Products/MegaRAID
	mkdir $fileName/LSI_Products/MegaRAID/Notes
	mkdir $fileName/LSI_Products/MegaRAID/storcli
	mkdir $fileName/LSI_Products/MegaRAID/MegaCli
	mkdir $fileName/LSI_Products/MegaRAID/AENs
	mkdir $fileName/LSI_Products/MegaRAID/FWTermLog
	mkdir $fileName/LSI_Products/HBA
	mkdir $fileName/LSI_Products/HBA/Notes

# Used in AEN Sort
echo Info > $fileName/script_workspace/mr_aen_types.txt
echo Warning >> $fileName/script_workspace/mr_aen_types.txt
echo Critical >> $fileName/script_workspace/mr_aen_types.txt
echo Fatal >> $fileName/script_workspace/mr_aen_types.txt
echo Progress >> $fileName/script_workspace/mr_aen_types.txt
echo Obsolete >> $fileName/script_workspace/mr_aen_types.txt

#For output files
	if [ "$CLEANED_UP" = "NO" ] ; then
		cp $BASECMD ./$fileName/script_workspace
		cp Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code ./$fileName/script_workspace/Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code.txt >> ./misc_output.txt 2>&1
		if [ "$TWGETSKIPHBA" != "YES" ] ; then
			cp Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code ./$fileName/LSI_Products/HBA/Notes/Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code.txt >> ./misc_output.txt 2>&1
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			cp Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code ./$fileName/LSI_Products/MegaRAID/Notes/Sense-Key_ASC-ASCQ_Opcodes_ATA-Command_Codes-Set_Features_Code.txt >> ./misc_output.txt 2>&1
			cp MegaRAID_Terminology ./$fileName/LSI_Products/MegaRAID/Notes/MegaRAID_Terminology.txt >> ./misc_output.txt 2>&1
		fi
		mv -f aens_mr.txt ./$fileName/script_workspace/aens_mr.txt >> ./misc_output.txt 2>&1



		if [ "$OS_LSI" = "linux" ] ; then
			#External		
			if [ "$all_cli" = "all_cli" ] ; then
				if [ "$TWGETSKIPHBA" != "YES" ] ; then
					cp ./linux_HBA_Debug_Notes_ext.txt ./$fileName/LSI_Products/HBA/Notes/HBA_Debug_Notes_ext.txt >> ./misc_output.txt 2>&1
					cp ./linux_mpt2sas_debug.h ./$fileName/LSI_Products/HBA/Notes/mpt2sas_debug.h >> ./misc_output.txt 2>&1
					cp ./linux_mpt3sas_debug.h ./$fileName/LSI_Products/HBA/Notes/mpt3sas_debug.h >> ./misc_output.txt 2>&1
			fi
			fi
			#Internal
			if [ "$all_cli" != "all_cli" ] ; then
				if [ "$TWGETSKIPHBA" != "YES" ] ; then
					cp ./linux_HBA_Debug_Notes_int.txt ./$fileName/LSI_Products/HBA/Notes/HBA_Debug_Notes_int.txt >> ./misc_output.txt 2>&1
					cp ./linux_mpt2sas_debug.h ./$fileName/LSI_Products/HBA/Notes/mpt2sas_debug.h >> ./misc_output.txt 2>&1
					cp ./linux_mpt3sas_debug.h ./$fileName/LSI_Products/HBA/Notes/mpt3sas_debug.h >> ./misc_output.txt 2>&1
				fi
			fi
		fi
	fi

	if [ "$TWPRINTFILENAME" = "YES" ] ; then echo $fileName.tar.gz >> LSICAPTUREFILES.TXT; fi

				

	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		###########################################################################################################################
		# 3W - Rename any old AEC files
		###########################################################################################################################
		for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do #Support for Controller IDs 0-15
			if [ -f evtcapture_c$i.dat ] ; then OLD_EVT_CAP_EXIST=YES >> ./misc_output.txt 2>&1 ; fi	
		
			if [ "$OLD_EVT_CAP_EXIST" = "YES" ] ; then 
				if [ ! -d PreExisting_AEC_$fileName ] ; then mkdir PreExisting_AEC_$fileName ; fi	
			fi
		
			if [ -f evtcapture_c$i.dat ] ; then mv evtcapture_c$i.dat PreExisting_AEC_$fileName >> ./misc_output.txt 2>&1 ; fi
		
		done
	fi

	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		###########################################################################################################################
		# MR - Rename any old AEC files
		# Reusing OLD_EVT_CAP_EXIST= variable from 3w, any error should get piped to /dev/null on missing file
		###########################################################################################################################
		if [ "$OS_LSI" != "macos" ] ; then
		
			for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do #Support for Adapter IDs 0-15
				if [ -f evtcapture_a$i.dat ] ; then OLD_EVT_CAP_EXIST=YES >> ./misc_output.txt 2>&1 ; fi	
			
				if [ "$OLD_EVT_CAP_EXIST" = "YES" ] ; then 
					if [ ! -d PreExisting_AEC_$fileName ] ; then mkdir PreExisting_AEC_$fileName ; fi	
				fi
			
				if [ -f evtcapture_a$i.dat ] ; then mv evtcapture_a$i.dat PreExisting_AEC_$fileName >> ./misc_output.txt 2>&1 ; fi
			
			done
		fi
	fi
	###########################################################################################################################
	# 3W - For G_*
	# Disabling and Getting Advanced Event Capture (AEC) IF Enabled
	# Advanced Event Capture must be first or the log gets filled with IOCTLs
	###########################################################################################################################
	if [ "$TWGETPARTIALMODE" != "STANDARD" ] ; then
		if [ "$TWGETPARTIALMODE" != "G_DPMSTAT" ] ; then

			if [ "$TWGETSKIP3WARE" != "YES" ] ; then	
				for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
					if $CLI_LOCATION$CLI_NAME /c$i show capture 2>> ./misc_output.txt | $grep "  State =" >> ./misc_output.txt 2>&1 ; then
						$CLI_LOCATION$CLI_NAME /c$i set capture=stop >> ./misc_output.txt 2>&1	 
						$CLI_LOCATION$CLI_NAME /c$i get capture >> ./misc_output.txt 2>&1
						$CLI_LOCATION$CLI_NAME /c$i set capture=disable >> ./misc_output.txt 2>&1
						if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
						if [ ! -d ./$fileName/LSI_Products/3ware/AEC ] ;then mkdir ./$fileName/LSI_Products/3ware/AEC ; fi
					
						if [ -f evtcapture_c$i.dat ] ; then mv evtcapture_c$i.dat ./$fileName/LSI_Products/3ware/AEC >> ./misc_output.txt 2>&1 ; fi
						TW_ECHO_AEC=YES
					fi
				done
			fi

			###########################################################################################################################
			# MR - For G_*
			# Disabling and Getting Advanced Event Capture (AEC) IF Enabled
			# Advanced Event Capture must be first or the log gets filled with IOCTLs
			###########################################################################################################################

			if [ "$OS_LSI" != "macos" ] ; then

				if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
				# storcli syntax not available for aec/dpmstat
	
					for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
						if $MCLI_LOCATION$MCLI_NAME adpaec show a$i nolog 2>> ./misc_output.txt | $grep "AEC enabled :" >> ./misc_output.txt 2>&1 ; then
							$MCLI_LOCATION$MCLI_NAME  adpaec stop a$i nolog >> ./misc_output.txt 2>&1	 
							$MCLI_LOCATION$MCLI_NAME adpaec get -f evtcapture_a$i.dat a$i nolog >> ./misc_output.txt 2>&1
							$MCLI_LOCATION$MCLI_NAME adpaec disable a$i nolog >> ./misc_output.txt 2>&1
						
							if [ ! -d ./$fileName/LSI_Products/MegaRAID/AEC ] ;then mkdir ./$fileName/LSI_Products/MegaRAID/AEC ; fi
						
							if [ -f evtcapture_a$i.dat ] ; then mv evtcapture_a$i.dat ./$fileName/LSI_Products/MegaRAID/AEC >> ./misc_output.txt 2>&1 ; fi
							MR_ECHO_AEC=YES
						fi
					done
				fi
			fi

			if [ "$TWGETSKIP3WARE" != "YES" ] ; then
				if [ "$TW_ECHO_AEC" = "YES" ] ; then
					echo "Disabling and Getting Advanced Event Capture (AEC) for 3ware..."
				fi
			fi

			if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then			
				if [ "$MR_ECHO_AEC" = "YES" ] ; then
					echo "Disabling and Getting Advanced Event Capture (AEC) for MegaRAID..."
				fi
			fi
		fi
	fi 

	
	###########################################################################################################################
	# 3W - Standard
	# Disabling and Getting Advanced Event Capture (AEC) IF Enabled
	# Advanced Event Capture must be first or the log gets filled with IOCTLs
	###########################################################################################################################
	if [ "$TWGETPARTIALMODE" != "G_AEC" ] ; then
		if [ "$TWGETPARTIALMODE" != "G_AEC_DPMSTAT" ] ; then
			if [ "$TWGETPARTIALMODE" != "G_DPMSTAT" ] ; then
			
				if [ "$TWGETSKIP3WARE" != "YES" ] ; then
					for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
						if $CLI_LOCATION$CLI_NAME /c$i show capture 2>> ./misc_output.txt | $grep "  State = Enabled" >> ./misc_output.txt 2>&1 ; then
							$CLI_LOCATION$CLI_NAME /c$i set capture=stop >> ./misc_output.txt 2>&1	 
							$CLI_LOCATION$CLI_NAME /c$i get capture >> ./misc_output.txt 2>&1
							$CLI_LOCATION$CLI_NAME /c$i set capture=disable >> ./misc_output.txt 2>&1
							if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi							
							if [ ! -d ./$fileName/LSI_Products/3ware/AEC ] ;then mkdir ./$fileName/LSI_Products/3ware/AEC ; fi
						
							if [ -f evtcapture_c$i.dat ] ; then mv evtcapture_c$i.dat ./$fileName/LSI_Products/3ware/AEC >> ./misc_output.txt 2>&1 ; fi
							TW_ECHO_AEC=YES
						fi
					done
				fi

				if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then			
					###########################################################################################################################
					# MR - Standard
					# Disabling and Getting Advanced Event Capture (AEC) IF Enabled
					# Advanced Event Capture must be first or the log gets filled with IOCTLs
					###########################################################################################################################
			
					if [ "$OS_LSI" != "macos" ] ; then
					
						for i in $($MCLI_LOCATION$MCLI_NAME show  | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
							if $MCLI_LOCATION$MCLI_NAME adpaec show  a$i nolog 2>> ./misc_output.txt | $grep "AEC enabled : Yes" >> ./misc_output.txt 2>&1 ; then
								$MCLI_LOCATION$MCLI_NAME adpaec stop a$i nolog >> ./misc_output.txt 2>&1	 
								$MCLI_LOCATION$MCLI_NAME adpaec get -f evtcapture_a$i.dat a$i nolog >> ./misc_output.txt 2>&1
								$MCLI_LOCATION$MCLI_NAME adpaec disable a$i nolog >> ./misc_output.txt 2>&1
								
								if [ ! -d ./$fileName/LSI_Products/MegaRAID/AEC ] ;then mkdir ./$fileName/LSI_Products/MegaRAID/AEC ; fi
								
								if [ -f evtcapture_a$i.dat ] ; then mv evtcapture_a$i.dat ./$fileName/LSI_Products/MegaRAID/AEC >> ./misc_output.txt 2>&1 ; fi
								MR_ECHO_AEC=YES
							fi
						done
					 
					fi
				fi				
				if [ "$TWGETSKIP3WARE" != "YES" ] ; then
					if [ "$TW_ECHO_AEC" = "YES" ] ; then
						echo "Disabling and Getting Advanced Event Capture (AEC) for 3ware..."
					fi
				fi
				if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
					if [ "$MR_ECHO_AEC" = "YES" ] ; then
						echo "Disabling and Getting Advanced Event Capture (AEC) for MegaRAID..."
					fi
				fi
				if [ "$TWGETSKIPHBA" != "YES" ] ; then
					###########################################################################################################################
					# HBA - Standard
					# Disabling and Getting HBA Ring Buffer IF Enabled
					###########################################################################################################################
				
					if [ "$NO_LSI_HBAs" != "YES" ] ; then
						for i in $(./$LSUT_NAME 0 | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
							./$LSUT_NAME -p $i -f IOC${i}_"$todayDate"."$currentTime".rb -a 43,0,4,3,5,,0 | $grep failed >> ./misc_output.txt 2>&1
							if [ "$?" = "1" ] ; then
								if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
								rbcurrentTime=`date '+%H%M%S.%N'`
								fi
								if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
								rbcurrentTime=`date '+%H%M%S'`
								fi						
								mv IOC${i}_"$todayDate"."$currentTime".rb $fileName/LSI_Products/HBA/IOC${i}_"$todayDate"."$rbcurrentTime".rb  >> ./misc_output.txt 2>&1
								echo "Captured Ring Buffer for IOC/HBA $i..."
							fi
							#mv data files
							for j in *inbandcli.tar.gz IOC${i}_*.*.*.rb trigger.out ; do
								ls ${j} >> ./misc_output.txt 2>&1
								if [ "$?" = "0" ] ; then
				 					for k in $(ls ${j}) ; do
										if [ -f ${k} ] ; then mv ${k} $fileName/LSI_Products/HBA/In-WD_${k} ; fi
									done
								fi
							done
						done
						#cp script files
						for i in LSITRIGGERCMDLINE.TXT ; do
							if [ -f ${i} ] ; then 
								for j in $(cat LSITRIGGERCMDLINE.TXT) ; do
									if [ -f ${j} ] ; then
										cp ./${j} $fileName/LSI_Products/HBA/.
									fi
								done
							fi
							if [ -f LSITRIGGERCMDLINE.TXT ] ; then rm -f LSITRIGGERCMDLINE.TXT ; fi
						done
					fi
				fi
			fi 
		fi
	fi 
fi


	
###########################################################################################################################
# 3W - Clearing and Enabling Advanced Event Capture (AEC)
###########################################################################################################################


if [ "$TWGETPARTIALMODE" = "E_AEC" ]||[ "$TWGETPARTIALMODE" = "E_AEC_DPMSTAT" ] ; then

	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		if [ "$tw_cli_Functional" != "NO" ] ; then
	
			echo "Clearing and Enabling Advanced Event Capture (AEC) for 3ware..."
			for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
				if $CLI_LOCATION$CLI_NAME /c$i set capture=disable >> ./misc_output.txt 2>&1; then	 
					$CLI_LOCATION$CLI_NAME /c$i set capture=enable numevents=150000 >> ./misc_output.txt 2>&1 
					$CLI_LOCATION$CLI_NAME /c$i set capturemode=stopped >> ./misc_output.txt 2>&1 
					$CLI_LOCATION$CLI_NAME /c$i set capture=start >> ./misc_output.txt 2>&1
				fi
			done
			
		fi
	fi

###########################################################################################################################
# MR - Clearing and Enabling Advanced Event Capture (AEC)
###########################################################################################################################
	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		if [ "$OS_LSI" != "macos" ] ; then
		
			if [ "$mcli_Functional" != "NO" ] ; then
			
				echo "Clearing and Enabling Advanced Event Capture (AEC) for MegaRAID..."
				for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
					if $MCLI_LOCATION$MCLI_NAME adpaec disable a$i nolog >> ./misc_output.txt 2>&1; then	 
						$MCLI_LOCATION$MCLI_NAME adpaec start numevents 150000 a$i nolog >> ./misc_output.txt 2>&1 
					fi
				done
			
			fi
		
		fi
	fi
	if [ "$tw_cli_Functional" = "NO" ] ; then
		if [ "$mcli_Functional" = "NO" ] ; then
			#cho ".................................................||................................................."
			echo "...................................................................................................."
			echo "############## CLI incompatible or NO 3ware or MegaRAID card in the system -E_AEC   ################"
			echo "################################################ OR ################################################"
			echo "################# You do not have root privileges which are required to run the CLI ################"
			echo ""
			echo "\"$BASECMD -H\" provides a help screen."
			echo ""
		fi
	fi
fi



###########################################
# If E_AEC exit
###########################################
if [ "$TWGETPARTIALMODE" = "E_AEC" ] ; then 
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N'
	fi
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S'
	fi

###########################################################################################################################
# Clean up
###########################################################################################################################


	for i in $( cat cleanup.txt ); do

		if [ -f ./$i ] ; then
			rm -f ./$i
		fi
	done
	
	CLEANED_UP=YES
	rm -f cleanup.txt
	exit 0
fi

	

###########################################################################################################################
# 3W - Clearing and Enabling Disk Performance Monitoring Statistics (DPMSTAT) 
###########################################################################################################################

if [ "$TWGETPARTIALMODE" = "E_DPMSTAT" ] || [ "$TWGETPARTIALMODE" = "E_AEC_DPMSTAT" ] ; then
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then	
		if [ "$tw_cli_Functional" != "NO" ] ; then
	
			echo "Clearing and Enabling Disk Performance Monitoring Statistics (DPMSTAT) for 3ware..."
			for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
				if $CLI_LOCATION$CLI_NAME /c$i show pmstat | $grep "Instantaneous" >> ./misc_output.txt 2>&1; then	 
			
					for l in $($CLI_LOCATION$CLI_NAME /c$i show | $grep ^p | $grep -v NOT-PRESENT | cut -b 2-4); do #Supports up to 999 Ports per Controller.
						$CLI_LOCATION$CLI_NAME /c$i/p$l set pmstat=clear type=ra >> ./misc_output.txt 2>&1 
						$CLI_LOCATION$CLI_NAME /c$i/p$l set pmstat=clear type=lct >> ./misc_output.txt 2>&1 
						$CLI_LOCATION$CLI_NAME /c$i/p$l set pmstat=clear type=ext >> ./misc_output.txt 2>&1 
					done
					$CLI_LOCATION$CLI_NAME /c$i set pmstat=on >> ./misc_output.txt 2>&1 
				fi
			done
			
		fi
	fi

###########################################################################################################################
# MR - Clearing and Enabling Disk Performance Monitoring Statistics (DPMSTAT)
###########################################################################################################################

	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		if [ "$OS_LSI" != "macos" ] ; then
		
			if [ "$mcli_Functional" != "NO" ] ; then
			
				echo "Clearing and Enabling Disk Performance Monitoring Statistics (DPMSTAT) for MegaRAID..."
				for i in $($MCLI_LOCATION$MCLI_NAME show  | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
					if $MCLI_LOCATION$MCLI_NAME adpgetprop dpmenable a$i nolog | $grep "DPM is " >> ./misc_output.txt 2>&1; then	 
				
						$MCLI_LOCATION$MCLI_NAME dpmstat clear lct a$i nolog >> ./misc_output.txt 2>&1 
						$MCLI_LOCATION$MCLI_NAME dpmstat clear hist a$i nolog >> ./misc_output.txt 2>&1 
						$MCLI_LOCATION$MCLI_NAME dpmstat clear ra a$i nolog >> ./misc_output.txt 2>&1 
						$MCLI_LOCATION$MCLI_NAME dpmstat clear ext a$i nolog >> ./misc_output.txt 2>&1 
						$MCLI_LOCATION$MCLI_NAME adpsetprop dpmenable 1 a$i nolog >> ./misc_output.txt 2>&1 
				
					fi
				done
			
			fi
		
		fi
	fi

	if [ "$tw_cli_Functional" = "NO" ] ; then
		if [ "$mcli_Functional" = "NO" ] ; then
			#cho ".................................................||................................................."
			echo "...................................................................................................."
			echo "############ CLI incompatible or NO 3ware or MegaRAID card in the system -E_DPMSTAT   ##############"
			echo "################################################ OR ################################################"
			echo "################# You do not have root privileges which are required to run the CLI ################"
			echo ""
			echo "\"$BASECMD -H\" provides a help screen."
			echo ""
		fi
	fi

###########################################
# If E_DPMSTAT or E_AEC_DPMSTAT exit
###########################################
	if [ "$TWGETPARTIALMODE" = "E_DPMSTAT" ] ; then 
		if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
			date '+%H:%M:%S.%N'
		fi
		if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
			date '+%H:%M:%S'
		fi

###########################################################################################################################
# Clean up
###########################################################################################################################


		for i in $( cat cleanup.txt ); do

			if [ -f ./$i ] ; then
				rm -f ./$i
			fi
		done

		CLEANED_UP=YES
		rm -f cleanup.txt
		exit 0
	fi
	if [ "$TWGETPARTIALMODE" = "E_AEC_DPMSTAT" ] ; then 
		if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
			date '+%H:%M:%S.%N'
		fi
		if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
			date '+%H:%M:%S'
		fi

###########################################################################################################################
# Clean up
###########################################################################################################################


		for i in $( cat cleanup.txt ); do

			if [ -f ./$i ] ; then
				rm -f ./$i
			fi
		done

		CLEANED_UP=YES
		rm -f cleanup.txt
		exit 0
	fi
fi



###########################################################################################################################
# 3W - PMSTAT Data Collection for G_*
###########################################################################################################################
if [ "$TWGETPARTIALMODE" != "STANDARD" ] ; then 
	if [ "$TWGETPARTIALMODE" != "G_AEC" ] ; then
		if [ "$TWSETONLY" != "YES" ] ; then
			if [ "$TWGETSKIP3WARE" != "YES" ] ; then
				for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
					if $CLI_LOCATION$CLI_NAME /c$i show pmstat 2>> ./misc_output.txt | $grep "Performance Monitor:" >> ./misc_output.txt 2>&1 ; then
			
						TW_ECHO_PMSTAT=YES
						if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
						if [ ! -d ./$fileName/LSI_Products/3ware/PMSTAT ] ; then mkdir ./$fileName/LSI_Products/3ware/PMSTAT ; fi
			
							$CLI_LOCATION$CLI_NAME /c$i set pmstat=off >> ./misc_output.txt 2>&1 
					
					
			
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=inst................................" >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_INST_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=inst 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_INST_C$i.txt 2>&1
					
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=ra.................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_RA_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=ra 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_RA_C$i.txt 2>&1
					
					
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=ext................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_EXT_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=ext 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_EXT_C$i.txt 2>&1
					
							for l in $($CLI_LOCATION$CLI_NAME /c$i show 2>> ./misc_output.txt | $grep ^p | $grep -v NOT-PRESENT | cut -b 2-4); do #Supports up to 999 Ports per Controller.
								#cho ".................................................||................................................."
								echo "............................../tw_cli /c$i/p$l show pmstat type=histdata.............................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_HISTDATA_C$i.txt
								$CLI_LOCATION$CLI_NAME /c$i/p$l show pmstat type=histdata >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_HISTDATA_C$i.txt 2>&1
										
								#cho ".................................................||................................................."
								echo "............................../tw_cli /c$i/p$l show pmstat type=lct..................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_LCT_C$i.txt
								$CLI_LOCATION$CLI_NAME /c$i/p$l show pmstat type=lct >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_LCT_C$i.txt 2>&1
					
						
							done
						fi
				done
			fi

###########################################################################################################################
# MR - PMSTAT Data Collection for G_*
###########################################################################################################################
			if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
				if [ "$OS_LSI" != "macos" ] ; then
				
					for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
						if $MCLI_LOCATION$MCLI_NAME adpgetprop dpmenable a$i nolog 2>> ./misc_output.txt | $grep "DPM is " >> ./misc_output.txt 2>&1; then	
				
							MR_ECHO_PMSTAT=YES
				
							if [ ! -d ./$fileName/LSI_Products/MegaRAID/PMSTAT ] ; then mkdir ./$fileName/LSI_Products/MegaRAID/PMSTAT ; fi
				
							$MCLI_LOCATION$MCLI_NAME adpsetprop dpmenable 0 a$i nolog >> ./misc_output.txt 2>&1 
						
							for l in $($MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -d" " -f1); do #Supports all disks on Controller.
				
								#cho ".................................................||................................................."
								echo "......................./$MCLI_NAME dpmstat dsply lct physdrv[$l] a$i nolog.........................." >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_LCT_A$i.txt
								$MCLI_LOCATION$MCLI_NAME dpmstat dsply lct physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_LCT_A$i.txt 2>&1
					
								#cho ".................................................||................................................."
								echo "........................../$MCLI_NAME dpmstat hist physdrv[$l] a$i nolog............................" >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_HIST_A$i.txt
								$MCLI_LOCATION$MCLI_NAME dpmstat dsply hist physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_HIST_A$i.txt 2>&1
					
								#cho ".................................................||................................................."
								echo ".........................../$MCLI_NAME dpmstat ra physdrv[$l] a$i nolog............................." >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_RA_A$i.txt
								$MCLI_LOCATION$MCLI_NAME dpmstat dsply ra physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_RA_A$i.txt 2>&1
					
								#cho ".................................................||................................................."
								echo "........................../$MCLI_NAME dpmstat ext physdrv[$l] a$i nolog............................." >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_EXT_A$i.txt
								$MCLI_LOCATION$MCLI_NAME dpmstat dsply ext physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_EXT_A$i.txt 2>&1
					
				
				
							done
						fi
					done
				
				fi
			fi	


			if [ "$TWGETSKIP3WARE" != "YES" ] ; then
				if [ "$TW_ECHO_PMSTAT" = "YES" ] ; then
					echo "Disabling and Getting Disk Performance Monitoring Statistics (DPMSTAT) for 3ware....."
				fi
			fi
			if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
				if [ "$MR_ECHO_PMSTAT" = "YES" ] ; then
					echo "Disabling and Getting Disk Performance Monitoring Statistics (DPMSTAT) for MegaRAID....."
				fi
			fi
		fi
	fi
fi



###########################################################################################################################
# 3W - PMSTAT Data Collection Standard
###########################################################################################################################
if [ "$TWGETPARTIALMODE" != "G_AEC_DPMSTAT" ] ; then 
	if [ "$TWGETPARTIALMODE" != "G_DPMSTAT" ] ; then 
		if [ "$TWGETPARTIALMODE" != "G_AEC" ] ; then 
			if [ "$TWSETONLY" != "YES" ] ; then
				if [ "$TWGETSKIP3WARE" != "YES" ] ; then
					for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
						if $CLI_LOCATION$CLI_NAME /c$i show pmstat 2>> ./misc_output.txt | $grep "Performance Monitor: ON" >> ./misc_output.txt 2>&1 ; then
				
							TW_ECHO_PMSTAT=YES
							if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
							if [ ! -d ./$fileName/LSI_Products/3ware/PMSTAT ] ; then mkdir ./$fileName/LSI_Products/3ware/PMSTAT ; fi
					
							$CLI_LOCATION$CLI_NAME /c$i set pmstat=off >> ./misc_output.txt 2>&1 
							
										
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=inst................................" >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_INST_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=inst 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_INST_C$i.txt 2>&1
					
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=ra.................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_RA_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=ra 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_RA_C$i.txt 2>&1
					
					
							#cho ".................................................||................................................."
							echo ".................................../tw_cli /c$i show pmstat type=ext................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_EXT_C$i.txt
							$CLI_LOCATION$CLI_NAME /c$i show pmstat type=ext 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_EXT_C$i.txt 2>&1
					
							for l in $($CLI_LOCATION$CLI_NAME /c$i show 2>> ./misc_output.txt | $grep ^p | $grep -v NOT-PRESENT | cut -b 2-4); do #Supports up to 999 Ports per Controller.
								#cho ".................................................||................................................."
								echo "............................../tw_cli /c$i/p$l show pmstat type=histdata.............................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_HISTDATA_C$i.txt
								$CLI_LOCATION$CLI_NAME /c$i/p$l show pmstat type=histdata 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_HISTDATA_C$i.txt 2>&1
										
								#cho ".................................................||................................................."
								echo "............................../tw_cli /c$i/p$l show pmstat type=lct..................................." >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_LCT_C$i.txt
								$CLI_LOCATION$CLI_NAME /c$i/p$l show pmstat type=lct 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/PMSTAT/PMSTAT_LCT_C$i.txt 2>&1
					
							done
						fi
					done
				fi

###########################################################################################################################
# MR - PMSTAT Data Collection Standard
###########################################################################################################################
				if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
					if [ "$OS_LSI" != "macos" ] ; then
					
						for i in $($MCLI_LOCATION$MCLI_NAME show 2>> ./misc_output.txt | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
							if $MCLI_LOCATION$MCLI_NAME adpgetprop dpmenable a$i nolog 2>> ./misc_output.txt | $grep "DPM is Enabled on Adapter" >> ./misc_output.txt 2>&1; then	
						
								MR_ECHO_PMSTAT=YES
						
								if [ ! -d ./$fileName/LSI_Products/MegaRAID/PMSTAT ] ; then mkdir ./$fileName/LSI_Products/MegaRAID/PMSTAT ; fi
						
								$MCLI_LOCATION$MCLI_NAME adpsetprop dpmenable 0 a$i nolog >> ./misc_output.txt 2>&1 
								
								for l in $($MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -d" " -f1); do #Supports all disks on Controller.
						
									#cho ".................................................||................................................."
									echo "......................./$MCLI_NAME dpmstat dsply lct physdrv[$l] a$i nolog............................" >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_LCT_A$i.txt
									$MCLI_LOCATION$MCLI_NAME dpmstat dsply lct physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_LCT_A$i.txt 2>&1
							
									#cho ".................................................||................................................."
									echo "......................./$MCLI_NAME dpmstat hist lct physdrv[$l] a$i nolog............................" >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_HIST_A$i.txt
									$MCLI_LOCATION$MCLI_NAME dpmstat dsply hist physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_HIST_A$i.txt 2>&1
							
									#cho ".................................................||................................................."
									echo "......................./$MCLI_NAME dpmstat ra lct physdrv[$l] a$i nolog............................" >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_RA_A$i.txt
									$MCLI_LOCATION$MCLI_NAME dpmstat dsply ra physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_RA_A$i.txt 2>&1
							
									#cho ".................................................||................................................."
									echo "......................./$MCLI_NAME dpmstat ext lct physdrv[$l] a$i nolog............................" >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_EXT_A$i.txt
									$MCLI_LOCATION$MCLI_NAME dpmstat dsply ext physdrv[$l] a$i nolog >> ./$fileName/LSI_Products/MegaRAID/PMSTAT/PMSTAT_EXT_A$i.txt 2>&1
								
								done
							fi
						done
					fi
				fi
				if [ "$TWGETSKIP3WARE" != "YES" ] ; then
					if [ "$TW_ECHO_PMSTAT" = "YES" ] ; then
						echo "Disabling and Getting Disk Performance Monitoring Statistics (DPMSTAT) for 3ware....."
					fi
				fi
				if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
					if [ "$MR_ECHO_PMSTAT" = "YES" ] ; then
						echo "Disabling and Getting Disk Performance Monitoring Statistics (DPMSTAT) for MegaRAID....."
					fi
				fi
			fi
		fi
	fi
fi

if [ "$TWGETSKIPHBA" != "YES" ] ; then
	###########################################
	# Enabling/Disabling HBA Linux Driver Logging, need to check other OS driver logging options.
	###########################################
	
	if [ "$OS_LSI" = "linux" ] ; then
		if [ "$TWGETPARTIALMODE" = "E_SAS2LOG" ]||[ "$TWGETPARTIALMODE" = "E_SAS2LOG_RB" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt2sas" ] ; then
					echo 00ffffff > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS2 HBAs..."
				fi
			done
		fi
		if [ "$TWGETPARTIALMODE" = "E_SAS2LOG1" ]||[ "$TWGETPARTIALMODE" = "E_SAS2LOG1_RB" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt2sas" ] ; then
					echo 000003f8 > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS2 HBAs..."
				fi
			done
		fi
		
		if [ "$TWGETPARTIALMODE" = "E_SAS3LOG" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG_RB" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt3sas" ] ; then
					echo 00ffffff > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS3 HBAs..."
				fi
			done
		fi
		if [ "$TWGETPARTIALMODE" = "E_SAS3LOG1" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG1_RB" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt3sas" ] ; then
					echo 000003f8 > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS3 HBAs..."
				fi
			done
		fi
	
		if [ "$TWGETPARTIALMODE" = "D_SAS2LOG" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt2sas" ] ; then
					echo 0 > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS2 HBAs..."
				fi
			done
		fi
		if [ "$TWGETPARTIALMODE" = "D_SAS3LOG" ] ; then
			for i in $(ls /sys/class/scsi_host -1);  do
				if [ `cat /sys/class/scsi_host/$i/proc_name` = "mpt3sas" ] ; then
					echo 0 > /sys/class/scsi_host/$i/logging_level
					logging_level=`cat /sys/class/scsi_host/$i/logging_level`
					echo "Setting the Driver Logging level to $logging_level for all SAS3 HBAs..."
				fi
			done
		fi
	###########################################
	# If E_SAS2LOG or E_SAS3LOG exit
	###########################################
	
	
		if [ "$TWGETPARTIALMODE" = "E_SAS2LOG" ]||[ "$TWGETPARTIALMODE" = "E_SAS2LOG1" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG1" ]||[ "$TWGETPARTIALMODE" = "D_SAS2LOG" ]||[ "$TWGETPARTIALMODE" = "D_SAS3LOG" ] ; then
			if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
				date '+%H:%M:%S.%N'
			fi
			if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
				date '+%H:%M:%S'
			fi
		
	
	###########################################################################################################################
	# Clean up
	###########################################################################################################################
	
	
			for i in $( cat cleanup.txt ); do
	
				if [ -f ./$i ] ; then
				rm -f ./$i
				fi
			done
		
			CLEANED_UP=YES
			rm -f cleanup.txt
			exit 0
		fi
	fi	
fi	
					
if [ "$TWGETSKIPHBA" != "YES" ] ; then
	###########################################
	# Enabling HBA Ring Buffer.
	###########################################
	
	
	if [ "$NO_LSI_HBAs" != "YES" ] ; then
		if [ "$TWGETPARTIALMODE" = "E_RB" ]||[ "$TWGETPARTIALMODE" = "E_SAS2LOG_RB" ]||[ "$TWGETPARTIALMODE" = "E_SAS2LOG1_RB" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG_RB" ]||[ "$TWGETPARTIALMODE" = "E_SAS3LOG1_RB" ] ; then
			for i in $(./$LSUT_NAME 0 | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
				./$LSUT_NAME -p $i -a 43,0,4,5,,0 >> ./misc_output.txt
				./$LSUT_NAME -p $i -a 43,0,1,3072,,,0,0, >> ./misc_output.txt
				echo "Enabling a 3MB Ring Buffer IOC/HBA $i..."
			done
	
											
			if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
				date '+%H:%M:%S.%N'
			fi
			if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
				date '+%H:%M:%S'
			fi
										
	
	###########################################################################################################################
	# Clean up
	###########################################################################################################################
	
	
			for i in $( cat cleanup.txt ); do
	
				if [ -f ./$i ] ; then
				rm -f ./$i
				fi
			done
		
			CLEANED_UP=YES
			rm -f cleanup.txt
			exit 0
							
		fi
	fi
fi
	
						
					
										
									




if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	###########################################################################################################################
	# Starting 3ware Controller Data Collection
	###########################################################################################################################
	
	# List of 3ware controller numbers
	$CLI_LOCATION$CLI_NAME show > ./$fileName/script_workspace/tw_cli_show.txt
	$CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3 > ./$fileName/script_workspace/controller_numbers.txt
	#if [ -s ./$fileName/script_workspace/controller_numbers.txt ] ; then
		if [ ! -d ./$fileName/LSI_Products/3ware ] ; then mkdir ./$fileName/LSI_Products/3ware ; fi
		echo "Starting 3ware Controller Data Collection..."
	#fi
	
	for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
		#cho ".................................................||................................................."
		echo $fileName >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
		echo "............................................/tw_cli show............................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
		$CLI_LOCATION$CLI_NAME show >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
		echo "......................................../tw_cli /c$i show all........................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
		$CLI_LOCATION$CLI_NAME /c$i show all 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
		echo "Collecting Controller Information for 3ware Controller C$i..." >> ./misc_output.txt 2>&1
	done
fi
if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	###########################################
	# If G_* Grab MR show/show all equivalent
	###########################################
	if [ "$TWPARTIALCAP" = "YES" ] ; then
	
		for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
			#cho ".................................................||................................................."
			echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
			echo "........................................./$MCLI_NAME show............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
			$MCLI_LOCATION$MCLI_NAME show 2>> ./misc_output.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
			echo "................................../$MCLI_NAME /c$i show all.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
			$MCLI_LOCATION$MCLI_NAME /c$i show all 2>> ./misc_output.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
			echo "Collecting Controller Information for MegaRAID Controller C$i..." >> ./misc_output.txt 2>&1
		done
	fi
fi



	
if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	###########################################
	# If G_* skip to the end
	###########################################
	if [ "$TWPARTIALCAP" != "YES" ] ; then
	#
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
			echo "Collecting Unit Information for Controller C$i..." >> ./misc_output.txt 2>&1
			for k in $($CLI_LOCATION$CLI_NAME /c$i show 2>> ./misc_output.txt | $grep ^u | cut -b 2-4); do #Supports up to 999 Units per Controller.
				#cho ".................................................||................................................."
				echo "....................................../tw_cli /c$i/u$k show all......................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$CLI_LOCATION$CLI_NAME /c$i/u$k show all >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".................................../tw_cli /c$i/u$k show writedpo....................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$CLI_LOCATION$CLI_NAME /c$i/u$k show writedpo >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			done	
	
	
			echo "Collecting Port Information for Controller C$i..." >> ./misc_output.txt 2>&1
			echo "This may take a few minutes..." >> ./misc_output.txt 2>&1
	
			#work around for bug in tw_cli 2.00.11.022, segfault in /cx/px show smart/all - Works with included build
			#if [ "$VMWARE_SUPPORTED" != "YES" ] ; then	
				for l in $($CLI_LOCATION$CLI_NAME /c$i show 2>> ./misc_output.txt | $grep ^p | $grep -v NOT-PRESENT | cut -b 2-4); do #Supports up to 999 Ports per Controller.
					#cho ".................................................||................................................."
					echo "....................................../tw_cli /c$i/p$l show all......................................." >> ./$fileName/script_workspace/px_show_all_c$i.txt
					$CLI_LOCATION$CLI_NAME /c$i/p$l show all >> ./$fileName/script_workspace/px_show_all_c$i.txt
				done
			#fi
			#if [ "$VMWARE_SUPPORTED" = "YES" ] ; then	
			#	for l in $($CLI_LOCATION$CLI_NAME /c$i show 2>> ./misc_output.txt | $grep ^p | $grep -v NOT-PRESENT | cut -b 2-4); do #Supports up to 999 Ports per Controller.
			#		#cho ".................................................||................................................."
			#		echo "....................................../tw_cli /c$i/p$l show all......................................." >> ./$fileName/script_workspace/px_show_all_c$i.txt
			#		for m in status model firmware serial capacity driveinfo ncq identify lspeed ports connections drvintf wwn rasect pohrs temperature spindlespd ; do
			#			$CLI_LOCATION$CLI_NAME /c$i/p$l show $m >> ./$fileName/script_workspace/px_show_all_c$i.txt 2>> ./misc_output.txt 
			#		done
			#	done
			#fi
	
			if [ -f ./$fileName/script_workspace/px_show_all_c$i.txt ] ; then 
				#cho ".................................................||................................................."
				echo ".............................................Disk Model............................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Model =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo ".......................................SAS Only - Drive Type........................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Drive Type =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo "...........................................Disk Firmware............................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Firmware Version =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo "...........................................Disk Serial #............................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Serial =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo "...........................................Disk Capacity............................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Capacity =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				#cho ".................................................||................................................."
				echo ".......................................Disk Reallocated Sectors....................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Reallocated Sectors =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo "..........................................Disk Temperature.........................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Temperature =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo "..........................................Disk Link Speed..........................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Link Speed =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				#cho ".................................................||................................................."
				echo "......................................SATA Only - NCQ Enabled......................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "NCQ Enabled" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				echo ".....................................SAS Only - Queuing Enabled....................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$grep "Queuing Enabled =" ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			fi	
	
	
	
			#cho ".................................................||................................................."
			echo "................................../tw_cli /c$i show alarms reverse..................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			$CLI_LOCATION$CLI_NAME /c$i show alarms reverse 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			
			for CLI_ARG in rebuild rebuildrate rebuildmode verify verifyrate verifymode selftest forcedrivecacheon avmode lbareorder modepgctl phy iostat pmstat capture ; do
				#cho ".................................................||................................................."
				echo "....................................../tw_cli /c$i show $CLI_ARG......................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$CLI_LOCATION$CLI_NAME /c$i show $CLI_ARG 2>> ./misc_output.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			done
	
	
	
			#cho ".................................................||................................................."
			echo "....................................../tw_cli /c$i/bbu show all......................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			$CLI_LOCATION$CLI_NAME /c$i/bbu show all >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
		
		
			echo "Collecting /Cx/Ex Enclosure Information for Controller C$i..." >> ./misc_output.txt 2>&1	
			for j in $($CLI_LOCATION$CLI_NAME show | $grep ^/c$i/e | cut -d/ -f3 | cut -b 2-4); do #Supports up to 999 Enclosure IDs
				#cho ".................................................||................................................."
				echo "......................................./tw_cli /c$i/e$j show all......................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
				$CLI_LOCATION$CLI_NAME /c$i/e$j show all >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			done
		
		
		
			echo "Collecting /Ex Enclosure Information for Controller C$i..." >> ./misc_output.txt 2>&1
			for j in $($CLI_LOCATION$CLI_NAME show | $grep ^e | cut -b 2-4); do  #Supports up to 999 Enclosure IDs 
				for k in $($CLI_LOCATION$CLI_NAME /e$j show | $grep ^e | cut -d/ -f2 | cut -b2-3); do #Supports up to 999 Controller IDs
					if [ "$k" = "$i" ] ; then # To keep the output in sync with the script flow, otherwise /ex prints at the top of output file.
						#cho ".................................................||................................................."
						echo "...................................../tw_cli /e$j show protocol......................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
						$CLI_LOCATION$CLI_NAME /e$j show protocol >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
						echo "......................................./tw_cli /e$j show all......................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
						$CLI_LOCATION$CLI_NAME /e$j show all >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
					fi
				done
			done
		
			if [ -f ./$fileName/script_workspace/px_show_all_c$i.txt ] ; then 
				cat ./$fileName/script_workspace/px_show_all_c$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			fi	
		
		
			#cho ".................................................||................................................."
			echo "......................................../tw_cli /c$i show diag......................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			echo ".........The diag output has been processed by strings in order to be viewable by GEDIT............." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			echo "......................The degree symbol was deleted to maintain formatting.........................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			echo "..............................The unmodified output is available in ................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			echo "............../$fileName/script_workspace/raw_diag_C$i.txt................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			echo "...................................................................................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
			#cho ".................................................||................................................."
			$CLI_LOCATION$CLI_NAME /c$i show diag 2>> ./misc_output.txt > ./$fileName/script_workspace/raw_diag_C$i.txt 2>&1
			
			#No tr in vmware
			if [ "$VMWARE_SUPPORTED" != "YES" ] ; then	
				#strings was deleting the degree symbol and changing the format, tr deletes the symbol but maintains format
				cat ./$fileName/script_workspace/raw_diag_C$i.txt | tr -d ° | strings > ./$fileName/script_workspace/strings_diag_C$i.txt
				cat ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			fi
			if [ "$VMWARE_SUPPORTED" = "YES" ] ; then	
				cat ./$fileName/script_workspace/raw_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt 2>&1
			fi
			###########################################################################################################################
			# 3ware Diag Output Error Checking
			###########################################################################################################################
	
	
			if [ -f ./$fileName/script_workspace/strings_diag_C$i.txt ] ; then 
				$grep "Check power cycles" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./misc_output.txt
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".....................tw_cli /c$i show diag - contains Drive Power Cycle Checks......................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo ".If the power cycle number is increasing this usually signifies a power supply or backplane problem." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					$grep "Check power cycles" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo "......................tw_cli /c$i show diag - contains Drive Power Cycle Checks......................" >> ./$fileName/LSI_Products/3ware/diag_CPC_C$i.txt
					echo ".If the power cycle number is increasing this usually signifies a power supply or backplane problem." >> ./$fileName/LSI_Products/3ware/diag_CPC_C$i.txt
					$grep "Check power cycles" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/diag_CPC_C$i.txt
				fi
			fi
	
			if [ -f ./$fileName/script_workspace/strings_diag_C$i.txt ] ; then 
				egrep "Assert|.cpp" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./misc_output.txt
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".............................tw_cli /c$i show diag - contains Asserts................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo "..................This is evidence of a possible FW bug, contact your support rep!.................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					egrep "Assert|.cpp" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo ".............................tw_cli /c$i show diag - contains Asserts................................" >> ./$fileName/LSI_Products/3ware/diag_Assert_C$i.txt
					echo "..................This is evidence of a possible FW bug, contact your support rep!.................." >> ./$fileName/LSI_Products/3ware/diag_Assert_C$i.txt
					egrep "Assert|.cpp" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/diag_Assert_C$i.txt
				fi
			fi
	
	
			if [ -f ./$fileName/script_workspace/strings_diag_C$i.txt ] ; then 
				egrep "BT1680" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./misc_output.txt
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".............................tw_cli /c$i show diag - contains BT1680................................" >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo "....................................The controller should be RMAed.................................." >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					egrep "BT1680" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/Controller_C$i.txt
					echo ".............................tw_cli /c$i show diag - contains BT1680................................" >> ./$fileName/LSI_Products/3ware/diag_BT1680_C$i.txt
					echo "....................................The controller should be RMAed.................................." >> ./$fileName/LSI_Products/3ware/diag_BT1680_C$i.txt
					egrep "BT1680" ./$fileName/script_workspace/strings_diag_C$i.txt >> ./$fileName/LSI_Products/3ware/diag_Assert_C$i.txt
				fi
			fi
	
	
	
			###########################################################################################################################
			# 3ware Smartctl Data Collection if Smartctl is installed
			###########################################################################################################################

		
	

			smartctl -h > ./$fileName/script_workspace/smartctl-h.txt 2>&1
			if [ "$?" -eq "0" ] ; then
				echo "Starting 3ware Smartctl Data Collection for Controller C$i..."
				if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
				if [ ! -d ./$fileName/LSI_Products/3ware/SMARTCTL ] ; then mkdir ./$fileName/LSI_Products/3ware/SMARTCTL ; fi

				for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do #Supports up to 16 character device node entries
					for j in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 ; do #Supports up to 128 Disks per controller, smartctl limitation still 16 as of 5.39??
				#		#cho ".................................................||................................................."
						smartctl -i -d 3ware,$j /dev/twa$i | egrep "START OF INFORMATION SECTION|Serial number:" 1>> ./misc_output.txt # Test for instances that exist
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "..........Port $j, TWA # is in order of Controller number, not equal too, lowest to highest.........." >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWA$i.txt 2>&1
							echo "..................................smartctl -a -d 3ware,$j /dev/twa$i.................................." >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWA$i.txt 2>&1
							smartctl -a -d 3ware,$j /dev/twa$i >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWA$i.txt 2>&1
						fi
					done
				done

				for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do #Supports up to 16 character device node entries
					for j in 0 1 2 3 4 5 6 7 8 9 10 11 ; do #Supports up to 12 Disks per controller, max port count for 7/8xxx controllers
				#		#cho ".................................................||................................................."
						smartctl -i -d 3ware,$j /dev/twe$i | egrep "START OF INFORMATION SECTION|Serial number:" 1>> ./misc_output.txt # Test for instances that exist
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "..........Port $j, TWE # is in order of Controller number, not equal too, lowest to highest.........." >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWE$i.txt 2>&1
							echo "..................................smartctl -a -d 3ware,$j /dev/twe$i.................................." >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWE$i.txt 2>&1
							smartctl -a -d 3ware,$j /dev/twe$i >> ./$fileName/LSI_Products/3ware/SMARTCTL/TWE$i.txt 2>&1
						fi
					done
				done
			fi
		done
	fi
fi



###########################################################################################################################
# LSI HBA & LSI Expander Info
###########################################################################################################################

if [ "$TWGETSKIPHBA" != "YES" ] ; then
	./smartctl -h > ./$fileName/script_workspace/smartctl-h.txt 2>&1
	
	
	if [ "$?" = "0" ] ; then
		echo "Starting Generic Smartctl Data Collection..."
		if [ ! -d ./$fileName/LSI_Products/HBA ] ;then mkdir ./$fileName/LSI_Products/HBA ; fi
		if [ ! -d ./$fileName/LSI_Products/HBA/SMARTCTL ] ; then mkdir ./$fileName/LSI_Products/HBA/SMARTCTL ; fi
	
		for i in $( ls /dev/sd* ); do #Supports all devices, doesn't currently distinguish per IOC/HBA/NON LSI/Avago controlled devices
			#cho ".................................................||................................................."
			echo ".....................................smartctl --all /dev/${dev}....................................." > ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_$i.txt 2>&1
			./smartctl --all /dev/${dev} >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_$i.txt 2>&1
			echo ".............................smartctl -g wcache /dev/${dev} - Write Cache:.........................." >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_$i.txt 2>&1
			./smartctl -g wcache /dev/${dev} | grep "Write cache is:" >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_$i.txt 2>&1
		done
	fi
	
	if [ ! -d ./$fileName/LSI_Products/HBA/SMARTCTL ] ; then
		smartctl -h > ./$fileName/script_workspace/smartctl-h.txt 2>&1
	
	
	
		if [ "$?" = "0" ] ; then
			echo "Starting Generic Smartctl Data Collection..."
			if [ ! -d ./$fileName/LSI_Products/HBA ] ;then mkdir ./$fileName/LSI_Products/HBA ; fi
			if [ ! -d ./$fileName/LSI_Products/HBA/SMARTCTL ] ; then mkdir ./$fileName/LSI_Products/HBA/SMARTCTL ; fi
		
			for i in $( ls /dev/sd* ); do #Supports all devices, doesn't currently distinguish per IOC/HBA/NON LSI/Avago controlled devices
				dev=$(basename $i)
				#cho ".................................................||................................................."
				echo ".....................................smartctl --all /dev/${dev}....................................." > ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_${dev}.txt 2>&1
				smartctl --all /dev/${dev} >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_${dev}.txt 2>&1
				echo ".............................smartctl -g wcache /dev/${dev} - Write Cache:.........................." >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_${dev}.txt 2>&1
				smartctl -g wcache /dev/${dev} | grep "Write cache is:" >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_${dev}.txt 2>&1
				echo ".............................smartctl -g wcache /dev/${dev} - Write Cache:.........................." >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_WC.txt 2>&1
				smartctl -g wcache /dev/${dev} | grep "Write cache is:" >> ./$fileName/LSI_Products/HBA/SMARTCTL/smartctl_WC.txt 2>&1
			done
	
		fi
	fi
fi




#Recursive, need to be more selective
#
#	#Grab all scsi_host info, need to see if functional prior to lsut
#	if [ -d /sys/class/scsi_host ] ; then
#		mkdir ./$fileName/scsi_host
#		for i in $( ls /sys/class/scsi_host ); do
#			if [ -d /sys/class/scsi_host/${i} ] ; then
#				$grep -i -e mpt -e megaraid -e 3ware /sys/class/scsi_host/${i}/proc_name>> ./misc_output.txt 2>&1
#				if [ "$?" = "0" ] ; then 				
#					mkdir ./$fileName/scsi_host/$i
#					for j in $( ls /sys/class/scsi_host/${i} ); do
#						if [ -f /sys/class/scsi_host/${i}/"${j}" ] ; then
#							#First Level
#							cat /sys/class/scsi_host/${i}/"${j}" > ./$fileName/scsi_host/${i}/"${j}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#							else 
#							mkdir ./$fileName/scsi_host/${i}/"${j}"
#							for k in $( ls /sys/class/scsi_host/${i}/"${j}" ); do
#								if [ -f /sys/class/scsi_host/${i}/"${j}"/"${k}" ] ; then
#									#Second Level
#									cat /sys/class/scsi_host/${i}/"${j}"/"${k}" > ./$fileName/scsi_host/${i}/"${j}"/"${k}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#									else 
#									mkdir ./$fileName/scsi_host/${i}/"${j}"/"${k}"
#									for l in $( ls /sys/class/scsi_host/${i}/"${j}"/"${k}" ); do
#										if [ -f /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}" ] ; then	
#											#Third Level
#											cat /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}" > ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#											else 
#											mkdir ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"
#											for m in $( ls /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}" ); do
#												if [ -f /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}" ] ; then
#													#Forth Level
#													cat /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}" > ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#													else 
#													mkdir ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"
#													for n in $( ls /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}" ); do
#														if [ -f /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" ] ; then
#															#Fifth Level
#															cat /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" > ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#															else 
#															mkdir ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"
#															for o in $( ls /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" ); do
#																if [ -f /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"/"${o}" ] ; then
#																	#Fifth Level/No Subdir
#																	cat /sys/class/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"/"${o}" > ./$fileName/scsi_host/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#																fi
#															done
#														fi
#													done
#												fi
#											done
#										fi
#									done
#								fi
#							done
#						fi
#					done
#				fi
#			fi
#		done
#	fi

#Grab all scsi_host info, need to see if functional prior to lsut
if [ -d /sys/class/scsi_host ] ; then
	mkdir ./$fileName/scsi_host
	for i in $( ls /sys/class/scsi_host ); do
		if [ -d /sys/class/scsi_host/${i} ] ; then
			$grep -i -e mpt2sas -e mpt3sas -e megaraid -e 3ware -e avago /sys/class/scsi_host/${i}/proc_name>> ./misc_output.txt 2>&1
			if [ "$?" = "0" ] ; then 				
				mkdir ./$fileName/scsi_host/$i
				for j in $( ls /sys/class/scsi_host/${i} ); do
					if [ -f /sys/class/scsi_host/${i}/"${j}" ] ; then
						#First Level
						cat /sys/class/scsi_host/${i}/"${j}" > ./$fileName/scsi_host/${i}/"${j}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
					fi
				done
			fi
		fi
	done
fi



#Recursive, need to be more selective
#	#Grab all /proc/scsi/ info, need to see if functional prior to lsut
#	if [ -d /proc/scsi ] ; then
#		mkdir ./$fileName/proc
#		for i in $( ls /proc/scsi ); do
#			if [ -d /proc/scsi/${i} ] ; then
#				mkdir ./$fileName/proc/$i
#				for j in $( ls /proc/scsi/${i} ); do
#					if [ -f /proc/scsi/${i}/${j} ] ; then
#						#First Level
#						cat /proc/scsi/${i}/${j} > ./$fileName/proc/${i}/${j}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#						else 
#						mkdir ./$fileName/proc/${i}/${j}
#						for k in $( ls /proc/scsi/${i}/${j} ); do
#							if [ -f /proc/scsi/${i}/${j}/${k} ] ; then
#								#Second Level
#								cat /proc/scsi/${i}/${j}/${k} > ./$fileName/proc/${i}/${j}/${k}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#								else 
#								mkdir ./$fileName/proc/${i}/${j}/${k}
#								for l in $( ls /proc/scsi/${i}/${j}/${k} ); do
#									if [ -f /proc/scsi/${i}/${j}/${k}/${l} ] ; then	
#										#Third Level
#										cat /proc/scsi/${i}/${j}/${k}/${l} > ./$fileName/proc/${i}/${j}/${k}/${l}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#										else 
#										mkdir ./$fileName/proc/${i}/${j}/${k}/${l}
#										for m in $( ls /proc/scsi/${i}/${j}/${k}/${l} ); do
#											if [ -f /proc/scsi/${i}/${j}/${k}/${l}/${m} ] ; then
#												#Forth Level
#												cat /proc/scsi/${i}/${j}/${k}/${l}/${m} > ./$fileName/proc/${i}/${j}/${k}/${l}/${m}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#												else 
#												mkdir ./$fileName/proc/${i}/${j}/${k}/${l}/${m}
#												for n in $( ls /proc/scsi/${i}/${j}/${k}/${l}/${m} ); do
#													if [ -f /proc/scsi/${i}/${j}/${k}/${l}/${m}/${n} ] ; then
#														#Fifth Level
#														cat /proc/scsi/${i}/${j}/${k}/${l}/${m}/${n} > ./$fileName/proc/${i}/${j}/${k}/${l}/${m}/${n}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#														else 
#														mkdir ./$fileName/proc/${i}/${j}/${k}/${l}/${m}/${n}
#														for o in $( ls /proc/scsi/${i}/${j}/${k}/${l}/${m}/${n} ); do
#															if [ -f /proc/scsi/${i}/${j}/${k}/${l}/${m}/${n}/"${o}" ] ; then
#																#Fifth Level/No Subdir
#																cat /proc/scsi/${i}/${j}/${k}/${l}/${m}/${n}/"${o}" > ./$fileName/proc/${i}/${j}/${k}/${l}/${m}/${n}.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#															fi
#														done
#													fi
#												done
#											fi
#										done
#									fi
#								done
#							fi
#						done
#					fi
#				done
#			fi
#		done
#	fi
	
#Grab all /proc/scsi/sg, need to see if functional prior to lsut, consolidated all /proc data collection here.

if [ -d /proc ] ; then
	if [ ! -d ./$fileName/proc ] ; then mkdir ./$fileName/proc ; fi
	for i in pci interrupts cmdline cpuinfo buddyinfo devices diskstats dma filesystems iomem ioports kallsyms mdstat meminfo misc modules mounts mtrr partitions pci slabinfo stat uptime version vmstat zoneinfo scsi ; do 
		if [ -f /proc/$i ] ; then
			cat /proc/$i > ./$fileName/proc/${i} 2>> ./misc_output.txt				
		fi
	done
	ls -latrR /proc > ./$fileName/proc/proc-latrR.txt 2>&1 ./$fileName/script_workspace/lsiget_errorlog.txt
fi

if [ -d /proc/scsi ] ; then
	if [ ! -d ./$fileName/proc/scsi ] ; then mkdir ./$fileName/proc/scsi; fi
	for i in $( ls /proc/scsi ) ; do
		if [ -f /proc/scsi/${i} ] ; then
			cat /proc/scsi/${i} > ./$fileName/proc/scsi/${i} 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
		fi
	done
fi
for i in sg 3w-xxxx 3w-9xxx megaraid_sas ; do
	if [ -d /proc/scsi/${i} ] ; then
		if [ ! -d ./$fileName/proc/scsi/${i} ] ; then mkdir ./$fileName/proc/scsi/${i}; fi
		for j in $( ls /proc/scsi/${i} ); do
			if [ -f /proc/scsi/${i}/${j} ] ; then
				cat /proc/scsi/${i}/${j} > ./$fileName/proc/scsi/${i}/${j} 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
			fi
		done
	fi		
done
if [ -f /proc/scsi/scsi ] ; then 
	cat /proc/scsi/scsi > ./$fileName/proc/scsi/scsi 2>> ./misc_output.txt
fi
if [ -d /proc/sys/kernel ] ; then
	if [ ! -d ./$fileName/proc/sys ] ; then mkdir ./$fileName/proc/sys; fi
	if [ ! -d ./$fileName/proc/sys/kernel ] ; then mkdir ./$fileName/proc/sys/kernel ; fi
	$grep -r . /proc/sys/kernel/ > ./$fileName/proc/sys/kernel/grep-r_._proc_sys_kernel.txt 2>> ./misc_output.txt
	for i in $( ls /proc/sys/kernel ); do
		if [ -f /proc/sys/kernel/${i} ] ; then
			cat /proc/scsi/kernel/${i} > ./$fileName/proc/sys/kernel/${i} 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
		fi
	done
fi
if [ -d /proc/sys/vm ] ; then 
	if [ ! -d ./$fileName/proc/sys ] ; then mkdir ./$fileName/proc/sys; fi
	if [ ! -d ./$fileName/proc/sys/vm ] ; then mkdir ./$fileName/proc/sys/vm ; fi
	for i in $( ls /proc/sys/vm ); do
		if [ -f /proc/sys/vm/${i} ] ; then 
			cat /proc/sys/vm/${i} > ./$fileName/proc/sys/vm/${i}.txt 2>> ./misc_output.txt
		fi
	done
fi

#Recursive, need to be more selective
#Capture 5 levels of /sys/block info
#	if [ -d /sys/block ] ; then
#		mkdir ./$fileName/sys_block
#		for i in $( ls /sys/block ); do
#			if [ -d /sys/block/${i} ] ; then
#				echo $i | grep sd >> ./misc_output.txt 2>&1
#				if [ "$?" = "0" ] ; then
#					mkdir ./$fileName/sys_block/$i
#					for j in $( ls /sys/block/${i} ); do
#						if [ -f /sys/block/${i}/"${j}" ] ; then
#							#First Level
#							cat /sys/block/${i}/"${j}" > ./$fileName/sys_block/${i}/"${j}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#							else 
#							mkdir ./$fileName/sys_block/${i}/"${j}"
#							for k in $( ls /sys/block/${i}/"${j}" ); do
#								if [ -f /sys/block/${i}/"${j}"/"${k}" ] ; then
#									#Second Level
#									cat /sys/block/${i}/"${j}"/"${k}" > ./$fileName/sys_block/${i}/"${j}"/"${k}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#									else 
#									mkdir ./$fileName/sys_block/${i}/"${j}"/"${k}"
#									for l in $( ls /sys/block/${i}/"${j}"/"${k}" ); do
#										if [ -f /sys/block/${i}/"${j}"/"${k}"/"${l}" ] ; then	
#											#Third Level
#											cat /sys/block/${i}/"${j}"/"${k}"/"${l}" > ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#											else 
#											mkdir ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"
#											for m in $( ls /sys/block/${i}/"${j}"/"${k}"/"${l}" ); do
#												if [ -f /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}" ] ; then
#													#Forth Level
#													cat /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}" > ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"/"${m}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#													else 
#													mkdir ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"/"${m}"
#													for n in $( ls /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}" ); do
#														if [ -f /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" ] ; then
#															#Fifth Level
#															cat /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" > ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#															else 
#															mkdir ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"
#															for o in $( ls /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}" ); do
#																if [ -f /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"/"${o}" ] ; then
#																	#Fifth Level/No Subdir
#																	cat /sys/block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}"/"${o}" > ./$fileName/sys_block/${i}/"${j}"/"${k}"/"${l}"/"${m}"/"${n}".txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
#																fi
#															done
#														fi
#													done
#												fi
#											done
#										fi
#									done
#								fi
#							done
#						fi
#					done
#				fi
#			fi
#		done
#	fi

scsi_logging_level -g >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then 
	scsi_logging_level -g > ./$fileName/proc/scsi_logging_level-g.txt 
fi
	

if [ -f /proc/bus/pci/devices ] ; then cat /proc/bus/pci/devices > ./$fileName/proc/bus-pci-devices ; fi 2>> ./misc_output.txt
if [ -f /proc/net/dev ] ; then cat /proc/net/dev > ./$fileName/proc/net-dev ; fi 2>> ./misc_output.txt



if [ "$TWGETSKIPHBA" != "YES" ] ; then
	if [ "$NO_LSI_HBAs" != "YES" ] ; then 
		echo "Starting the LSI HBA Data Collection..."
	
		for i in $(./$LSUT_NAME 0 2>> ./misc_output.txt | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
		
			#cho ".................................................||................................................."
			echo $fileName >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo "..............................................LSITool..............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME 0 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo "" >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 1.............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 1 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 8.............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 8 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 16............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 16 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 17............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 17 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo "...........................................LSITool 21-1-2..........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i -a 21,1,2,0,0 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo "..........................................LSITool 25-2-0-0.........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i -a 25,2,0,0 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 42............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 42 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 47............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 47 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo ".............................................LSITool 66............................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i 66 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			echo "...........................................LSITool 64-0-0..........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
			./$LSUT_NAME -p $i -a 64,,,0,0, 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
	
			#misc pl cli commands
			for j in dbg info param mids dev iocount status phy cphy port smp encl expconn exp cmdbuf eminfo inittbl hwftrs stallnf txinfo temp mreq bt1680 port ; do
				./$LSUT_NAME -p $i -a 65,pl_${j}_hba${i}.txt,"pl ${j}",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
				if [ ! -d ./$fileName/LSI_Products/HBA/Details ] ; then mkdir ./$fileName/LSI_Products/HBA/Details ; fi
				if [ -f ./pl_${j}_hba${i}.txt ] ; then
					$grep -e "pl --   PL Commands" -e "pl -- PL Commands" ./pl_${j}_hba${i}.txt >> ./misc_output.txt 2>&1
					if [ "$?" != "0" ] ; then
						#cho ".................................................||................................................."
						echo "......................................pl_${j}_hba${i}.txt..........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						cat ./pl_${j}_hba${i}.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						mv ./pl_${j}_hba${i}.txt ./$fileName/LSI_Products/HBA/Details
						else
						mv ./pl_${j}_hba${i}.txt ./$fileName/script_workspace
					fi
				fi
			done		
			
			./$LSUT_NAME -p $i -a 65,pl_int_0_hba${i}.txt,"pl int 0",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_int_1_hba${i}.txt,"pl int 1",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_fpe_stats_hba${i}.txt,"pl fpe stats",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_fpe_iopcq_hba${i}.txt,"pl fpe iopcq",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_tm_outstanding_hba${i}.txt,"pl tm outstanding",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_tm_task_hba${i}.txt,"pl tm task",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
			./$LSUT_NAME -p $i -a 65,pl_tbm_reg_all_hba${i}.txt,"pl tbm reg all",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
	
			for j in pl_int_0_hba${i}.txt pl_int_1_hba${i}.txt pl_fpe_stats_hba${i}.txt pl_fpe_iopcq_hba${i}.txt pl_tm_outstanding_hba${i}.txt pl_tm_task_hba${i}.txt pl_tbm_reg_all_hba${i}.txt ; do
				if [ ! -d ./$fileName/LSI_Products/HBA/Details ] ; then mkdir ./$fileName/LSI_Products/HBA/Details ; fi
				if [ -f ./${j} ] ; then
					$grep -e "pl --   PL Commands" -e "pl -- PL Commands" ./pl_${j}_hba${i}.txt >> ./misc_output.txt 2>&1
					if [ "$?" != "0" ] ; then 
						#cho ".................................................||................................................."
						echo ".............................................${j}..........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						cat ./${j} >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						mv ./${j} ./$fileName/LSI_Products/HBA/Details
						else
						mv ./${j} ./$fileName/script_workspace
					fi
				fi
			done	
	
			#misc iop show commands
			for j in mem msgctx tlb ver sbr cfg stack fifo reqfifo diag snap hwftrs temp; do
				./$LSUT_NAME -p $i -a 65,iop_show_${j}_hba${i}.txt,"iop show ${j}",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
				if [ ! -d ./$fileName/LSI_Products/HBA/Details ] ; then mkdir ./$fileName/LSI_Products/HBA/Details ; fi
				if [ -f ./iop_show_${j}_hba${i}.txt ] ; then 
					$grep -e "iop --  IOP commands" -e "iop -- IOP Commands" ./iop_${j}_hba${i}.txt >> ./misc_output.txt 2>&1
					if [ "$?" != "0" ] ; then 
						#cho ".................................................||................................................."
						echo ".....................................iop_${j}_hba${i}.txt......................................" >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						cat ./iop_show_${j}_hba${i}.txt >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						mv ./iop_show_${j}_hba${i}.txt ./$fileName/LSI_Products/HBA/Details/.
						else
						mv ./iop_show_${j}_hba${i}.txt ./$fileName/script_workspace
					fi
				fi
			done
	
	
			./$LSUT_NAME -p $i -a 65,iop_mreq_m_hba${i}.txt,"iop mreq m",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_mreq_s_hba${i}.txt,"iop mreq s",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_mreq_r_hba${i}.txt,"iop mreq r",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_pm_getgmode_hba${i}.txt,"iop pm getgmode",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_fpe_maxex_hba${i}.txt,"iop fpe maxex",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_fpe_maxev_hba${i}.txt,"iop fpe maxev",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
			./$LSUT_NAME -p $i -a 65,iop_perfmon_show_hba${i}.txt,"iop perfmon show",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt	
		
	
			for j in iop_gpio_hba${i}.txt iop_mreq_m_hba${i}.txt iop_mreq_s_hba${i}.txt iop_mreq_r_hba${i}.txt iop_pm_getgmode_hba${i}.txt iop_fpe_maxex_hba${i}.txt iop_fpe_maxev_hba${i}.txt iop_perfmon_show_hba${i}.txt; do
				if [ ! -d ./$fileName/LSI_Products/HBA/Details ] ; then mkdir ./$fileName/LSI_Products/HBA/Details ; fi
				if [ -f ./${j} ] ; then 
					$grep -e "iop --  IOP commands" -e "iop -- IOP Commands" ./${j} >> ./misc_output.txt 2>&1
					if [ "$?" != "0" ] ; then 
						#cho ".................................................||................................................."
						echo ".............................................${j}..........................................." >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						cat ./${j} >> ./$fileName/LSI_Products/HBA/properties_hba$i.txt
						mv ./${j} ./$fileName/LSI_Products/HBA/Details/.
						else
						mv ./${j} ./$fileName/script_workspace
					fi
				fi
			done	
	
		
			#echo "............................................LSITool 100............................................." >> ./$fileName/LSI_Products/HBA/100_hba$i.txt
			./$LSUT_NAME -p $i 100 2>> ./misc_output.txt >> ./$fileName/LSI_Products/HBA/100_hba$i.txt >> ./$fileName/LSI_Products/HBA/100_hba$i.txt
			
			echo "REM lsipage is an LSI internal utility" > ./$fileName/LSI_Products/HBA/100_hba$i.bat
			echo "copy 100_hba$i.txt 100_hba$i" >> ./$fileName/LSI_Products/HBA/100_hba$i.bat
			echo "lsipage -h -i 100_hba$i" >> ./$fileName/LSI_Products/HBA/100_hba$i.bat
			echo "del 100_hba$i" >> ./$fileName/LSI_Products/HBA/100_hba$i.bat
	
			#cho ".................................................||................................................."
			echo ".................ManPages 17-31, option 100 does not capture these currently........................" >> ./$fileName/LSI_Products/HBA/ManPages_16-31_hba$i.txt
			echo ".............ManPages 30 Offset 0x8 shows Databolt, 00200000 Enabled, 00200001 Disabled............." >> ./$fileName/LSI_Products/HBA/ManPages_16-31_hba$i.txt		
			echo ".......................00200002 DataBOLT and Fast Context Switching Disabled........................" >> ./$fileName/LSI_Products/HBA/ManPages_16-31_hba$i.txt		
			for j in 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31; do 
				#cho ".................................................||................................................."
				echo ".............................................ManPage $j............................................." >> ./$fileName/LSI_Products/HBA/ManPages_16-31_hba$i.txt
				./$LSUT_NAME -p $i -a 9,9,$j,0,,,0 2>> ./misc_output.txt | grep " : " >> ./$fileName/LSI_Products/HBA/ManPages_16-31_hba$i.txt
			done
		done
	fi

	###########################################################################################################################
	#Starting scl HBA Data Collection
	############################################################################################################################
	echo "Before scli HBA section initial test"  >> ./misc_output.txt 2>&1

	if [ "$NO_scl_EXP_or_IOC" = "NO" ] ; then
		echo "After scli HBA section initial test"  >> ./misc_output.txt 2>&1
		./scl --list | $grep DeviceId -A 100 | $grep ")" | $grep -v -e SAS3x -e SAS35x | $grep -v -e SAS35 >> ./$fileName/script_workspace/sc_gen3_hbas_raw.txt 2>&1 
		if [ ! -s ./$fileName/script_workspace/sc_gen3_hbas_raw.txt ] ; then mv ./$fileName/script_workspace/sc_gen3_hbas_raw.txt ./$fileName/script_workspace/0Byte_sc_gen3_hbas_raw.txt >> ./misc_output.txt 2>&1 ; fi
		if [ -f ./$fileName/script_workspace/sc_gen3_hbas_raw.txt ] ; then	
		 	NO_scl_IOC=NO
			echo IN__scl_IOC_Branch_1 >> ./misc_output.txt 2>&1
			if [ ! -d $fileName/LSI_Products ] ; then mkdir $fileName/LSI_Products ; fi
			if [ ! -d $fileName/LSI_Products/HBA ] ; then mkdir $fileName/LSI_Products/HBA ; fi
			if [ ! -d $fileName/LSI_Products/HBA/12Gb ] ; then mkdir $fileName/LSI_Products/HBA/12Gb ; fi
			if [ ! -d $fileName/LSI_Products/HBA/12Gb/Details ] ; then mkdir $fileName/LSI_Products/HBA/12Gb/Details ; fi
			./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x -e SAS35x | $grep -v -e SAS35 > $fileName/LSI_Products/HBA/12Gb/Details/sc_gen3_hbas.txt
			./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x -e SAS35x | $grep -v -e SAS35 | grep -v DeviceId | cut -d')' -f1 > ./$fileName/script_workspace/sc_gen3_hba_nums.txt
			echo IN__scl_IOC_Branch_2 >> ./misc_output.txt 2>&1
			for i in $( cat ./$fileName/script_workspace/sc_gen3_hba_nums.txt); do
				echo IN__scl_IOC_Branch_3 >> ./misc_output.txt 2>&1
				./scl -i $i ioc -wwid | $grep "Current WWID" | cut -d: -f2 | cut -d' ' -f2 > ./$fileName/script_workspace/sc_hba_${i}_sasaddr.txt
				for j in $( cat ./$fileName/script_workspace/sc_hba_${i}_sasaddr.txt); do 
					echo IN__scl_IOC_Branch_4_HBA${i} >> ./misc_output.txt 2>&1
					#cho "......................................::......................................."
					echo .................................scli HBAs-IOCs................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................show -all.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i show -all >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ......................................scan...................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i scan >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ....................................phy -da..................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -da >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................phy -port.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -port >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ....................................phy -err.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -err >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................phy -evnt.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -evnt >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................ioc -facts -decode................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -facts -decode >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................ioc -diserr................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -diserr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................ioc -displaysas.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -displaysas >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci status............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci status >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci lnkspdr.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci lnkspdr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci lnkwdr............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci lnkwdr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli iop pci reg................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci reg >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop show stack............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop show stack >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop show diag................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop show diag >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl status.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl status >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................cli pl dev................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl dev >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ................................cli pl iocount.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl iocount >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................cli pl mids................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl mids >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl reg link................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg link >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl tm task................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl tm task >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .............................cli pl tm outstanding.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl tm outstanding >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwcontext.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwcontext >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwdevtab............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwdevtab >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwsatatab.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwsatatab >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................cfg -dumpall.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cfg -dumpall >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ......................................logs...................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i logs >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
				done
			done
		fi
		echo "After scli HBA section initial test-35"  >> ./misc_output.txt 2>&1
		./scl --list | $grep DeviceId -A 100 | $grep ")" | $grep -v -e SAS3x -e SAS35x | $grep -e SAS35 >> ./$fileName/script_workspace/sc_gen35_hbas_raw.txt 2>&1 
		if [ ! -s ./$fileName/script_workspace/sc_gen35_hbas_raw.txt ] ; then mv ./$fileName/script_workspace/sc_gen35_hbas_raw.txt ./$fileName/script_workspace/0Byte_sc_gen35_hbas_raw.txt >> ./misc_output.txt 2>&1 ; fi
		if [ -f ./$fileName/script_workspace/sc_gen35_hbas_raw.txt ] ; then	
		 	NO_scl_IOC=NO
			echo IN__scl_IOC_Branch_1-35 >> ./misc_output.txt 2>&1
			if [ ! -d $fileName/LSI_Products ] ; then mkdir $fileName/LSI_Products ; fi
			if [ ! -d $fileName/LSI_Products/HBA ] ; then mkdir $fileName/LSI_Products/HBA ; fi
			if [ ! -d $fileName/LSI_Products/HBA/12Gb ] ; then mkdir $fileName/LSI_Products/HBA/12Gb ; fi
			if [ ! -d $fileName/LSI_Products/HBA/12Gb/Details ] ; then mkdir $fileName/LSI_Products/HBA/12Gb/Details ; fi
			./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x -e SAS35x | $grep -e SAS35  > $fileName/LSI_Products/HBA/12Gb/Details/sc_gen35_hbas.txt
			./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x -e SAS35x | $grep -e SAS35  | grep -v DeviceId | cut -d')' -f1 > ./$fileName/script_workspace/sc_gen35_hba_nums.txt
			echo IN__scl_IOC_Branch_2 >> ./misc_output.txt 2>&1
			for i in $( cat ./$fileName/script_workspace/sc_gen35_hba_nums.txt); do
				echo IN__scl_IOC_Branch_3-35 >> ./misc_output.txt 2>&1
				./scl -i $i ioc -wwid | $grep "Current WWID" | cut -d: -f2 | cut -d' ' -f2 > ./$fileName/script_workspace/sc_hba_${i}_sasaddr.txt
				for j in $( cat ./$fileName/script_workspace/sc_hba_${i}_sasaddr.txt); do 
					echo IN__scl_IOC_Branch_4_HBA${i}-35 >> ./misc_output.txt 2>&1
					#cho "......................................::......................................."
					echo .................................scli HBAs-IOCs................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl --list | $grep DeviceId -A 100 | $grep -v -e SAS3x >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................show -all.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i show -all >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ......................................scan...................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i scan >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ....................................phy -da..................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -da >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................phy -port.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -port >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ....................................phy -err.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -err >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................phy -evnt.................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i phy -evnt >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................ioc -facts -decode................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -facts -decode >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................ioc -diserr................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -diserr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................ioc -displaysas.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -displaysas >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................ioc -displaypcie.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i ioc -displaysas >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci status............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci status >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci lnkspdr.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci lnkspdr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci lnkwdr............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci lnkwdr >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli iop pci reg................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci reg >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop pci resize............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci resize >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ................................cli iop pci msix................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop pci msix >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop show mem............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop show mem >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...............................cli iop show diag................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli iop show diag >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl status.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl status >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ...................................cli pl dev................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl dev >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ................................cli pl iocount.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl iocount >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................cli pl mids................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl mids >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl reg link................................ >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg link >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .................................cli pl tm task................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl tm task >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo .............................cli pl tm outstanding.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl tm outstanding >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwcontext.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwcontext >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwdevtab............................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwdevtab >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..............................cli pl reg hwsatatab.............................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cli pl reg hwsatatab >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ..................................cfg -dumpall.................................. >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i cfg -dumpall >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					#cho "......................................::......................................."
					echo ......................................logs...................................... >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
					./scl -i $i logs >> $fileName/LSI_Products/HBA/12Gb/${j}_scli_hba${i}.txt 2>&1
				done
			done
		fi
	fi
fi



#Expander Data Collection


if [ "$NO_LSI_HBAs" != "YES" ] ; then 
	if [ "$TWGETSKIPEXPANDER" != "YES" ] ; then
		for i in $(./$LSUT_NAME 0 2>> ./misc_output.txt | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
	
		        for j in $(./$LSUT_NAME -p $i -a 64,,,0,0, 2>> ./misc_output.txt | grep EnclServ | grep LSI | cut -d. -f1); do
	
				
		
				./$LSUT_NAME -p $i -a 64,$j,temp_hba${i}exp${j}.txt,"adcread 1 1 1000",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
	
				./$LSUT_NAME -p $i -a 64,$j,showlogs_detail_hba${i}exp${j}.txt,"showlogs detail",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
	
				./$LSUT_NAME -p $i -a 64,$j,enclLogicalId_hba${i}exp${j}.txt,"rdcfg 0xfe01 0",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
	
				for k in date sasaddr showmfg showpost rev phyinfo serdesinfo counters rtr scedebug flashtblinfo sgpioinfo edfbinfo thread memstat coredump showtrace ssptdebug rreg ipconfig; do
					./$LSUT_NAME -p $i -a 64,$j,${k}_hba${i}exp${j}.txt,$k,exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
				done
		
				for k in starfish edfb link connection config; do
					./$LSUT_NAME -p $i -a 64,$j,debuginfo_${k}_hba${i}exp${j}.txt,"debuginfo $k",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
				done

				for k in tx rx; do
					./$LSUT_NAME -p $i -a 64,$j,serdesinfo_${k}_hba${i}exp${j}.txt,"serdesinfo $k",exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
				done
			
				echo $fileName > debuginfo_manual_hba${i}exp${j}.txt
		
				for k in date sasaddr enclLogicalId showmfg showpost temp rev phyinfo serdesinfo serdesinfo_tx serdesinfo_rx counters rtr scedebug flashtblinfo sgpioinfo edfbinfo thread memstat coredump showtrace showlogs_detail ssptdebug debuginfo_starfish debuginfo_edfb debuginfo_link debuginfo_connection debuginfo_config rreg ipconfig; do
					if [ -e ./${k}_hba${i}exp${j}.txt ] ; then
						#cho ".................................................||................................................."
						echo "...........................................$k............................................" >> debuginfo_manual_hba${i}exp${j}.txt
						cat ./${k}_hba${i}exp${j}.txt >> debuginfo_manual_hba${i}exp${j}.txt
						echo >> debuginfo_manual_hba${i}exp${j}.txt
	
						if [ -e ./${k}_hba${i}exp${j}.txt ] ; then 
							mv ${k}_hba${i}exp${j}.txt ./$fileName/script_workspace
						fi
					fi
				done
	
				if [ -e ./debuginfo_manual_hba${i}exp${j}.txt ] ; then grep -e SAS3x -e SAS35x debuginfo_manual_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/12Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/12Gb; fi
						mv debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/12Gb/debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb/${k}_debuginfo_manual_hba${i}exp${j}.txt
	
								$grep COREDUMP ./$fileName/script_workspace/flashtblinfo_hba${i}exp${j}.txt | tr -s ' ' | cut -d' ' -f3 | $grep -c COREDUMP | $grep 1 >> ./misc_output.txt
								if [ "$?" = "0" ] ; then
									$grep COREDUMP ./$fileName/script_workspace/flashtblinfo_hba${i}exp${j}.txt | tr -s ' ' | cut -d' ' -f2 > ./$fileName/script_workspace/${k}_coredump_region.txt
								fi
							done
						fi
					fi
				fi
	
	
				if [ -e ./debuginfo_manual_hba${i}exp${j}.txt ] ; then grep SAS2x debuginfo_manual_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/6Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/6Gb; fi
						mv debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/6Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/6Gb/debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/6Gb/${k}_debuginfo_manual_hba${i}exp${j}.txt
							done
						fi
					fi
				fi
				if [ -e ./debuginfo_manual_hba${i}exp${j}.txt ] ; then grep SASx debuginfo_manual_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/3Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/3Gb; fi
						mv debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/3Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/3Gb/debuginfo_manual_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/3Gb/${k}_debuginfo_manual_hba${i}exp${j}.txt
							done
						fi
					fi
				fi
	
	
				if [ -e ./debuginfo_manual_hba${i}exp${j}.txt ] ; then mv debuginfo_manual_hba${i}exp${j}.txt ./$fileName/script_workspace; fi
	
	
				echo $fileName > debuginfo_hba${i}exp${j}.txt	
				#cho ".................................................||................................................."
				echo ".............................................debuginfo.............................................." >> debuginfo_hba${i}exp${j}.txt	
	
	        	        ./$LSUT_NAME -p $i -a 64,$j,debuginfo_hba${i}exp${j}.txt,debuginfo,exit,0, 2>> ./misc_output.txt >> ./misc_output.txt
	
	
	
				if [ -e ./debuginfo_hba${i}exp${j}.txt ] ; then	grep -e SAS3x -e SAS35x debuginfo_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/12Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/12Gb; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/12Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/12Gb/Details; fi
						mv debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/12Gb/debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb/${k}_debuginfo_hba${i}exp${j}.txt
								for l in date sasaddr enclLogicalId showmfg showpost temp rev phyinfo serdesinfo serdesinfo_tx serdesinfo_rx counters rtr scedebug flashtblinfo sgpioinfo edfbinfo thread memstat coredump showtrace showlogs_detail ssptdebug debuginfo_starfish debuginfo_edfb debuginfo_link debuginfo_connection debuginfo_config rreg ipconfig; do
									if [ -e ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ] ; then mv ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb/Details/${k}_${l}_hba${i}exp${j}.txt ; fi
								done
								if [ -e ./$fileName/LSI_Products/Expander/12Gb/Details/${k}_coredump_hba${i}exp${j}.txt ] ; then 
									$grep "Checking for valid coredump image...SUCCESS." ./$fileName/LSI_Products/Expander/12Gb/Details/${k}_coredump_hba${i}exp${j}.txt >> ./misc_output.txt 2>&1
									if [ "$?" = "0" ] ; then
										mv ./$fileName/LSI_Products/Expander/12Gb/Details/${k}_coredump_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb/Details/COREDUMP_${k}_hba${i}exp${j}.txt
										if [ -e ./$fileName/LSI_Products/Expander/12Gb/Details/COREDUMP_${k}_hba${i}exp${j}.txt ] ; then cp ./$fileName/LSI_Products/Expander/12Gb/Details/COREDUMP_${k}_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/12Gb/COREDUMP_${k}_hba${i}exp${j}.txt ; fi
									fi
								fi
							done
						fi
					fi
	
		
	
	
				fi
	
	
	
	
				if [ -e ./debuginfo_hba${i}exp${j}.txt ] ; then	grep SAS2x debuginfo_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/6Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/6Gb; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/6Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/6Gb/Details; fi
						mv debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/6Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/6Gb/debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/6Gb/${k}_debuginfo_hba${i}exp${j}.txt
								for l in date sasaddr enclLogicalId showmfg showpost temp rev phyinfo serdesinfo serdesinfo_tx serdesinfo_rx counters rtr scedebug flashtblinfo sgpioinfo edfbinfo thread memstat coredump showtrace showlogs_detail ssptdebug debuginfo_starfish debuginfo_edfb debuginfo_link debuginfo_connection debuginfo_config rreg ipconfig; do
									if [ -e ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ] ; then mv ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/6Gb/Details/${k}_${l}_hba${i}exp${j}.txt ; fi
								done
							done
						fi
					fi
				fi
	
				if [ -e ./debuginfo_hba${i}exp${j}.txt ] ; then	grep SASx debuginfo_hba${i}exp${j}.txt >> ./misc_output.txt
					if [ "$?" -eq "0" ] ; then 
						if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/3Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/3Gb; fi
						if [ ! -d ./$fileName/LSI_Products/Expander/3Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/3Gb/Details; fi
						mv debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/3Gb
						if [ -e ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt ] ; then
							for k in $($grep "SxP Port 1 SAS Address:" ./$fileName/script_workspace/sasaddr_hba${i}exp${j}.txt | cut -d: -f2 | cut -dx -f2 | tr -d '\r') ; do
								mv ./$fileName/LSI_Products/Expander/3Gb/debuginfo_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/3Gb/${k}_debuginfo_hba${i}exp${j}.txt
								for l in date sasaddr enclLogicalId showmfg showpost temp rev phyinfo serdesinfo serdesinfo_tx serdesinfo_rx counters rtr scedebug flashtblinfo sgpioinfo edfbinfo thread memstat coredump showtrace showlogs_detail ssptdebug debuginfo_starfish debuginfo_edfb debuginfo_link debuginfo_connection debuginfo_config rreg ipconfig; do
									if [ -e ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ] ; then mv ./$fileName/script_workspace/${l}_hba${i}exp${j}.txt ./$fileName/LSI_Products/Expander/3Gb/Details/${k}_${l}_hba${i}exp${j}.txt ; fi
								done
							done
						fi
					fi
				fi
	
				if [ -e ./debuginfo_hba${i}exp${j}.txt ] ; then	mv debuginfo_hba${i}exp${j}.txt ./$fileName/script_workspace; fi
	
	
	
	
		        done
		done

		###########################################################################################################################
		#Starting scl Expander Data Collection
		############################################################################################################################
		echo "Before scli expander section initial test"  >> ./misc_output.txt 2>&1
		
		if [ "$NO_scl_EXP_or_IOC" = "NO" ] ; then
		echo "After scli expander section initial test"  >> ./misc_output.txt 2>&1
			./scl --list | $grep DeviceId -A 100 | $grep ")" | $grep -e SAS3x -e SAS35x >> ./$fileName/script_workspace/sc_gen3_exps_raw.txt 2>&1 
			if [ ! -s ./$fileName/script_workspace/sc_gen3_exps_raw.txt ] ; then mv ./$fileName/script_workspace/sc_gen3_exps_raw.txt ./$fileName/script_workspace/0Byte_sc_gen3_exps_raw.txt >> ./misc_output.txt 2>&1 ; fi
			if [ -f ./$fileName/script_workspace/sc_gen3_exps_raw.txt ] ; then	
			 	NO_scl_EXP=NO
				echo IN__scl_EXP_Branch_1 >> ./misc_output.txt 2>&1
				if [ ! -d $fileName/LSI_Products ] ; then mkdir $fileName/LSI_Products ; fi
				if [ ! -d $fileName/LSI_Products/Expander ] ; then mkdir $fileName/LSI_Products/Expander ; fi
				if [ ! -d $fileName/LSI_Products/Expander/12Gb ] ; then mkdir $fileName/LSI_Products/Expander/12Gb ; fi
				if [ ! -d $fileName/LSI_Products/Expander/12Gb/Details ] ; then mkdir $fileName/LSI_Products/Expander/12Gb/Details ; fi
				./scl --list | $grep DeviceId -A 100 | $grep -e SAS3x -e SAS35x > $fileName/LSI_Products/Expander/12Gb/Details/sc_expanders.txt
				./scl --list | $grep DeviceId -A 100 | $grep -e SAS3x -e SAS35x | grep -v DeviceId | cut -d')' -f1 > ./$fileName/script_workspace/sc_gen3_exp_nums.txt
				echo IN__scl_EXP_Branch_2 >> ./misc_output.txt 2>&1
				for i in $( cat ./$fileName/script_workspace/sc_gen3_exp_nums.txt); do
					echo IN__scl_EXP_Branch_3 >> ./misc_output.txt 2>&1
					./scl -i $i show | grep "SAS Address" | cut -d: -f2 | tr -d ' ' | tr -d '-' > ./$fileName/script_workspace/sc_exp_${i}_sasaddr.txt
					for j in $( cat ./$fileName/script_workspace/sc_exp_${i}_sasaddr.txt); do 
						echo IN__scl_EXP_Branch_4_Expander${i} >> ./misc_output.txt 2>&1



						#cho "......................................::......................................."
						echo "................................scli Expanders................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl --list | $grep DeviceId >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl --list | $grep DeviceId -A 100 | $grep -e SAS3x -e SAS35x >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "..................................show -all...................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i show -all >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "...................................phy -da....................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i phy -da >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "...................................phy -edfb..................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i phy -edfb >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "...................................phy -err...................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i phy -err >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "..................................phy -speed..................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i phy -speed >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo ".....................................trace....................................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						echo "Commented out, hanging intermittantly" >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#./scl -i $i trace >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						#cho "......................................::......................................."
						echo "...............................healthlogs -decode.............................." >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i healthlogs -decode >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}.txt 2>&1
						./scl -i $i ul -fw >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}_up_fw.bin 2>&1
						./scl -i $i ul -mfg >> $fileName/LSI_Products/Expander/12Gb/${j}_scli_exp${i}_up_mfg.bin 2>&1
					done
				done
			fi
		fi
	fi
	
	if [ "$TWGETSKIPHBA" != "YES" ] ; then
		###########################################################################################################################
		# Starting sas2ircu & sas3ircu Controller Data Collection
		###########################################################################################################################
	
	
	
		for i in $(./sas2ircu list 2>> ./misc_output.txt | grep h: | cut -b 4-5); do # Support for 100 HBAs
			NO_2ircu_HBAs=NO
			#cho ".................................................||................................................."
			echo $fileName >> ./$fileName/LSI_Products/HBA/sas2ircu_hba$i.txt
			#cho ".................................................||................................................."
			echo "...........................................sas2ircu list............................................" >> ./$fileName/LSI_Products/HBA/sas2ircu_hba$i.txt
			./sas2ircu list >> ./$fileName/LSI_Products/HBA/sas2ircu_hba$i.txt
			echo "........................................sas2ircu $i display........................................." >> ./$fileName/LSI_Products/HBA/sas2ircu_hba$i.txt
			./sas2ircu $i display >> ./$fileName/LSI_Products/HBA/sas2ircu_hba$i.txt
		done
	
		for i in $(./sas3ircu list 2>> ./misc_output.txt | grep h: | cut -b 4-5); do # Support for 100 HBAs
			NO_3ircu_HBAs=NO
			#cho ".................................................||................................................."
			echo $fileName >> ./$fileName/LSI_Products/HBA/sas3ircu_hba$i.txt
			#cho ".................................................||................................................."
			echo "...........................................sas3ircu list............................................" >> ./$fileName/LSI_Products/HBA/sas3ircu_hba$i.txt
			./sas3ircu list >> ./$fileName/LSI_Products/HBA/sas3ircu_hba$i.txt
			echo "........................................sas3ircu $i display........................................." >> ./$fileName/LSI_Products/HBA/sas3ircu_hba$i.txt
			./sas3ircu $i display >> ./$fileName/LSI_Products/HBA/sas3ircu_hba$i.txt
		done
	fi
fi

if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	if [ "$USEMEGACLISYNTAX" = "YES" ] ; then

		###########################################################################################################################
		# Starting MegaCli MegaRAID Controller Data Collection
		###########################################################################################################################
		if [ "$OS_LSI" != "macos" ] ; then
	
			
			#MegaRAID Adapter #'s
			#Changed to storcli syntax to avoid using tr
			$MCLI_LOCATION$MCLI_NAME show > ./$fileName/script_workspace/storcli_show.txt
			$MCLI_LOCATION$MCLI_NAME show ctrlcount | grep "Controller Count" | cut -d" " -f 4 > ./$fileName/script_workspace/num_mraid_adapters.txt
			if [ -n `cat ./$fileName/script_workspace/num_mraid_adapters.txt` ] ; then
				for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do
					if [ "$i" -lt "`cat ./$fileName/script_workspace/num_mraid_adapters.txt`" ] ; then echo $i >> ./$fileName/script_workspace/adapter_numbers.txt ; fi
				done
			fi
		
			# Make sure at least 1 MegaRAID Adapter is identified
			if [ -f ./$fileName/script_workspace/adapter_numbers.txt ] ; then

				echo "Start of the Optional MegaRAID Adapter Data Collection with MegaCli syntax..."
		
				for i in `cat ./$fileName/script_workspace/adapter_numbers.txt` ; do #Support for all adapter IDs 
	
					# MegaRAID Logical Disk #'s
					#Changed to storcli syntax to avoid using tr
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show j | grep "DG/VD" | wc -l > ./$fileName/script_workspace/num_lds_A$i.txt
					if [ -n `cat ./$fileName/script_workspace/num_lds_A$i.txt` ] ; then
						for j in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255; do #Supports up to 256 LDs
							if [ "$j" -lt "`cat ./$fileName/script_workspace/num_lds_A$i.txt`" ] ; then
						 		echo $j >> ./$fileName/script_workspace/ld_numbers_A$i.txt
								else
						  		break
							fi
						done
					fi
	
	
		
					if [ "$LimitMegaCliCMDs" != "YES" ] ; then 
					#MegaRAID Enclosure #'s
						$MCLI_LOCATION$MCLI_NAME encinfo a$i | grep "Device ID" | awk  '{ print $4 }' >> ./$fileName/script_workspace/enclosure_numbers_A$i.txt
					fi
		
					#MegaRAID phy #'s
					$MCLI_LOCATION$MCLI_NAME phyerrorcounters a$i | grep "Phy No:" | awk '{ print $3 }' >> ./$fileName/script_workspace/phy_numbers_A$i.txt
		
					#MegaRAID PCI data, used for grepping.
					$MCLI_LOCATION$MCLI_NAME adpgetpciinfo a$i nolog 2>> ./misc_output.txt >> ./$fileName/script_workspace/pci_info_A$i.txt
		
					#MegaRAID PDList data, used for grepping.
					echo $fileName >> ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt 2>&1
					$MCLI_LOCATION$MCLI_NAME pdlist a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt 2>&1
		
					#MegaRAID Disk Device ID #'s
					$grep -e "Device Id:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt | cut -d" " -f3 >> ./$fileName/script_workspace/disk_dev_id_numbers_A$i.txt
		
					#MegaRAID adpallinfo data, used for grepping.
					echo $fileName >> ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt 2>&1
					$MCLI_LOCATION$MCLI_NAME adpallinfo a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt 2>&1
		
					#MegaRAID adpalilog data, used for grepping.
					if [ "$LimitMegaCliCMDs" != "YES" ] ; then
						echo $fileName >> ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAliLog_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpalilog a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAliLog_A$i.txt 2>&1
					fi
		
		

					
					#cho ".................................................||................................................."
					echo $fileName >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					echo "......................................./$MCLI_NAME adpcount.........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME adpcount nolog | $grep Count: >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "............................................Adapter a$i.............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ -f ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt ] ; then
						$grep -e "Product Name    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Memory Size     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Host Interface  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Serial No       :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "SAS Address     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "FW Package Build:" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "FW Version         :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Mfg. Date       :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "BBU             :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Battery FRU     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Serial Debugger :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "On board Expander:" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "On board Expander FW version :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Driver Name:" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAliLog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grepA -m 1 -e "Driver Version:" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAliLog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "..............................................PCI Info.............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Bus Number      :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Device Number   :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Function Number :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						echo "..........................................MegaRAID PCI Info........................................." >> ./$fileName/Controller_Disk_Association.txt
						$grep -e "PCI information for Controller" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/Controller_Disk_Association.txt 2>&1
						$grep -e "Bus Number      :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/Controller_Disk_Association.txt 2>&1
						$grep -e "Device Number   :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/Controller_Disk_Association.txt 2>&1
						$grep -e "Function Number :" ./$fileName/script_workspace/pci_info_A$i.txt >> ./$fileName/Controller_Disk_Association.txt 2>&1
						echo "...............................................Errors..............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Memory Correctable Errors   :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Memory Uncorrectable Errors :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Any Offline VD Cache Preserved   :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	$grep -e "ECC Bucket Count                 :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "...........................................Rate Settings............................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					#	$grep -e "Ecc Bucket Size                  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	$grep -e "Ecc Bucket Leak Rate             :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Predictive Fail Poll Interval    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	$grep -e "Interrupt Throttle Active Count  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	$grep -e "Interrupt Throttle Completion    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Rebuild Rate                     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "PR Rate                          :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	echo "Note: Resynch Rate is BgiRate" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "BGI Rate                         :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Check Consistency Rate           :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Reconstruction Rate              :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Physical Drive Coercion Mode     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo ".............................................Performance............................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Cache Flush Interval             :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Host Request Reordering          :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Load Balance Mode                :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop NCQDsply a$i nolog | $grep "NCQ" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop WBSupport a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop perfmode a$i nolog | $grep "Perf Tuned Mode :" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "................................Enclosures/Backplanes and Connectors................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Auto Detect BackPlane Enabled    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop ExposeEnclDevicesEnbl a$i nolog | $grep "Expose" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetconnectormode connectorall a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "........................................Alarms and Warnings........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Alarm                            :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Battery Warning                  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "........................................Rebuild and Hotspare........................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Auto Rebuild                     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Restore HotSpare on Insertion    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop AutoEnhancedImportDsply a$i nolog | $grep "Auto" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop MaintainPdFailHistoryEnbl a$i nolog | $grep "Maintain" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo ".............................................Copy Back.............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME adpgetprop CopyBackDsbl a$i nolog | $grep "Copyback" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop SMARTCpyBkEnbl a$i nolog | $grep "Copyback" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop SSDSMARTCpyBkEnbl a$i nolog | $grep "Copyback" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo ".............................................PR and CC.............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Enable SSD Patrol Read                  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop PrCorrectUncfgdAreas a$i nolog | $grep "Unconfigured" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop AbortCCOnError a$i nolog | $grep "Abort" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "..........................................ELF/Advanced SW..........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME elf getsafeid a$i nolog | $grep "Safe" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME elf rehostinfo a$i nolog | $grep "Needs"  >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME elf ControllerFeatures a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo ".............................................Encryption............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Security Key Assigned            :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Security Key Failed              :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Security Key Not Backedup        :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop UseFDEOnlyEncrypt a$i nolog | $grep "FDE Only Encryption:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "..........................................Power Management.........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Max Drives to Spinup at One Time :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Delay Among Spinup Groups        :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						echo "#### Dimmer Switch 1" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME adpgetprop EnblSpinDownUnConfigDrvs a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						echo "#### Dimmer Switch 2" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME adpgetprop DsblSpinDownHSP a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						echo "#### Dimmer Switch 3" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME adpgetprop DefaultLdPSPolicy a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop DisableLdPsInterval a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop DisableLdPsTime a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop SpinDownTime a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop SpinUpEncDrvCnt a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpgetprop SpinUpEncDelay a$i nolog | $grep "Adapter" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo ".............................................BIOS/Boot.............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$MCLI_LOCATION$MCLI_NAME adpbios dsply a$i nolog | $grep "BIOS" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME adpbootdrive get a$i nolog | $grep "boot" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#	$grep -e "Cluster Mode                     :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "...............................................Drives..............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						$grep -e "Virtual Drives    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Degraded        :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Offline         :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Physical Devices  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Disks           :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Critical Disks  :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						$grep -e "Failed Disks    :" ./$fileName/LSI_Products/MegaRAID/MegaCli/AdpAllInfo_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
					fi
					
					#cho ".................................................||................................................."
					echo "....................................Adapter/System Time Sync A$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					echo "Adapter Date/Time:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$MCLI_LOCATION$MCLI_NAME adpgettime a$i nolog | $grep -e "Date:" -e "Time:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					echo "System Date/Time:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					date >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					echo "System Date/Time UTC:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					date -u >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
					#cho ".................................................||................................................."
					echo ".............................../$MCLI_NAME getpreservedcachelist a$i.................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME getpreservedcachelist a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
					if [ -f ./$fileName/script_workspace/ld_numbers_A$i.txt ] ; then
						for j in `cat ./$fileName/script_workspace/ld_numbers_A$i.txt` ; do #Support for up to 256 LDs
							#cho ".................................................||................................................."
							echo "...................................../$MCLI_NAME ldinfo l$j a$i........................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
								date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
								date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							$MCLI_LOCATION$MCLI_NAME ldinfo l$j a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							#cho ".................................................||................................................."
							echo ".........................../$MCLI_NAME ldgetprop consistency l$j a$i..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
								date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
								date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							$MCLI_LOCATION$MCLI_NAME ldgetprop consistency l$j a$i nolog | $grep "Virtual" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							#cho ".................................................||................................................."
							echo "............................./$MCLI_NAME ldgetprop pspolicy l$j a$i...................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
								date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
								date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							fi
							$MCLI_LOCATION$MCLI_NAME ldgetprop pspolicy l$j a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							#cho ".................................................||................................................."
							echo ".....................................Init/CC/Recon Status l$j a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
							$MCLI_LOCATION$MCLI_NAME ldinit showprog l$j a$i nolog | $grep "Initialization" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							$MCLI_LOCATION$MCLI_NAME ldbi showprog l$j a$i nolog | $grep "Background" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							$MCLI_LOCATION$MCLI_NAME ldcc showprog l$j a$i nolog | $grep "Check" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
							$MCLI_LOCATION$MCLI_NAME ldrecon showprog l$j a$i nolog | $grep "Reconstruction" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						done
					fi
					
					echo "..................................../$MCLI_NAME adppr info a$i........................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME adppr info a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
				
					echo "................................../$MCLI_NAME adpccsched info a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
	
					$MCLI_LOCATION$MCLI_NAME adpccsched info a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
				
					echo ".................................../$MCLI_NAME pdgetmissing a$i......................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
	
					$MCLI_LOCATION$MCLI_NAME pdgetmissing a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "...................................Logical Disks & Physical Disks..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
	
					$MCLI_LOCATION$MCLI_NAME ldpdinfo a$i nolog | $grep -e "Virtual Disk:" -e "RAID Level:" -e "Number Of Drives:" -e "PD:" -e "Enclosure Device ID:" -e "Slot Number:" -e "Device Id:" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo ".....................................Physical Disk - Slot Number...................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Slot Number:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "......................................Physical Disk - Device Id....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Device Id:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo ".....................................Physical Disk - Inquiry Data..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Inquiry Data:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo ".....................................Physical Disk - SAS Address...................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "SAS Address" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "....................................Physical Disk - Firmware state.................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Firmware state:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "....................................Physical Disk - Foreign State..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e  "Foreign State:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "...............................Physical Disk - Predictive Failure Count............................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Predictive Failure Count:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo ".....................................Physical Disk - Link Speed....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Link Speed:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo ".......................................Physical Disk - Type........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "PD Type:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "......................................Physical Disk - Common........................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					$grep -e "Slot Number:" -e "Enclosure Device ID:" -e "Device Id:" -e "Inquiry Data:" -e "Firmware state:" -e "Predictive Failure Count:" -e "Link Speed:" -e "PD Type:" ./$fileName/LSI_Products/MegaRAID/MegaCli/PDList_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
				
					
					
					if [ "$LimitMegaCliCMDs" != "YES" ] ; then 
						echo "Collecting Enclosure Information for Adapter A$i with MegaCli syntax..." >> ./misc_output.txt 2>&1
						#cho ".................................................||................................................."
						echo "....................................../$MCLI_NAME encinfo a$i........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
	
						$MCLI_LOCATION$MCLI_NAME encinfo a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					fi
					
					echo "Collecting Information for Adapter A$i with MegaCli syntax..." >> ./misc_output.txt 2>&1
					
					#cho ".................................................||................................................."
	
					#Solaris/storcli work around - segmentation fault 
	
					if [ "$OS_LSI" != "solaris" ] ; then
						echo "...................................../$MCLI_NAME adpbbucmd a$i........................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
		
						$MCLI_LOCATION$MCLI_NAME adpbbucmd a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					fi			
	
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME cfgforeign scan a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgforeign scan a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					echo "................................./$MCLI_NAME cfgforeign dsply a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgforeign dsply a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					echo "................................/$MCLI_NAME cfgforeign preview a$i...................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgforeign preview a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					
					echo "................................./$MCLI_NAME cfgfreespaceinfo a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgfreespaceinfo a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					echo "............................/$MCLI_NAME cfgsave -f cfgsave_A$i.cfg a$i................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgsave -f ./$fileName/LSI_Products/MegaRAID/MegaCli/cfgsave_A$i.cfg a$i nolog >> ./misc_output.txt 2>&1
				
					#cho ".................................................||................................................."
					echo "............................/$MCLI_NAME adpeventlog geteventloginfo a$i..............................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME adpeventlog geteventloginfo a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1

					if [ "SKIPPING" = "THIS" ] ; then
						#cho ".................................................||................................................."
						echo ".............../$MCLI_NAME adpeventlog getevents -f eventlog_getevents_A$i.txt a$i....................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						$MCLI_LOCATION$MCLI_NAME adpeventlog getevents -f ./$fileName/LSI_Products/MegaRAID/AENs/eventlog_getevents_A$i.txt a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "........./$MCLI_NAME adpeventlog getsinceshutdown -f eventlog_getsinceshutdown_A$i.txt a$i............" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						$MCLI_LOCATION$MCLI_NAME adpeventlog getsinceshutdown -f ./$fileName/LSI_Products/MegaRAID/AENs/eventlog_getsinceshutdown_A$i.txt a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
						#cho ".................................................||................................................."
						echo "........./$MCLI_NAME adpeventlog getsincereboot -f eventlog_getsincereboot_A$i.txt a$i................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						$MCLI_LOCATION$MCLI_NAME adpeventlog getsincereboot -f ./$fileName/LSI_Products/MegaRAID/AENs/eventlog_getsincereboot_A$i.txt a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
						#cho ".................................................||................................................."
						echo "........./$MCLI_NAME adpeventlog includedeleted -f eventlog_includedeleted_A$i.txt a$i................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						$MCLI_LOCATION$MCLI_NAME adpeventlog includedeleted -f ./$fileName/LSI_Products/MegaRAID/AENs/eventlog_includedeleted_A$i.txt a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					fi		
					
					#cho ".................................................||................................................."
					echo "...................................../$MCLI_NAME cfgdsply a$i.........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME cfgdsply a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
					echo "Collecting Logical Disk Information for Adapter A$i with MegaCli syntax..." >> ./misc_output.txt 2>&1
					#cho ".................................................||................................................."
					echo "...................................../$MCLI_NAME ldpdinfo a$i........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME ldpdinfo a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					echo "................................/$MCLI_NAME phyerrorcounters a$i......................................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME phyerrorcounters a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					
					for j in `cat ./$fileName/script_workspace/phy_numbers_A$i.txt` ; do
						#cho ".................................................||................................................."
						echo ".................................../$MCLI_NAME phyinfo phy$j a$i......................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						$MCLI_LOCATION$MCLI_NAME phyinfo phy$j a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					done
					#cho ".................................................||................................................."
					echo "............................../$MCLI_NAME directpdmapping dsply a$i..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME directpdmapping dsply a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					echo "Collecting Physical Disk Information for Adapter A$i with MegaCli syntax..." >> ./misc_output.txt 2>&1
					echo "....................................../$MCLI_NAME pdlist a$i.........................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME pdlist a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					#cho ".................................................||................................................."
					
			

					###########################################################################################################################
					# MegaOEM INI dump if installed
					###########################################################################################################################
	
					if [ -f /opt/MegaRAID/MegaOEM/MegaOEM ] ; then 
						#cho ".................................................||................................................."
						echo "................................./opt/MegaRAID/MegaOEM/MegaOEM -v..................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						/opt/MegaRAID/MegaOEM/MegaOEM -v >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						echo "...........................MegaOEM adpsettings write -f MFC_Settings_opt_A$i.ini a$i......................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						fi
						/opt/MegaRAID/MegaOEM/MegaOEM adpsettings write -f ./$fileName/LSI_Products/MegaRAID/MFC_Settings_opt_A$i.ini a$i >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
	
	
					if [ -f ./MegaOEM ] ; then 
						#cho ".................................................||................................................."
						echo ".............................................MegaOEM -v............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						./MegaOEM -v >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
			
						echo "...........................MegaOEM adpsettings write -f MFC_Settings_A$i.ini a$i......................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
						./MegaOEM adpsettings write -f ./$fileName/LSI_Products/MegaRAID/MFC_Settings_A$i.ini a$i >> ./$fileName/LSI_Products/MegaRAID/Adapter_A$i.txt
					fi
	
					MegaOEM -v >> ./misc_output.txt 2>&1
					if [ "$?" -eq "0" ] ; then 
						#cho ".................................................||................................................."
						echo ".............................................MegaOEM -v............................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						MegaOEM -v >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
			
						echo "...........................MegaOEM adpsettings write -f MFC_Settings_path_A$i.ini a$i......................" >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
						MegaOEM adpsettings write -f ./$fileName/LSI_Products/MegaRAID/MFC_Settings_path_A$i.ini a$i >> ./$fileName/LSI_Products/MegaRAID/Adapter_A$i.txt
					fi
	
	
					echo "Collecting Internal Logs for Adapter A$i..." >> ./misc_output.txt 2>&1
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME fwtermlog dsply a$i....................................." >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S' >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt
					fi
					$MCLI_LOCATION$MCLI_NAME fwtermlog dsply a$i nolog >> ./$fileName/LSI_Products/MegaRAID/MegaCli/Adapter_A$i.txt 2>&1
					echo $fileName >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt 2>&1
					$MCLI_LOCATION$MCLI_NAME fwtermlog dsply a$i nolog >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt 2>&1
	
	
	
					###########################################################################################################################
					# MegaCli fwtermlog dsply Ax - fwtermlog error screening
					# 
					# Words to look out for... "Fatal firmware error: Line" Fault Panic BAIL_OUT Paused REC CRC Unrecoverable Sense "Battery Put to Sleep" 
					###########################################################################################################################
	
	
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "Fatal firmware error: Line" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...............$MCLI_NAME fwtermlog dsply a$i - contains Fatal firmware error: Line...................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fatal_FW_Error_FWTermLog_A$i.txt
							echo "........This indicates that a SERIOUS FW Issue has occurred, contact Tech. Support or your FAE......." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fatal_FW_Error_FWTermLog_A$i.txt
							$grep "Fatal firmware error: Line" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fatal_FW_Error_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep " Fault " ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains Fault............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fault_FWTermLog_A$i.txt
							echo "........This indicates that a SERIOUS FW Issue has occurred, contact Tech. Support or your FAE......." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fault_FWTermLog_A$i.txt
							$grep " Fault " ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Fault_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "Panic" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains Panic............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Panic_FWTermLog_A$i.txt
							echo "........This indicates that a SERIOUS FW Issue has occurred, contact Tech. Support or your FAE......." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Panic_FWTermLog_A$i.txt
							$grep "Panic" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Panic_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "BAIL_OUT" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains BAIL_OUT.........................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BAIL_OUT_FWTermLog_A$i.txt
							echo "........This indicates that a SERIOUS FW Issue has occurred, contact Tech. Support or your FAE......." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BAIL_OUT_FWTermLog_A$i.txt
							$grep "BAIL_OUT" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BAIL_OUT_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "Paused" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains Paused............................" >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Paused_FWTermLog_A$i.txt
							echo "....................................This should be investigated....................................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Paused_FWTermLog_A$i.txt
							$grep "Paused" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Paused_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep -e "MPI2_EVENT_SAS_QUIESCE_RC_STARTED" -e "MPI2_EVENT_SAS_QUIESCE_RC_COMPLETED" -e "Test event: An unexpected data IO error occurred on PD" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "..........................$MCLI_NAME fwtermlog dsply a$i - contains Test_Event............................" >> ./$fileName/LSI_Products/MegaRAID/Adapter_A$i.txt
							echo "....................................The controller should be RMAed.................................." >> ./$fileName/LSI_Products/MegaRAID/Adapter_A$i.txt
							echo "..........................$MCLI_NAME fwtermlog dsply a$i - contains Test_Event............................" >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Test_Event_FWTermLog_A$i.txt
							echo "....................................The controller should be RMAed.................................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Test_Event_FWTermLog_A$i.txt
							$grep -e "MPI2_EVENT_SAS_QUIESCE_RC_STARTED" -e "MPI2_EVENT_SAS_QUIESCE_RC_COMPLETED" -e "Test event: An unexpected data IO error occurred on PD" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Test_Event_FWTermLog_A$i.txt
						fi
					fi
			
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "REC" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains REC..............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/REC_FWTermLog_A$i.txt
							echo "....................................This should be investigated....................................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/REC_FWTermLog_A$i.txt
							$grep "REC" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/REC_FWTermLog_A$i.txt
						fi
					fi
			
					
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "Unrecoverable" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...................$MCLI_NAME fwtermlog dsply a$i - contains Unrecoverable............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Unrecoverable_FWTermLog_A$i.txt
							echo "....................................This should be investigated....................................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Unrecoverable_FWTermLog_A$i.txt
							$grep "Unrecoverable" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Unrecoverable_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep "CRC" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains CRC..............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/CRC_FWTermLog_A$i.txt
							echo "....................................This should be investigated....................................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/CRC_FWTermLog_A$i.txt
							$grep "CRC" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/CRC_FWTermLog_A$i.txt
						fi
					fi
					
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep -i "Sense" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "...........................$MCLI_NAME fwtermlog dsply a$i - contains Sense............................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Sense_FWTermLog_A$i.txt
							echo "...........................................Informational............................................" >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Sense_FWTermLog_A$i.txt
							$grep -i "Sense" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/Sense_FWTermLog_A$i.txt
						fi
					fi
			
					if [ -f ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt ] ; then 
					$grep -i "Battery Put to Sleep" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./misc_output.txt
						if [ "$?" -eq "0" ] ; then
							#cho ".................................................||................................................."
							echo "..................$MCLI_NAME fwtermlog dsply a$i - contains Battery Put to Sleep......................." >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BatterySleep_FWTermLog_A$i.txt
							echo "...........................................Informational............................................" >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BatterySleep_FWTermLog_A$i.txt
							$grep -i "Battery Put to Sleep" ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_A$i.txt >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/BatterySleep_FWTermLog_A$i.txt
						fi
					fi
	
				# Adapter number
				done
			fi
		fi
	fi			



###########################################################################################################################
# Starting storcli MegaRAID Controller Data Collection
###########################################################################################################################
				
	if [ "$OS_LSI" != "macos" ] ; then

		#MegaRAID Adapter #'s
		$MCLI_LOCATION$MCLI_NAME show > ./$fileName/script_workspace/storcli_show.txt
		$MCLI_LOCATION$MCLI_NAME show ctrlcount | grep "Controller Count" | cut -d" " -f 4 > ./$fileName/script_workspace/num_mraid_adapters.txt
		if [ -n `cat ./$fileName/script_workspace/num_mraid_adapters.txt` ] ; then
			for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do
				if [ "$i" -lt "`cat ./$fileName/script_workspace/num_mraid_adapters.txt`" ] ; then echo $i >> ./$fileName/script_workspace/adapter_numbers.txt ; fi
			done
		fi
	
		# Make sure at least 1 MegaRAID Adapter is identified
		if [ -f ./$fileName/script_workspace/adapter_numbers.txt ] ; then
						
			
			echo "Starting MegaRAID Controller Data Collection with storcli..."
		
		
			echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			# Using /cx instead - need to get rid of duplicate "Controller =" entries.	
		
			#cho ".................................................||................................................."
			#echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt
			#echo ".................................../$MCLI_NAME /call show all........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt
			#$MCLI_LOCATION$MCLI_NAME /call show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt
		
			# Doesn't add additional data
			#cho ".................................................||................................................."
			#echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Dall_show_all.txt
			#echo "................................/$MCLI_NAME /call/dall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Dall_show_all.txt
			#$MCLI_LOCATION$MCLI_NAME /call/dall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Dall_show_all.txt
		
			# Doing C$i/vall show all instead
			#cho ".................................................||................................................."
			#echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Vall_show_all.txt
			#echo "................................/$MCLI_NAME /call/vall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Vall_show_all.txt
			#$MCLI_LOCATION$MCLI_NAME /call/vall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Vall_show_all.txt
		
			# Embeddedd in Controller_C$i	
			#cho ".................................................||................................................."
			#echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_all.txt
			#echo "................................../$MCLI_NAME /call/eall show all...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_all.txt
			#$MCLI_LOCATION$MCLI_NAME /call/eall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_all.txt
		
			#cho ".................................................||................................................."
			#echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_status.txt
			#echo "................................/$MCLI_NAME /call/eall show status..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_status.txt
			#$MCLI_LOCATION$MCLI_NAME /call/eall show status >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_show_status.txt
		
		
		
			#cho ".................................................||................................................."
			echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt
			echo ".............................../$MCLI_NAME /call/eall/sall show all...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt
			$MCLI_LOCATION$MCLI_NAME /call/eall/sall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt
		
			if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt ] ; then 
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...............................................Drives..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$MCLI_LOCATION$MCLI_NAME /call/eall/sall show | $grep -e SATA -e SAS >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................Shield Counter.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Shield Counter =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Media Error Count.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Media Error Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Other Error Count.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Other Error Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Drive Temperature.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Drive Temperature =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "......................................Predictive Failure Count......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Predictive Failure Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...................................S.M.A.R.T alert flagged by drive................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "S.M.A.R.T alert flagged by drive =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".................................................SN................................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "SN =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".................................................WWN................................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "WWN =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Firmware Revision.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Firmware Revision =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".............................................Raw size..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Raw size =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................Coerced size............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Coerced size =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt | $grep -v "Non" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "..........................................Non Coerced size.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Non Coerced size =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................Device Speed............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Device Speed =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".............................................Link Speed............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Link Speed =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				#Per VD not PD
				#echo "..........................................Drive write cache............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#$grep "Drive write cache =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Logical Sector Size........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Logical Sector Size =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Physical Sector Size......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Physical Sector Size =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...........................................Drive position..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Drive position =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Enclosure position........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Enclosure position =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "........................................Connected Port Number......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Connected Port Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...........................................Sequence Number.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Sequence Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt | $grep -v "Predictive" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Commissioned Spare........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Commissioned Spare =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...........................................Emergency Spare.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Emergency Spare =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".............................Last Predictive Failure Event Sequence Number.........................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Last Predictive Failure Event Sequence Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".................................Successful diagnostics completion on..............................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Successful diagnostics completion on =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................SED Capable............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "SED Capable =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................SED Enabled..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "SED Enabled =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "..............................................Secured..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Secured =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "...............................................Locked..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Locked =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo ".........................................Needs EKM Attention..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Needs EKM Attention =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................PI Eligible..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "PI Eligible =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "..........................................Wide Port Capable..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Wide Port Capable =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "................................Port # - Status - Linkspeed - SAS Address..........................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$grep "Gb/s   0x" ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				#cho ".................................................||................................................."
				echo "............................................Inquiry Data............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				$MCLI_LOCATION$MCLI_NAME /call/eall/sall show all j | $grep "Inquiry Data" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_Eall_Sall_show_all-Compare-All-Parms.txt
				
			fi	
		
		
			
			for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
		
		
				#Work around for storcli bug
		
				$MCLI_LOCATION$MCLI_NAME /c$i show 2>> ./misc_output.txt | grep "iBBU" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					echo "iBBU is on Controller" > ./$fileName/script_workspace/BBU_PRESENT_C$i.txt
				fi
				$MCLI_LOCATION$MCLI_NAME /c$i show 2>> ./misc_output.txt | grep "Cachevault_Info" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					echo "SuperCaP is on Controller" > ./$fileName/script_workspace/SuperCaP_PRESENT_C$i.txt
				fi
				
			
				echo "Collecting Information for Controller C$i with storcli..." >> ./misc_output.txt 2>&1	
			
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt
				echo "..................................../$MCLI_NAME /c$i show all........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show all 2>> ./misc_output.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt

				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_alilog_C$i.txt
				echo "..................................../$MCLI_NAME /c$i show alilog........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_alilog_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show alilog 2>> ./misc_output.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_alilog_C$i.txt

			
				echo "Collecting Enclosure Information for Controller C$i with storcli..." >> ./misc_output.txt 2>&1
			
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_show_all-status_C$i.txt
				echo "................................../$MCLI_NAME /c$i/eall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_show_all-status_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_show_all-status_C$i.txt
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/eall show status......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_show_all-status_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall show status >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_show_all-status_C$i.txt
		
		
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt
				echo ".............................../$MCLI_NAME /c$i/eall/sall show all...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show all >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt
		
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_phyerrorcounters_C$i.txt
				echo "........................../$MCLI_NAME /c$i/eall/sall show phyerrorcounters............................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_phyerrorcounters_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show phyerrorcounters >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_phyerrorcounters_C$i.txt
			
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_initialization_C$i.txt
				echo "............................/$MCLI_NAME /c$i/eall/sall show initialization............................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_initialization_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show initialization >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_initialization_C$i.txt
		
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_smart_C$i.txt
				echo ".............................../$MCLI_NAME /c$i/eall/sall show smart.................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_smart_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show smart >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_smart_C$i.txt
		
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_erase_C$i.txt
				echo ".............................../$MCLI_NAME /c$i/eall/sall show erase.................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_erase_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show erase >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_erase_C$i.txt
		
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_copyback_C$i.txt
				echo "............................./$MCLI_NAME /c$i/eall/sall show copyback................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_copyback_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show copyback >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_copyback_C$i.txt
		
		
		
		
				# Not really "/call show all" - need to get rid of duplicate "Controller =" entries.
			
				$MCLI_LOCATION$MCLI_NAME /c$i show all 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt		
			
			
			
				#cho ".................................................||................................................."
				echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				echo "......................................./$MCLI_NAME show all.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME show all | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
		
				if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt ] ; then 
					#cho ".................................................||................................................."
					echo "................................................Time................................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Current Controller Date/Time" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Current System Date/time" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi	
			
				echo "Collecting Information for Controller C$i with storcli..." >> ./misc_output.txt 2>&1
				echo "Collecting Logical Disk Information for Controller C$i with storcli..." >> ./misc_output.txt 2>&1
			
				#cho ".................................................||................................................."
				echo "......................................./$MCLI_NAME /c$i show.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				$MCLI_LOCATION$MCLI_NAME /c$i show personality | grep -i "Adapter does not support personality" >> ./misc_output.txt
				if [ "$?" != "0" ] ; then 
					#cho ".................................................||................................................."
					echo "................................/$MCLI_NAME /c$i show personality....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i show personality 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
				$MCLI_LOCATION$MCLI_NAME /c$i show sesmonitoring | grep -i "Adapter does not support sesMonitoring" >> ./misc_output.txt
				if [ "$?" != "0" ] ; then 
					#cho ".................................................||................................................."
					echo "................................/$MCLI_NAME /c$i show sesmonitoring....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i show sesmonitoring 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
				#cho ".................................................||................................................."
				echo "............................./$MCLI_NAME /c$i show failpdonsmarterror................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show failpdonsmarterror 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				$MCLI_LOCATION$MCLI_NAME /c$i show fshinting | grep -i "Show FSHinting failed" >> ./misc_output.txt
				if [ "$?" != "0" ] ; then 
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME /c$i show fshinting....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i show fshinting 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
		
				#cho ".................................................||................................................."
				echo ".....................................Persistent Nvdata version......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog 2>> ./misc_output.txt | $grep "Persistent Nvdata version =" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "..........................................PCI-E Link Speed.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog 2>> ./misc_output.txt | $grep "PCIE Link Status/Ctrl" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
		
				#cho ".................................................||................................................."
				echo "..................................../$MCLI_NAME /c$i show bios........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show bios 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "...........................................LDBBM Settings..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show all | $grep -e "Supported VD Operations :" -e "Enable LDBBM =" -e "Defaults :" -e "EnableLDBBM =" 2>> ./misc_output.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/vall show bbmt....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show bbmt |2>> ./misc_output.txt  sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show trim | grep -i "command invalid" >> ./misc_output.txt
				if [ "$?" != "0" ] ; then 
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME /c$i/vall show trim....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show trim 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/vall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show all |2>> ./misc_output.txt  sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
		
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show autobgi 2>> ./misc_output.txt | $grep "No VDs have been configured" >> ./misc_output.txt 2>&1
				if [ "$?" -ne "0" ] ; then
					#cho ".................................................||................................................."
					echo "........................................VD Auto BGI Status.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show autobgi 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show rebuild 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo "........................................Drive Rebuild Status........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show rebuild 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
					 
				$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show copyback 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo "........................................Drive CopyBack Status......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show copyback 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show initialization 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".....................................Drive Initialization Status...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show initialization 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show erase 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".........................................Drive Erase Status........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall/sall show erase 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show init 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".......................................VD Initialization Status....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show init 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show bgi 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo ".................................VD Background Initialization Status................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show bgi 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show cc 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo "....................................VD Consistency Check Status....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show cc 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show migrate 2>> ./misc_output.txt | $grep "In progress" >> ./misc_output.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					#cho ".................................................||................................................."
					echo "........................................VD Migration Status........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show migrate 2>> ./misc_output.txt | sed '1,/Description/d' | $grep -v "Not in progress" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i/vall show expansion 2>> ./misc_output.txt | $grep "Controller has no VD" >> ./misc_output.txt 2>&1
				if [ "$?" -ne "0" ] ; then
					#cho ".................................................||................................................."
					echo "........................................VD Expansion Status........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/vall show expansion 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi
		
		
		
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i show freespace..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show freespace 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "...................................../$MCLI_NAME /c$i show cc......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show cc 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "...................................../$MCLI_NAME /c$i show pr......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show pr 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................./$MCLI_NAME /c$i show copyback....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show copyback 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "...................................../$MCLI_NAME /c$i show eghs....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show enableesmarter 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................/$MCLI_NAME /c$i show perfmode......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show perfmode 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "............................./$MCLI_NAME /c$i show perfmodevalues....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show perfmodevalues 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "..................................../$MCLI_NAME /c$i show ncq........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show ncq 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".............................../$MCLI_NAME /c$i show largeiosupport..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show largeiosupport 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".............................../$MCLI_NAME /c$i show largeqd..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show largeqd 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".................................../$MCLI_NAME /c$i show ds........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show ds 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".................................../$MCLI_NAME /c$i show aso........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show aso 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................./$MCLI_NAME /c$i show bootdrive...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show bootdrive 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "............................./$MCLI_NAME /c$i show bootwithpinnedcache................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show bootwithpinnedcache 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
		
				#cho ".................................................||................................................."
				echo "................................./$MCLI_NAME /c$i show cachebypass.................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show cachebypass 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
		
				$MCLI_LOCATION$MCLI_NAME /c$i show preservedcache 2>> ./misc_output.txt | $grep "No Virtual" >> ./misc_output.txt 2>&1
				if [ "$?" -ne "0" ] ; then
					#cho ".................................................||................................................."
					echo ".............................../$MCLI_NAME /c$i show preservedcache................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i show preservedcache 2>> ./misc_output.txt | sed '1,/===/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
		
		
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/pall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/pall show all 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".........................../$MCLI_NAME /c$i/eall show phyerrorcounters................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall show phyerrorcounters 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/eall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall show all 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................./$MCLI_NAME /c$i/eall show status...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/eall show status 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
			
				if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt ] ; then 
					#cho ".................................................||................................................."
					echo "............................................Temperatures............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Temperature Sensor for" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "ROC temperature" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				fi	
		
				$MCLI_LOCATION$MCLI_NAME /c$i show health | grep Unsupported >> ./misc_output.txt
				if [ "$?" != "0" ] ; then 
					#cho ".................................................||................................................."
					echo ".................................../$MCLI_NAME /c$i show health......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i show health 2>> ./misc_output.txt | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
				
				if [ -f ./$fileName/script_workspace/BBU_PRESENT_C$i.txt ] ; then
					#cho ".................................................||................................................."
					echo "..........................................iBBU Temperature.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/bbu show all 2>> ./misc_output.txt | $grepA -m 1 Temperature >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
					$MCLI_LOCATION$MCLI_NAME /c$i/bbu show all 2>> ./misc_output.txt | $grep "Over Temperature" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
				
				if [ -f ./$fileName/script_workspace/SuperCaP_PRESENT_C$i.txt ] ; then
					#cho ".................................................||................................................."
					echo "..........................................CV Temperature............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/cv show all 2>> ./misc_output.txt | $grepA -m 1 Temperature >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
					$MCLI_LOCATION$MCLI_NAME /c$i/cv show all 2>> ./misc_output.txt | $grep "Over Temperature" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
		
		
				# MegaCli syntax - PR to add output to storcli
		
				#cho ".................................................||................................................."
				echo ".......................................Enclosure Temperature........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME encinfo a$i 2>> ./misc_output.txt | $grep -e "Temp Sensor                  :" -e "Temperature                  :" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
							
		
				echo "Collecting Physical Disk Information for Controller C$i with storcli..." >> ./misc_output.txt 2>&1
			
				if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt ] ; then 
					#cho ".................................................||................................................."
					echo ".........................................Drive Temperature.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Drive Temperature =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Media Error Count.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Media Error Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					#cho ".................................................||................................................."
					echo "......................................Predictive Failure Count......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Predictive Failure Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................Device Speed............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Device Speed =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					#cho ".................................................||................................................."
					echo "................................Port # - Status - Linkspeed - SAS Address..........................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					#echo "...................................Port #/Status/Linkspeed/SAS Address.............................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$grep "Gb/s   0x" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
			
				fi	
		
		
				if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt ] ; then 
					echo $fileName >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
		
					echo "...............................................Drives..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /call/eall/sall show | $grep -e SATA -e SAS >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
		
					#cho ".................................................||................................................."
					echo "............................................Shield Counter.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Shield Counter =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Media Error Count.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Media Error Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Other Error Count.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Other Error Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Drive Temperature.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Drive Temperature =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "......................................Predictive Failure Count......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Predictive Failure Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "...................................S.M.A.R.T alert flagged by drive................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "S.M.A.R.T alert flagged by drive =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".................................................SN................................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "SN =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".................................................WWN................................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "WWN =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Firmware Revision.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Firmware Revision =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".............................................Raw size..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Raw size =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................Coerced size............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Coerced size =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt | $grep -v "Non" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "..........................................Non Coerced size.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Non Coerced size =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................Device Speed............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Device Speed =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".............................................Link Speed............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Link Speed =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "..........................................Drive write cache............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Drive write cache =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Logical Sector Size........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Logical Sector Size =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Physical Sector Size......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Physical Sector Size =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "...........................................Drive position..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Drive position =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Enclosure position........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Enclosure position =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "........................................Connected Port Number......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Connected Port Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "...........................................Sequence Number.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Sequence Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt | $grep -v "Predictive" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Commissioned Spare........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Commissioned Spare =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "...........................................Emergency Spare.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Emergency Spare =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".............................Last Predictive Failure Event Sequence Number.........................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Last Predictive Failure Event Sequence Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".................................Successful diagnostics completion on..............................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Successful diagnostics completion on =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................SED Capable............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "SED Capable =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................SED Enabled..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "SED Enabled =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "..............................................Secured..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Secured =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "...............................................Locked..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Locked =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo ".........................................Needs EKM Attention..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Needs EKM Attention =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "............................................PI Eligible..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "PI Eligible =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "..........................................Wide Port Capable..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Wide Port Capable =" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
					echo "................................Port # - Status - Linkspeed - SAS Address..........................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$grep "Gb/s   0x" ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all_C$i.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					#cho ".................................................||................................................."
		
					echo "............................................Inquiry Data............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show all j | $grep "Inquiry Data" >> ./$fileName/LSI_Products/MegaRAID/storcli/Cx_Eall_Sall_show_all-Compare-All-Parms_C$i.txt
				
		
			
				fi	
			
				if [ -f ./$fileName/script_workspace/BBU_PRESENT_C$i.txt ] ; then			
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME /c$i/bbu show all......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/bbu show all | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
				if [ -f ./$fileName/script_workspace/SuperCaP_PRESENT_C$i.txt ] ; then
					#cho ".................................................||................................................."
					echo "................................../$MCLI_NAME /c$i/cv show all........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /c$i/cv show all | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
				fi
		
				#cho ".................................................||................................................."
				echo "................................../$MCLI_NAME /c$i/fall show all......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/fall show all | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo ".............................../$MCLI_NAME /c$i/fall import preview..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i/fall import preview | sed '1,/Description/d' >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "............................../$MCLI_NAME /c$i show termlog type=config..............................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog type=config >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_C$i.txt 2>&1
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog type=config >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1
		
				#cho ".................................................||................................................."
				echo "................................./$MCLI_NAME /c$i show termlog........................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog  >> ./$fileName/LSI_Products/MegaRAID/FWTermLog/fwtermlog_C$i.txt 2>&1
				$MCLI_LOCATION$MCLI_NAME /c$i show termlog  >> ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt 2>&1		

				###########################################################################################################################
				# MegaRAID Smartctl Data Collection if Smartctl is installed
				# Start Linux Only Section for now
				# Must test on other OS's
				###########################################################################################################################
			
					
				smartctl -h > ./$fileName/script_workspace/smartctl-h.txt 2>&1
				if [ "$?" -eq "0" ] ; then
					# Smartctl added support for MegaRAID in 5.39
				
					# Only have one sd letter associated per 
					# controller to eliminate duplicate
					# entries.
					
					# Used with smartctl, Don't want duplicates with multiple controllers.
						
					for j in $( ls /dev/sd* ); do
						dev=$(basename $j)
						if [ -e /sys/block/${dev} ] ; then echo ${dev} >> ./$fileName/script_workspace/sd_letters.txt ; fi
					done
				
					$MCLI_LOCATION$MCLI_NAME /c$i/eall/sall show j | grep DID | cut -d: -f 2 | cut -d, -f 1 >> ./$fileName/script_workspace/disk_dev_id_numbers_C$i.txt
				
					if [ -f ./$fileName/script_workspace/sd_letters.txt ] ; then
						for j in `cat ./$fileName/script_workspace/sd_letters.txt` ; do #Supports up to 26 character device node entries
							for k in `cat ./$fileName/script_workspace/disk_dev_id_numbers_C$i.txt` ; do #Limit?
								smartctl -T permissive -i -d megaraid,$k /dev/$j | egrep "INQUIRY failed|No such device" >> ./misc_output.txt 2>&1
									if [ "$?" -ne "0" ] ; then
										echo $j > ./$fileName/script_workspace/sd_letter_C$i.txt
									fi
							done
						done
					fi	
				
					# Differentiate SATA from SAS				
					if [ -f ./$fileName/script_workspace/sd_letter_C$i.txt ] ; then
						for j in `cat ./$fileName/script_workspace/sd_letter_C$i.txt` ; do #Supports up to 26 character device node entries
							for k in `cat ./$fileName/script_workspace/disk_dev_id_numbers_C$i.txt` ; do #Limit?
								smartctl -T permissive -i -d megaraid,$k /dev/$j | grep "SATA device detected" >> ./misc_output.txt 2>&1
									if [ "$?" -ne "0" ] ; then
										echo $k >> ./$fileName/script_workspace/sas_disk_dev_id_numbers_C$i.txt
										else	
										echo $k >> ./$fileName/script_workspace/sata_disk_dev_id_numbers_C$i.txt
									fi
							done
						done
					fi		
										
				
					echo "Starting MegaRAID Smartctl Data Collection for Controller C$i..."
					if [ ! -d ./$fileName/LSI_Products/MegaRAID/SMARTCTL ] ; then mkdir ./$fileName/LSI_Products/MegaRAID/SMARTCTL ; fi
				
					#cho ".................................................||................................................."
					echo "...................All SAS disks are listed first and then all SATA disks follow...................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
					echo "...................................................................................................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
				
					# SAS Disks		
					
					if [ -f ./$fileName/script_workspace/sd_letter_C$i.txt ] ; then
						if [ -f ./$fileName/script_workspace/sas_disk_dev_id_numbers_C$i.txt ] ; then
							for j in `cat ./$fileName/script_workspace/sd_letter_C$i.txt` ; do #Supports up to 26 character device node entries
								for k in `cat ./$fileName/script_workspace/sas_disk_dev_id_numbers_C$i.txt` ; do #Limit?
									#cho ".................................................||................................................."
									echo ".................................megaraid,$k is the Disk Device ID #................................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
									echo ".......................smartctl -T permissive -a -d megaraid,$k /dev/$j............................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
									smartctl -T permissive -a -d megaraid,$k /dev/$j >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
								done
							done
						fi
					fi
				
					# SATA Disks
				
					if [ -f ./$fileName/script_workspace/sd_letter_C$i.txt ] ; then
						if [ -f ./$fileName/script_workspace/sata_disk_dev_id_numbers_C$i.txt ] ; then
							for j in `cat ./$fileName/script_workspace/sd_letter_C$i.txt` ; do #Supports up to 26 character device node entries
								for k in `cat ./$fileName/script_workspace/sata_disk_dev_id_numbers_C$i.txt` ; do #Limit?
									#cho ".................................................||................................................."
									echo ".................................megaraid,$k is the Disk Device ID #................................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
									echo ".......................smartctl -T permissive -a -d sat+megaraid,$k /dev/$j............................." >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
									smartctl -T permissive -a -d sat+megaraid,$k /dev/$j >> ./$fileName/LSI_Products/MegaRAID/SMARTCTL/megaraid_C$i.txt 2>&1
								done
							done
						fi
					fi
				
				# return for smartctl -h
				fi
	
			
			#done for controller #ing i.e. $i
			done

	
			#cho ".................................................||................................................."
			echo "..............................................Basics :.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
							
			#cho ".................................................||................................................."
			echo "............................................Controller #............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Controller =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt | $grep -v "Temperature" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...............................................Model #.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Model =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt | $grep -v "Support Config Page Model" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...........................................Serial Number #.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Serial Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "....................................Current Controller Date/Time...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Current Controller Date/Time =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "......................................Current System Date/time......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Current System Date/time =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "..............................................Mfg Date.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Mfg Date =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................Rework Date............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Rework Date =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................Revision No............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Revision No =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
		
			#cho ".................................................||................................................."
			echo "..............................................Version :............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			echo ".......................................Firmware Package Build......................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Firmware Package Build =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................Bios Version............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Bios Version =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...........................................NVDATA Version..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "NVDATA Version =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".........................................Boot Block Version........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Boot Block Version =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".........................................Bootloader Version........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Bootloader Version =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................Driver Name............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Driver Name =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...........................................Driver Version..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Driver Version =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
		
			#cho ".................................................||................................................."
			echo "................................................Bus :..............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			echo ".............................................Vendor Id.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Vendor Id =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt | $grep -v "SubVendor" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".............................................Device Id.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Device Id =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt | $grep -v "SubDevice" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................SubVendor Id............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "SubVendor Id =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................SubDevice Id............................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "SubDevice Id =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...........................................Host Interface..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Host Interface =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "..........................................Device Interface.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Device Interface =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".............................................Bus Number............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Bus Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "............................................Device Number..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Device Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...........................................Function Number.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Function Number =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			#cho ".................................................||................................................."
			echo "......................................Pending Images in Flash :....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			#cho ".................................................||................................................."
			echo ".............................................Image name............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Image name =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			#cho ".................................................||................................................."
			echo "..............................................Status :.............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
			#cho ".................................................||................................................."
			echo ".........................................Controller Status.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Controller Status =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".....................................Memory Correctable Errors......................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Memory Correctable Errors =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "....................................Memory Uncorrectable Errors....................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Memory Uncorrectable Errors =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".........................................ECC Bucket Count..........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "ECC Bucket Count =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...................................Any Offline VD Cache Preserved..................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Any Offline VD Cache Preserved =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".............................................BBU Status............................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "BBU Status =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "....................................Support PD Firmware Download...................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Support PD Firmware Download =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo ".........................................Lock Key Assigned.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Lock Key Assigned =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...................................Failed to get lock key on bootup................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Failed to get lock key on bootup =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "...................................Lock key has not been backed up.................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Lock key has not been backed up =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "..................................Bios was not detected during boot................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Bios was not detected during boot =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "......................Controller must be rebooted to complete security operation...................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Controller must be rebooted to complete security operation =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "................................A rollback operation is in progress................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "A rollback operation is in progress =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "..................................At least one PFK exists in NVRAM.................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "At least one PFK exists in NVRAM =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "..........................................SSC Policy is WB.........................................." >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "SSC Policy is WB =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			#cho ".................................................||................................................."
			echo "................................Controller has booted into safe mode................................" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
			$grep "Controller has booted into safe mode =" ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all.txt >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt
		
		
			echo "##################################Supported Adapter Operations :####################################" >> ./$fileName/LSI_Products/MegaRAID/storcli/Call_show_all-Compare-All-Parms.txt

			if [ "$TWGETSKIPRECORDCAPTURE" != "YES" ] ; then

				echo .
				echo "Start of the MegaRAID Event Record Capture..."
				echo "Start of the MegaRAID Event Record Capture..."  >> ./misc_output.txt 2>&1
				if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
					date '+%H:%M:%S.%N'
				fi
				if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
					date '+%H:%M:%S'
				fi
				echo "This will take approx. 1 minute per 78K/0x130B0 Event Records per Controller..."
				echo "Start of the MegaRAID Event Record Capture..."  >> ./misc_output.txt 2>&1
				echo "This will take approx. 1 minute per 78K/0x130B0 Event Records per Controller..." >> ./misc_output.txt 2>&1
				
				for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
				
					echo .
					echo Start on Controller $i...
					$MCLI_LOCATION$MCLI_NAME /C$i show eventloginfo | $grep -e "Newest" -e "Oldest"
					
					if [ ! -d ./$fileName/LSI_Products/MegaRAID/Event_Logs ] ; then mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs ; fi
					echo $fileName > ./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt 2>&1							
					echo "..................storcli /C$i show events type=includedeleted.................." >> ./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt
					$MCLI_LOCATION$MCLI_NAME /C$i show events type=includedeleted file=./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt >> ./misc_output.txt 2>&1

					echo "End on Controller $i..." >> ./misc_output.txt 2>&1
					if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
						date '+%H:%M:%S.%N'
					fi
					if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
						date '+%H:%M:%S'
					fi
											
				done					
				
				echo "End of the MegaRAID Event Record Capture..." >> ./misc_output.txt 2>&1
			fi

			if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then	
				if [ "$OS_LSI" != "macos" ] ; then
					if [ "$TWGETSKIPRECORDSORT" != "YES" ] ; then
		
						###########################################################################################################################
						#MegaRAID Adapter Event log sorting - Deleted MegaCli syntax version
						###########################################################################################################################
		
							
						if [ ! -d ./$fileName/script_workspace/mr_aen_sort ] ; then mkdir ./$fileName/script_workspace/mr_aen_sort ; fi >> ./misc_output.txt 2>&1
						if [ -f ./$fileName/script_workspace/aens_mr.txt ] ; then mv -f  ./$fileName/script_workspace/aens_mr.txt ./$fileName/script_workspace/mr_aen_sort/aens_mr.txt ; fi >> ./misc_output.txt 2>&1
		
						echo .
						echo "Start of the MegaRAID Event Record Sort..."
						echo "Start of the MegaRAID Event Record Sort..." >> ./misc_output.txt 2>&1
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N'
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S'
						fi
						echo This will take approx. 1 minute per 78,000 Event Records per Controller...
						echo Start of the MegaRAID Event Record Sort... >> ./misc_output.txt 2>&1
						echo This will take approx. 1 minute per 78,000 Event Records per Controller... >> ./misc_output.txt 2>&1
		
						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199	
							echo "Start on Controller $i..."
							echo "Start on Controller $i..." >> ./misc_output.txt 2>&1
							$grep "Code: 0x" ./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt > ./$fileName/script_workspace/mr_aen_sort/all_aen_iterations_C$i.txt
#						done

#						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199	
							while IFS=' ' read j k ; do
								echo $j $k > ./$fileName/script_workspace/mr_aen_sort/"$k"_used_aens_C$i.txt
							done < ./$fileName/script_workspace/mr_aen_sort/all_aen_iterations_C$i.txt
#						done

#						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199									
							ls ./$fileName/script_workspace/mr_aen_sort/ | grep 0x | grep _used_aens_C$i.txt > ./$fileName/script_workspace/mr_aen_sort/all_used_aens_C$i.txt
#						done


#						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
			
							while IFS=_ read j k; do
								echo "Number of Events :" > ./$fileName/script_workspace/mr_aen_sort/"$j"_AEN_"C$i".txt
								$grep "Code: ${j}" ./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt -c >> ./$fileName/script_workspace/mr_aen_sort/"$j"_AEN_"C$i".txt
								echo .................................................................................................... >> ./$fileName/script_workspace/mr_aen_sort/"$j"_AEN_"C$i".txt
								$grep -B 3 -A 5 "Code: ${j}" ./$fileName/LSI_Products/MegaRAID/Event_Logs/evtlog_ID_C$i.txt >> ./$fileName/script_workspace/mr_aen_sort/"$j"_AEN_"C$i".txt
								$grep "Code: ${j}" ./$fileName/script_workspace/mr_aen_sort/aens_mr.txt >> ./$fileName/script_workspace/mr_aen_sort/aens_mr_used_C$i.txt
							done < ./$fileName/script_workspace/mr_aen_sort/all_used_aens_C$i.txt

#						done

#						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199							
							
							while IFS=' ' read j k l ; do
								mv -f ./$fileName/script_workspace/mr_aen_sort/"$k"_AEN_"C$i".txt ./$fileName/LSI_Products/MegaRAID/Event_Logs/"$k"_"$l"_"C$i".txt >> ./misc_output.txt 2>&1
							done < ./$fileName/script_workspace/mr_aen_sort/aens_mr_used_C$i.txt
							echo "ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_C$i.txt" >> ./misc_output.txt 2>&1											
							ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_C$i.txt >> ./misc_output.txt 2>&1
							if [ "$?" -eq "0" ] ; then
								mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i >> ./misc_output.txt 2>&1
								cp ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_C$i.txt ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i >> ./misc_output.txt 2>&1
							fi			
						done				
														

						
						for i in $(cat ./$fileName/script_workspace/mr_aen_types.txt) ; do	
							echo "ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_${i}_*" >> ./misc_output.txt 2>&1
							ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_${i}_* >> ./misc_output.txt 2>&1
							if [ "$?" -eq "0" ] ; then
								mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs/$i >> ./misc_output.txt 2>&1
								echo "mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs/$i" >> ./misc_output.txt 2>&1
								mv -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_${i}_* ./$fileName/LSI_Products/MegaRAID/Event_Logs/$i >> ./misc_output.txt 2>&1
								echo "mv -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/*_${i}_* ./$fileName/LSI_Products/MegaRAID/Event_Logs/$i" >> ./misc_output.txt 2>&1
							fi
						done						
		
							
						
						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199	
							for j in $(cat ./$fileName/script_workspace/mr_aen_types.txt) ; do
								echo "ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_${j}_*" >> ./misc_output.txt 2>&1
								ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_${j}_* >> ./misc_output.txt 2>&1
								if [ "$?" -eq "0" ] ; then
									mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/${j} >> ./misc_output.txt 2>&1
									echo "mkdir ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/${j}" >> ./misc_output.txt 2>&1
									mv -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_${j}_* ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/${j} >> ./misc_output.txt 2>&1
									echo "mv -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_${j}_* ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/${j}" >> ./misc_output.txt 2>&1
								fi
							done
						done							
		
						
			
						
						for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199	
							echo "ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_C${i}.txt" >> ./misc_output.txt 2>&1
							ls ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C${i}/*_C${i}.txt >> ./misc_output.txt 2>&1
							if [ "$?" -eq "0" ] ; then
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GE_C$i.txt ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GE_C$i.txt ; fi >> ./misc_output.txt 2>&1
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GE_C$i.txt ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GE_C$i.txt ; fi >> ./misc_output.txt 2>&1
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GSR_C$i.txt ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GSR_C$i.txt ; fi >> ./misc_output.txt 2>&1
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GSS_C$i.txt ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_GSS_C$i.txt ; fi >> ./misc_output.txt 2>&1
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_ID_C$i.txt ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/evtlog_ID_C$i.txt ; fi >> ./misc_output.txt 2>&1
								if [ -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/Return_getlatest_* ] ; then rm -f ./$fileName/LSI_Products/MegaRAID/Event_Logs/Controller_C$i/Return_getlatest_* ; fi >> ./misc_output.txt 2>&1
							fi
							echo End on Controller $i... >> ./misc_output.txt 2>&1
						done
						
						echo "End of the MegaRAID Event Record Sort..." >> ./misc_output.txt 2>&1
						echo "End of the MegaRAID Event Record Sort..." 
						if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
							date '+%H:%M:%S.%N'
						fi
						if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
							date '+%H:%M:%S'
						fi
					fi
				fi
			fi

					
		# Returns from - Make sure at least 1 MegaRAID Adapter is identified
		fi


		###########################################################################################################################
		# Component version number collection
		###########################################################################################################################
		#Get the MegaCli version that came bundled with the script
		if [ "$mcli_Bundled_work" = "YES" ] ; then
			./$MCLI_NAME -v | grep -i StorCli | awk '{ print$6 }' > ./$fileName/script_workspace/mcli_Bundled_version.txt
		fi

		# Get the MegaCli version that was pre-existing
		if [ "$tw_cli_Existing_work" = "YES" ] ; then
			$MCLI_NAME -v | grep -i StorCli | awk '{ print$6 }' > ./$fileName/script_workspace/mcli_Existing_version.txt
		fi

	###########################################################################################################################
	# Done with MegaCli!
	###########################################################################################################################				
			
	
	# Return if MacOS for MegaRAID
	fi
# Return for TWGETSKIPMEGARAID
fi
###########################################################################################################################
# Script Version
###########################################################################################################################
###Update on Code Set Change
echo "$Capture_Script_Version" > ./$fileName/script_workspace/lsigetlunix_version.txt

###########################################################################################################################
# Data to help troubleshoot script issues.
###########################################################################################################################


#cho ".................................................||................................................."
echo "............................whoami - user executing lsigetlunix.sh script..........................." >> ./$fileName/script_workspace/script_diag.txt
whoami >> ./$fileName/script_workspace/script_diag.txt 2>&1
#cho ".................................................||................................................."
echo "..................groups - groups user executing lsigetlunix.sh script belongs to..................." >> ./$fileName/script_workspace/script_diag.txt
groups >> ./$fileName/script_workspace/script_diag.txt 2>&1
#cho ".................................................||................................................."
echo "..........................ls -latr - files in subdir script was executed from......................." >> ./$fileName/script_workspace/script_diag.txt
ls -latr >> ./$fileName/script_workspace/script_diag.txt
#cho ".................................................||................................................."
echo "....................................set - environment for script...................................." >> ./$fileName/script_workspace/script_diag.txt
set >> ./$fileName/script_workspace/script_diag.txt 2>&1
#cho ".................................................||................................................."
echo "....................................env - environment for script...................................." >> ./$fileName/script_workspace/script_diag.txt
env >> ./$fileName/script_workspace/script_diag.txt 2>&1
echo "......................................Command Line and Options......................................" >> ./$fileName/script_workspace/script_diag.txt
echo "$0 $@" > ./$fileName/script_workspace/cmd_line.txt
TWCMDLINE=`cat ./$fileName/script_workspace/cmd_line.txt`


export TWCMDLINE 


echo "$0 $@" >> ./$fileName/script_workspace/script_diag.txt

###########################################################################################################################
# System capture comment
###########################################################################################################################

if [ "$TWcomment" != "" ] ; then  
	echo "$TWcomment" > ./$fileName/Comment.txt 
fi


###########################################################################################################################
# Component version number collection
###########################################################################################################################
#Get the tw_cli version that came bundled with the script
if [ "$tw_cli_Bundled_work" = "YES" ] ; then
	./$CLI_NAME help 2>> ./misc_output.txt | $grep version | awk  '{ print $4 }' | sed -e 's/)//' > ./$fileName/script_workspace/tw_cli_Bundled_version.txt
fi

# Get the tw_cli version that was pre-existing
if [ "$tw_cli_Existing_work" = "YES" ] ; then
	$CLI_NAME help 2>> ./misc_output.txt | $grep version | awk  '{ print $4 }' | sed -e 's/)//' > ./$fileName/script_workspace/tw_cli_Existing_version.txt
fi


########################################################################################################################### 
###########################################################################################################################
# Common - Collect system information 
###########################################################################################################################
###########################################################################################################################


echo "Collecting System info..."

if [ -f re_execute_variable_shell.txt ] ; then
	mv  re_execute_variable_shell.txt ./$fileName/script_workspace
fi

uname -a > ./$fileName/uname-a.txt 2>> ./misc_output.txt



echo "Start of var_log section" >> ./misc_output.txt 2>&1
if [ -d /var/log ] ; then
	mkdir ./$fileName/var_log
	ls /var/log > ./$fileName/script_workspace/var_log.txt
	for i in $( cat ./$fileName/script_workspace/var_log.txt ); do
		if [ -f /var/log/$i ] ; then 
			if [ "$i" != "lastlog" ] ; then cp /var/log/$i ./$fileName/var_log 2>> ./misc_output.txt ; fi
		fi
	done
	ls ./$fileName/var_log >> ./misc_output.txt 2>&1
	rm -f ./$fileName/var_log/maillog* 2>> ./misc_output.txt
	rm -f ./$fileName/var_log/secure* 2>> ./misc_output.txt
	#Linux
	if [ -f ./$fileName/var_log/messages ] ; then ls -latr ./$fileName/var_log/messages* >> ./$fileName/script_workspace/all_copied_messages.txt 2>> ./misc_output.txt ; fi
	#MacOS
	if [ -f ./$fileName/var_log/system.log ] ; then ls -latr ./$fileName/var_log/system.log* >> ./$fileName/script_workspace/all_copied_messages.txt 2>> ./misc_output.txt ; fi
	#VMWare
	if [ -f /var/log/vmkernel ] ; then ls -latr ./$fileName/var_log/vmkernel* >> ./$fileName/script_workspace/all_copied_messages.txt 2>> ./misc_output.txt ; fi
	if [ -d /var/log/vmware ] ; then
		mkdir ./$fileName/var_log/vmware
		cp /var/log/vmware/* ./$fileName/var_log/vmware 2>> ./misc_output.txt
	fi
fi

dmesg >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	dmesg > ./$fileName/dmesg_${currentTime}.txt 2>> ./misc_output.txt
	if [ -d /var/log ] ; then mv -f ./$fileName/dmesg_${currentTime}.txt ./$fileName/var_log 2>> ./misc_output.txt ; fi
fi

echo "End of var_log section" >> ./misc_output.txt 2>&1



###########################################################################################################################
#Solaris
###########################################################################################################################
if [ -f /var/adm/messages ] ; then cp -p /var/adm/messages* ./$fileName/
	ls -latr ./$fileName/messages* >> ./$fileName/script_workspace/all_copied_messages.txt 2>> ./misc_output.txt
fi




if [ -d /etc ] ; then
	mkdir ./$fileName/etc
	cp /etc/* ./$fileName/etc 2>> ./misc_output.txt
# Do NOT copy any password files.
	if [ -f ./$fileName/etc/passwd ] ; then rm -f ./$fileName/etc/passwd 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/passwd- ] ; then rm -f ./$fileName/etc/passwd- 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/shadow ] ; then rm -f ./$fileName/etc/shadow 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/shadow- ] ; then rm -f ./$fileName/etc/shadow- 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/gshadow ] ; then rm -f ./$fileName/etc/gshadow 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/gshadow- ] ; then rm -f ./$fileName/etc/gshadow- 2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/master.passwd  ] ; then rm -f ./$fileName/etc/master.passwd  2>> ./misc_output.txt ; fi
	if [ -f ./$fileName/etc/spwd.db ] ; then rm -f ./$fileName/etc/spwd.db 2>> ./misc_output.txt ; fi
        if [ -f ./$fileName/etc/group ] ; then rm -f ./$fileName/etc/group 2>> ./misc_output.txt ; fi
        if [ -f ./$fileName/etc/sudoers ] ; then rm -f ./$fileName/etc/sudoers 2>> ./misc_output.txt ; fi
        if [ -f ./$fileName/etc/vsftpd.user_list ] ; then rm -f ./$fileName/etc/vsftpd.user_list 2>> ./misc_output.txt ; fi
        if [ -f ./$fileName/etc/mail.log ] ; then rm -f ./$fileName/etc/mail.log 2>> ./misc_output.txt ; fi
fi


#Get the rest of the conf files
if [ -d /etc ] ; then
	mkdir ./$fileName/etc/other_conf_files
	for i in $( ls /etc/ ); do
		if [ -d /etc/$i ] ; then cp /etc/$i/*.conf ./$fileName/etc/other_conf_files/. 2>> ./misc_output.txt ; fi
	done
	if [ -f /etc/sysconfig/irqbalance ] ; then cp /etc/sysconfig/irqbalance ./$fileName/etc/other_conf_files/. 2>> ./misc_output.txt ; fi 
fi

#Brackets get rid of messages on FreeBSD if files don't exist.
#{ cp -p /etc/*.conf ./$fileName/conf; } 2>> ./misc_output.txt 
#{ cp -p /etc/*release ./$fileName/; } 2>> ./misc_output.txt


for i in sysconfig/diskdump sysconfig/harddisks sysconfig/hwconf fstab raidtab ; do
	if [ -f /etc/$i ] ; then cp -p /etc/$i ./$fileName/etc ; fi
done


for i in $( ls /dev/sd* ); do
		dev=$(basename $i)
	for j in vendor model timeout;do
		if [ -e /sys/block/${dev}/device/$j ] ;then
			#cho ".................................................||................................................."
			echo "....................................cat /sys/block/${dev}/device/$j..................................." >> ./$fileName/sd_time_out_value.txt 2>&1
			cat /sys/block/${dev}/device/$j >> ./$fileName/sd_time_out_value.txt 2>&1
		fi
	done
done

uptime >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then 
	uptime > ./$fileName/uptime.txt 2>> ./misc_output.txt
fi

lsmod >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	lsmod > ./$fileName/lsmod.txt 2>> ./misc_output.txt
fi

lspci >> ./misc_output.txt 2>&1
if [ "$?" -eq "0" ] ; then
	if [ ! -d ./$fileName/lspci ] ; then mkdir ./$fileName/lspci ; fi

	lspci > ./$fileName/lspci/lspci.txt 2>> ./misc_output.txt
	
	lspci -e >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -e > ./$fileName/lspci/lspci-e.txt 2>> ./misc_output.txt
	fi
	
	lspci -p >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -p > ./$fileName/lspci/lspci-p.txt 2>> ./misc_output.txt
	fi
	
	lspci -t >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -t > ./$fileName/lspci/lspci-t.txt 2>> ./misc_output.txt
	fi
	
	lspci -x >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -x > ./$fileName/lspci/lspci-x.txt 2>> ./misc_output.txt
	fi
	
	lspci -xxx >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -xxx > ./$fileName/lspci/lspci-xxx.txt 2>> ./misc_output.txt
	fi
	
	lspci -xxxx >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -xxxx > ./$fileName/lspci/lspci-xxxx.txt 2>> ./misc_output.txt
	fi
	
	lspci -v >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -v > ./$fileName/lspci/lspci-v.txt 2>> ./misc_output.txt
	fi
	
	lspci -vv >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -vv > ./$fileName/lspci/lspci-vv.txt 2>> ./misc_output.txt
	fi
	
	lspci -vvv >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -vvv > ./$fileName/lspci/lspci-vvv.txt 2>> ./misc_output.txt
	fi
	
	lspci -tvvv >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -tvvv > ./$fileName/lspci/lspci-tvvv.txt 2>> ./misc_output.txt
	fi
	
	lspci -vvvxxxx >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		lspci -vvvxxxx > ./$fileName/lspci/lspci-vvvxxxx.txt 2>> ./misc_output.txt
	fi
fi
scanpci -v >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	scanpci -v > ./$fileName/scanpci-v.txt 2>> ./misc_output.txt
fi

rpm -q -a -i >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	rpm -q -a -i > ./$fileName/rpm-q-a-i.txt 2>> ./misc_output.txt
fi

df >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df > ./$fileName/df.txt 2>> ./misc_output.txt
fi

df -h >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -h > ./$fileName/df-h.txt 2>> ./misc_output.txt
fi

df -ha >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -ha > ./$fileName/df-ha.txt 2>> ./misc_output.txt
fi

df -hat >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -hat > ./$fileName/df-hat.txt 2>> ./misc_output.txt
fi

df -haT >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -haT > ./$fileName/df-haT.txt 2>> ./misc_output.txt
fi

df -Hai >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -Hai > ./$fileName/df-Hai.txt 2>> ./misc_output.txt
fi

df -Hami >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	df -Hami > ./$fileName/df-Hami.txt 2>> ./misc_output.txt
fi


who >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	who > ./$fileName/who.txt  2>> ./misc_output.txt
fi

who -b >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	who -b > ./$fileName/who-b.txt  2>> ./misc_output.txt
fi

who -m >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	who -m > ./$fileName/who-m.txt  2>> ./misc_output.txt
fi

top -b -n 1 >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
top -b -n 1 > ./$fileName/top-b-n1.txt 2>> ./misc_output.txt
fi

top -l1 >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	top -l1 > ./$fileName/top-l1.txt 2>> ./misc_output.txt
fi

if [ -d /boot ] ; then ls -latrR /boot > ./$fileName/boot-latrR.txt 2>&1 ./$fileName/script_workspace/lsiget_errorlog.txt; fi

if [ -d /sys ] ; then ls -latrR /sys > ./$fileName/sys-latrR.txt 2>&1 ./$fileName/script_workspace/lsiget_errorlog.txt; fi

if [ -d /dev ] ; then ls -latrR /dev > ./$fileName/dev-latrR.txt 2>&1 ./$fileName/script_workspace/lsiget_errorlog.txt; fi

if [ -f /boot/grub/menu.lst ] ; then cp -p /boot/grub/menu.lst ./$fileName/; fi

ps >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	ps > ./$fileName/ps.txt 2>> ./misc_output.txt
fi

ps -e >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	ps -e > ./$fileName/ps-e.txt 2>> ./misc_output.txt
fi

ps -ef >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	ps -ef > ./$fileName/ps-ef.txt 2>> ./misc_output.txt
fi

ps -ea >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	ps -ea > ./$fileName/ps-ea.txt 2>> ./misc_output.txt
fi

ps -auxw >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	ps -auxw > ./$fileName/ps-auxw.txt 2>> ./misc_output.txt
fi

fdisk -l >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	fdisk -l > ./$fileName/fdisk-l.txt 2>> ./misc_output.txt
fi

fdisk -lu >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	fdisk -lu > ./$fileName/fdisk-lu.txt 2>> ./misc_output.txt
fi

vgs >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	vgs > ./$fileName/lvm/vgs.txt 2>> ./misc_output.txt
fi

lvs >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	lvs > ./$fileName/lvm/lvs.txt 2>> ./misc_output.txt
fi

pvs >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	pvs > ./$fileName/lvm/pvs.txt 2>> ./misc_output.txt
fi

lvdisplay >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	lvdisplay > ./$fileName/lvm/lvdisplay.txt 2>> ./misc_output.txt
fi

pvdisplay >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	pvdisplay > ./$fileName/lvm/pvdisplay.txt 2>> ./misc_output.txt
fi

sysctl -a >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	sysctl -a > ./$fileName/sysctl-a.txt 2>> ./misc_output.txt
fi

sysctl -ad >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	sysctl -ad > ./$fileName/sysctl-ad.txt 2>> ./misc_output.txt
fi

sysctl -ah >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	sysctl -ah > ./$fileName/sysctl-ah.txt 2>> ./misc_output.txt
fi

sysctl -adh >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	sysctl -adh > ./$fileName/sysctl-adh.txt 2>> ./misc_output.txt
fi

vmstat >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	vmstat > ./$fileName/vmstat.txt 2>> ./misc_output.txt
fi

vmstat -i >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	vmstat -i > ./$fileName/vmstat-i.txt 2>> ./misc_output.txt
fi

dmidecode >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	dmidecode > ./$fileName/dmidecode.txt 2>> ./misc_output.txt
fi

biosdecode >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	biosdecode > ./$fileName/biosdecode.txt 2>> ./misc_output.txt
fi

vpddecode >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	vpddecode > ./$fileName/vpddecode.txt 2>> ./misc_output.txt
fi

if [ -f /etc/lvm/lvm.conf ] ; then 
	if [ ! -d ./$fileName/lvm ] ; then mkdir ./$fileName/lvm; fi
	cp /etc/lvm/lvm.conf ./$fileName/lvm 2>> ./misc_output.txt 
fi

lsscsi >> ./misc_output.txt 2>&1
if [ $? = 0 ] ; then

	#cho ".................................................||................................................."
	echo "...............................................lsscsi..............................................." >> ./$fileName/lsscsi-all.txt
	lsscsi  >> ./$fileName/lsscsi-all.txt 2>> ./misc_output.txt
	for i in c d g H k l v ; do
		echo "..............................................lsscsi -$i............................................" >> ./$fileName/lsscsi-all.txt
		lsscsi -$i >> ./$fileName/lsscsi-all.txt 2>> ./misc_output.txt
	done

	# Non standard out, leave separate
	echo "..............................................lsscsi -V............................................" >> ./$fileName/lsscsi-all.txt
	lsscsi -V >> ./$fileName/lsscsi-all.txt 2>&1 
	
	lsscsi -vg >> ./$fileName/lsscsi-vg.txt 2>> ./misc_output.txt
	
	#cho ".................................................||................................................."
	echo ".......................All lines lspci -vv with AVAGO, LSI OR 3ware in the line....................." >> ./$fileName/Controller_Disk_Association.txt 2>> ./misc_output.txt
	lspci -vv | $grep -i -e LSI -e 3ware -e avago >> ./$fileName/Controller_Disk_Association.txt 2>> ./misc_output.txt
	
	#cho ".................................................||................................................."
	echo ".............................................lsscsi -vg............................................." >> ./$fileName/Controller_Disk_Association.txt 2>> ./misc_output.txt
	lsscsi -vg >> ./$fileName/Controller_Disk_Association.txt 2>> ./misc_output.txt
	
	else

	#cho ".................................................||................................................."
	echo "...............lsscsi NOT installed! Recommend Installation if supported on this OS................." >> ./$fileName/Controller_Disk_Association.txt

fi

lsblk >> ./misc_output.txt 2>&1

if [ $? = 0 ] ; then

	#cho ".................................................||................................................."
	echo "...............................................lsblk................................................" >> ./$fileName/lsblk-all.txt
	lsblk  >> ./$fileName/lsblk-all.txt 2>> ./misc_output.txt
	for i in a b t V ; do
		#cho ".................................................||................................................."
		echo "..............................................lsblk -$i............................................." >> ./$fileName/lsblk-all.txt
		lsblk -$i >> ./$fileName/lsblk-all.txt 2>> ./misc_output.txt
	done
fi

showsel  >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	showsel  > ./$fileName/showsel.txt 2>> ./misc_output.txt
fi

lshw  >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then
	lshw > ./$fileName/lshw.txt 2>> ./misc_output.txt
fi

if [ "$TWSKIPNETWORK" = "NO" ] ; then

	ifconfig  >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		#cho ".................................................||................................................."
		echo "..............................................ifconfig.............................................." > ./$fileName/network_config.txt
		ifconfig > ./$fileName/network_config.txt 2>> ./misc_output.txt
	fi

	ipmitool lan print  >> ./misc_output.txt 2>&1
	if [ "$?" = "0" ] ; then
		#cho ".................................................||................................................."
		echo ".........................................ipmitool_lan_print........................................." >> ./$fileName/network_config.txt
		ipmitool lan print >> ./$fileName/network_config.txt 2>> ./misc_output.txt
	fi
fi

if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	###########################################################################################################################
	# Common - 3DM(2) data collection
	###########################################################################################################################
	echo "Collecting 3DM(2) version info..."

	#3dm data collection
	
	if [ -f  /usr/sbin/3dmd ] ; then
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		mkdir ./$fileName/LSI_Products/3ware/3DM
		strings /usr/sbin/3dmd | fgrep 1.13. > ./$fileName/LSI_Products/3ware/3DM/3dm_version.txt
	fi
	
	if [ -f  /etc/3dm.conf ] ; then 
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		cp /etc/3dm.conf ./$fileName/LSI_Products/3ware/3DM/ 
	fi

	if [ -f  /var/log/3w-aenlog.txt ] ; then 
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		cp /var/log/3w-* ./$fileName/LSI_Products/3ware/3DM/ 
	fi

	#Other 3dm logs????

	#3dm2 data collection
	if [ -f  /usr/sbin/3dm2 ] ; then 
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		mkdir ./$fileName/LSI_Products/3ware/3DM2 
	fi

	###Update on Code Set Change
	for i in 2.00.00.038 2.01. 2.02. 2.03. 2.04. 2.05. 2.06. 2.07. 2.08. 2.09.; do
		if [ -f  /usr/sbin/3dm2 ] ; then 
			strings /usr/sbin/3dm2 | fgrep $i >> ./$fileName/script_workspace/3dm2_api_ver.txt
		fi
	done

	if [ -f  ./$fileName/script_workspace/3dm2_api_ver.txt ] ; then 
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		grep -n . ./$fileName/script_workspace/3dm2_api_ver.txt | $grep 1: | cut -d: -f2 > ./$fileName/LSI_Products/3ware/3DM2/3dm2api_ver.txt
		grep -n . ./$fileName/script_workspace/3dm2_api_ver.txt | $grep 3: | cut -d: -f2 > ./$fileName/LSI_Products/3ware/3DM2/3dm2_ver.txt
	fi


	if [ -f /etc/3dm2/3dm2.conf ] ; then 
		if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
		cp /etc/3dm2/3dm2.conf ./$fileName/LSI_Products/3ware/3DM2 
	fi

	cp /var/log/tdm_aen* ./$fileName/LSI_Products/3ware/3DM2 2>> ./misc_output.txt

	if [ -f /opt/AMCC/log.txt ] ; then cp /opt/AMCC/log.txt ./$fileName/LSI_Products/3ware/3DM2 2>> ./misc_output.txt ; fi

	if [ -f /var/log/tw_mgmt.log ] ; then 
		if [ -f ./$fileName/LSI_Products/3ware/3DM2 ] ; then cp /var/log/tw_mgmt.log ./$fileName/LSI_Products/3ware/3DM2; else
			if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
			cp /var/log/tw_mgmt.log ./$fileName/LSI_Products/3ware/ 2>> ./misc_output.txt
		fi
	fi

	#Other 3dm2 logs????
fi


if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	###########################################################################################################################
	# MegaRAID Storage Manager Log files - Other Unix distros?
	###########################################################################################################################



	if [ -d "$MSM_HOME"/MegaMonitor ] ; then
		mkdir ./$fileName/LSI_Products/MegaRAID/MSM
		echo "Collecting MSM logs..."
		cp "$MSM_HOME"/MegaMonitor/* ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
		cp "$MSM_HOME"/*.log ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
		cp "$MSM_HOME"/*.txt ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
	fi

	if [ "$MSM_HOME" = "" ] ; then
		if [ -d /usr/local/"MegaRAID Storage Manager"/MegaMonitor ] ; then
			mkdir ./$fileName/LSI_Products/MegaRAID/MSM
			echo "Collecting MSM logs..."
			cp /usr/local/"MegaRAID Storage Manager"/MegaMonitor/* ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
			cp /usr/local/"MegaRAID Storage Manager"/*.log ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
			cp /usr/local/"MegaRAID Storage Manager"/*.txt ./$fileName/LSI_Products/MegaRAID/MSM 2>> ./misc_output.txt
		fi
	fi

	if [ -f ./$fileName/LSI_Products/MegaRAID/MSM ] ; then
		if [ -f /etc/init.d/vivaldiframeworkd  ] ; then
			/etc/init.d/vivaldiframeworkd status 2 > ./$fileName/LSI_Products/MegaRAID/MSM/Status.txt 2>> ./misc_output.txt
		fi
	fi

	if [ -f ./$fileName/LSI_Products/MegaRAID/MSM ] ; then
		if [ -f /etc/init.d/mrmonitor   ] ; then
			/etc/init.d/mrmonitor status 2 >> ./$fileName/LSI_Products/MegaRAID/MSM/Status.txt 2>> ./misc_output.txt
			/etc/init.d/mrmonitor -v 2 >> ./$fileName/LSI_Products/MegaRAID/MSM/Status.txt 2>> ./misc_output.txt
			#Keep separate, mrmonitord_version.txt used in other parts of the script
			/etc/init.d/mrmonitor -v | cut -d m -f 2 | cut -d r -f 2 > ./$fileName/LSI_Products/MegaRAID/MSM/mrmonitord_version.txt 2>> ./misc_output.txt
		fi
	fi


###########################################################################################################################
# Nytro MegaRaid (CSA)
###########################################################################################################################
	if [ -n `cat ./$fileName/script_workspace/num_mraid_adapters.txt` ] ; then
		csa_list=`$MCLI_LOCATION$MCLI_NAME -cfgcachecadedsply -aall nolog 2>&1 | awk '{if ($1 == "Adapter:") print $2}'`
			if [ -n "$csa_list" ] ; then
				echo "Collecting Nytro MegaRAID info..."
				mkdir ./$fileName/LSI_Products/NytroMegaRAID
				$MCLI_LOCATION$MCLI_NAME -cfgcachecadedsply -aall nolog > $fileName/LSI_Products/NytroMegaRAID/mr-cfgcachecadedsply-all.txt 2>&1
				for i in $csa_list ; do
					for slot in 4 6 ; do
						$MCLI_LOCATION$MCLI_NAME -PDDffDiag -DumpSmartLog -PhysDrv[252:$slot] -a$i nolog > $fileName/LSI_Products/NytroMegaRAID/mr-smartlog-c$i-slot$slot.txt 2>&1
						$MCLI_LOCATION$MCLI_NAME -PDDffDiag -DumpPanicLog -query -PhysDrv[252:$slot] -a$i nolog > $fileName/LSI_Products/NytroMegaRAID/mr-paniclog-query-c$i-slot$slot.txt 2>&1
						# Add check for valid panic logs and save them
					done
				done
			fi
	fi

fi	

###########################################################################################################################
echo "Collecting and Processing system logs/messages files..."
###########################################################################################################################
#Linux, FreeBSD
###########################################################################################################################


journalctl -h > ./$fileName/script_workspace/journalctl-h.txt 2>> ./misc_output.txt
if [ "$?" = "0" ] ; then
	if [ "$TWPRINTFILENAMETRIGGER" != "YES" ] ; then
		journalctl -a > ./$fileName/journalctl-a.txt 2>> ./misc_output.txt
		#journalctl -o verbose > ./$fileName/journalctl-o_verbose.txt 2>> ./misc_output.txt
	fi
	if [ -f ./journalctl_time.txt ] ; then
		journalctl_time=`(cat journalctl_time.txt)`
		if [ "$TWPRINTFILENAMETRIGGER" = "YES" ] ; then
			journalctl -a --since=${journalctl_time} > ./$fileName/journalctl-a.txt 2>> ./misc_output.txt
			#journalctl -o verbose > ./$fileName/journalctl-o_verbose.txt 2>> ./misc_output.txt
		fi
	fi
fi



###########################################################################################################################
# Start additional info collection
###########################################################################################################################
if [ -f ./local.sh ] ; then
	echo "Collecting additional info via local.sh..."
	mkdir ./$fileName/addedInfo
	./local.sh ./$fileName/addedInfo
fi 

###########################################################################################################################
#Start Linux Only Section
###########################################################################################################################


#Added udevadm info back in

udevadm --version >> ./misc_output.txt 2>&1
if [ "$?" = "0" ] ; then

	mkdir $fileName/udev
	udevadm --version > $fileName/udev/udev_version.txt 2>&1

	for i in $( ls /dev/sd* ); do
		dev=$(basename $i)
		if [ -f /usr/lib/udev/scsi_id ] ; then
			/usr/lib/udev/scsi_id  --export --whitelisted -d $i > $fileName/udev/scsi_id_whitelisted_${dev}.txt 2>&1
		fi
		if [ -f /usr/lib/udev/ata_id ] ; then
			/usr/lib/udev/ata_id  --export $i > $fileName/udev/ata_id_${dev}.txt 2>&1
		fi
		#cho ".................................................||................................................."
		echo "....................................udevadm_info_dev_${dev}......................................." >> ./$fileName/udev/udevadm_info.txt	
		udevadm info /dev/${dev} >> ./$fileName/udev/udevadm_info.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
		echo "....................................udevadm_info-a_dev_${dev}......................................." >> ./$fileName/udev/udevadm_info-a.txt
		udevadm info -a /dev/${dev} >> ./$fileName/udev/udevadm_info-a.txt 2>> ./$fileName/script_workspace/lsiget_errorlog.txt
	done
fi


#Device Mapping Info Linux
for i in by-id by-label by-path by-uuid; do
	if [ -d  /dev/disk/$i ] ; then
		if [  ! -d $fileName/udev ] ; then mkdir $fileName/udev ; fi
		ls -latr /dev/disk/$i > ./$fileName/udev/ls-latr_dev_disk_$i.txt 
		ls /dev/disk/$i > ./$fileName/udev/ls_dev_disk_$i.txt 
	fi
done


if [ "$OS_LSI" = "linux" ] ; then 


	#cho ".................................................||................................................."
	echo $fileName > ./$fileName/get_cpu_affinity.txt

	
	for i in $(ls -1 /sys/class/scsi_host);  do
		if [ `cat /sys/class/scsi_host/${i}/proc_name` = "mpt2sas" ] ; then MPT2SASLOADED=YES ; fi
		if [ `cat /sys/class/scsi_host/${i}/proc_name` = "mpt3sas" ] ; then MPT3SASLOADED=YES ; fi
		if [ `cat /sys/class/scsi_host/${i}/proc_name` = "megaraid_sas" ] ; then MEGARAID_SASLOADED=YES ; fi
	done
		
	if [ "${MPT2SASLOADED}" = "YES" ] ; then

		#cho ".................................................||................................................."
		echo "..............................................mpt2sas..............................................." >> ./$fileName/get_cpu_affinity.txt		

		# this is count of the number cpu's we have set the affinity for
		cpu_affinity_count=0
		
		# This is total number of cores
		cpu_count=0
		for i in /sys/devices/system/cpu/cpu[0-9]* ; do
			let cpu_count=${cpu_count}+1
		done
		
		# line below for is for debugging
		#cpu_count=24
		

		for i in $(ls -1 /sys/class/scsi_host);  do
			if [ `cat /sys/class/scsi_host/${i}/proc_name` != "mpt2sas" ] ; then
				continue;
			fi;
			if [ ! -f /sys/class/scsi_host/${i}/reply_queue_count ] ; then
				echo "reply_queue_count sysfs attribute doesn't exist" >> ./$fileName/get_cpu_affinity.txt
				continue;
			fi;
			if [ `cat /sys/class/scsi_host/${i}/reply_queue_count` -lt 2 ] ; then
				continue;
			fi;
			ioc=`cat /sys/class/scsi_host/${i}/unique_id`
			index=0
			affinity=1
			cpu_affinity_count=0
			msix_count=`cat /proc/interrupts | grep -c mpt2sas${ioc}-msix`
			if [ ${cpu_count} -gt ${msix_count} ] ; then
				let grouping=${cpu_count}/${msix_count}
				let grouping_mod=${cpu_count}%${msix_count}
		# line below for is for debugging
		#		echo grouping = ${grouping}, grouping_mod = ${grouping_mod}
				if [ ${grouping} -lt 2 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -eq 2 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -lt 4 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -eq 4 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -lt 8 ] ; then
					cpu_grouping=8;
				elif [ ${grouping} -eq 8 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=8;
				else
					cpu_grouping=16;
				fi;
			else
				cpu_grouping=0;
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			echo "ioc number = ${ioc}" >> ./$fileName/get_cpu_affinity.txt
			echo "number of core processors = ${cpu_count}" >> ./$fileName/get_cpu_affinity.txt
			echo "msix vector count = ${msix_count}" >> ./$fileName/get_cpu_affinity.txt
			if [ ${cpu_grouping} -eq 0 ] ; then
				echo "number of cores per msix vector = 1" >> ./$fileName/get_cpu_affinity.txt
			else
				echo "number of cores per msix vector = ${cpu_grouping}" >> ./$fileName/get_cpu_affinity.txt
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			while [ ${msix_count} -gt ${index} ] && \
				( [ ${cpu_affinity_count} -lt ${cpu_count} ] || \
				  [ ${cpu_affinity_count} -eq ${cpu_count} ] ); do
				b=0
				a=`cat /proc/interrupts | grep -w mpt2sas${ioc}-msix${index} | cut -d : -f 1`
				irq_number=`basename ${a}`
				echo -e "\tmsix index = ${index}, irq number =  ${irq_number}, cpu affinity mask = `cat /proc/irq/${irq_number}/smp_affinity` \thint = `cat /proc/irq/${irq_number}/affinity_hint`" >> ./$fileName/get_cpu_affinity.txt
				let index=${index}+1
				let affinity=${affinity}*2
			done
			echo -e "\nWe have set affinity for ${index} msix vectors and ${cpu_count} core processors\n" >> ./$fileName/get_cpu_affinity.txt
		done
	fi

	if [ "${MPT3SASLOADED}" = "YES" ] ; then

		#cho ".................................................||................................................."
		echo "..............................................mpt3sas..............................................." >> ./$fileName/get_cpu_affinity.txt		

		# this is count of the number cpu's we have set the affinity for
		cpu_affinity_count=0
		
		# This is total number of cores
		cpu_count=0
		for i in /sys/devices/system/cpu/cpu[0-9]* ; do
			let cpu_count=${cpu_count}+1
		done
		
		# line below for is for debugging
		#cpu_count=24
		

		for i in $(ls -1 /sys/class/scsi_host);  do
			if [ `cat /sys/class/scsi_host/${i}/proc_name` != "mpt3sas" ] ; then
				continue;
			fi;
			if [ ! -f /sys/class/scsi_host/${i}/reply_queue_count ] ; then
				echo "reply_queue_count sysfs attribute doesn't exist" >> ./$fileName/get_cpu_affinity.txt
				continue;
			fi;
			if [ `cat /sys/class/scsi_host/${i}/reply_queue_count` -lt 2 ] ; then
				continue;
			fi;
			ioc=`cat /sys/class/scsi_host/${i}/unique_id`
			index=0
			affinity=1
			cpu_affinity_count=0
			msix_count=`cat /proc/interrupts | grep -c mpt3sas${ioc}-msix`
			if [ ${cpu_count} -gt ${msix_count} ] ; then
				let grouping=${cpu_count}/${msix_count}
				let grouping_mod=${cpu_count}%${msix_count}
		# line below for is for debugging
		#		echo grouping = ${grouping}, grouping_mod = ${grouping_mod}
				if [ ${grouping} -lt 2 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -eq 2 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -lt 4 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -eq 4 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -lt 8 ] ; then
					cpu_grouping=8;
				elif [ ${grouping} -eq 8 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=8;
				else
					cpu_grouping=16;
				fi;
			else
				cpu_grouping=0;
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			echo "ioc number = ${ioc}" >> ./$fileName/get_cpu_affinity.txt
			echo "number of core processors = ${cpu_count}" >> ./$fileName/get_cpu_affinity.txt
			echo "msix vector count = ${msix_count}" >> ./$fileName/get_cpu_affinity.txt
			if [ ${cpu_grouping} -eq 0 ] ; then
				echo "number of cores per msix vector = 1" >> ./$fileName/get_cpu_affinity.txt
			else
				echo "number of cores per msix vector = ${cpu_grouping}" >> ./$fileName/get_cpu_affinity.txt
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			while [ ${msix_count} -gt ${index} ] && \
				( [ ${cpu_affinity_count} -lt ${cpu_count} ] || \
				  [ ${cpu_affinity_count} -eq ${cpu_count} ] ); do
				b=0
				a=`cat /proc/interrupts | grep -w mpt3sas${ioc}-msix${index} | cut -d : -f 1`
				irq_number=`basename ${a}`
				echo -e "\tmsix index = ${index}, irq number =  ${irq_number}, cpu affinity mask = `cat /proc/irq/${irq_number}/smp_affinity` \thint = `cat /proc/irq/${irq_number}/affinity_hint`" >> ./$fileName/get_cpu_affinity.txt
				let index=${index}+1
				let affinity=${affinity}*2
			done
			echo -e "\nWe have set affinity for ${index} msix vectors and ${cpu_count} core processors\n" >> ./$fileName/get_cpu_affinity.txt
		done
	fi

	if [ "${MEGARAID_SASLOADED}" = "YES" ] ; then

		#cho ".................................................||................................................."
		echo "............................................megaraid_sas............................................" >> ./$fileName/get_cpu_affinity.txt		

		# this is count of the number cpu's we have set the affinity for
		cpu_affinity_count=0
		
		# This is total number of cores
		cpu_count=0
		for i in /sys/devices/system/cpu/cpu[0-9]* ; do
			let cpu_count=${cpu_count}+1
		done
		
		# line below for is for debugging
		#cpu_count=24
		

		for i in $(ls -1 /sys/class/scsi_host);  do
			if [ `cat /sys/class/scsi_host/${i}/proc_name` != "megaraid_sas" ] ; then
				continue;
			fi;
			if [ ! -f /sys/class/scsi_host/${i}/reply_queue_count ] ; then
				echo "reply_queue_count sysfs attribute doesn't exist" >> ./$fileName/get_cpu_affinity.txt
				continue;
			fi;
			if [ `cat /sys/class/scsi_host/${i}/reply_queue_count` -lt 2 ] ; then
				continue;
			fi;
			ioc=`cat /sys/class/scsi_host/${i}/unique_id`
			index=0
			affinity=1
			cpu_affinity_count=0
			msix_count=`cat /proc/interrupts | grep -c megaraid_sas${ioc}-msix`
			if [ ${cpu_count} -gt ${msix_count} ] ; then
				let grouping=${cpu_count}/${msix_count}
				let grouping_mod=${cpu_count}%${msix_count}
		# line below for is for debugging
		#		echo grouping = ${grouping}, grouping_mod = ${grouping_mod}
				if [ ${grouping} -lt 2 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -eq 2 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=2;
				elif [ ${grouping} -lt 4 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -eq 4 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=4;
				elif [ ${grouping} -lt 8 ] ; then
					cpu_grouping=8;
				elif [ ${grouping} -eq 8 ] && [ ${grouping_mod} -eq 0 ] ; then
					cpu_grouping=8;
				else
					cpu_grouping=16;
				fi;
			else
				cpu_grouping=0;
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			echo "ioc number = ${ioc}" >> ./$fileName/get_cpu_affinity.txt
			echo "number of core processors = ${cpu_count}" >> ./$fileName/get_cpu_affinity.txt
			echo "msix vector count = ${msix_count}" >> ./$fileName/get_cpu_affinity.txt
			if [ ${cpu_grouping} -eq 0 ] ; then
				echo "number of cores per msix vector = 1" >> ./$fileName/get_cpu_affinity.txt
			else
				echo "number of cores per msix vector = ${cpu_grouping}" >> ./$fileName/get_cpu_affinity.txt
			fi;
			echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
			while [ ${msix_count} -gt ${index} ] && \
				( [ ${cpu_affinity_count} -lt ${cpu_count} ] || \
				  [ ${cpu_affinity_count} -eq ${cpu_count} ] ); do
				b=0
				a=`cat /proc/interrupts | grep -w megaraid_sas${ioc}-msix${index} | cut -d : -f 1`
				irq_number=`basename ${a}`
				echo -e "\tmsix index = ${index}, irq number =  ${irq_number}, cpu affinity mask = `cat /proc/irq/${irq_number}/smp_affinity` \thint = `cat /proc/irq/${irq_number}/affinity_hint`" >> ./$fileName/get_cpu_affinity.txt
				let index=${index}+1
				let affinity=${affinity}*2
			done
			echo -e "\nWe have set affinity for ${index} msix vectors and ${cpu_count} core processors\n" >> ./$fileName/get_cpu_affinity.txt
		done
	fi


	#cho ".................................................||................................................."
	echo "...................................................................................................." >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt 
	echo "Ideally "cpu affinity mask" and "hint" should match. See example snippet below." >> ./$fileName/get_cpu_affinity.txt
	echo "This can cause CPU  "BUG: soft lockup"." >> ./$fileName/get_cpu_affinity.txt
	echo "To correct this if not matching, modify /etc/sysconfig/irqbalance as follows;" >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt 
	echo "# append any args here to the irqbalance daemon as documented in the man page" >> ./$fileName/get_cpu_affinity.txt
	echo "#" >> ./$fileName/get_cpu_affinity.txt
	echo "IRQBALANCE_ARGS=-hexact" >> ./$fileName/get_cpu_affinity.txt 
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo "ps -elf | grep irqbalance will show you the option in use" >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo "Your current setting;" >> ./$fileName/get_cpu_affinity.txt 
	if [ -f /etc/sysconfig/irqbalance ] ; then grep "IRQBALANCE_ARGS=" /etc/sysconfig/irqbalance >> ./$fileName/get_cpu_affinity.txt ; fi
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo "Now do an irqbalance service restart (command may be different for different Distro)." >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo "service irqbalance restart" >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo -e "\n" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 0, irq number =  98, cpu affinity mask = 00,00000001 hint = 00,00000001" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 1, irq number =  99, cpu affinity mask = 00,00000002 hint = 00,00000002" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 2, irq number =  100, cpu affinity mask = 00,00000004 hint = 00,00000004" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 3, irq number =  101, cpu affinity mask = 00,00000008 hint = 00,00000008" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 4, irq number =  102, cpu affinity mask = 00,00000010 hint = 00,00000010" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 5, irq number =  103, cpu affinity mask = 00,00000020 hint = 00,00000020" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 6, irq number =  104, cpu affinity mask = 00,00000040 hint = 00,00000040" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 7, irq number =  105, cpu affinity mask = 00,00000080 hint = 00,00000080" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 8, irq number =  106, cpu affinity mask = 00,00000100 hint = 00,00000100" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 9, irq number =  107, cpu affinity mask = 00,00000200 hint = 00,00000200" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 10, irq number =  108, cpu affinity mask = 00,00000400 hint = 00,00000400" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 11, irq number =  109, cpu affinity mask = 00,00000800 hint = 00,00000800" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 12, irq number =  110, cpu affinity mask = 00,00001000 hint = 00,00001000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 13, irq number =  111, cpu affinity mask = 00,00002000 hint = 00,00002000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 14, irq number =  112, cpu affinity mask = 00,00004000 hint = 00,00004000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 15, irq number =  113, cpu affinity mask = 00,00008000 hint = 00,00008000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 16, irq number =  114, cpu affinity mask = 00,00010000 hint = 00,00010000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 17, irq number =  115, cpu affinity mask = 00,00020000 hint = 00,00020000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 18, irq number =  116, cpu affinity mask = 00,00040000 hint = 00,00040000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 19, irq number =  117, cpu affinity mask = 00,00080000 hint = 00,00080000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 20, irq number =  118, cpu affinity mask = 00,00100000 hint = 00,00100000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 21, irq number =  119, cpu affinity mask = 00,00200000 hint = 00,00200000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 22, irq number =  120, cpu affinity mask = 00,00400000 hint = 00,00400000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 23, irq number =  121, cpu affinity mask = 00,00800000 hint = 00,00800000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 24, irq number =  122, cpu affinity mask = 00,01000000 hint = 00,01000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 25, irq number =  123, cpu affinity mask = 00,02000000 hint = 00,02000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 26, irq number =  124, cpu affinity mask = 00,04000000 hint = 00,04000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 27, irq number =  125, cpu affinity mask = 00,08000000 hint = 00,08000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 28, irq number =  126, cpu affinity mask = 00,10000000 hint = 00,10000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 29, irq number =  127, cpu affinity mask = 00,20000000 hint = 00,20000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 30, irq number =  128, cpu affinity mask = 00,40000000 hint = 00,40000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 31, irq number =  129, cpu affinity mask = 00,80000000 hint = 00,80000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 32, irq number =  130, cpu affinity mask = 01,00000000 hint = 01,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 33, irq number =  131, cpu affinity mask = 02,00000000 hint = 02,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 34, irq number =  132, cpu affinity mask = 04,00000000 hint = 04,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 35, irq number =  133, cpu affinity mask = 08,00000000 hint = 08,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 36, irq number =  134, cpu affinity mask = 10,00000000 hint = 10,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 37, irq number =  135, cpu affinity mask = 20,00000000 hint = 20,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 38, irq number =  136, cpu affinity mask = 40,00000000 hint = 40,00000000" >> ./$fileName/get_cpu_affinity.txt
	echo "msix index = 39, irq number =  137, cpu affinity mask = 80,00000000 hint = 80,00000000" >> ./$fileName/get_cpu_affinity.txt
	



	

	echo "...................................................................................................." >> ./$fileName/modinfo.txt

	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		TwareDriver=`lsmod | awk '{print $1}' | $grep 3w`
		if [ "$?" -eq "0" ] ; then
			for i in $TwareDriver ;	do
				echo "LSI 3ware driver module name $i" >> ./$fileName/modinfo.txt
				modinfo $i >> ./$fileName/modinfo.txt 2>> ./misc_output.txt
				echo "...................................................................................................." >> ./$fileName/modinfo.txt
					if  modinfo -n $i >> ./misc_output.txt 2>> ./misc_output.txt; then  
						if [ ! -d ./$fileName/LSI_Products/3ware ] ;then mkdir ./$fileName/LSI_Products/3ware ; fi
						cp -p `modinfo -n $i` ./$fileName/LSI_Products/3ware
					fi  
		
			done
		
			else
			
			echo "LSI 3ware driver module not installed or IN-Kernel driver used if driver installed." >> ./$fileName/modinfo.txt
			echo "...................................................................................................." >> ./$fileName/modinfo.txt
		fi
	fi

	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		MRSASDriver=`lsmod | awk '{print $1}' | $grep mega`
		
		if [ "$?" -eq "0" ] ; then
			for i in $MRSASDriver ;	do
				echo "LSI MegaRAID SAS driver module name $i" >> ./$fileName/modinfo.txt
				modinfo $i >> ./$fileName/modinfo.txt
				echo "...................................................................................................." >> ./$fileName/modinfo.txt
				if  modinfo -n $i >> ./misc_output.txt 2>> ./misc_output.txt; then  
					cp -p `modinfo -n $i` ./$fileName/LSI_Products/MegaRAID
				fi  
		
			done
			
			else
			
			echo "LSI MegaRAID SAS driver module not installed or IN-Kernel driver used if driver installed." >> ./$fileName/modinfo.txt
			echo "...................................................................................................." >> ./$fileName/modinfo.txt
		fi
	fi		

	if [ "$TWGETSKIPHBA" != "YES" ] ; then
		LSISASDriver=`lsmod | awk '{print $1}' | $grep mpt`
		if [ "$?" -eq "0" ] ; then
			for i in $LSISASDriver ; do
				echo "LSI SAS HBA driver module name $i" >> ./$fileName/modinfo.txt
				modinfo $i >> ./$fileName/modinfo.txt
				echo "...................................................................................................." >> ./$fileName/modinfo.txt
				if  modinfo -n $i >> ./misc_output.txt 2>> ./misc_output.txt; then  
					cp -p `modinfo -n $i` ./$fileName/LSI_Products/HBA
					objdump -ld `modinfo -n $i` 2>> ./misc_output.txt 1>> ./$fileName/LSI_Products/HBA/${i}_objdump-ld.txt
				fi  
	
			done
	
			else
	
			echo "LSI SAS HBA driver module not installed or IN-Kernel driver used if driver installed." >> ./$fileName/modinfo.txt
			echo "...................................................................................................." >> ./$fileName/modinfo.txt
		fi
	fi
	

	
	#
	#Add additional md data gathering lines
	#
	
	


	#
	#All Generic Performance Tuning parameters should be gathered here!
	#
	echo "Collecting Generic SD Device Performance Tuning Data..."
	echo "Collecting - parted -s /dev/sdX print..."
	echo "Note: An active mkfs will cause this script to pause..."



	for t in $( ls /dev/sd* ); do
		dev=$(basename $t)

		if [ ! -d ./$fileName/Generic_Perf_Tuning ] ; then mkdir ./$fileName/Generic_Perf_Tuning ; fi 

		if [ -e /sys/block/${dev} ] ; then
				
			#cho ".................................................||................................................."
			echo "...................................................................................................." >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			echo "Parameters that impact performance, see the various KB articles on Performance Tuning." >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			echo "...................................................................................................." >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt


			echo "cat /sys/block/${dev}/queue/max_sectors_kb" >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			cat /sys/block/${dev}/queue/max_sectors_kb >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
	
			echo "cat /sys/block/${dev}/queue/nr_requests" >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			cat /sys/block/${dev}/queue/nr_requests >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
		 
			echo "cat /sys/block/${dev}/queue/scheduler" >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			cat /sys/block/${dev}/queue/scheduler >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
	
			echo "blockdev --getra /dev/${dev}" >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
			blockdev --getra /dev/${dev} >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt        	

			if [ -d /sys/block/${dev} ] ; then 				
				echo "...................................................................................................."  >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
				echo "...................................................................................................."  >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
				echo "The following dumps all info in the /sys/block/${dev}/queue/ directory" >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
				echo "...................................................................................................."  >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
				
				for j in /sys/block/${dev}/queue/*; do
					if [ -f ${j} ] ; then
						echo $j >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt 2>&1
						cat $j >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt 2>&1
					fi
				done
				
				if [ -d /sys/block/${dev}/queue/iosched ] ; then 				
					for j in /sys/block/${dev}/queue/iosched/*; do
						if [ -f ${j} ] ; then 
							echo $j >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
							cat $j >> ./$fileName/Generic_Perf_Tuning/Generic_Perf_Tuning_${dev}.txt
						fi
					done
				fi
				
				#cho ".................................................||................................................."
				echo ".....................................parted -s /dev/${dev} print......................................." >> ./$fileName/parted_print_sdX.txt
				parted -s /dev/${dev} print >> ./$fileName/parted_print_sdX.txt 2>> ./misc_output.txt
			fi	
		fi
	done
			

	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then				
		echo "Notice: MR_MONITOR Error Codes are in Decimal, megasas_aen_polling Error Codes are in HEX, they report the same information." > ./$fileName/LSI_Products/MegaRAID/Hex_Decimal_Numbering_Conventions.txt
		echo "I.E. '"'megasas_aen_polling[1]: event code 0x0087'"' is the same as '"'MR_MONITOR[3226]: <MRMON135> Controller ID: 0  Global Hot Spare created: --:--:2'"'" >> ./$fileName/LSI_Products/MegaRAID/Hex_Decimal_Numbering_Conventions.txt
		echo "The MR_MONITOR error files have the Decimal equivalent embedded after the severity level for ease of identification." >> ./$fileName/LSI_Products/MegaRAID/Hex_Decimal_Numbering_Conventions.txt
		echo "The megasas error files have the Hex equivalent embedded after the severity level for ease of identification." >> ./$fileName/LSI_Products/MegaRAID/Hex_Decimal_Numbering_Conventions.txt
	fi
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with ' 3w' in the line#" > ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
	fi

	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'MR_MONITOR' in the line#" > ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'MR_MONITOR' OR 'megasas' OR 'megaraid_sas' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'megasas' OR 'megaraid_sas' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
	fi

	if [ "$TWGETSKIPHBA" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'kernel: mpt' OR '] mpt' in the line#" > ./$fileName/LSI_Products/HBA/hba_driver_messages.txt
	fi
	

	
	echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with ' kernel: sd ' OR '] sd ' in the line#" > ./$fileName/OS_Disk_driver_messages.txt
	
	#Make sure zipped messages don't overwrite unzipped.
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f ./$fileName/var_log/message$i ] ; then 
			if [ -f  ./$fileName/var_log/message$i.gz ] ; then mv ./$fileName/var_log/message$i.gz ./$fileName/var_log/message$i.gz.dupe_name ; fi
		fi
	done


	
	#Checks for compressed/dated message files.
	
	gunzip ./$fileName/messages-????????.gz 2>> ./misc_output.txt
	ls ./$fileName/messages-???????? > ./$fileName/script_workspace/messages_dated.txt 2>> ./misc_output.txt
	for i in $( cat ./$fileName/script_workspace/messages_dated.txt );do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			grep " 3w" $i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			grep "MR_MONITOR" $i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
			grep "kernel: megasas" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
			egrep "MR_MONITOR|kernel: megasas" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt
		fi
		if [ "$TWGETSKIPHBA" != "YES" ] ; then
			grep "kernel: mpt" $i >> ./$fileName/LSI_Products/HBA/hba_driver_messages.txt
		fi
		grep " kernel: sd " $i >> ./$fileName/OS_Disk_driver_messages.txt
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f  ./$fileName/var_log/message$i.gz ] ; then gunzip ./$fileName/var_log/message$i.gz ; fi
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep " 3w" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt ; fi
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "MR_MONITOR" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt ; fi
			if [ -f ./$fileName/var_log/message$i ] ; then egrep "megasas|megaraid_sas" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt ; fi
			if [ -f ./$fileName/var_log/message$i ] ; then egrep "MR_MONITOR|megasas|megaraid_sas" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt ; fi
		fi
		if [ "$TWGETSKIPHBA" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then egrep "kernel: mpt|] mpt" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/HBA/hba_driver_messages.txt ; fi
		fi
		if [ -f ./$fileName/var_log/message$i ] ; then egrep " kernel: sd |] sd " ./$fileName/var_log/message$i >> ./$fileName/OS_Disk_driver_messages.txt ; fi
	done
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		grep " (0x03:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x03_9000_ERROR.txt 
		grep " (0x04:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt 
		grep " (0x06:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x06_9000_DRIVER.txt 
		grep " (0x09:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x09_LINUX_OS.txt 
		grep " (0x0B:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0B_API.txt 
		grep " (0x0C:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0C_3DMPLUS.txt 
		grep " (0x0D:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0D_CLI.txt 
		grep " (0x0E:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0E_7000_ERROR.txt 
		grep " (0x0F:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0F_7000_EVENT.txt 
		grep " (0x11:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_LINUX_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ; fi
		done
	
	
		grep " (0x03:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x03_9000_ERROR.txt 
		grep " (0x04:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x04_9000_EVENT.txt 
		grep " (0x06:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_9000_DRIVER.txt 
		grep " (0x09:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x09_LINUX_OS.txt 
		grep " (0x0B:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0B_API.txt 
		grep " (0x0C:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0C_3DMPLUS.txt 
		grep " (0x0D:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0D_CLI.txt 
		grep " (0x0E:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0E_7000_ERROR.txt 
		grep " (0x0F:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0F_7000_EVENT.txt 
		grep " (0x11:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_LINUX_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
			if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ; fi
		done
	fi
	
	
	
# Final fi
fi
#
# End Linux Only Section
#

###########################################################################################################################
#Start VMWare Only Section
###########################################################################################################################

if [ "$VMWARE_SUPPORTED" = "YES" ] ; then 

#Leftover from original 3.x/4.x vmware support	
#		TwareDriver=`vmkload_mod -b | awk '{print $1}' | $grep 3w`
#		if [ "$?" -eq "0" ] ; then
#			for i in $TwareDriver ; do
#				echo "3ware driver module name " $i "\n" >> ./$fileName/modinfo.txt
#				modinfo $i >> ./$fileName/modinfo.txt
#					if  modinfo -n $i >> ./misc_output.txt 2>> ./misc_output.txt; then  
#						cp -p `modinfo -n $i` ./$fileName/
#					fi  
#
#			done
#
#			else
#
#			echo "Driver Module not installed or IN-Kernel Driver used if driver installed." >> ./$fileName/modinfo.txt
#		fi
#
#		echo "" >> ./$fileName/modinfo.txt
#
#		MRSASDriver=`vmkload_mod -b | awk '{print $1}' | $grep megaraid_sas`
#		if [ "$?" -eq "0" ] ; then
#			for i in $MRSASDriver ;	do
#				echo "MegaRAID SAS driver module name " $i "\n" >> ./$fileName/modinfo.txt
#				modinfo $i >> ./$fileName/modinfo.txt
#				if  modinfo -n $i >> ./misc_output.txt 2>> ./misc_output.txt; then  
#					cp -p `modinfo -n $i` ./$fileName/
#				fi  
#		
#			done
#		
#			else
#			
#			echo "3ware MegaRAID SAS Driver Module not installed or IN-Kernel Driver used if driver installed." >> ./$fileName/modinfo.txt
#		fi

	vmkload_mod -l > ./$fileName/vmkload_mod-l.txt 2>> ./misc_output.txt
	vmkload_mod -b > ./$fileName/vmkload_mod-b.txt 2>> ./misc_output.txt
	
	if [ -d /proc/vmware ] ; then
		for x in watchpoints vmkstor version uptime timers thermmon stats procstats pci mem log intr-tracker interrupts cpuinfo chipset ; do 
			if [ -f /proc/vmware/$x ] ; then
				cat /proc/vmware/$x > ./$fileName/proc/vmware-$x 2>> ./misc_output.txt
			fi
		done
	fi

	if [ -d /proc/vmware/config/Disk ] ; then
		for x in  UseReportLUN UseLunReset UseDeviceReset ThroughputCap SupportSparseLUN SPCmdsToSwitch SPBlksToSwitch SharesNormal SharesLow SharesHigh SectorMaxDiff SchedQuantum SchedQControlVMSwitches SchedQControlSeqReqs SchedNumReqOutstanding SANDevicesWithAPFailover RetryUnitAttention ResetThreadMin ResetThreadMax ResetThreadExpires ResetPeriod ResetOverdueLogPeriod ResetOnFailover ResetMaxRetries ResetLatency PreventVMFSOverwrite PathEvalTime MaxVCNotReadyTime MaxResetLatency MaxLUN MaxDS400NotReadyTime MaskLUNs EnableNaviReg DumpMaxRetries DiskMaxIOSize DelayOnBusy BandwidthCap ; do 
			if [ -f /proc/vmware/config/Disk/$x ] ; then
				cat /proc/vmware/config/Disk/$x > ./$fileName/proc/vmware-config-Disk-$x 2>> ./misc_output.txt
			fi
		done
	fi
	
	if [ -d /proc/vmware/config/Scsi ] ; then
		for x in TimeoutTMThreadRetry TimeoutTMThreadMin TimeoutTMThreadMax TimeoutTMThreadLatency TimeoutTMThreadExpires SCSITimeout_ScanTime SCSITimeout_ReabortTime ScsiRestartStalledQueueLatency ScsiRescanAllHbas ScanOnDriverLoad ReserveBacktrace PrintCmdErrors PassthroughLocking MaxReserveTotalTime MaxReserveTime MaxReserveBacktrace LogMultiPath LogAborts ConflictRetries CompareLUNNumber ; do 
			if [ -f /proc/vmware/config/Scsi/$x ] ; then
				cat /proc/vmware/config/Scsi/$x > ./$fileName/proc/vmware-config-Scsi-$x 2>> ./misc_output.txt
			fi
		done
	fi
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		echo "#All lines from all vmkernel files (vmkernel, vmkernel.0-20(.gz), vmkernel-????????(.gz) with '3w-9xxx or 3ware' in the line#" > ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
	fi

	echo "#All lines from all vmkernel files (vmkernel, vmkernel.0-20(.gz), vmkernel-????????(.gz) with 'SCSI or Scsi' in the line#" > ./$fileName/OS_Disk_driver_messages.txt
	
	#Make sure zipped vmkernel don't overwrite unzipped.
	
	for i in l.20 l.19 l.18 l.17 l.16 l.15 l.14 l.13 l.12 l.11 l.10 l.9 l.8 l.7 l.6 l.5 l.4 l.3 l.2 l.1 l.0 s; do
		if [ -f ./$fileName/vmkerne$i ] ; then 
			if [ -f  ./$fileName/vmkerne$i.gz ] ; then mv ./$fileName/vmkerne$i.gz ./$fileName/vmkerne$i.gz.dupe_name ; fi
		fi
	done
	
	#Checks for compressed/dated vmkernel files.
	
	gunzip ./$fileName/vmkernel-????????.gz 2>> ./misc_output.txt
	ls ./$fileName/vmkernel-???????? > ./$fileName/script_workspace/messages_dated.txt 2>> ./misc_output.txt
	for i in $( cat ./$fileName/script_workspace/messages_dated.txt ) ; do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			grep "3w-9xxx\|3ware" $i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
		fi
		grep "SCSI\|Scsi" $i >> ./$fileName/OS_Disk_driver_messages.txt
	done
	
	
	for i in l.20 l.19 l.18 l.17 l.16 l.15 l.14 l.13 l.12 l.11 l.10 l.9 l.8 l.7 l.6 l.5 l.4 l.3 l.2 l.1 l.0 s; do
		if [ -f  ./$fileName/vmkerne$i.gz ] ; then gunzip ./$fileName/vmkerne$i.gz ; fi
	done
	
	
	for i in l.20 l.19 l.18 l.17 l.16 l.15 l.14 l.13 l.12 l.11 l.10 l.9 l.8 l.7 l.6 l.5 l.4 l.3 l.2 l.1 l.0 s; do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			if [ -f ./$fileName/vmkerne$i ] ; then $grep "3w-9xxx\|3ware" ./$fileName/vmkerne$i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt ; fi
		fi
		if [ -f ./$fileName/vmkerne$i ] ; then $grep "SCSI\|Scsi" ./$fileName/vmkerne$i >> ./$fileName/OS_Disk_driver_messages.txt ; fi
	done
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		grep -i " (0x03:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x03_9000_ERROR.txt 
		grep -i " (0x04:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt 
		grep -i " (0x06:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x06_9000_DRIVER.txt 
		grep -i " (0x09:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x09_LINUX_OS.txt 
		grep -i " (0x0B:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0B_API.txt 
		grep -i " (0x0C:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0C_3DMPLUS.txt 
		grep -i " (0x0D:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0D_CLI.txt 
		grep -i " (0x0E:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0E_7000_ERROR.txt 
		grep -i " (0x0F:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0F_7000_EVENT.txt 
		grep -i " (0x11:0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_LINUX_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ; fi
		done
		
		grep -i " (0x03:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x03_9000_ERROR.txt 
		grep -i " (0x04:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x04_9000_EVENT.txt 
		grep -i " (0x06:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_9000_DRIVER.txt 
		grep -i " (0x09:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x09_LINUX_OS.txt 
		grep -i " (0x0B:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0B_API.txt 
		grep -i " (0x0C:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0C_3DMPLUS.txt 
		grep -i " (0x0D:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0D_CLI.txt 
		grep -i " (0x0E:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0E_7000_ERROR.txt 
		grep -i " (0x0F:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0F_7000_EVENT.txt 
		grep -i " (0x11:0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_LINUX_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
			if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ; fi
		done
	fi
	

	if [ "$VMWARE_5x" = "YES" ]||[ "$VMWARE_6x" = "YES" ] ; then 
	echo $fileName >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".....................................esxcli hardware cpu list......................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware cpu list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt 
	#cho ".................................................||................................................."
	echo "...................................esxcli hardware ipmi fru list...................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware ipmi fru list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...................................esxcli hardware ipmi sdr list...................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware ipmi sdr list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...................................esxcli hardware ipmi sel list...................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware ipmi sel list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "..................................esxcli hardware bootdevice list..................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware bootdevice list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".....................................esxcli hardware clock get......................................" >> ./$fileName/esxcli_output.txt
	esxcli hardware clock get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".....................................esxcli hardware memory get....................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware memory get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "......................................esxcli hardware pci list......................................" >> ./$fileName/esxcli_output.txt
	esxcli hardware pci list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "....................................esxcli hardware platform get...................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware platform get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...................................esxcli hardware trustedboot get.................................." >> ./$fileName/esxcli_output.txt
	esxcli hardware trustedboot get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".......................................esxcli network ip get........................................" >> ./$fileName/esxcli_output.txt		
	esxcli network ip get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "....................................esxcli network ip connection...................................." >> ./$fileName/esxcli_output.txt
	esxcli network ip connection 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo "......................................esxcli software vib get......................................." >> ./$fileName/esxcli_output.txt		
	esxcli software vib get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".....................................esxcli software vib list......................................." >> ./$fileName/esxcli_output.txt
	esxcli software vib list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".................................esxcli software vib list | grep LSI................................" >> ./$fileName/esxcli_output.txt
	esxcli software vib list | grep LSI 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".................................esxcli software vib list | grep mpt................................" >> ./$fileName/esxcli_output.txt
	esxcli software vib list | grep mpt 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".................................esxcli software vib list | grep 3w................................." >> ./$fileName/esxcli_output.txt
	esxcli software vib list | grep 3w 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo "................................esxcli storage core adapter stats get..............................." >> ./$fileName/esxcli_output.txt		
	esxcli storage core adapter stats get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...................................esxcli storage core device list.................................." >> ./$fileName/esxcli_output.txt
	esxcli storage core device list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "..............................esxcli storage core device vaai status get............................" >> ./$fileName/esxcli_output.txt
	esxcli storage core device vaai status get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "..............................esxcli storage core device detached list.............................." >> ./$fileName/esxcli_output.txt
	esxcli storage core device detached list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".............................esxcli storage core device partition list.............................." >> ./$fileName/esxcli_output.txt
	esxcli storage core device partition list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "................................esxcli storage core device stats get................................" >> ./$fileName/esxcli_output.txt
	esxcli storage core device stats get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...............................esxcli storage core device world list................................" >> ./$fileName/esxcli_output.txt
	esxcli storage core device world list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".....................................esxcli system module list......................................" >> ./$fileName/esxcli_output.txt	
	esxcli system module list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "..............................esxcli system process stats running get..............................." >> ./$fileName/esxcli_output.txt
	esxcli system process stats running get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "....................................esxcli system process list......................................" >> ./$fileName/esxcli_output.txt
	esxcli system process list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "...................................esxcli system stats uptime get..................................." >> ./$fileName/esxcli_output.txt
	esxcli system stats uptime get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".....................................esxcli system hostname get....................................." >> ./$fileName/esxcli_output.txt
	esxcli system hostname get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".......................................esxcli system time get......................................." >> ./$fileName/esxcli_output.txt
	esxcli system time get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo ".......................................esxcli system uuid get......................................." >> ./$fileName/esxcli_output.txt
	esxcli system uuid get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "......................................esxcli system version get....................................." >> ./$fileName/esxcli_output.txt
	esxcli system version get 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	fi

	if [ "$VMWARE_4x" = "YES" ] ; then 
	echo $fileName >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".................................esxcli corestorage claimrule list.................................." >> ./$fileName/esxcli_output.txt
	esxcli corestorage claimrule list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt 
	#cho ".................................................||................................................."
	echo "...................................esxcli corestorage device list..................................." >> ./$fileName/esxcli_output.txt
	esxcli corestorage device list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo "...................................esxcli network connection list..................................." >> ./$fileName/esxcli_output.txt		
	esxcli network connection list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".......................................esxcli nmp device list......................................." >> ./$fileName/esxcli_output.txt		
	esxcli nmp device list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/esxcli_output.txt	
	echo ".........................................esxcli vms vm list........................................." >> ./$fileName/esxcli_output.txt		
	esxcli vms vm list 2>> ./misc_output.txt >> ./$fileName/esxcli_output.txt
	fi
	
# Final fi
fi
#
# End VMWare Only Section
#

###########################################################################################################################
#Start FreeBSD Only Section
###########################################################################################################################

if [ "$OS_LSI" = "freebsd" ] ; then 

	devinfo -r > ./$fileName/devinfo-r.txt 2>> ./misc_output.txt
	devinfo -ru > ./$fileName/devinfo-ru.txt 2>> ./misc_output.txt
	devinfo -rv > ./$fileName/devinfo-rv.txt 2>> ./misc_output.txt
	devinfo -ruv > ./$fileName/devinfo-ruv.txt 2>> ./misc_output.txt
	pciconf -l -cv > ./$fileName/pciconf-l-cv.txt 2>> ./misc_output.txt
	# kldstat like modinfo, kldload & kldunload are like modload & mod unload
	kldstat -v > ./$fileName/kldstat-v.txt 2>> ./misc_output.txt
	camcontrol devlist  > ./$fileName/camcontrol_inquiry.txt 2>> ./misc_output.txt
	pkg_info > ./$fileName/pkg_info.txt 2>> ./misc_output.txt
	lsvfs > ./$fileName/lsvfs.txt 2>> ./misc_output.txt
	
	df | $grep dev/ | cut -d "/" -f 3 | cut -d " " -f 1 > ./$fileName/script_workspace/df.out
	for i in $( cat ./$fileName/script_workspace/df.out );do
		dumpfs -m /dev/$i > ./$fileName/dumpfs-m_$i.txt
	done
	
	
	
	#
	#Add additional SW Raid data gathering lines
	#
	

	#Can be used for Generic Performance Info, no doing TWGETSKIP3WARE
	#All 3ware Performance Tuning parameters should be gathered here!
	#
	echo "Collecting System Level Disk Performance Tuning Data..."
	#cho ".................................................||................................................."
	echo "...................................................................................................." >> ./$fileName/Perf_Tuning.txt
	
	echo "http://mycusthelp.info/LSI/_cs/AnswerDetail.aspx?inc=7008" >> ./$fileName/Perf_Tuning.txt
	echo "http://mycusthelp.info/LSI/_cs/AnswerDetail.aspx?inc=6456" >> ./$fileName/Perf_Tuning.txt
	echo "These are some parameters that you can tune under FreeBSD." >> ./$fileName/Perf_Tuning.txt
	echo  >> ./$fileName/Perf_Tuning.txt
	echo "Read ahead caching:" >> ./$fileName/Perf_Tuning.txt
	echo "Current: sysctl vfs.read_max (default is 8)" >> ./$fileName/Perf_Tuning.txt
	echo "Recommended: sysctl vfs.read_max=256" >> ./$fileName/Perf_Tuning.txt 
	echo  >> ./$fileName/Perf_Tuning.txt
	echo "Write caching:" >> ./$fileName/Perf_Tuning.txt
	echo "Current: sysctl vfs.write_behind (default is 1)" >> ./$fileName/Perf_Tuning.txt
	echo "Recommended: sysctl vfs.write_behind=1" >> ./$fileName/Perf_Tuning.txt
	echo >> ./$fileName/Perf_Tuning.txt
	echo "Bytes Outstanding before Write Cache Flush:" >> ./$fileName/Perf_Tuning.txt
	echo "Current: sysctl vfs.hirunningspace" >> ./$fileName/Perf_Tuning.txt
	echo "Recommended: sysctl vfs.hirunningspace=1048576" >> ./$fileName/Perf_Tuning.txt
	echo >> ./$fileName/Perf_Tuning.txt
	echo "You will need to experiment with different values to" >> ./$fileName/Perf_Tuning.txt
	echo "see what yields the best performance for you in your application." >> ./$fileName/Perf_Tuning.txt
	echo "...................................................................................................." >> ./$fileName/Perf_Tuning.txt
	if [ -f  ./$fileName/sysctl-ad.txt ] ; then 
		echo "Current settings..." >> ./$fileName/Perf_Tuning.txt
		grep vfs.read_max ./$fileName/sysctl-ad.txt >> ./$fileName/Perf_Tuning.txt
		grep vfs.read_max ./$fileName/sysctl-a.txt >> ./$fileName/Perf_Tuning.txt
		grep vfs.write_behind ./$fileName/sysctl-ad.txt >> ./$fileName/Perf_Tuning.txt
		grep vfs.write_behind ./$fileName/sysctl-a.txt >> ./$fileName/Perf_Tuning.txt
		grep vfs.hirunningspace ./$fileName/sysctl-ad.txt >> ./$fileName/Perf_Tuning.txt
		grep vfs.hirunningspace ./$fileName/sysctl-a.txt >> ./$fileName/Perf_Tuning.txt
	fi

	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.bz2), messages-????????(.bz2) with 'kernel: twa' in the line#" > ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
	fi

	echo "#All lines from all message files (messages, messages.0-200(.bz2), messages-????????(.bz2) with 'kernel: da' in the line#" > ./$fileName/OS_Disk_driver_messages.txt
	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		#echo "#All lines from all message files (messages, messages.0-200(.bz2), messages-????????(.bz2) with 'MR_MONITOR' in the line#" > ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
		#echo "#All lines from all message files (messages, messages.0-200(.bz2), messages-????????(.bz2) with 'MR_MONITOR' OR 'kernel: mfi' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt	
		echo "#All lines from all message files (messages, messages.0-200(.bz2), messages-????????(.bz2) with 'kernel: mfi' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
	fi
	if [ "$TWGETSKIPHBA" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'kernel: mpt' in the line#" > ./$fileName/LSI_Products/HBA/hba_driver_messages.txt
	fi
	

	
	
	#Make sure zipped messages don't overwrite unzipped.
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f ./$fileName/var_log/message$i ] ; then 
			if [ -f  ./$fileName/var_log/message$i.bz2 ] ; then mv ./$fileName/var_log/message$i.bz2 ./$fileName/var_log/message$i.bz2.dupe_name ; fi
		fi
	done
	
	#Checks for compressed/dated message files.
	
	bzip2 -d ./$fileName/messages-????????.bz2 2>> ./misc_output.txt
	ls ./$fileName/messages-???????? > ./$fileName/script_workspace/messages_dated.txt 2>> ./misc_output.txt
	for i in $( cat ./$fileName/script_workspace/messages_dated.txt );do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			grep "kernel: twa" $i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			#grep "MR_MONITOR" $i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
			grep "kernel: mfi" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
			#egrep "MR_MONITOR|kernel: mfi" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt
		fi
		if [ "$TWGETSKIPHBA" != "YES" ] ; then
			grep "kernel: mpt" $i >> ./$fileName/LSI_Products/HBA/hba_driver_messages.txt
		fi
		grep "kernel: da" $i >> ./$fileName/OS_Disk_driver_messages.txt
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f  ./$fileName/var_log/message$i.bz2 ] ; then bzip2 -d ./$fileName/var_log/message$i.bz2 ; fi
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "kernel: twa" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt ; fi
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			#if [ -f ./$fileName/var_log/message$i ] ; then $grep "MR_MONITOR" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt ; fi
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "kernel: mfi" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt ; fi
			#if [ -f ./$fileName/var_log/message$i ] ; then egrep "MR_MONITOR|kernel: mfi" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt ; fi
		fi
		if [ "$TWGETSKIPHBA" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "kernel: mpt" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/HBA/hba_driver_messages.txt ; fi
		fi
		if [ -f ./$fileName/var_log/message$i ] ; then $grep "kernel: da" ./$fileName/var_log/message$i >> ./$fileName/OS_Disk_driver_messages.txt ; fi
	done
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		grep " (0x03: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x03_9000_ERROR.txt 
		grep " (0x04: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt 
		grep " (0x06: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x06_9000_DRIVER.txt 
		grep " (0x09: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x09_FreeBSD_OS.txt 
		grep " (0x0B: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0B_API.txt 
		grep " (0x0C: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0C_3DMPLUS.txt 
		grep " (0x0D: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0D_CLI.txt 
		grep " (0x0E: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0E_7000_ERROR.txt 
		grep " (0x0F: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x0F_7000_EVENT.txt 
		grep " (0x11: 0x" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_FreeBSD_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ; fi
		done
		
		grep " (0x03: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x03_9000_ERROR.txt 
		grep " (0x04: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x04_9000_EVENT.txt 
		grep " (0x06: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_9000_DRIVER.txt 
		grep " (0x09: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x09_FreeBSD_OS.txt 
		grep " (0x0B: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0B_API.txt 
		grep " (0x0C: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0C_3DMPLUS.txt 
		grep " (0x0D: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0D_CLI.txt 
		grep " (0x0E: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0E_7000_ERROR.txt 
		grep " (0x0F: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x0F_7000_EVENT.txt 
		grep " (0x11: 0x" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x11_7000_DRIVER.txt  
		
		for i in _x03_9000_ERROR _x04_9000_EVENT _x06_9000_DRIVER _x09_FreeBSD_OS _x0B_API _x0C_3DMPLUS _x0D_CLI _x0E_7000_ERROR _x0F_7000_EVENT _x11_7000_DRIVER; do
		if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ; fi
		done
	fi
	
	
# Final fi
fi
#
# End FreeBSD Only Section
#

if [ "$TWGETSKIP3WARE" != "YES" ] ; then
###########################################################################################################################
#Start MacOS Only Section
###########################################################################################################################


	if [ "$OS_LSI" = "macos" ] ; then 
	
	
		system_profiler -detaillevel full > ./$fileName/system_profile.txt 2>> ./misc_output.txt
		grep "System Version" ./$fileName/system_profile.txt > ./$fileName/Mac_OS_X_Version.txt 2>> ./misc_output.txt
		grep "Kernel Version" ./$fileName/system_profile.txt >> ./$fileName/Mac_OS_X_Version.txt 2>> ./misc_output.txt
		
		
		kextstat > ./$fileName/kextstat.txt 2>> ./misc_output.txt
		kextstat | $grep amcc > ./$fileName/LSI_Products/3ware/kextstat_amcc.txt 2>> ./misc_output.txt
		kextstat | $grep 3ware > ./$fileName/LSI_Products/3ware/kextstat_3ware.txt 2>> ./misc_output.txt
		

		
		#Add new logs as needed
		
		
		echo "#All lines from all system.log files (system.log, system.log.0-20(.gz)) with 'AMCC3ware9000' OR 'LSI3ware9000' in the line#" > ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
		echo "#All lines from all system.log files (system.log, system.log.0-20(.gz)) with 'diskarbitrationd' in the line#" > ./$fileName/OS_Disk_driver_messages.txt
		
		
		#Make sure zipped messages don't overwrite unzipped.
		
		for i in g.20 g.19 g.18 g.17 g.16 g.15 g.14 g.13 g.12 g.11 g.10 g.9 g.8 g.7 g.6 g.5 g.4 g.3 g.2 g.1 g.0 g; do
			if [ -f ./$fileName/system.lo$i ] ; then 
				if [ -f  ./$fileName/system.lo$i.gz ] ; then mv ./$fileName/system.lo$i.gz ./$fileName/system.lo$i.gz.dupe_name ; fi
			fi
		done
	
		for i in g.20 g.19 g.18 g.17 g.16 g.15 g.14 g.13 g.12 g.11 g.10 g.9 g.8 g.7 g.6 g.5 g.4 g.3 g.2 g.1 g.0 g; do
			if [ -f  ./$fileName/system.lo$i.gz ] ; then gunzip ./$fileName/system.lo$i.gz ; fi
		done
	
	
		for i in g.20 g.19 g.18 g.17 g.16 g.15 g.14 g.13 g.12 g.11 g.10 g.9 g.8 g.7 g.6 g.5 g.4 g.3 g.2 g.1 g.0 g; do
			if [ -f ./$fileName/system.lo$i ] ; then $grep "AMCC3ware9000" ./$fileName/system.lo$i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt ; 	fi
			if [ -f ./$fileName/system.lo$i ] ; then $grep "diskarbitrationd" ./$fileName/system.lo$i >> ./$fileName/OS_Disk_driver_messages.txt ; fi
		done
	
		egrep "AMCC3ware9000 AEN|LSI3ware9000 AEN" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt 
		
		for i in _x04_9000_EVENT; do
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver$i.txt >> ./misc_output.txt 2>&1 ; fi
		done
		
		grep "diskarbitrationd" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x04_9000_EVENT.txt 
		
		for i in _x04_9000_EVENT; do
			if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt >> ./misc_output.txt 2>&1 ; fi
		done
		
		
		
# Final fi
	fi
#
# End MacOS Only Section
#
fi

###########################################################################################################################
#Start Solaris Only Section
###########################################################################################################################

if [ "$OS_LSI" = "solaris" ] ; then 

	uptime > ./$fileName/uptime.txt 2>> ./misc_output.txt
	df -ha > ./$fileName/df-ha.txt 2>> ./misc_output.txt
	who > ./$fileName/who.txt  2>> ./misc_output.txt
	top -b -n 1 > ./$fileName/top-b-n1.txt 2>> ./misc_output.txt
	ps -ef > ./$fileName/ps-ef.txt 2>> ./misc_output.txt
	
	
	#
	# All 3ware Performance Tuning parameters should be gathered here!
	#
	# echo "Collecting 3ware SD Device Performance Tuning Data..."
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with ': tw' in the line#" > ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
	fi	
	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with ': mega_sas' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'MR_MONITOR' in the line#" > ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
		echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with 'MR_MONITOR' OR ': mega_sas' in the line#" > ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt
	fi
	echo "#All lines from all message files (messages, messages.0-200(.gz), messages-????????(.gz) with '] sd' in the line#" > ./$fileName/OS_Disk_driver_messages.txt

	
	
	
	
	
	
	#Make sure zipped messages don't overwrite unzipped.
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f ./$fileName/var_log/message$i ] ; then 
			if [ -f  ./$fileName/var_log/message$i.gz ] ; then mv ./$fileName/var_log/message$i.gz ./$fileName/var_log/message$i.gz.dupe_name ; fi
		fi
	done

#Checks for compressed/dated message files.

	gunzip ./$fileName/messages-????????.gz 2>> ./misc_output.txt
	ls ./$fileName/messages-???????? > ./$fileName/script_workspace/messages_dated.txt 2>> ./misc_output.txt
	for i in $( cat ./$fileName/script_workspace/messages_dated.txt );do
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		grep ": tw" $i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt
	fi
	if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
		grep ": mega_sas" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt
		grep "MR_MONITOR" $i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt
		egrep "MR_MONITOR|: mega_sas" $i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt
	fi
	grep "] sd" $i >> ./$fileName/OS_Disk_driver_messages.txt
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ -f  ./$fileName/var_log/message$i.gz ] ; then gunzip ./$fileName/var_log/message$i.gz ; fi
	done
	
	
	for i in s.200 s.199 s.198 s.197 s.196 s.195 s.194 s.193 s.192 s.191 s.190 s.189 s.188 s.187 s.186 s.185 s.184 s.183 s.182 s.181 s.180 s.179 s.178 s.177 s.176 s.175 s.174 s.173 s.172 s.171 s.170 s.169 s.168 s.167 s.166 s.165 s.164 s.163 s.162 s.161 s.160 s.159 s.158 s.157 s.156 s.155 s.154 s.153 d.152 s.151 s.150 s.149 s.148 s.147 s.146 s.145 s.144 s.143 s.142 s.141 s.140 s.139 s.138 s.137 s.136 s.135 s.134 s.133 s.132 s.131 s.130 s.129 s.128 s.127 s.126 s.125 s.124 s.123 s.122 s.121 s.120 s.119 s.118 s.117 s.116 s.115 s.114 s.113 s.112 s.111 s.110 s.109 s.108 s.107 s.106 s.105 s.104 s.103 s.102 s.101 s.100 s.99 s.98 s.97 s.96 s.95 s.94 s.93 s.92 s.91 s.90 s.89 s.88 s.87 s.86 s.85 s.84 s.83 s.82 s.81 s.80 s.79 s.78 s.77 s.76 s.75 s.74 s.73 s.72 s.71 s.70 s.69 s.68 s.67 s.66 s.65 s.64 s.63 s.62 s.61 s.60 s.59 s.58 s.57 s.56 s.55 s.54 s.53 d.52 s.51 s.50 s.49 s.48 s.47 s.46 s.45 s.44 s.43 s.42 s.41 s.40 s.39 s.38 s.37 s.36 s.35 s.34 s.33 s.32 s.31 s.30 s.29 s.28 s.27 s.26 s.25 s.24 s.23 s.22 s.21 s.20 s.19 s.18 s.17 s.16 s.15 s.14 s.13 s.12 s.11 s.10 s.9 s.8 s.7 s.6 s.5 s.4 s.3 s.2 s.1 s.0 s; do
		if [ "$TWGETSKIP3WARE" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep ": tw" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt ; fi
		fi
		if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
			if [ -f ./$fileName/var_log/message$i ] ; then $grep ": mega_sas" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_driver_messages.txt ; fi
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "MR_MONITOR" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/AENs/mrmonitord_messages.txt ; fi
			if [ -f ./$fileName/var_log/message$i ] ; then $grep "MR_MONITOR|: mega_sas" ./$fileName/var_log/message$i >> ./$fileName/LSI_Products/MegaRAID/megaraid_messages.txt ; fi
		fi
		if [ -f ./$fileName/var_log/message$i ] ; then $grep "] sd" ./$fileName/var_log/message$i >> ./$fileName/OS_Disk_driver_messages.txt ; fi
	done
	
	if [ "$TWGETSKIP3WARE" != "YES" ] ; then
		grep "tw_aen_task AEN" ./$fileName/LSI_Products/3ware/3ware_driver_messages.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt 
			
	
		for i in _x04_9000_EVENT; do
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver$i.txt >> ./misc_output.txt 2>&1 ; fi
		done
		
		grep "] sd" ./$fileName/OS_Disk_driver_messages.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x04_9000_EVENT.txt 
		
		
		for i in _x04_9000_EVENT; do
			if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver$i.txt >> ./misc_output.txt 2>&1 ; fi
		done
	fi
	
			
	
# Final fi
fi
#
# End Solaris Only Section
#

###########################################################################################################################
###########################################################################################################################
#Start Generic Section
###########################################################################################################################
###########################################################################################################################




if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	########################################################################################################################### 
	###Update on Code Set Change
	# Based on errorcode.h 5.12.00.016FW 10.2.2.1 codeset 
	# Errors = 0x03 
	# Legacy error_codes, these are reserved on Apache 
	########################################################################################################################### 


	x0000="no_error_status_dependent_code_for_this_error_status" 
	x0001="no_request_ID_available_for_this_error_status" 
	x0002="CP_queue_became_full" 
	x0003="illegal_SGL_offset_in_CP_Header" 
	x0004="illegal_number_of_SGL_entries_in_CPH" 
	x0005="could_not_allocate_additional_memory" 
	x0006="some_PCI_read_error_occurred" 
	x0007="timeout_during_PCI_transaction" 
	x0008="PCI_ERR_bit_for_a_PCI_transaction" 
	x0009="unrecoverable_disk_error" 
	x000A="completion_token_queue_overflow" 
	x000B="error_reading_SGL" 
	x000C="error_reading_CP_Header" 
	x000D="abort_req_for_cmd_that_wasn't_active" 
	x000E="illegal_size_in_CP_Header" 
	x000F="got_a_CPH_with_an_already_active_req_ID" 
	x0010="lengths_in_SGLs_not_match_block_count_in_CP" 
	x0011="requested_LBA_greater_than_maximum_LBA_of_the_unit" 
	x0012="host_address_or_SGL_size_not_on_8_dw_boundary" 
	x0013="data_integrity_error_on_read_or_write_test" 
	x0014="subcmd_num_in_CP_hdr_is_undef_for_this_cmd" 
	x0015="undefined_table_requested" 
	x0016="param_requested_is_out_of_bounds_of_table" 
	x0017="given_param_size_doesn't_match_param_size_in_table" 
	x0018="host_address_or_SGL_size_not_on_a_sector_boundary" 
	x0019="Achip_Unit__number_exceeds_maximum" 
	x001A="bad_pairing_of_disk-op_or_xer-op_from_Aop_queue" 
	x001B="Aport_timed_out_doing_an_Aop" 
	x001C="an_Achip_GSR_interrupt_occurred" 
	x001D="Aop_encountered_with_unknown_cmd_byte" 
	x001E="PCI_was_busy_when_tried_to_do_a_PCI_read_or_write" 
	x001F="next_Aop_should_have_been_a_disk-op_but_it_wasnt" 
	x0020="param_0_unsupported_for_now_for_get_or_set_param" 
	x0021="an_unimplemented_method_was_invoked" 
	x0022="request_ID_took_too_long_to_complete" 
	x0023="a_disk_task_file_error_occurred_during_a_1F_test" 
	x0024="data_lines_shorted_in_Sbuf_RAM" 
	x0025="data_lines_open_in_Sbuf_RAM" 
	x0026="addr_line_problem_in_Sbuf_RAM" 
	x0027="Sbuf_RAM_unreadable" 
	x0028="command_requires_at_least_1_SGL" 
	x0029="no_unit_num_is_available_for_use" 
	x002A="CP_queue_was_empty_when_tried_to_get_a_CP_ptr_from_it" 
	x002B="test_firmware_not_downloaded_to_RAM" 
	x002C="an_attempt_was_made_to_nest_internal_requests" 
	x002D="error_in_downloading_hex_file" 
	x002E="error_in_programming_the_flash_ROM" 
	x002F="error_in_rollcall" 
	x0030="error_in_wait_disks_rdy" 
	x0031="error_in_UIT_formatting" 
	x0032="incorrect_unit_type_for_the_request" 
	x0033="unit_does_not_have_the_logical_sub-unit_specified" 
	x0034="unit_has_corrupted_data_on_it" 
	x0035="could_not_write_DCB_to_disk" 
	x0036="could_not_get_profiler_from_disk" 
	x0037="unit_is_not_exportable" 
	x0038="unit_is_missing_a_sub-unit" 
	x0039="unit_is_not_operating_normally" 
	x003A="not_enough_SBUF_segments_to_service_the_request" 
	x003B="User_area_in_unit_was_not_written_with_zeroes" 
	x003C="user_data_on_unit_did_not_verify" 
	x003D="error_while_writing_0s_to_user_data" 
	x003E="a_unit_does_not_contain_a_logical_mapping_for_a_phys_drive" 
	x003F="drive_replacement_would_cause_a_double-degrade" 
	x0040="capacity_of_replacement_drive_is_too_small" 
	x0041="no_drive_detected" 
	x0042="drive_detected_to_be_busy" 
	x0043="aport_unavailable" 
	x0044="unable_to_clear_sbuf" 
	x0045="can_not_replace_drive_because_unit_not_degraded" 
	x0046="we_have_no_routine_to_program_this_mfr_or_type_of_flash" 
	x0047="cant_fill_Sbuf_with_zeros" 
	x0048="byte_count_in_PARAM_field_is_too_big_for_this_cmd" 
	x0049="timeout_while_waiting_for_data_from_display_panel" 
	x004A="CRC_error_reported_on_BFB_transfer" 
	x0051="for_drive_errors_the_status_register_gets_stuffed_into_LSB_of_ESDC" 
	x0060="reserved_-_this_is_a_valid_drive_status_error_code" 
	x0061="reserved_-_this_is_a_valid_drive_status_error_code" 
	
	x0100="SGL_entry_contains_zero_data" 
	x0101="Invalid_command_opcode" 
	x0102="SGL_entry_has_unaligned_address" 
	x0103="SGL_size_does_not_match_command" 
	x0104="SGL_entry_has_illegal_length" 
	x0105="Command_packet_is_not_aligned" 
	x0106="Invalid_request_ID" 
	x0107="Duplicate_request_ID" 
	x0108="ID_not_locked" 
	x0109="LBA_out_of_range" 
	x010A="Logical_unit_not_present" 
	x010B="Parameter_table_does_not_exist" 
	x010C="Parameter_index_does_not_exist" 
	x010D="Invalid_field_in_CDB" 
	x010E="Invalid_operation_for_specified_port" 
	x010F="Parameter_item_size_mismatch" 
	
	x0110="Failed_memory_allocation" 
	x0111="Memory_request_too_large" 
	x0112="Out_of_memory_segments" 
	x0113="Invalid_address_to_deallocate" 
	x0114="Out_of_memory" 
	x0115="Out_of_heap" 
	x0116="Invalid_BIOS_buffer_id" 
	
	x0117="Host_lock_not_available" 
	
	x011E="Unrecovered_Read_Error"
	x011F="Recovered_Data_with_error_correction_applied"
	
	x0120="Double_degrade" 
	x0121="Drive_not_degraded" 
	x0122="Reconstruct_error" 
	x0123="Replace_not_accepted" 
	x0124="Drive_capacity_too_small" 
	x0125="Sector_count_not_allowed" 
	x0126="No_spares_left" 
	x0127="Reconstruct_error" 
	x0128="Unit_offline" 
	x0129="Cannot_update_status_to_DCB" 
	x012A="Invalid_configuration_for_split" 
	x012B="Invalid_configuration_for_join" 
	x012C="No_migration_recovery" 
	x012D="No_SATA_spares" 
	x012E="No_SAS_spares"  
	x012F="Mixed_SAS_SATA_not_allowed_in_same_unit" 
	
	x0130="Invalid_stripe_handle" 
	x0131="Handle_that_was_not_locked" 
	x0132="Handle_that_was_not_empty" 
	x0133="Handle_has_different_owner" 
	
	x0140="IPR_has_parent" 
	
	x0150="Illegal_Pbuf_address_alignment" 
	x0151="Illegal_Pbuf_transfer_length" 
	x0152="Illegal_Sbuf_address_alignment" 
	x0153="Illegal_Sbuf_transfer_length" 
	
	x0160="Command_packet_too_large" 
	x0161="SGL_exceeds_maximum_length" 
	x0162="SGL_has_too_many_entries" 
	
	x0170="Insufficient_resources_for_rebuilder" 
	x0171="Verify_error_data_doesn't_equal_parity" 
	
	x0180="Requested_segment_not_in_directory_of_this_DCB" 
	x0181="DCB_segment_has_unsupported_version" 
	x0182="DCB_segment_has_checksum_error" 
	x0183="DCB_support_settings_segment_invalid" 
	x0184="DCB_UDB_unit_descriptor_block_segment_invalid" 
	x0185="DCB_GUID_globally_unique_identifier_segment_invalid" 
	
	x01A0="Could_not_clear_Sbuf" 
	
	x01C0="Flash_device_unsupported" 
	x01C1="Flash_out_of_bounds" 
	x01C2="Flash_write_verify_failed" 
	x01C3="Flash_file_object_not_found" 
	x01C4="Flash_file_already_present" 
	x01C5="Flash_file_system_full" 
	x01C6="Flash_file_not_present" 
	x01C7="Flash_file_size_mismatch" 
	x01C8="Flash_file_checksum_error" 
	x01C9="Flash_file_version_unsupported" 
	x01CA="Flash_file_system_error_detected" 
	x01CB="Flash_file_component_directory_not_found" 
	x01CC="Flash_file_component_not_found" 
	x01CD="Flash_write_cycle_failed" 
	x01CE="Flash_erase_cycle_failed" 
	
	x01D0="Invalid_field_in_parameter_list" 
	x01D1="Parameter_list_length_error" 
	x01D2="Parameter_not_changeable" 
	x01D3="Parameter_not_saveable" 
	x01D4="Invalid_mode_page" 
	
	x0200="Drive_CRC_error" 
	x0201="Internal_bus_CRC_error" 
	x0202="Drive_ECC_Medium_error" 
	x0203="Drive_TFR_readback_error" 
	x0204="Drive_timeout" 
	x0205="Drive_power_on_reset" 
	x0206="ADP_level_2_error" 
	x0207="Drive_soft_reset_failed" 
	x0208="Drive_not_ready" 
	x0209="Unclassified_drive_error" 
	x020A="Drive_aborted_command" 
	x020B="Port_link_error_detected" 
	x020C="Port_internal_error_detected" 
	x020D="Drive_not_ready_require_Spinup" 
	x020E="Uninitialized_drive_handle" 
	
	x0210="Internal_bus_CRC_error" 
	x0211="PCI_bus_abort_error" 
	x0212="PCI_bus_parity_error" 
	x0213="Port_handler_error" 
	x0214="Token_interrupt_count_error" 
	x0215="PCI_bus_timeout" 
	x0216="Buffer_ECC_error_corrected" 
	x0217="Buffer_ECC_error_not_corrected" 
	x0218="Xop_pool_parity_error" 
	
	x0230="Unsupported_command_during_flash_recovery" 
	x0231="Next_image_buffer_expected" 
	x0232="Binary_image_architecture_ID_incompatible" 
	x0233="Binary_image_no_signature_detected" 
	x0234="Binary_image_checksum_error_detected" 
	x0235="Binary_image_buffer_overflow_detected" 
	x0236="Binary_image_SRL_incompatible" 
	
	x0240="I2C_device_not_detected" 
	x0241="I2C_transaction_aborted" 
	x0242="SO-DIMM_parameters_incompatible_using_defaults" 
	x0243="SO-DIMM_unsupported" 
	x0244="I2C_clock_is_held_low_transfer_aborted" 
	x0245="I2C_data_is_held_low_transfer_aborted" 
	x0246="I2C_slave_device_NACKed_the_transfer"
	x0247="I2C_buffer_in-sufficient" 
	x0248="SPI_transfer_status_error" 
	x024A="I2C_interface_is_active" 
	x024B="Lost_arbitration" 
	x024C="I2C_transfer_error" 
	
	x0250="Unit_descriptor_size_invalid" 
	x0251="Unit_descriptor_size_exceeds_data_buffer" 
	x0252="Invalid_value_in_unit_descriptor" 
	x0253="Inadequate_disk_space_to_support_descriptor" 
	x0254="Unable_to_create_data_channel_for_this_unit_descriptor" 
	x0255="Unit_descriptor_specifies_a_drive_already_in_use" 
	x0256="Unable_to_write_configuration_to_all_disks" 
	x0257="Unit_descriptor_version_not_supported" 
	x0258="Invalid_subunit_for_RAID_0_or_5" 
	x0259="Too_many_unit_descriptors" 
	x025A="Invalid_configuration_in_unit_descriptor" 
	x025B="Invalid_LBA_offset_in_unit_descriptor" 
	x025C="Invalid_stripelet_size_in_unit_descriptor" 
	x025D="JBOD_unit_is_not_allowed" 
	x025E="Operation_not_allowed_retained_cache_data" 
	x025F="Exceeded_maximum_number_of_active_drives" 
	x0260="SMART_threshold_exceeded" 
	x0261="Maximum_number_of_units_reached" 
	
	
	x0270="Unit_not_in_NORMAL_state" 
	x0271="Invalid_drive_members" 
	x0272="Converted_unit_not_supported" 
	
	x0280="ResponseIU_status_code_EC_STATUS_BUSY" 
	x0281="ResponseIU_status_code_EC_STATUS_QUEUE_FULL" 
	x0282="ResponseIU_status_code_EC_STATUS_UNEXPECTED" 
	x0283="IO_Hold_Error"
	
	x0290="No_Sense_Info_EC_SK_NO_SENSE_INFO"  
	x0291="Recovered_Error_EC_SK_RECOVERED_ERROR"  
	x0293="Hardware_Error_EC_SK_HARDWARE_ERROR"  
	x0294="Hardware_ECC_Error_EC_SK_HARDWARE_ECC"  
	x0295="Illegal_Req_EC_SK_ILLEGAL_REQ"  
	x0296="Unit_Attention_EC_SK_UNIT_ATTENTION"  
	x0297="Unit_Attention_Reset_EC_SK_UNIT_ATTENTION" 
	# note: The error code below is reused - Commented Out 
	#x0205="ResponseIU_status_codeEC_SK_UNIT_REset" 
	x0298="Aborted_Cmd_EC_SK_ABORTED_CMD"  
	x0299="Sense_Keys_Unexpected_EC_SK_UNEXPECTED"  
	x029A="Unit_Attention_Mode_Page_Changed_EC_SK_UNIT_ATTENTION" 
	x029B="Current_command_Write_Fault"
	x029C="Deferred_Drive_Write_Fault"
	
	x029D="Lba_Out_Of_Range"
	
	x02A0="SAS_error_code_EC_PAYLOAD_PARITY" 
	x02A1="SAS_error_code_EC_UNDER_RUN_IN_RW" 
	x02A2="SAS_error_code_EC_UNDER_RUN_OUT" 
	x02A3="SAS_error_code_EC_OVER_RUN" 
	x02A4="SAS_error_code_EC_OPEN_REJECT_BUSY"  
	x02A5="SAS_error_code_EC_OPEN_REJECT_RETRY"   
	x02A6="SAS_error_code_EC_OPEN_REJECT_ABANDON"   
	x02A7="SAS_error_code_EC_OPEN_REJECT_STP_RES_BUSY"   
	x02A8="SAS_error_code_EC_RX_FRAME_ERROR"   
	x02A9="SAS_error_code_EC_RX_TRANSPORT_ERROR" 
	x02AA="SAS_error_code_EC_PORT_OFFLINE"   
	x02AB="SAS_Response_data_present"   
	
	x02B0="Tx_failure_-_SATA_SYNC_received" 
	x02B1="Tx_failure_-_BREAK_received" 
	x02B2="Protocol_overrun" 
	x02B3="Protocol_underrun" 
	x02B4="Open_failure_-_Connection_rejected" 
	x02B5="Open_failure_-_Bad_destination" 
	x02B6="Open_failure_-_Wrong_destination" 
	x02B7="Open_failure_-_Connection_rate_not_supported" 
	x02B8="Open_failure_-_Protocol_not_supported" 
	x02B9="Open_failure_-_STP_Resources_busy" 
	x02BA="Open_failure_-_No_destination" 
	x02BB="Open_failure_-_Pathway_blocked" 
	x02BC="Open_failure_-_Retry" 
	x02BD="Open_failure_-_Open_frame_timeout" 
	x02BE="STP_Inactivity" 
	x02BF="Failed_to_discover_Emulex_chip" 
	x02C0="Emulex_flash_file_is_corrupted" 
	x02C1="Error_while_flashing_Emulex" 
	x02C2="Target_returned_valid_sense_data_during_a_SCSI_PASSTHROUGH" 
	x02C3="Failed_to_unlock_flash_block_while_flashing_Emulex_ROM" 
	x02C4="Failed_to_erase_flash_block_while_flashing_Emulex_ROM" 
	x02C5="Failed_to_write_flash_block_while_flashing_Emulex_ROM" 
	x02C6="Failed_to_lock_flash_block_while_flashing_Emulex_ROM" 
	x02C7="ROM_size_is_not_a_multiple_of_128k" 
	x02C8="Emulex_SLI_command_timeout" 
	x02C9="Emulex_mailbox_status_error" 
	x02CA="Emulex_mailbox_format_error" 
	x02CB="Emulex_no_resources" 
	x02CC="Emulex_protocol_check_error" 
	x02CD="Emulex_protocol_fis_error" 
	x02CE="IOC_firmware_update_error" 
	
	x02D0="Discovery_module_resource_error" 
	x02D1="Delete_the_port_in_discovery_Manager" 
	x02D2="Discovery_module_bad_pointer_error" 
	x02D3="Discovery_module_unknown_SMP_function" 
	x02D4="Target_Unregistration_with_IOC_failed" 
	
	x02E0="SAS_Error_PAYLOAD_PARITY" 
	x02E1="SAS_Error_UNDER_RUN_IN_RW" 
	x02E4="SAS_Error_OPEN_REJECT_BUSY" 
	x02E5="SAS_Error_OPEN_REJECT_RETRY" 
	x02E6="SAS_Error_OPEN_REJECT_ABANDON" 
	x02E7="SAS_Error_OPEN_REJECT_STP_RES_BUSY" 
	x02E8="SAS_Error_RX_FRAME_ERROR" 
	x02EA="SAS_Error_PORT_OFFLINE" 
	
	x02EB="Suspend_IO_during_PL_TMF" 
	
	x02F0="Tx_failure_-_SATA_R_ERR_received" 
	x02F1="Tx_failure_-_SATA_DMAT_received" 
	x02F2="Non_specific_NCQ_error" 
	x02F3="Task_File_error" 
	x02F4="SATA_Register_Set_error" 
	
	x0300="Internal_errorcode_BBU_base_-_should_not_occur" 
	x0301="Invalid_BBU_state_change_request"  
	x0302="The_BBU_resource_needed_is_in_use_retry_command_after_a_delay"  
	x0303="Command_requires_a_battery_pack_to_be_present_and_enabled" 
	
	x0310="BBU_command_packet_error" 
	x0311="BBU_command_not_implemented" 
	x0312="BBU_command_buffer_underflow" 
	x0313="BBU_command_buffer_overflow" 
	x0314="BBU_command_incomplete" 
	x0315="BBU_command_checksum_error" 
	x0316="BBU_command_timeout" 
	
	x0317="BBU_flash_operation_failed" 
	x0318="BBU_flash_Vpp_voltage_out_of_progamming_range" 
	x0319="BBU_flash_incorrect_command_or_parameter_or_not_enough_space_in_stack" 
	x031A="BBU_flash_not_yet_completed" 
	x031B="BBU_flash_write_skip" 
	x031C="BBU_flash_invalid_erase_sector" 
	
	x0320="BBU_parameter_not_defined" 
	x0321="BBU_parameter_size_mismatch" 
	x0322="Cannot_write_a_read-only_BBU_parameter" 
	x0323="Invalid_state_bits_in_BBU_SetportPins_command" 
	
	x0330="FBU_nif_error"
	x0331="FBU_ERASE_BLOCK_ERROR"
	x0332="FBU_Program_error"
	x0333="FBU_ecc_uncorrectable"
	x0334="FBU_ecc_correctable"
	x0335="FBU_Program_error"
	x0336="FBU_Erase_error"
	x0337="FBU_No_defect_guard"
	x0338="FBU_Read_Error"
	x0339="FBU_Read_Error"
	
	x0350="Uncorrectable_ECC_Error"
	x0351="Correctable_ECC_Error"
	x0352="Access_out_of_memory_range"
	x0353="Dram_Controller_Fatal_Error"
	x0354="Dram_ECC_error_log_full"
	
	x0340="Invalid_discharge-learn_cycle_in_Battery_test" 
	x0341="Battery_test_failed"
	
	x0380="BBU_firmware_version_string_not_found" 
	x0381="BBU_operating_state_not_available" 
	x0382="BBU_not_present" 
	x0383="BBU_not_ready" 
	x0384="BBU_S1_not_compatible_with_HBA" 
	x0385="BBU_S0_not_compatible_with_HBA" 
	x0386="BBU_not_compatible_with_HBA" 
	x0387="BBU_not_in_S0" 
	x0388="BBU_not_in_S1" 
	x0389="Timeout_on_BBU_power_fail_interrupt" 
	x038A="BBU_invalid_response_length" 
	x038B="Not_S1_ident_or_event_packet" 
	x038C="HBA_has_backup_data" 
	x038D="Invalid_BBU_state" 
	x038E="BBU_invalid_response_code" 
	
	x0390="Log_updates_not_allowed" 
	x0391="Logs_are_invalid" 
	x0392="Logs_not_found" 
	
	x0400="Invalid_enclosure_port_defined" 
	x0401="Enclosure_resource_reserved" 
	x0402="Enclosure_parameter_not_defined" 
	x0403="Enclosure_parameter_re-defined" 
	x0404="Enclosure_port_is_input_port" 
	x0405="Invalid_SAFTE_page_requested" 
	x0406="Invalid_SES_page_requested" 
	x0407="EPCT_device_description_error" 
	x0408="EPCT_device_redefined" 
	x0409="EPCT_element_unknown" 
	x040A="EPCT_device_unknown" 
	x040B="EPCT_ID_unknown" 
	x040C="Invalid_enclosure_device" 
	x040D="EPCT_LED_descriptor_redefinition" 
	x040F="Enclosure_device_not_initialized" 
	x0410="Temperature_sensor_reading_unknown" 
	x0411="Enclosure_not_present" 
	x0412="Bad_EPCT_version" 
	x0413="Failed_to_create_SEP_object" 
	x0414="Too_many_enclosures_cannot_add_more" 
	x0415="Failed_to_create_enclosure_object" 
	x0416="Invalid_Sep_Command" 
	
	x1000="EC_NEED_CMD_PROCESS" 
	x1001="EC_UDMA_UPGRADE_SKIPPED" 
	x1002="EC_DRIVE_NOT_IN_UDMA" 
	x1003="EC_OFFLINE_TIMER_RUNNING" 
	x1004="EC_BIN_HANDLE_NOT_EMPTY" 
	x1005="EC_BIN_HANDLE_WRONG_OWNER" 
	x1006="EC_CMD_IN_PBUF" 
	x1007="EC_INVALID_DATA_CHKSUM" 
	x1008="EC_LBA_OVERLAP" 
	x1009="EC_SGL_NON_SECTOR_SIZE" 
	x100A="Retry_CMD"
	
	x100B="BT1680_Liberator_Error"
	x100C="BT1680_Liberator_Fail"
	
	x1010="Error_recovery_in_progress" 
	x1011="Error_recovery_complete" 
	x1012="No_LBA_to_repair_sector" 
	x1013="Retry_recovery_step" 
	x1014="Do_error_action" 
	x1015="Degrade_unit" 
	x1016="Sector_repair_was_not_completed" 
	x1017="Command_aborted" 
	x1018="Drive_added" 
	
	x1019="Drive_removed" 
	x101A="Retry_queued_command" 
	x101B="Drive_error" 
	x101C="Non-Dma_Retry_no_recovery" 
	x101D="Drive_removed_no_wait" 
	x1020="Simulate_power_fail" 
	x1021="Simulate_uC_error" 
	
	x1022="Need_to_transition_to_Drive_remove_error" 
	x1023="Simulate_exception_error" 
	x1024="Simulate_illegal_instruction_error" 
	
	x2000="Checksum_of_cache_meta_data_is_bad" 
	x2001="Signature_of_cache_meta_data_is_bad" 
	x2002="Cache_meta_data_is_bad_due_to_bad_parity_link" 
	
	x2100="SATA_NCQ_Error_to_trigger_error_handler" 
	x2101="SAS_Error_to_trigger_error_handler" 
	x2102="Drive_must_be_reset_either_locally_or_via_SMP"
	
	x2FFF="EC_FEATURE_NOT_IMPLEMENTED" 
	
	x3013="Data_integrity_error_in_diagnostic_test" 
	x3014="Undefined_Sub-command_for_diag_test" 
	x3017="Drive_reset_error_thru_Marvell" 
	x3018="Reading_config_space_error" 
	x3019="Read_or_write_memory_space_data_integrity_error" 
	x301A="Pchip_UCbuf_data_integrity_error" 
	x301B="Pchip_Xop_Pool_data_integrity_error" 
	x301C="Pchip_Cmd_Ram_data_integrity_error" 
	x301E="iHandler_was_busy_before_diag_test" 
	x3021="An_unimplemented_method_was_invoked" 
	x3024="Data_lines_shorted_in_Sbuf_RAM" 
	x3025="Data_lines_open_in_Sbuf_RAM" 
	x3026="Addr_line_problem_in_Sbuf_RAM" 
	x3027="Sbuf_RAM_unreadable" 
	x3047="Cant_fill_Sbuf_with_zeros" 
	x3062="iHandler_error_during_xfer_op" 
	x3063="Bad_disk_sequencer_cmd_issued_to_Aport" 
	x3064="Bad_RAM_location" 
	x3065="Bad_Shadowed_RAM_location" 
	x3066="Cant_determine_Sbuf_size" 
	x3067="Pbuf_read_or_write_error" 
	x3068="XOR_error" 
	x3069="No_disk_found_on_requested_Aport" 
	x306A="Interrupt_line_error" 
	x306B="Unable_to_calculate_checksum" 
	x306C="BBU_S0_or_S1_firmware_boundary_error" 
	x306D="Old_method_selected_wrong_value_EC_DQS_FIFO_set_WRONG" 
	x3070="NvRam_read_or_write_test_error" 
	x3071="SpdRom_read_test_error" 
	x3072="Hareware_strap_error" 
	x3073="Clock_Generator_data_integrity_error" 
	
	x3074="Inject_SBUF_ECC_incorrect_param"
	
	x3100="Error_manufacturing_diagnostic_test_failed" 
	x3101="XScale_Core_Processor_Subtest"
	x3102="SRAM_Subtest"
	x3103="TPMI_Subtest"
	x3104="Component_Internal_RAM_Subtest"
	x3105="ASIC_Register_Subtest"
	x3106="PCIX-E_Loopback_Subtest"
	x3107="4.0G_FC_or_3.0G_SAS_Link_Internal_Analog_Loopback_Subtest" 
	x3108="1.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x3109="2.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x310A="4.0G_FC_or_3.0G_SAS_Link_External_Loopback_Subtest" 
	x310B="TDMA_Subtest"
	x310C="Concurrent_DMA_Subtest"
	x310D="SDRAM_Subtest"
	
	x310E="Error_manufacturing_diagnostic_test-Coordinated_Reset_failed" 
	x310F="Error_manufacturing_diagnostic_test-Preemptive_Reset_failed" 
	x3110="Error_manufacturing_diagnostic_test-Invalid_Request_failed"
	
	x3111="Hareware_strap_error"
	
	x3112="Dma_completed_Miscompare_error"
	x3113="Dma_completed_error"
	
	x3200="No_RAID_key-s_found"
	x3201="RAID_key_bus_in_use"
	x3202="RAID_key_CRC_error"
	x3203="RAID_key_authentication_failure"
	x3204="RAID_key_contains_data_that_did_not_pass_validation" 
	x3205="Address_requested_is_not_valid_or_page_aligned"
	
	x3300="Invalid_board_id"
	
	x7E00="Error_manufacturing_diagnostic_test_failed" 
	x7E01="XScale_Core_Processor_Subtest"
	x7E02="SRAM_Subtest"
	x7E03="TPMI_Subtest"
	x7E04="Component_Internal_RAM_Subtest"
	x7E05="ASIC_Register_Subtest"
	x7E06="PCIX-E_Loopback_Subtest"
	x7E07="4.0G_FC_or_3.0G_SAS_Link_Internal_Analog_Loopback_Subtest" 
	x7E08="1.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x7E09="2.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x7E0A="4.0G_FC_or_3.0G_SAS_Link_External_Loopback_Subtest" 
	x7E0B="TDMA_Subtest"
	x7E0C="Concurrent_DMA_Subtest"
	x7E0D="SDRAM_Subtest"
	x7E0E="Error_manufacturing_diagnostic_test-Coordinated_Reset_failed" 
	x7E0F="Error_manufacturing_diagnostic_test-Preemptive_Reset_failed" 
	x7E10="Error_manufacturing_diagnostic_test-Invalid_Request_failed" 
	x7E11="EDMA_error"
	
	for i in "x0000 $x0000" "x0001 $x0001" "x0002 $x0002" "x0003 $x0003" "x0004 $x0004" "x0005 $x0005" "x0006 $x0006" "x0007 $x0007" "x0008 $x0008" "x0009 $x0009" "x000A $x000A" "x000B $x000B" "x000C $x000C" "x000D $x000D" "x000E $x000E" "x000F $x000F" "x0010 $x0010" "x0011 $x0011" "x0012 $x0012" "x0013 $x0013" "x0014 $x0014" "x0015 $x0015" "x0016 $x0016" "x0017 $x0017" "x0018 $x0018" "x0019 $x0019" "x001A $x001A" "x001B $x001B" "x001C $x001C" "x001D $x001D" "x001E $x001E" "x001F $x001F" "x0020 $x0020" "x0021 $x0021" "x0022 $x0022" "x0023 $x0023" "x0024 $x0024" "x0025 $x0025" "x0026 $x0026" "x0027 $x0027" "x0028 $x0028" "x0029 $x0029" "x002A $x002A" "x002B $x002B" "x002C $x002C" "x002D $x002D" "x002E $x002E" "x002F $x002F" "x0030 $x0030" "x0031 $x0031" "x0032 $x0032" "x0033 $x0033" "x0034 $x0034" "x0035 $x0035" "x0036 $x0036" "x0037 $x0037" "x0038 $x0038" "x0039 $x0039" "x003A $x003A" "x003B $x003B" "x003C $x003C" "x003D $x003D" "x003E $x003E" "x003F $x003F" "x0040 $x0040" "x0041 $x0041" "x0042 $x0042" "x0043 $x0043" "x0044 $x0044" "x0045 $x0045" "x0046 $x0046" "x0047 $x0047" "x0048 $x0048" "x0049 $x0049" "x004A $x004A" "x0051 $x0051" "x0060 $x0060" "x0061 $x0061" "x0100 $x0100" "x0101 $x0101" "x0102 $x0102" "x0103 $x0103" "x0104 $x0104" "x0105 $x0105" "x0106 $x0106" "x0107 $x0107" "x0108 $x0108" "x0109 $x0109" "x010A $x010A" "x010B $x010B" "x010C $x010C" "x010D $x010D" "x010E $x010E" "x010F $x010F" "x0110 $x0110" "x0111 $x0111" "x0112 $x0112" "x0113 $x0113" "x0114 $x0114" "x0115 $x0115" "x0116 $x0116" "x0117 $x0117" "x011E $x011E" "x011F $x011F" "x0120 $x0120" "x0121 $x0121" "x0122 $x0122" "x0123 $x0123" "x0124 $x0124" "x0125 $x0125" "x0126 $x0126" "x0127 $x0127" "x0128 $x0128" "x0129 $x0129" "x012A $x012A" "x012B $x012B" "x012C $x012C" "x012D $x012D" "x012E $x012E" "x012F $x012F" "x0130 $x0130" "x0131 $x0131" "x0132 $x0132" "x0133 $x0133" "x0140 $x0140" "x0150 $x0150" "x0151 $x0151" "x0152 $x0152" "x0153 $x0153" "x0160 $x0160" "x0161 $x0161" "x0162 $x0162" "x0170 $x0170" "x0171 $x0171" "x0180 $x0180" "x0181 $x0181" "x0182 $x0182" "x0183 $x0183" "x0184 $x0184" "x0185 $x0185" "x01A0 $x01A0" "x01C0 $x01C0" "x01C1 $x01C1" "x01C2 $x01C2" "x01C3 $x01C3" "x01C4 $x01C4" "x01C5 $x01C5" "x01C6 $x01C6" "x01C7 $x01C7" "x01C8 $x01C8" "x01C9 $x01C9" "x01CA $x01CA" "x01CB $x01CB" "x01CC $x01CC" "x01CD $x01CD" "x01CE $x01CE" "x01D0 $x01D0" "x01D1 $x01D1" "x01D2 $x01D2" "x01D3 $x01D3" "x01D4 $x01D4" "x0200 $x0200" "x0201 $x0201" "x0202 $x0202" "x0203 $x0203" "x0204 $x0204" "x0205 $x0205" "x0206 $x0206" "x0207 $x0207" "x0208 $x0208" "x0209 $x0209" "x020A $x020A" "x020B $x020B" "x020C $x020C" "x020D $x020D" "x0210 $x0210" "x0211 $x0211" "x0212 $x0212" "x0213 $x0213" "x0214 $x0214" "x0215 $x0215" "x0216 $x0216" "x0217 $x0217" "x0218 $x0218" "x0230 $x0230" "x0231 $x0231" "x0232 $x0232" "x0233 $x0233" "x0234 $x0234" "x0235 $x0235" "x0236 $x0236" "x0240 $x0240" "x0241 $x0241" "x0242 $x0242" "x0243 $x0243" "x0244 $x0244" "x0245 $x0245" "x0246 $x0246" "x0247 $x0247" "x0248 $x0248" "x024A $x024A" "x024B $x024B" "x024C $x024C" "x0250 $x0250" "x0251 $x0251" "x0252 $x0252" "x0253 $x0253" "x0254 $x0254" "x0255 $x0255" "x0256 $x0256" "x0257 $x0257" "x0258 $x0258" "x0259 $x0259" "x025A $x025A" "x025B $x025B" "x025C $x025C" "x025D $x025D" "x025E $x025E" "x025F $x025F" "x0261 $x0261" "x0260 $x0260" "x0270 $x0270" "x0271 $x0271" "x0272 $x0272" "x0280 $x0280" "x0281 $x0281" "x0282 $x0282" "x0283 $x0283" "x0290 $x0290" "x0291 $x0291" "x0293 $x0293" "x0294 $x0294" "x0295 $x0295" "x0296 $x0296" "x0297 $x0297" "x0298 $x0298" "x0299 $x0299" "x029A $x029A" "x029B $x029B" "x029C $x029C" "x029D $x029D" "x02A0 $x02A0" "x02A1 $x02A1" "x02A2 $x02A2" "x02A3 $x02A3" "x02A4 $x02A4" "x02A5 $x02A5" "x02A6 $x02A6" "x02A7 $x02A7" "x02A8 $x02A8" "x02A9 $x02A9" "x02AA $x02AA" "x02AB $x02AB" "x02B0 $x02B0" "x02B1 $x02B1" "x02B2 $x02B2" "x02B3 $x02B3" "x02B4 $x02B4" "x02B5 $x02B5" "x02B6 $x02B6" "x02B7 $x02B7" "x02B8 $x02B8" "x02B9 $x02B9" "x02BA $x02BA" "x02BB $x02BB" "x02BC $x02BC" "x02BD $x02BD" "x02BE $x02BE" "x02BF $x02BF" "x02C0 $x02C0" "x02C1 $x02C1" "x02C2 $x02C2" "x02C3 $x02C3" "x02C4 $x02C4" "x02C5 $x02C5" "x02C6 $x02C6" "x02C7 $x02C7" "x02C8 $x02C8" "x02C9 $x02C9" "x02CA $x02CA" "x02CB $x02CB" "x02CC $x02CC" "x02CD $x02CD" "x02CE $x02CE" "x02D0 $x02D0" "x02D1 $x02D1" "x02D2 $x02D2" "x02D3 $x02D3" "x02D4 $x02D4" "x02E0 $x02E0" "x02E1 $x02E1" "x02E4 $x02E4" "x02E5 $x02E5" "x02E6 $x02E6" "x02E7 $x02E7" "x02E8 $x02E8" "x02EA $x02EA" "x02EB $x02EB" "x02F0 $x02F0" "x02F1 $x02F1" "x02F2 $x02F2" "x02F3 $x02F3" "x02F4 $x02F4" "x0300 $x0300" "x0301 $x0301" "x0302 $x0302" "x0303 $x0303" "x0310 $x0310" "x0311 $x0311" "x0312 $x0312" "x0313 $x0313" "x0314 $x0314" "x0315 $x0315" "x0316 $x0316" "x0317 $x0317" "x0318 $x0318" "x0319 $x0319" "x031A $x031A" "x031B $x031B" "x031C $x031C" "x0320 $x0320" "x0321 $x0321" "x0322 $x0322" "x0323 $x0323" "x0330 $x0330" "x0331 $x0331" "x0332 $x0332" "x0333 $x0333" "x0334 $x0334" "x0335 $x0335" "x0336 $x0336" "x0337 $x0337" "x0338 $x0338" "x0339 $x0339" "x0340 $x0340" "x0341 $x0341" "x0350 $x0350" "x0351 $x0351" "x0352 $x0352" "x0353 $x0353" "x0354 $x0354" "x0380 $x0380" "x0381 $x0381" "x0382 $x0382" "x0383 $x0383" "x0384 $x0384" "x0385 $x0385" "x0386 $x0386" "x0387 $x0387" "x0388 $x0388" "x0389 $x0389" "x038A $x038A" "x038B $x038B" "x038C $x038C" "x038D $x038D" "x038E $x038E" "x0390 $x0390" "x0391 $x0391" "x0392 $x0392" "x0400 $x0400" "x0401 $x0401" "x0402 $x0402" "x0403 $x0403" "x0404 $x0404" "x0405 $x0405" "x0406 $x0406" "x0407 $x0407" "x0408 $x0408" "x0409 $x0409" "x040A $x040A" "x040B $x040B" "x040C $x040C" "x040D $x040D" "x040F $x040F" "x0410 $x0410" "x0411 $x0411" "x0412 $x0412" "x0413 $x0413" "x0414 $x0414" "x0415 $x0415" "x0416 $x0416" "x1000 $x1000" "x1001 $x1001" "x1002 $x1002" "x1003 $x1003" "x1004 $x1004" "x1005 $x1005" "x1006 $x1006" "x1007 $x1007" "x1008 $x1008" "x1009 $x1009" "x100A $x100A" "x100B $x100B" "x100C $x100C" "x1010 $x1010" "x1011 $x1011" "x1012 $x1012" "x1013 $x1013" "x1014 $x1014" "x1015 $x1015" "x1016 $x1016" "x1017 $x1017" "x1018 $x1018" "x1019 $x1019" "x101A $x101A" "x101B $x101B" "x101C $x101C" "x101D $x101D" "x1020 $x1020" "x1021 $x1021" "x1022 $x1022" "x1023 $x1023" "x1024 $x1024" "x2000 $x2000" "x2001 $x2001" "x2002 $x2002" "x2100 $x2100" "x2101 $x2101" "x2102 $x2102" "x2FFF $x2FFF" "x3013 $x3013" "x3014 $x3014" "x3017 $x3017" "x3018 $x3018" "x3019 $x3019" "x301A $x301A" "x301B $x301B" "x301C $x301C" "x301E $x301E" "x3021 $x3021" "x3024 $x3024" "x3025 $x3025" "x3026 $x3026" "x3027 $x3027" "x3047 $x3047" "x3062 $x3062" "x3063 $x3063" "x3064 $x3064" "x3065 $x3065" "x3066 $x3066" "x3067 $x3067" "x3068 $x3068" "x3069 $x3069" "x306A $x306A" "x306B $x306B" "x306C $x306C" "x306D $x306D" "x3070 $x3070" "x3071 $x3071" "x3072 $x3072" "x3073 $x3073" "x3074 $x3074" "x3100 $x3100" "x3101 $x3101" "x3102 $x3102" "x3103 $x3103" "x3104 $x3104" "x3105 $x3105" "x3106 $x3106" "x3107 $x3107" "x3108 $x3108" "x3109 $x3109" "x310A $x310A" "x310B $x310B" "x310C $x310C" "x310D $x310D" "x310E $x310E" "x310F $x310F" "x3110 $x3110" "x3111 $x3111" "x3112 $x3112" "x3113 $x3113" "x3200 $x3200" "x3201 $x3201" "x3202 $x3202" "x3203 $x3203" "x3204 $x3204" "x3205 $x3205" "x3300 $x3300" "x7E00 $x7E00" "x7E01 $x7E01" "x7E02 $x7E02" "x7E03 $x7E03" "x7E04 $x7E04" "x7E05 $x7E05" "x7E06 $x7E06" "x7E07 $x7E07" "x7E08 $x7E08" "x7E09 $x7E09" "x7E0A $x7E0A" "x7E0B $x7E0B" "x7E0C $x7E0C" "x7E0D $x7E0D" "x7E0E $x7E0E" "x7E0F $x7E0F" "x7E10 $x7E10" "x7E11 $x7E11"; do
	
		set $i
	
		if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x03_9000_ERROR.txt ] ; then
	       		$grep -i $1 ./$fileName/LSI_Products/3ware/3ware_driver_x03_9000_ERROR.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x03_$1_$2.txt
	
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_x03_$1_$2.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_x03_$1_$2.txt >> ./misc_output.txt 2>&1 ; fi
	
		fi
	
	done
	
	########################################################################################################################### 
	# Deferred errors (AENs) 0x04 
	########################################################################################################################### 
	x0000="AEN_queue_empty" 
	x0001="Controller_reset_occurred" 
	x0002="Degraded_unit" 
	x0003="Controller_error_occurred" 
	x0004="Rebuild_failed" 
	x0005="Rebuild_completed" 
	x0006="Incomplete_unit_detected" 
	x0007="Initialize_completed" 
	x0008="Unclean_shutdown_detected" 
	x0009="Drive_timeout_detected" 
	x000A="Drive_error_detected" 
	x000B="Rebuild_started" 
	x000C="Initialize_started" 
	x000D="Unit_deleted" 
	x000E="Initialize_failed" 
	x000F="SMART_threshold_exceeded" 
	x0010="Power_supply_reported_AC_under_range" 
	x0011="Power_supply_reported_DC_out_of_range" 
	x0012="Power_supply_reported_a_malfunction" 
	x0013="Power_supply_predicted_malfunction" 
	x0014="Battery_charge_below_threshold" 
	x0015="Fan_speed_below_threshold" 
	x0016="Temperature_sensor_above_threshold" 
	x0017="Power_supply_removed" 
	x0018="Power_supply_inserted" 
	x0019="Drive_removed" 
	x001A="Drive_inserted" 
	x001B="Drive_bay_cover_door_was_opened" 
	x001C="Drive_bay_cover_door_was_closed" 
	x001D="Product_case_was_opened" 
	x001E="Unit_inoperable" 
	x001F="Unit_operational" 
	x0020="Prepare_for_shutdown_power-off" 
	x0021="Downgrade_UDMA_mode" 
	x0022="Upgrade_UDMA_mode" 
	x0023="Sector_repair_completed" 
	x0024="Buffer_integrity_test_failed" 
	x0025="Cache_flush_failed_some_data_lost" 
	x0026="Drive_ECC_error_reported" 
	x0027="DCB_checksum_error_detected" 
	x0028="DCB_version_unsupported" 
	x0029="Verify_started" 
	x002A="Verify_failed" 
	x002B="Verify_completed" 
	x002C="Source_drive_ECC_error_overwritten" 
	x002D="Source_drive_error_occurred" 
	x002E="Replacement_drive_capacity_too_small" 
	x002F="Verify_not_started_unit_never_initialized" 
	x0030="Drive_not_supported" 
	x0031="Synchronize_host_or_controller_time" 
	x0032="Spare_capacity_too_small_for_some_units" 
	x0033="Migration_started" 
	x0034="Migration_failed" 
	x0035="Migration_completed" 
	x0036="Verify_fixed_data_or_parity_mismatch" 
	x0037="SO-DIMM_not_compatible" 
	x0038="SO-DIMM_not_detected" 
	x0039="Buffer_ECC_error_corrected" 
	x003A="Drive_power_on_reset_detected" 
	x003B="Rebuild_paused" 
	x003C="Initialize_paused" 
	x003D="Verify_paused" 
	x003E="Migration_paused" 
	x003F="Flash_file_system_error_detected" 
	x0040="Flash_file_system_repaired" 
	x0041="Unit_number_assignments_lost" 
	x0042="Primary_DCB_read_error_occurred" 
	x0043="Backup_DCB_read_error_detected" 
	x0044="Battery_voltage_is_normal" 
	x0045="Battery_voltage_is_low" 
	x0046="Battery_voltage_is_high" 
	x0047="Battery_voltage_is_too_low" 
	x0048="Battery_voltage_is_too_high" 
	x0049="Battery_temperature_is_normal" 
	x004A="Battery_temperature_is_low" 
	x004B="Battery_temperature_is_high" 
	x004C="Battery_temperature_is_too_low" 
	x004D="Battery_temperature_is_too_high" 
	x004E="Battery_capacity_test_started" 
	x004F="Cache_synchronization_skipped" 
	x0050="Battery_capacity_test_completed" 
	x0051="Battery_health_check_started" 
	x0052="Battery_health_check_completed" 
	x0053="Battery_capacity_test_is_overdue" 
	x0054="Charge_termination_voltage_is_at_high_level" 
	x0055="Battery_charging_started" 
	x0056="Battery_charging_completed" 
	x0057="Battery_charging_fault" 
	x0058="Battery_capacity_is_below_warning_level" 
	x0059="Battery_capacity_is_below_error_level" 
	x005A="Battery_is_present" 
	x005B="Battery_is_not_present" 
	x005C="Battery_is_weak" 
	x005D="Battery_health_check_failed" 
	x005E="Cache_synchronization_completed" 
	x005F="Cache_synchronization_failed_some_data_lost" 
	x0060="Redundancy_level_decreased" 
	x0061="Redundancy_level_increased" 
	#x0062="Enclosure_removed__9.5.0_CodeSet_and_Later" 
	#x0062="Local_link_up__PRE_9.5.0_CodeSet" 
	x0062="Shared_x0062" 
	#x0063="Enclosure_added__9.5.0_CodeSet_and_Later" 
	#x0063="Local_link_down__PRE_9.5.0_CodeSet" 
	x0063="Shared_x0063" 
	#x0064="Local_link_up__9.5.0_CodeSet_and_Later" 
	#x0064="Enclosure_removed__Only_9.4.3_CodeSet"  
	x0064="Shared_x0064"  
	#x0065="Local_link_down__9.5.0_CodeSet_and_Later" 
	#x0065="Enclosure_added__Only_9.4.3_CodeSet"  
	x0065="Shared_x0065"  
	x0067="NVRAM_data_preserved_over_power_fail" 
	x0068="NVRAM_data_not_preserved_over_power_fail" 
	x0070="Drive_Write_Fault"
	x00F7="Bad_cache_metadata_checksum" 
	x00F8="Bad_cache_metadata_signature" 
	x00F9="Cache_metadata_restore_failed" 
	x00FA="BBU_not_found_after_power_fail" 
	x00FB="BBU_and_HBA_state_out_of_sync" 
	x00FC="Recovered_or_finished_array_membership_update" 
	x00FD="Handler_lockup" 
	x00FE="Retrying_PCI_transfer" 
	x00FF="AEN_queue_is_full" 
	x0100="Error_occurred_during_device_discovery" 
	x8000="Enclosure_fan_normal"
	x8001="Enclosure_fan_error"
	x8002="Enclosure_fan_removed"
	x8003="Enclosure_fan_added"
	x8004="Enclosure_fan_status_unknown"
	x8005="Enclosure_fan_on"
	x8006="Enclosure_fan_off"
	x8010="Enclosure_fan_stopped"
	x8011="Enclosure_fan_started"
	x8012="Enclosure_fan_speed_increased"
	x8013="Enclosure_fan_speed_decreased"
	x8020="Enclosure_temperature_normal"
	x8021="Enclosure_temperature_low"
	x8022="Enclosure_temperature_high"
	x8023="Enclosure_temperature_too_low"
	x8024="Enclosure_temperature_too_high"
	x8025="Enclosure_temperature_sensor_removed" 
	x8026="Enclosure_temperature_sensor_added" 
	x8027="Enclosure_temperature_sensor_error" 
	x8028="Enclosure_temperature_status_unknown" 
	x8030="Enclosure_power_supply_normal"
	x8031="Enclosure_power_supply_failed"
	x8032="Enclosure_power_supply_removed"
	x8033="Enclosure_power_supply_added"
	x8034="Enclosure_power_supply_status_unknown" 
	x8036="Enclosure_power_supply_on"
	x8037="Enclosure_power_supply_off"
	x8040="Enclosure_power_supply_voltage_normal" 
	x8041="Enclosure_power_supply_voltage_high" 
	x8042="Enclosure_power_supply_voltage_low" 
	x8043="Enclosure_power_supply_voltage_status_unknown" 
	x8044="Enclosure_power_supply_current_normal" 
	x8045="Enclosure_power_supply_current_high" 
	x8046="Enclosure_power_supply_current_status_unknown"
	
	x8047="Enclosure_audio_alarm_activated_and_muted" 
	x8048="Enclosure_audio_alarm_failed"
	x8049="Enclosure_audio_alarm_removed"
	x804A="Enclosure_audio_alarm_added"
	x804B="Enclosure_audio_alarm_status_unknown"
	 
	#Deferred errors(AENs)0x04 
	for i in "x0000 $x0000" "x0001 $x0001" "x0002 $x0002" "x0003 $x0003" "x0004 $x0004" "x0005 $x0005" "x0006 $x0006" "x0007 $x0007" "x0008 $x0008" "x0009 $x0009" "x000A $x000A" "x000B $x000B" "x000C $x000C" "x000D $x000D" "x000E $x000E" "x000F $x000F" "x0010 $x0010" "x0011 $x0011" "x0012 $x0012" "x0013 $x0013" "x0014 $x0014" "x0015 $x0015" "x0016 $x0016" "x0017 $x0017" "x0018 $x0018" "x0019 $x0019" "x001A $x001A" "x001B $x001B" "x001C $x001C" "x001D $x001D" "x001E $x001E" "x001F $x001F" "x0020 $x0020" "x0021 $x0021" "x0022 $x0022" "x0023 $x0023" "x0024 $x0024" "x0025 $x0025" "x0026 $x0026" "x0027 $x0027" "x0028 $x0028" "x0029 $x0029" "x002A $x002A" "x002B $x002B" "x002C $x002C" "x002D $x002D" "x002E $x002E" "x002F $x002F" "x0030 $x0030" "x0031 $x0031" "x0032 $x0032" "x0033 $x0033" "x0034 $x0034" "x0035 $x0035" "x0036 $x0036" "x0037 $x0037" "x0038 $x0038" "x0039 $x0039" "x003A $x003A" "x003B $x003B" "x003C $x003C" "x003D $x003D" "x003E $x003E" "x003F $x003F" "x0040 $x0040" "x0041 $x0041" "x0042 $x0042" "x0043 $x0043" "x0044 $x0044" "x0045 $x0045" "x0046 $x0046" "x0047 $x0047" "x0048 $x0048" "x0049 $x0049" "x004A $x004A" "x004B $x004B" "x004C $x004C" "x004D $x004D" "x004E $x004E" "x004F $x004F" "x0050 $x0050" "x0051 $x0051" "x0052 $x0052" "x0053 $x0053" "x0054 $x0054" "x0055 $x0055" "x0056 $x0056" "x0057 $x0057" "x0058 $x0058" "x0059 $x0059" "x005A $x005A" "x005B $x005B" "x005C $x005C" "x005D $x005D" "x005E $x005E" "x005F $x005F" "x0060 $x0060" "x0061 $x0061" "x0062 $x0062" "x0063 $x0063" "x0064 $x0064" "x0065 $x0065" "x0067 $x0067" "x0068 $x0068" "x0070 $x0070" "x00F7 $x00F7" "x00F8 $x00F8" "x00F9 $x00F9" "x00FA $x00FA" "x00FB $x00FB" "x00FC $x00FC" "x00FD $x00FD" "x00FE $x00FE" "x00FF $x00FF" "x0100 $x0100" "x8000 $x8000" "x8001 $x8001" "x8002 $x8002" "x8003 $x8003" "x8004 $x8004" "x8005 $x8005" "x8006 $x8006" "x8010 $x8010" "x8011 $x8011" "x8012 $x8012" "x8013 $x8013" "x8020 $x8020" "x8021 $x8021" "x8022 $x8022" "x8023 $x8023" "x8024 $x8024" "x8025 $x8025" "x8026 $x8026" "x8027 $x8027" "x8028 $x8028" "x8030 $x8030" "x8031 $x8031" "x8032 $x8032" "x8033 $x8033" "x8034 $x8034" "x8036 $x8036" "x8037 $x8037" "x8040 $x8040" "x8041 $x8041" "x8042 $x8042" "x8043 $x8043" "x8044 $x8044" "x8045 $x8045" "x8046 $x8046" "x8047 $x8047" "x8049 $x8049" "x804A $x804A" "x804B $x804B" ; do
	
		set $i
	
		if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt ] ; then
	
	       		$grep -i $1 ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_$1_$2.txt
	
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_x04_$1_$2.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_x04_$1_$2.txt >> ./misc_output.txt 2>&1 ; fi
	
		fi
	done
	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Shared_x0062.txt ] ; then
		grep -i "Enclosure removed" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Shared_x0062.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Enclosure_removed.txt
		grep -i "Local link up" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Shared_x0062.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Local_link_up.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0062_Shared_x0062.txt
	fi
	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Shared_x0063.txt ] ; then
		grep -i "Enclosure added" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Shared_x0063.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Enclosure_added.txt
		grep -i "Local link down" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Shared_x0063.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Local_link_down.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0063_Shared_x0063.txt
	fi
	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Shared_x0064.txt ] ; then
		grep -i "Local link up" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Shared_x0064.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Local_link_up.txt
		grep -i "Enclosure removed" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Shared_x0064.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Enclosure_removed.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0064_Shared_x0064.txt
	fi
	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Shared_x0065.txt ] ; then
		grep -i "Local link down" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Shared_x0065.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Local_link_down.txt
		grep -i "Enclosure added" ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Shared_x0065.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Enclosure_added.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x04_x0065_Shared_x0065.txt
	fi
	
	for i in _x0062_Enclosure_removed.txt _x0062_Local_link_up.txt _x0063_Enclosure_added.txt _x0063_Local_link_down.txt _x0064_Local_link_up.txt _x0064_Enclosure_removed.txt _x0065_Local_link_down.txt _x0065_Enclosure_added.txt; do
	
		if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_x04$l ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_x04$l >> ./misc_output.txt 2>&1 ; fi
	
	done
	
	###########################################################################################################################
	###Update on CodeSet Change
	# Based on errorcode.h 5.12.00.016FW 10.2.2.1 codeset 
	# Errors = 0x03 that accompany AEN's 0x04's don't have preceeding 0's
	# Duplicate of 0x03 errors without preceeding 0's
	# Example -  3w-9xxx: scsi0: AEN: WARNING (0x04:0x0043): Backup DCB read error detected:port=2, error=0x202.
	# Legacy error_codes, these are reserved on Apache 
	###########################################################################################################################
	x0="no_error_status_dependent_code_for_this_error_status" 
	x1="no_request_ID_available_for_this_error_status" 
	x2="CP_queue_became_full" 
	x3="illegal_SGL_offset_in_CP_Header" 
	x4="illegal_number_of_SGL_entries_in_CPH" 
	x5="could_not_allocate_additional_memory" 
	x6="some_PCI_read_error_occurred" 
	x7="timeout_during_PCI_transaction" 
	x8="PCI_ERR_bit_for_a_PCI_transaction" 
	x9="unrecoverable_disk_error" 
	xA="completion_token_queue_overflow" 
	xB="error_reading_SGL" 
	xC="error_reading_CP_Header" 
	xD="abort_req_for_cmd_that_wasn't_active" 
	xE="illegal_size_in_CP_Header" 
	xF="got_a_CPH_with_an_already_active_req_ID" 
	x10="lengths_in_SGLs_not_match_block_count_in_CP" 
	x11="requested_LBA_greater_than_maximum_LBA_of_the_unit" 
	x12="host_address_or_SGL_size_not_on_8_dw_boundary" 
	x13="data_integrity_error_on_read_or_write_test" 
	x14="subcmd_num_in_CP_hdr_is_undef_for_this_cmd" 
	x15="undefined_table_requested" 
	x16="param_requested_is_out_of_bounds_of_table" 
	x17="given_param_size_doesn't_match_param_size_in_table" 
	x18="host_address_or_SGL_size_not_on_a_sector_boundary" 
	x19="Achip_Unit_number_exceeds_maximum" 
	x1A="bad_pairing_of_disk-op_or_xer-op_from_Aop_queue" 
	x1B="Aport_timed_out_doing_an_Aop" 
	x1C="an_Achip_GSR_interrupt_occurred" 
	x1D="Aop_encountered_with_unknown_cmd_byte" 
	x1E="PCI_was_busy_when_tried_to_do_a_PCI_read_or_write" 
	x1F="next_Aop_should_have_been_a_disk-op_but_it_wasnt" 
	x20="param_0_unsupported_for_now_for_get_or_set_param" 
	x21="an_unimplemented_method_was_invoked" 
	x22="request_ID_took_too_long_to_complete" 
	x23="a_disk_task_file_error_occurred_during_a_1F_test" 
	x24="data_lines_shorted_in_Sbuf_RAM" 
	x25="data_lines_open_in_Sbuf_RAM" 
	x26="addr_line_problem_in_Sbuf_RAM" 
	x27="Sbuf_RAM_unreadable" 
	x28="command_requires_at_least_1_SGL" 
	x29="no_unit_num_is_available_for_use" 
	x2A="CP_queue_was_empty_when_tried_to_get_a_CP_ptr_from_it" 
	x2B="test_firmware_not_downloaded_to_RAM" 
	x2C="an_attempt_was_made_to_nest_internal_requests" 
	x2D="error_in_downloading_hex_file" 
	x2E="error_in_programming_the_flash_ROM" 
	x2F="error_in_rollcall" 
	x30="error_in_wait_disks_rdy" 
	x31="error_in_UIT_formatting" 
	x32="incorrect_unit_type_for_the_request" 
	x33="unit_does_not_have_the_logical_sub-unit_specified" 
	x34="unit_has_corrupted_data_on_it" 
	x35="could_not_write_DCB_to_disk" 
	x36="could_not_get_profiler_from_disk" 
	x37="unit_is_not_exportable" 
	x38="unit_is_missing_a_sub-unit" 
	x39="unit_is_not_operating_normally" 
	x3A="not_enough_SBUF_segments_to_service_the_request" 
	x3B="User_area_in_unit_was_not_written_with_zeroes" 
	x3C="user_data_on_unit_did_not_verify" 
	x3D="error_while_writing_0s_to_user_data" 
	x3E="a_unit_does_not_contain_a_logical_mapping_for_a_phys_drive" 
	x3F="drive_replacement_would_cause_a_double-degrade" 
	x40="capacity_of_replacement_drive_is_too_small" 
	x41="no_drive_detected" 
	x42="drive_detected_to_be_busy" 
	x43="aport_unavailable" 
	x44="unable_to_clear_sbuf" 
	x45="can_not_replace_drive_because_unit_not_degraded" 
	x46="we_have_no_routine_to_program_this_mfr_or_type_of_flash" 
	x47="cant_fill_Sbuf_with_zeros" 
	x48="byte_count_in_PARAM_field_is_too_big_for_this_cmd" 
	x49="timeout_while_waiting_for_data_from_display_panel" 
	x4A="CRC_error_reported_on_BFB_transfer" 
	x51="for_drive_errors_the_status_register_gets_stuffed_into_LSB_of_ESDC" 
	x60="reserved_-_this_is_a_valid_drive_status_error_code" 
	x61="reserved_-_this_is_a_valid_drive_status_error_code" 
	
	x100="SGL_entry_contains_zero_data" 
	x101="Invalid_command_opcode" 
	x102="SGL_entry_has_unaligned_address" 
	x103="SGL_size_does_not_match_command" 
	x104="SGL_entry_has_illegal_length" 
	x105="Command_packet_is_not_aligned" 
	x106="Invalid_request_ID" 
	x107="Duplicate_request_ID" 
	x108="ID_not_locked" 
	x109="LBA_out_of_range" 
	x10A="Logical_unit_not_present" 
	x10B="Parameter_table_does_not_exist" 
	x10C="Parameter_index_does_not_exist" 
	x10D="Invalid_field_in_CDB" 
	x10E="Invalid_operation_for_specified_port" 
	x10F="Parameter_item_size_mismatch" 
	
	x110="Failed_memory_allocation" 
	x111="Memory_request_too_large" 
	x112="Out_of_memory_segments" 
	x113="Invalid_address_to_deallocate" 
	x114="Out_of_memory" 
	x115="Out_of_heap" 
	x116="Invalid_BIOS_buffer_id" 
	
	x117="Host_lock_not_available" 
	
	x11E="Unrecovered_Read_Error"
	x11F="Recovered_Data_with_error_correction_applied"
	
	x120="Double_degrade" 
	x121="Drive_not_degraded" 
	x122="Reconstruct_error" 
	x123="Replace_not_accepted" 
	x124="Drive_capacity_too_small" 
	x125="Sector_count_not_allowed" 
	x126="No_spares_left" 
	x127="Reconstruct_error" 
	x128="Unit_offline" 
	x129="Cannot_update_status_to_DCB" 
	x12A="Invalid_configuration_for_split" 
	x12B="Invalid_configuration_for_join" 
	x12C="No_migration_recovery" 
	x12D="No_SATA_spares" 
	x12E="No_SAS_spares"  
	x12F="Mixed_SAS_SATA_not_allowed_in_same_unit" 
	
	x130="Invalid_stripe_handle" 
	x131="Handle_that_was_not_locked" 
	x132="Handle_that_was_not_empty" 
	x133="Handle_has_different_owner" 
	
	x140="IPR_has_parent" 
	
	x150="Illegal_Pbuf_address_alignment" 
	x151="Illegal_Pbuf_transfer_length" 
	x152="Illegal_Sbuf_address_alignment" 
	x153="Illegal_Sbuf_transfer_length" 
	
	x160="Command_packet_too_large" 
	x161="SGL_exceeds_maximum_length" 
	x162="SGL_has_too_many_entries" 
	
	x170="Insufficient_resources_for_rebuilder" 
	x171="Verify_error_data_doesn't_equal_parity" 
	
	x180="Requested_segment_not_in_directory_of_this_DCB" 
	x181="DCB_segment_has_unsupported_version" 
	x182="DCB_segment_has_checksum_error" 
	x183="DCB_support_settings_segment_invalid" 
	x184="DCB_UDB_unit_descriptor_block_segment_invalid" 
	x185="DCB_GUID_globally_unique_identifier_segment_invalid" 
	
	x1A0="Could_not_clear_Sbuf" 
	
	x1C0="Flash_device_unsupported" 
	x1C1="Flash_out_of_bounds" 
	x1C2="Flash_write_verify_failed" 
	x1C3="Flash_file_object_not_found" 
	x1C4="Flash_file_already_present" 
	x1C5="Flash_file_system_full" 
	x1C6="Flash_file_not_present" 
	x1C7="Flash_file_size_mismatch" 
	x1C8="Flash_file_checksum_error" 
	x1C9="Flash_file_version_unsupported" 
	x1CA="Flash_file_system_error_detected" 
	x1CB="Flash_file_component_directory_not_found" 
	x1CC="Flash_file_component_not_found" 
	x1CD="Flash_write_cycle_failed" 
	x1CE="Flash_erase_cycle_failed" 
	
	x1D0="Invalid_field_in_parameter_list" 
	x1D1="Parameter_list_length_error" 
	x1D2="Parameter_not_changeable" 
	x1D3="Parameter_not_saveable" 
	x1D4="Invalid_mode_page" 
	
	x200="Drive_CRC_error" 
	x201="Internal_bus_CRC_error" 
	x202="Drive_ECC_Medium_error" 
	x203="Drive_TFR_readback_error" 
	x204="Drive_timeout" 
	x205="Drive_power_on_reset" 
	x206="ADP_level_2_error" 
	x207="Drive_soft_reset_failed" 
	x208="Drive_not_ready" 
	x209="Unclassified_drive_error" 
	x20A="Drive_aborted_command" 
	x20B="Port_link_error_detected" 
	x20C="Port_internal_error_detected" 
	x20D="Drive_not_ready_require_Spinup" 
	x20E="Uninitialized_drive_handle" 
	
	x210="Internal_bus_CRC_error" 
	x211="PCI_bus_abort_error" 
	x212="PCI_bus_parity_error" 
	x213="Port_handler_error" 
	x214="Token_interrupt_count_error" 
	x215="PCI_bus_timeout" 
	x216="Buffer_ECC_error_corrected" 
	x217="Buffer_ECC_error_not_corrected" 
	x218="Xop_pool_parity_error" 
	
	x230="Unsupported_command_during_flash_recovery" 
	x231="Next_image_buffer_expected" 
	x232="Binary_image_architecture_ID_incompatible" 
	x233="Binary_image_no_signature_detected" 
	x234="Binary_image_checksum_error_detected" 
	x235="Binary_image_buffer_overflow_detected" 
	x236="Binary_image_SRL_incompatible" 
	
	x240="I2C_device_not_detected" 
	x241="I2C_transaction_aborted" 
	x242="SO-DIMM_parameters_incompatible_using_defaults" 
	x243="SO-DIMM_unsupported" 
	x244="I2C_clock_is_held_low_transfer_aborted" 
	x245="I2C_data_is_held_low_transfer_aborted" 
	x246="I2C_slave_device_NACKed_the_transfer"
	x247="I2C_buffer_in-sufficient" 
	x248="SPI_transfer_status_error" 
	x24A="I2C_interface_is_active" 
	x24B="Lost_arbitration" 
	x24C="I2C_transfer_error" 
	
	x250="Unit_descriptor_size_invalid" 
	x251="Unit_descriptor_size_exceeds_data_buffer" 
	x252="Invalid_value_in_unit_descriptor" 
	x253="Inadequate_disk_space_to_support_descriptor" 
	x254="Unable_to_create_data_channel_for_this_unit_descriptor" 
	x255="Unit_descriptor_specifies_a_drive_already_in_use" 
	x256="Unable_to_write_configuration_to_all_disks" 
	x257="Unit_descriptor_version_not_supported" 
	x258="Invalid_subunit_for_RAID_0_or_5" 
	x259="Too_many_unit_descriptors" 
	x25A="Invalid_configuration_in_unit_descriptor" 
	x25B="Invalid_LBA_offset_in_unit_descriptor" 
	x25C="Invalid_stripelet_size_in_unit_descriptor" 
	x25D="JBOD_unit_is_not_allowed" 
	x25E="Operation_not_allowed_retained_cache_data" 
	x25F="Exceeded_maximum_number_of_active_drives" 
	x260="SMART_threshold_exceeded" 
	x261="Maximum_number_of_units_reached" 
	
	
	x270="Unit_not_in_NORMAL_state" 
	x271="Invalid_drive_members" 
	x272="Converted_unit_not_supported" 
	
	x280="ResponseIU_status_code_EC_STATUS_BUSY" 
	x281="ResponseIU_status_code_EC_STATUS_QUEUE_FULL" 
	x282="ResponseIU_status_code_EC_STATUS_UNEXPECTED" 
	x283="IO_Hold_Error"
	
	x290="No_Sense_Info_EC_SK_NO_SENSE_INFO"  
	x291="Recovered_Error_EC_SK_RECOVERED_ERROR"  
	x293="Hardware_Error_EC_SK_HARDWARE_ERROR"  
	x294="Hardware_ECC_Error_EC_SK_HARDWARE_ECC"  
	x295="Illegal_Req_EC_SK_ILLEGAL_REQ"  
	x296="Unit_Attention_EC_SK_UNIT_ATTENTION"  
	x297="Unit_Attention_Reset_EC_SK_UNIT_ATTENTION" 
	# note: The error code below is reused - Commented Out 
	#x205="ResponseIU_status_codeEC_SK_UNIT_REset" 
	x298="Aborted_Cmd_EC_SK_ABORTED_CMD"  
	x299="Sense_Keys_Unexpected_EC_SK_UNEXPECTED"  
	x29A="Unit_Attention_Mode_Page_Changed_EC_SK_UNIT_ATTENTION" 
	x29B="Current_command_Write_Fault"
	x29C="Deferred_Drive_Write_Fault"
	
	x29D="Lba_Out_Of_Range"
	
	x2A0="SAS_error_code_EC_PAYLOAD_PARITY" 
	x2A1="SAS_error_code_EC_UNDER_RUN_IN_RW" 
	x2A2="SAS_error_code_EC_UNDER_RUN_OUT" 
	x2A3="SAS_error_code_EC_OVER_RUN" 
	x2A4="SAS_error_code_EC_OPEN_REJECT_BUSY"  
	x2A5="SAS_error_code_EC_OPEN_REJECT_RETRY"   
	x2A6="SAS_error_code_EC_OPEN_REJECT_ABANDON"   
	x2A7="SAS_error_code_EC_OPEN_REJECT_STP_RES_BUSY"   
	x2A8="SAS_error_code_EC_RX_FRAME_ERROR"   
	x2A9="SAS_error_code_EC_RX_TRANSPORT_ERROR" 
	x2AA="SAS_error_code_EC_PORT_OFFLINE"   
	x2AB="SAS_Response_data_present"   
	
	x2B0="Tx_failure_-_SATA_SYNC_received" 
	x2B1="Tx_failure_-_BREAK_received" 
	x2B2="Protocol_overrun" 
	x2B3="Protocol_underrun" 
	x2B4="Open_failure_-_Connection_rejected" 
	x2B5="Open_failure_-_Bad_destination" 
	x2B6="Open_failure_-_Wrong_destination" 
	x2B7="Open_failure_-_Connection_rate_not_supported" 
	x2B8="Open_failure_-_Protocol_not_supported" 
	x2B9="Open_failure_-_STP_Resources_busy" 
	x2BA="Open_failure_-_No_destination" 
	x2BB="Open_failure_-_Pathway_blocked" 
	x2BC="Open_failure_-_Retry" 
	x2BD="Open_failure_-_Open_frame_timeout" 
	x2BE="STP_Inactivity" 
	x2BF="Failed_to_discover_Emulex_chip" 
	x2C0="Emulex_flash_file_is_corrupted" 
	x2C1="Error_while_flashing_Emulex" 
	x2C2="Target_returned_valid_sense_data_during_a_SCSI_PASSTHROUGH" 
	x2C3="Failed_to_unlock_flash_block_while_flashing_Emulex_ROM" 
	x2C4="Failed_to_erase_flash_block_while_flashing_Emulex_ROM" 
	x2C5="Failed_to_write_flash_block_while_flashing_Emulex_ROM" 
	x2C6="Failed_to_lock_flash_block_while_flashing_Emulex_ROM" 
	x2C7="ROM_size_is_not_a_multiple_of_128k" 
	x2C8="Emulex_SLI_command_timeout" 
	x2C9="Emulex_mailbox_status_error" 
	x2CA="Emulex_mailbox_format_error" 
	x2CB="Emulex_no_resources" 
	x2CC="Emulex_protocol_check_error" 
	x2CD="Emulex_protocol_fis_error" 
	x2CE="IOC_firmware_update_error" 
	
	x2D0="Discovery_module_resource_error" 
	x2D1="Delete_the_port_in_discovery_Manager" 
	x2D2="Discovery_module_bad_pointer_error" 
	x2D3="Discovery_module_unknown_SMP_function" 
	x2D4="Target_Unregistration_with_IOC_failed" 
	
	x2E0="SAS_Error_PAYLOAD_PARITY" 
	x2E1="SAS_Error_UNDER_RUN_IN_RW" 
	x2E4="SAS_Error_OPEN_REJECT_BUSY" 
	x2E5="SAS_Error_OPEN_REJECT_RETRY" 
	x2E6="SAS_Error_OPEN_REJECT_ABANDON" 
	x2E7="SAS_Error_OPEN_REJECT_STP_RES_BUSY" 
	x2E8="SAS_Error_RX_FRAME_ERROR" 
	x2EA="SAS_Error_PORT_OFFLINE" 
	
	x2EB="Suspend_IO_during_PL_TMF" 
	
	x2F0="Tx_failure_-_SATA_R_ERR_received" 
	x2F1="Tx_failure_-_SATA_DMAT_received" 
	x2F2="Non_specific_NCQ_error" 
	x2F3="Task_File_error" 
	x2F4="SATA_Register_Set_error" 
	
	x300="Internal_errorcode_BBU_base_-_should_not_occur" 
	x301="Invalid_BBU_state_change_request"  
	x302="The_BBU_resource_needed_is_in_use_retry_command_after_a_delay"  
	x303="Command_requires_a_battery_pack_to_be_present_and_enabled" 
	
	x310="BBU_command_packet_error" 
	x311="BBU_command_not_implemented" 
	x312="BBU_command_buffer_underflow" 
	x313="BBU_command_buffer_overflow" 
	x314="BBU_command_incomplete" 
	x315="BBU_command_checksum_error" 
	x316="BBU_command_timeout" 
	
	x317="BBU_flash_operation_failed" 
	x318="BBU_flash_Vpp_voltage_out_of_progamming_range" 
	x319="BBU_flash_incorrect_command_or_parameter_or_not_enough_space_in_stack" 
	x31A="BBU_flash_not_yet_completed" 
	x31B="BBU_flash_write_skip" 
	x31C="BBU_flash_invalid_erase_sector" 
	
	x320="BBU_parameter_not_defined" 
	x321="BBU_parameter_size_mismatch" 
	x322="Cannot_write_a_read-only_BBU_parameter" 
	x323="Invalid_state_bits_in_BBU_SetportPins_command" 
	
	x330="FBU_nif_error"
	x331="FBU_ERASE_BLOCK_ERROR"
	x332="FBU_Program_error"
	x333="FBU_ecc_uncorrectable"
	x334="FBU_ecc_correctable"
	x335="FBU_Program_error"
	x336="FBU_Erase_error"
	x337="FBU_No_defect_guard"
	x338="FBU_Read_Error"
	x339="FBU_Read_Error"
	
	x350="Uncorrectable_ECC_Error"
	x351="Correctable_ECC_Error"
	x352="Access_out_of_memory_range"
	x353="Dram_Controller_Fatal_Error"
	x354="Dram_ECC_error_log_full"
	
	x340="Invalid_discharge-learn_cycle_in_Battery_test" 
	x341="Battery_test_failed"
	
	x380="BBU_firmware_version_string_not_found" 
	x381="BBU_operating_state_not_available" 
	x382="BBU_not_present" 
	x383="BBU_not_ready" 
	x384="BBU_S1_not_compatible_with_HBA" 
	x385="BBU_S0_not_compatible_with_HBA" 
	x386="BBU_not_compatible_with_HBA" 
	x387="BBU_not_in_S0" 
	x388="BBU_not_in_S1" 
	x389="Timeout_on_BBU_power_fail_interrupt" 
	x38A="BBU_invalid_response_length" 
	x38B="Not_S1_ident_or_event_packet" 
	x38C="HBA_has_backup_data" 
	x38D="Invalid_BBU_state" 
	x38E="BBU_invalid_response_code" 
	
	x390="Log_updates_not_allowed" 
	x391="Logs_are_invalid" 
	x392="Logs_not_found" 
	
	x400="Invalid_enclosure_port_defined" 
	x401="Enclosure_resource_reserved" 
	x402="Enclosure_parameter_not_defined" 
	x403="Enclosure_parameter_re-defined" 
	x404="Enclosure_port_is_input_port" 
	x405="Invalid_SAFTE_page_requested" 
	x406="Invalid_SES_page_requested" 
	x407="EPCT_device_description_error" 
	x408="EPCT_device_redefined" 
	x409="EPCT_element_unknown" 
	x40A="EPCT_device_unknown" 
	x40B="EPCT_ID_unknown" 
	x40C="Invalid_enclosure_device" 
	x40D="EPCT_LED_descriptor_redefinition" 
	x40F="Enclosure_device_not_initialized" 
	x410="Temperature_sensor_reading_unknown" 
	x411="Enclosure_not_present" 
	x412="Bad_EPCT_version" 
	x413="Failed_to_create_SEP_object" 
	x414="Too_many_enclosures_cannot_add_more" 
	x415="Failed_to_create_enclosure_object" 
	x416="Invalid_Sep_Command" 
	
	x1000="EC_NEED_CMD_PROCESS" 
	x1001="EC_UDMA_UPGRADE_SKIPPED" 
	x1002="EC_DRIVE_NOT_IN_UDMA" 
	x1003="EC_OFFLINE_TIMER_RUNNING" 
	x1004="EC_BIN_HANDLE_NOT_EMPTY" 
	x1005="EC_BIN_HANDLE_WRONG_OWNER" 
	x1006="EC_CMD_IN_PBUF" 
	x1007="EC_INVALID_DATA_CHKSUM" 
	x1008="EC_LBA_OVERLAP" 
	x1009="EC_SGL_NON_SECTOR_SIZE" 
	x100A="Retry_CMD"
	
	x100B="BT1680_Liberator_Error"
	x100C="BT1680_Liberator_Fail"
	
	x1010="Error_recovery_in_progress" 
	x1011="Error_recovery_complete" 
	x1012="No_LBA_to_repair_sector" 
	x1013="Retry_recovery_step" 
	x1014="Do_error_action" 
	x1015="Degrade_unit" 
	x1016="Sector_repair_was_not_completed" 
	x1017="Command_aborted" 
	x1018="Drive_added" 
	
	x1019="Drive_removed" 
	x101A="Retry_queued_command" 
	x101B="Drive_error" 
	x101C="Non-Dma_Retry_no_recovery" 
	x101D="Drive_removed_no_wait" 
	x1020="Simulate_power_fail" 
	x1021="Simulate_uC_error" 
	
	x1022="Need_to_transition_to_Drive_remove_error" 
	x1023="Simulate_exception_error" 
	x1024="Simulate_illegal_instruction_error" 
	
	x2000="Checksum_of_cache_meta_data_is_bad" 
	x2001="Signature_of_cache_meta_data_is_bad" 
	x2002="Cache_meta_data_is_bad_due_to_bad_parity_link" 
	
	x2100="SATA_NCQ_Error_to_trigger_error_handler" 
	x2101="SAS_Error_to_trigger_error_handler" 
	x2102="Drive_must_be_reset_either_locally_or_via_SMP"
	
	x2FFF="EC_FEATURE_NOT_IMPLEMENTED" 
	
	x3013="Data_integrity_error_in_diagnostic_test" 
	x3014="Undefined_Sub-command_for_diag_test" 
	x3017="Drive_reset_error_thru_Marvell" 
	x3018="Reading_config_space_error" 
	x3019="Read_or_write_memory_space_data_integrity_error" 
	x301A="Pchip_UCbuf_data_integrity_error" 
	x301B="Pchip_Xop_Pool_data_integrity_error" 
	x301C="Pchip_Cmd_Ram_data_integrity_error" 
	x301E="iHandler_was_busy_before_diag_test" 
	x3021="An_unimplemented_method_was_invoked" 
	x3024="Data_lines_shorted_in_Sbuf_RAM" 
	x3025="Data_lines_open_in_Sbuf_RAM" 
	x3026="Addr_line_problem_in_Sbuf_RAM" 
	x3027="Sbuf_RAM_unreadable" 
	x3047="Cant_fill_Sbuf_with_zeros" 
	x3062="iHandler_error_during_xfer_op" 
	x3063="Bad_disk_sequencer_cmd_issued_to_Aport" 
	x3064="Bad_RAM_location" 
	x3065="Bad_Shadowed_RAM_location" 
	x3066="Cant_determine_Sbuf_size" 
	x3067="Pbuf_read_or_write_error" 
	x3068="XOR_error" 
	x3069="No_disk_found_on_requested_Aport" 
	x306A="Interrupt_line_error" 
	x306B="Unable_to_calculate_checksum" 
	x306C="BBU_S0_or_S1_firmware_boundary_error" 
	x306D="Old_method_selected_wrong_value_EC_DQS_FIFO_set_WRONG" 
	x3070="NvRam_read_or_write_test_error" 
	x3071="SpdRom_read_test_error" 
	x3072="Hareware_strap_error" 
	x3073="Clock_Generator_data_integrity_error" 
	
	x3074="Inject_SBUF_ECC_incorrect_param"
	
	x3100="Error_manufacturing_diagnostic_test_failed" 
	x3101="XScale_Core_Processor_Subtest"
	x3102="SRAM_Subtest"
	x3103="TPMI_Subtest"
	x3104="Component_Internal_RAM_Subtest"
	x3105="ASIC_Register_Subtest"
	x3106="PCIX-E_Loopback_Subtest"
	x3107="4.0G_FC_or_3.0G_SAS_Link_Internal_Analog_Loopback_Subtest" 
	x3108="1.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x3109="2.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x310A="4.0G_FC_or_3.0G_SAS_Link_External_Loopback_Subtest" 
	x310B="TDMA_Subtest"
	x310C="Concurrent_DMA_Subtest"
	x310D="SDRAM_Subtest"
	
	x310E="Error_manufacturing_diagnostic_test-Coordinated_Reset_failed" 
	x310F="Error_manufacturing_diagnostic_test-Preemptive_Reset_failed" 
	x3110="Error_manufacturing_diagnostic_test-Invalid_Request_failed"
	
	x3111="Hareware_strap_error"
	
	x3112="Dma_completed_Miscompare_error"
	x3113="Dma_completed_error"
	
	x3200="No_RAID_key-s_found"
	x3201="RAID_key_bus_in_use"
	x3202="RAID_key_CRC_error"
	x3203="RAID_key_authentication_failure"
	x3204="RAID_key_contains_data_that_did_not_pass_validation" 
	x3205="Address_requested_is_not_valid_or_page_aligned"
	
	x3300="Invalid_board_id"
	
	x7E00="Error_manufacturing_diagnostic_test_failed" 
	x7E01="XScale_Core_Processor_Subtest"
	x7E02="SRAM_Subtest"
	x7E03="TPMI_Subtest"
	x7E04="Component_Internal_RAM_Subtest"
	x7E05="ASIC_Register_Subtest"
	x7E06="PCIX-E_Loopback_Subtest"
	x7E07="4.0G_FC_or_3.0G_SAS_Link_Internal_Analog_Loopback_Subtest" 
	x7E08="1.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x7E09="2.0G_FC_or_1.5G_SAS_Link_External_Loopback_Subtest" 
	x7E0A="4.0G_FC_or_3.0G_SAS_Link_External_Loopback_Subtest" 
	x7E0B="TDMA_Subtest"
	x7E0C="Concurrent_DMA_Subtest"
	x7E0D="SDRAM_Subtest"
	x7E0E="Error_manufacturing_diagnostic_test-Coordinated_Reset_failed" 
	x7E0F="Error_manufacturing_diagnostic_test-Preemptive_Reset_failed" 
	x7E10="Error_manufacturing_diagnostic_test-Invalid_Request_failed" 
	x7E11="EDMA_error"
	
	for i in "x0 $x0" "x1 $x1" "x2 $x2" "x3 $x3" "x4 $x4" "x5 $x5" "x6 $x6" "x7 $x7" "x8 $x8" "x9 $x9" "xA $xA" "xB $xB" "xC $xC" "xD $xD" "xE $xE" "xF $xF" "x10 $x10" "x11 $x11" "x12 $x12" "x13 $x13" "x14 $x14" "x15 $x15" "x16 $x16" "x17 $x17" "x18 $x18" "x19 $x19" "x1A $x1A" "x1B $x1B" "x1C $x1C" "x1D $x1D" "x1E $x1E" "x1F $x1F" "x20 $x20" "x21 $x21" "x22 $x22" "x23 $x23" "x24 $x24" "x25 $x25" "x26 $x26" "x27 $x27" "x28 $x28" "x29 $x29" "x2A $x2A" "x2B $x2B" "x2C $x2C" "x2D $x2D" "x2E $x2E" "x2F $x2F" "x30 $x30" "x31 $x31" "x32 $x32" "x33 $x33" "x34 $x34" "x35 $x35" "x36 $x36" "x37 $x37" "x38 $x38" "x39 $x39" "x3A $x3A" "x3B $x3B" "x3C $x3C" "x3D $x3D" "x3E $x3E" "x3F $x3F" "x40 $x40" "x41 $x41" "x42 $x42" "x43 $x43" "x44 $x44" "x45 $x45" "x46 $x46" "x47 $x47" "x48 $x48" "x49 $x49" "x4A $x4A" "x51 $x51" "x60 $x60" "x61 $x61" "x100 $x100" "x101 $x101" "x102 $x102" "x103 $x103" "x104 $x104" "x105 $x105" "x106 $x106" "x107 $x107" "x108 $x108" "x109 $x109" "x10A $x10A" "x10B $x10B" "x10C $x10C" "x10D $x10D" "x10E $x10E" "x10F $x10F" "x110 $x110" "x111 $x111" "x112 $x112" "x113 $x113" "x114 $x114" "x115 $x115" "x116 $x116" "x117 $x117" "x11E $x11E" "x11F $x11F" "x120 $x120" "x121 $x121" "x122 $x122" "x123 $x123" "x124 $x124" "x125 $x125" "x126 $x126" "x127 $x127" "x128 $x128" "x129 $x129" "x12A $x12A" "x12B $x12B" "x12C $x12C" "x12D $x12D" "x12E $x12E" "x12F $x12F" "x130 $x130" "x131 $x131" "x132 $x132" "x133 $x133" "x140 $x140" "x150 $x150" "x151 $x151" "x152 $x152" "x153 $x153" "x160 $x160" "x161 $x161" "x162 $x162" "x170 $x170" "x171 $x171" "x180 $x180" "x181 $x181" "x182 $x182" "x183 $x183" "x184 $x184" "x185 $x185" "x1A0 $x1A0" "x1C0 $x1C0" "x1C1 $x1C1" "x1C2 $x1C2" "x1C3 $x1C3" "x1C4 $x1C4" "x1C5 $x1C5" "x1C6 $x1C6" "x1C7 $x1C7" "x1C8 $x1C8" "x1C9 $x1C9" "x1CA $x1CA" "x1CB $x1CB" "x1CC $x1CC" "x1CD $x1CD" "x1CE $x1CE" "x1D0 $x1D0" "x1D1 $x1D1" "x1D2 $x1D2" "x1D3 $x1D3" "x1D4 $x1D4" "x200 $x200" "x201 $x201" "x202 $x202" "x203 $x203" "x204 $x204" "x205 $x205" "x206 $x206" "x207 $x207" "x208 $x208" "x209 $x209" "x20A $x20A" "x20B $x20B" "x20C $x20C" "x20D $x20D" "x210 $x210" "x211 $x211" "x212 $x212" "x213 $x213" "x214 $x214" "x215 $x215" "x216 $x216" "x217 $x217" "x218 $x218" "x230 $x230" "x231 $x231" "x232 $x232" "x233 $x233" "x234 $x234" "x235 $x235" "x236 $x236" "x240 $x240" "x241 $x241" "x242 $x242" "x243 $x243" "x244 $x244" "x245 $x245" "x246 $x246" "x247 $x247" "x248 $x248" "x24A $x24A" "x24B $x24B" "x24C $x24C" "x250 $x250" "x251 $x251" "x252 $x252" "x253 $x253" "x254 $x254" "x255 $x255" "x256 $x256" "x257 $x257" "x258 $x258" "x259 $x259" "x25A $x25A" "x25B $x25B" "x25C $x25C" "x25D $x25D" "x25E $x25E" "x25F $x25F" "x261 $x261" "x260 $x260" "x270 $x270" "x271 $x271" "x272 $x272" "x280 $x280" "x281 $x281" "x282 $x282" "x283 $x283" "x290 $x290" "x291 $x291" "x293 $x293" "x294 $x294" "x295 $x295" "x296 $x296" "x297 $x297" "x298 $x298" "x299 $x299" "x29A $x29A" "x29B $x29B" "x29C $x29C" "x29D $x29D" "x2A0 $x2A0" "x2A1 $x2A1" "x2A2 $x2A2" "x2A3 $x2A3" "x2A4 $x2A4" "x2A5 $x2A5" "x2A6 $x2A6" "x2A7 $x2A7" "x2A8 $x2A8" "x2A9 $x2A9" "x2AA $x2AA" "x2AB $x2AB" "x2B0 $x2B0" "x2B1 $x2B1" "x2B2 $x2B2" "x2B3 $x2B3" "x2B4 $x2B4" "x2B5 $x2B5" "x2B6 $x2B6" "x2B7 $x2B7" "x2B8 $x2B8" "x2B9 $x2B9" "x2BA $x2BA" "x2BB $x2BB" "x2BC $x2BC" "x2BD $x2BD" "x2BE $x2BE" "x2BF $x2BF" "x2C0 $x2C0" "x2C1 $x2C1" "x2C2 $x2C2" "x2C3 $x2C3" "x2C4 $x2C4" "x2C5 $x2C5" "x2C6 $x2C6" "x2C7 $x2C7" "x2C8 $x2C8" "x2C9 $x2C9" "x2CA $x2CA" "x2CB $x2CB" "x2CC $x2CC" "x2CD $x2CD" "x2CE $x2CE" "x2D0 $x2D0" "x2D1 $x2D1" "x2D2 $x2D2" "x2D3 $x2D3" "x2D4 $x2D4" "x2E0 $x2E0" "x2E1 $x2E1" "x2E4 $x2E4" "x2E5 $x2E5" "x2E6 $x2E6" "x2E7 $x2E7" "x2E8 $x2E8" "x2EA $x2EA" "x2EB $x2EB" "x2F0 $x2F0" "x2F1 $x2F1" "x2F2 $x2F2" "x2F3 $x2F3" "x2F4 $x2F4" "x300 $x300" "x301 $x301" "x302 $x302" "x303 $x303" "x310 $x310" "x311 $x311" "x312 $x312" "x313 $x313" "x314 $x314" "x315 $x315" "x316 $x316" "x317 $x317" "x318 $x318" "x319 $x319" "x31A $x31A" "x31B $x31B" "x31C $x31C" "x320 $x320" "x321 $x321" "x322 $x322" "x323 $x323" "x330 $x330" "x331 $x331" "x332 $x332" "x333 $x333" "x334 $x334" "x335 $x335" "x336 $x336" "x337 $x337" "x338 $x338" "x339 $x339" "x340 $x340" "x341 $x341" "x350 $x350" "x351 $x351" "x352 $x352" "x353 $x353" "x354 $x354" "x380 $x380" "x381 $x381" "x382 $x382" "x383 $x383" "x384 $x384" "x385 $x385" "x386 $x386" "x387 $x387" "x388 $x388" "x389 $x389" "x38A $x38A" "x38B $x38B" "x38C $x38C" "x38D $x38D" "x38E $x38E" "x390 $x390" "x391 $x391" "x392 $x392" "x400 $x400" "x401 $x401" "x402 $x402" "x403 $x403" "x404 $x404" "x405 $x405" "x406 $x406" "x407 $x407" "x408 $x408" "x409 $x409" "x40A $x40A" "x40B $x40B" "x40C $x40C" "x40D $x40D" "x40F $x40F" "x410 $x410" "x411 $x411" "x412 $x412" "x413 $x413" "x414 $x414" "x415 $x415" "x416 $x416" "x1000 $x1000" "x1001 $x1001" "x1002 $x1002" "x1003 $x1003" "x1004 $x1004" "x1005 $x1005" "x1006 $x1006" "x1007 $x1007" "x1008 $x1008" "x1009 $x1009" "x100A $x100A" "x100B $x100B" "x100C $x100C" "x1010 $x1010" "x1011 $x1011" "x1012 $x1012" "x1013 $x1013" "x1014 $x1014" "x1015 $x1015" "x1016 $x1016" "x1017 $x1017" "x1018 $x1018" "x1019 $x1019" "x101A $x101A" "x101B $x101B" "x101C $x101C" "x101D $x101D" "x1020 $x1020" "x1021 $x1021" "x1022 $x1022" "x1023 $x1023" "x1024 $x1024" "x2000 $x2000" "x2001 $x2001" "x2002 $x2002" "x2100 $x2100" "x2101 $x2101" "x2102 $x2102" "x2FFF $x2FFF" "x3013 $x3013" "x3014 $x3014" "x3017 $x3017" "x3018 $x3018" "x3019 $x3019" "x301A $x301A" "x301B $x301B" "x301C $x301C" "x301E $x301E" "x3021 $x3021" "x3024 $x3024" "x3025 $x3025" "x3026 $x3026" "x3027 $x3027" "x3047 $x3047" "x3062 $x3062" "x3063 $x3063" "x3064 $x3064" "x3065 $x3065" "x3066 $x3066" "x3067 $x3067" "x3068 $x3068" "x3069 $x3069" "x306A $x306A" "x306B $x306B" "x306C $x306C" "x306D $x306D" "x3070 $x3070" "x3071 $x3071" "x3072 $x3072" "x3073 $x3073" "x3074 $x3074" "x3100 $x3100" "x3101 $x3101" "x3102 $x3102" "x3103 $x3103" "x3104 $x3104" "x3105 $x3105" "x3106 $x3106" "x3107 $x3107" "x3108 $x3108" "x3109 $x3109" "x310A $x310A" "x310B $x310B" "x310C $x310C" "x310D $x310D" "x310E $x310E" "x310F $x310F" "x3110 $x3110" "x3111 $x3111" "x3112 $x3112" "x3113 $x3113" "x3200 $x3200" "x3201 $x3201" "x3202 $x3202" "x3203 $x3203" "x3204 $x3204" "x3205 $x3205" "x3300 $x3300" "x7E00 $x7E00" "x7E01 $x7E01" "x7E02 $x7E02" "x7E03 $x7E03" "x7E04 $x7E04" "x7E05 $x7E05" "x7E06 $x7E06" "x7E07 $x7E07" "x7E08 $x7E08" "x7E09 $x7E09" "x7E0A $x7E0A" "x7E0B $x7E0B" "x7E0C $x7E0C" "x7E0D $x7E0D" "x7E0E $x7E0E" "x7E0F $x7E0F" "x7E10 $x7E10" "x7E11 $x7E11"; do
	
		set $i
	
		if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt ] ; then
	       		$grep -i error'='0$1"\." ./$fileName/LSI_Products/3ware/3ware_driver_x04_9000_EVENT.txt > ./$fileName/LSI_Products/3ware/3ware_driver_Ext_AEN_Error_$1_$2.txt
	
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_Ext_AEN_Error_$1_$2.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_Ext_AEN_Error_$1_$2.txt >> ./misc_output.txt 2>&1 ; fi
	
		fi
	
	done
	###########################################################################################################################
	# 0x06 Driver Errors from 9.0, 9.4.1.2 & 9.5.1 3w-9xxx.c - Need to create a new section for 3w-sas.c driver, different errors
	###########################################################################################################################
	x0001="Found_unaligned_address_during_AEN_drain"
	x0002="Error_posting_request_sense"
	x0003="No_valid_response_while_draining_AEN_queue"
	x0004="Post_failed_while_reading_AEN_queue"
	x0005="Memory_allocation_failed"
	x0006="Failed_to_allocate_correctly_aligned_memory"
	x0007="Initconnection_failed_while_checking_SRL"
	x000A="Initconnection_base_mode_failed_while_checking_SRL"
	x000B="Incompatible_firmware_on_controller"
	#x000C="Character_ioctl_0xXX_timed_out_resetting_card." # 9.4.1.2
	#x000C="PCI_Parity_Error_clearing" # 9.4.1.2
	x000C="Shared_x000C"
	x000D="PCI_Abort_clearing"
	x000E="Controller_Queue_Error_clearing"
	x0010="Microcontroller_Error_clearing"
	x0011="Failed_to_allocate_memory_for_firmware_flash"
	x0012="No_valid_response_while_flashing_firmware"
	x0013="No_valid_response_during_get_param"
	x0014="No_valid_response_during_hard_reset"
	x0015="No_valid_response_during_init_connection"
	x0016="Command_packet_memory_allocation_failed"
	x0017="Generic_memory_allocation_failed"
	x0018="Event_info_memory_allocation_failed"
	x0019="Found_request_id_that_wasnt_pending"
	x001A="Received_a_request_id_that_wasnt_posted"
	x001B="Error_completing_AEN_during_attention_interrupt"
	x001C="Failed_to_map_scatter_gather_list"
	x001D="Failed_to_map_page"
	x001E="Found_unexpected_request_id_while_polling_for_response"
	x001F="Microcontroller_not_ready_during_reset_sequence"
	x0020="Response_queue_empty_failed_during_reset_sequence"
	x0021="Compatibility_check_failed_during_reset_sequence"
	x0022="AEN_drain_failed_during_reset_sequence"
	x0023="Failed_to_set_dma_mask"
	x0024="Failed_to_allocate_memory_for_device_extension"
	x0025="Failed_to_initialize_device_extension"
	x0026="Failed_to_get_mem_region"
	x0027="scsi_add_host_failed"
	x0028="Bad_scsi_host_data"
	x0029="Failed_to_register_character_device"
	x002B="Controller_reset_failed_during_scsi_host_reset"
	#x002C="Command_0xXX_timed_out_resetting_card" # 9.4.1.2
	#x002C="No_device_extension_for_proc_operation" # 9.0
	x002C="Shared_x002C" 
	x002D="Found_unaligned_address_during_execute_scsi"
	x002E="Found_unaligned_sgl_address_during_execute_scsi"
	x002F="Found_unaligned_sgl_address_during_internal_post"
	x0030="Error_requesting_IRQ"
	x0031="Connection_shutdown_failed"
	x0032="Firmware_and_driver_incompatibility_please_upgrade_firmware"
	x0033="Firmware_and_driver_incompatibility_please_upgrade_driver"
	x0034="Failed_to_enable_pci_device"
	x0035="Failed_to_ioremap"
	x0036="Response_queue_large_empty_failed_during_reset_sequence"
	x0037="Character_ioctl_timed_out_resetting_card"
	
	for i in "x0001 $x0001" "x0002 $x0002" "x0003 $x0003" "x0004 $x0004" "x0005 $x0005" "x0006 $x0006" "x0007 $x0007" "x000A $x000A" "x000B $x000B" "x000C $x000C" "x000D $x000D" "x000E $x000E" "x0010 $x0010" "x0011 $x0011" "x0012 $x0012" "x0013 $x0013" "x0014 $x0014" "x0015 $x0015" "x0016 $x0016" "x0017 $x0017" "x0018 $x0018" "x0019 $x0019" "x001A $x001A" "x001B $x001B" "x001C $x001C" "x001D $x001D" "x001E $x001E" "x001F $x001F" "x0020 $x0020" "x0021 $x0021" "x0022 $x0022" "x0023 $x0023" "x0024 $x0024" "x0025 $x0025" "x0026 $x0026" "x0027 $x0027" "x0028 $x0028" "x0029 $x0029" "x002B $x002B" "x002C $x002C" "x002D $x002D" "x002E $x002E" "x002F $x002F" "x0030 $x0030" "x0031 $x0031" "x0032 $x0032" "x0033 $x0033" "x0034 $x0034" "x0035 $x0035" "x0036 $x0036" "x0037 $x0037" ; do
	
		set $i
#Note: Appending to file rather than creating (>> vs >) due to duplicate error codes 0xc & 0x2c.
	
		if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x06_9000_DRIVER.txt ] ; then
		
			$grep -i $1 ./$fileName/LSI_Products/3ware/3ware_driver_x06_9000_DRIVER.txt >> ./$fileName/LSI_Products/3ware/3ware_driver_x06_$1_$2.txt
		
			if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_x06_$1_$2.txt ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_x06_$1_$2.txt >> ./misc_output.txt 2>&1 ; fi
		fi
		
		if [ -f ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_9000_DRIVER.txt ] ; then
		
		       	$grep -i $1 ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_9000_DRIVER.txt >> ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_$1_$2.txt
		
			if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_$1_$2.txt ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_$1_$2.txt >> ./misc_output.txt 2>&1 ; fi
		
		fi
	
	done



	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_Shared_x000C.txt ] ; then
		grep -i "Character ioctl" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_Shared_x000C.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_Character_ioctl_0xXX_timed_out_resetting_card.txt
		grep -i "PCI Parity Error" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_Shared_x000C.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_PCI_Parity_Error_clearing.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x06_x000C_Shared_x000C.txt
	fi
	
	
	if [ -f ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt ] ; then
		grep -i "resetting card" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Command_0xXX_timed_out_resetting_card.txt
		grep -i "No device extension" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_No_device_extension_for_proc_operation.txt
		rm ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt
	fi
	
	for i in _x002C_Command_0xXX_timed_out_resetting_card.txt _x002C_No_device_extension_for_proc_operation.txt _x000C_Character_ioctl_0xXX_timed_out_resetting_card.txt _x000C_PCI_Parity_Error_clearing.txt; do
		if [ ! -s ./$fileName/LSI_Products/3ware/3ware_driver_x06$i ] ; then rm ./$fileName/LSI_Products/3ware/3ware_driver_x06$i >> ./misc_output.txt 2>&1 ; fi
	done
	
	if [ -f ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x000C_Shared_x000C.txt ] ; then
		grep -i "Character ioctl" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x000C_Character_ioctl_0xXX_timed_out_resetting_card.txt
		grep -i "PCI Parity Error" ./$fileName/LSI_Products/3ware/3ware_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x000C_PCI_Parity_Error_clearing.txt
		rm ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x000C_Shared_x000C.txt
	fi
	
	
	if [ -f ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_Shared_x002C.txt ] ; then
		grep -i "resetting card" ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_Command_0xXX_timed_out_resetting_card.txt
		grep -i "No device extension" ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_Shared_x002C.txt > ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_No_device_extension_for_proc_operation.txt
		rm ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06_x002C_Shared_x002C.txt
	fi
	
	for i in _x002C_Command_0xXX_timed_out_resetting_card.txt _x002C_No_device_extension_for_proc_operation.txt _x000C_Character_ioctl_0xXX_timed_out_resetting_card.txt _x000C_PCI_Parity_Error_clearing.txt; do
		if [ ! -s ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06$i ] ; then rm ./$fileName/LSI_Products/3ware/OS_Disk_driver_x06$i >> ./misc_output.txt 2>&1 ; fi
	done
fi

		

if [ "$TWGETSKIPEXPANDER" != "YES" ] ; then
	###########################################################################################################################
	# LSI Expander Check Start
	###########################################################################################################################
	
	###########################################################################################################################
	# Set Gen 1 & 2 EXPANDER_INFO=YES If Linux/Solaris get LSI Expander Info
	###########################################################################################################################
	
	if [ "$OS_LSI" = "linux" ] ; then
		EXPANDER_INFO=YES
	fi
	
	if [ "$OS_LSI" = "solaris" ] ; then
		EXPANDER_INFO=YES
	fi
	
	if [ "$TWSKIPXUTILS" = "NO" ] ; then
	
		if [ "$EXPANDER_INFO" = "YES" ] ; then
	
			#Gen 1 #'s
			./xu -i get avail > ./$fileName/script_workspace/xu_-i_get_avail.txt  2>&1
			./xu -i get avail | $grepA -i SASx | $grepA -E -o '[0-9,A-F]+:[0-9,A-F]+' | cut -b 1-8,10-17 > ./$fileName/script_workspace/Gen1_EXP_SAS_Address.txt  2>&1
	
			# Make sure at least 1 SAS Address is identified
			if [ -s ./$fileName/script_workspace/Gen1_EXP_SAS_Address.txt ] ; then
				NO_xu_GEN1_or_2_EXPs=NO
				echo "Collecting LSI Gen 1 Expander Information..."
				if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/3Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/3Gb; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/3Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/3Gb/Details; fi
				./xu -i get avail > ./$fileName/LSI_Products/Expander/3Gb/get_avail.txt 2>&1
				for i in `cat ./$fileName/script_workspace/Gen1_EXP_SAS_Address.txt` ; do 
					for l in 0 1 2 3 ; do
						#cho ".................................................||................................................."
						echo ".............................................Region $l..............................................." >> ./$fileName/LSI_Products/Expander/3Gb/${i}_get_ver.txt
						./xu -i $i get ver $l >> ./$fileName/LSI_Products/Expander/3Gb/${i}_get_ver.txt  2>&1
					done
					for l in 0 1 2 ; do
						#cho ".................................................||................................................."
						./xu -i $i up fw ./$fileName/LSI_Products/Expander/3Gb/${i}_region-${l}_up_fw.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					for l in 3 ; do
						#cho ".................................................||................................................."
						./xu -i $i up mfg ./$fileName/LSI_Products/Expander/3Gb/${i}_region-${l}_up_mfg.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					if [ -f ./$fileName/script_workspace/${i}_coredump_region.txt ] ; then 
						for l in $( cat ./$fileName/script_workspace/${i}_coredump_region.txt ) ; do
							#cho ".................................................||................................................."
							./xu -i $i up cfg ./$fileName/LSI_Products/Expander/3Gb/${i}_region-${l}_COREDUMP.bin ${l} -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
						done
					fi
	
					
					./xu -i $i get exp > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_exp.txt  2>&1
					./xu -i $i get region > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_region.txt  2>&1
					./xu -i $i get attach > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_attach.txt  2>&1
					./xu -i $i get port > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_port.txt  2>&1
					./xu -i $i get phy > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_phy.txt  2>&1
					#Causes segfault on gen1
					#./xu -i $i get trace > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_trace.txt  2>&1				
					./xu -i $i get log > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_log.txt  2>&1
					./xu -i $i get zgrp > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_zgrp.txt  2>&1
					./xu -i $i get zperm > ./$fileName/LSI_Products/Expander/3Gb/${i}_get_zperm.txt  2>&1
	
	
					if [ ! -f ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt ] ; then 
						echo $fileName > ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt
						#cho ".................................................||................................................."
						echo ".............................................get avail.............................................." >> ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt
						cat ./$fileName/LSI_Products/Expander/3Gb/get_avail.txt >> ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt 
					fi
					
					for j in ${i}_get_exp ${i}_get_ver ${i}_get_region ${i}_get_attach ${i}_get_port ${i}_get_phy ${i}_get_trace ${i}_get_log ${i}_get_zgrp ${i}_get_zperm ; do
				
						if [ -e ./$fileName/LSI_Products/Expander/3Gb/$j.txt ] ; then
							#cho ".................................................||................................................."
							echo "...........................................$j............................................" >> ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt
							cat ./$fileName/LSI_Products/Expander/3Gb/$j.txt >> ./$fileName/LSI_Products/Expander/3Gb/${i}_xu.txt 
	
							if [ -e ./$fileName/LSI_Products/Expander/3Gb/$j.txt ] ; then 
								mv ./$fileName/LSI_Products/Expander/3Gb/${j}.txt ./$fileName/LSI_Products/Expander/3Gb/Details/${j}.txt
							fi
						fi
					done
				done			
			fi	
	
			#Gen 2 #'s
			./xu -i get avail > ./$fileName/script_workspace/xu_-i_get_avail.txt  2>&1
			./xu -i get avail | $grepA -i SAS2x | $grepA -E -o '[0-9,A-F]+:[0-9,A-F]+' | cut -b 1-8,10-17 > ./$fileName/script_workspace/Gen2_EXP_SAS_Address.txt  2>&1
	
			# Make sure at least 1 SAS Address is identified
			if [ -s ./$fileName/script_workspace/Gen2_EXP_SAS_Address.txt ] ; then
				echo "Collecting LSI Gen 2 Expander Information..."
				NO_xu_GEN1_or_2_EXPs=NO
				if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/6Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/6Gb; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/6Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/6Gb/Details; fi
				./xu -i get avail > ./$fileName/LSI_Products/Expander/6Gb/get_avail.txt 2>&1
				for i in `cat ./$fileName/script_workspace/Gen2_EXP_SAS_Address.txt` ; do 
					for l in 0 1 2 3 ; do
						#cho ".................................................||................................................."
						echo ".............................................Region $l..............................................." >> ./$fileName/LSI_Products/Expander/6Gb/${i}_get_ver.txt
						./xu -i $i get ver $l >> ./$fileName/LSI_Products/Expander/6Gb/${i}_get_ver.txt  2>&1
					done
					for l in 0 1 2 ; do
						#cho ".................................................||................................................."
						./xu -i $i up fw ./$fileName/LSI_Products/Expander/6Gb/${i}_region-${l}_up_fw.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					for l in 3 ; do
						#cho ".................................................||................................................."
						./xu -i $i up mfg ./$fileName/LSI_Products/Expander/6Gb/${i}_region-${l}_up_mfg.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					if [ -f ./$fileName/script_workspace/${i}_coredump_region.txt ] ; then 
						for l in $( cat ./$fileName/script_workspace/${i}_coredump_region.txt ) ; do
							#cho ".................................................||................................................."
							./xu -i $i up cfg ./$fileName/LSI_Products/Expander/6Gb/${i}_region-${l}_COREDUMP.bin ${l} -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
						done
					fi
	
					
					./xu -i $i get exp > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_exp.txt  2>&1
					./xu -i $i get region > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_region.txt  2>&1
					./xu -i $i get attach > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_attach.txt  2>&1
					./xu -i $i get port > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_port.txt  2>&1
					./xu -i $i get phy > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_phy.txt  2>&1
					./xu -i $i get trace > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_trace.txt  2>&1				
					./xu -i $i get log > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_log.txt  2>&1
					./xu -i $i get zgrp > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_zgrp.txt  2>&1
					./xu -i $i get zperm > ./$fileName/LSI_Products/Expander/6Gb/${i}_get_zperm.txt  2>&1
	
	
					if [ ! -f ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt ] ; then 
						echo $fileName > ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt
						#cho ".................................................||................................................."
						echo ".............................................get avail.............................................." >> ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt
						cat ./$fileName/LSI_Products/Expander/6Gb/get_avail.txt >> ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt 
					fi
					
					for j in ${i}_get_exp ${i}_get_ver ${i}_get_region ${i}_get_attach ${i}_get_port ${i}_get_phy ${i}_get_trace ${i}_get_log ${i}_get_zgrp ${i}_get_zperm ; do
				
						if [ -e ./$fileName/LSI_Products/Expander/6Gb/$j.txt ] ; then
							#cho ".................................................||................................................."
							echo "...........................................$j............................................" >> ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt
							cat ./$fileName/LSI_Products/Expander/6Gb/$j.txt >> ./$fileName/LSI_Products/Expander/6Gb/${i}_xu.txt 
	
							if [ -e ./$fileName/LSI_Products/Expander/6Gb/$j.txt ] ; then 
								mv ./$fileName/LSI_Products/Expander/6Gb/${j}.txt ./$fileName/LSI_Products/Expander/6Gb/Details/${j}.txt
							fi
						fi
					done
				done			
			fi
		fi
	fi	
	
	###########################################################################################################################
	# Set Gen 3 EXPANDER_INFO=YES If Linux/Solaris/FreeBSD get LSI Expander Info
	###########################################################################################################################
	
	
	if [ "$OS_LSI" = "linux" ] ; then
		if [ "$Arch32or64" = "64" ] ; then
			EXPANDER_INFO=YES
		fi
	fi
	
	if [ "$OS_LSI" = "solaris" ] ; then
		EXPANDER_INFO=YES
	fi
	
	if [ "$OS_LSI" = "freebsd" ] ; then
		EXPANDER_INFO=YES
	fi
	
	if [ "$TWSKIPXUTILS" = "NO" ] ; then
	
		if [ "$EXPANDER_INFO" = "YES" ] ; then
	
	
			#Gen 3 #'s
			./g3xu -i get avail > ./$fileName/script_workspace/g3xu_-i_get_avail.txt  2>&1
			./g3xu -i get avail | $grepA -i -e SAS_3X -e SAS_35X | $grepA -E -o '[0-9,A-F]+:[0-9,A-F]+' | cut -b 1-8,10-17 > ./$fileName/script_workspace/Gen3_EXP_SAS_Address.txt  2>&1
	
			# Make sure at least 1 SAS Address is identified
			if [ -s ./$fileName/script_workspace/Gen3_EXP_SAS_Address.txt ] ; then
				echo "Collecting LSI Gen 3 Expander Information..."
				NO_g3xu_GEN3_EXPs=NO
				if [ ! -d ./$fileName/LSI_Products/Expander ] ; then mkdir ./$fileName/LSI_Products/Expander; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/12Gb ] ; then mkdir ./$fileName/LSI_Products/Expander/12Gb; fi
				if [ ! -d ./$fileName/LSI_Products/Expander/12Gb/Details ] ; then mkdir ./$fileName/LSI_Products/Expander/12Gb/Details; fi
				./g3xu -i get avail > ./$fileName/LSI_Products/Expander/12Gb/get_avail.txt 2>&1
				for i in `cat ./$fileName/script_workspace/Gen3_EXP_SAS_Address.txt` ; do 
					for l in 0 1 2 3 ; do
						#cho ".................................................||................................................."
						echo ".............................................Region $l..............................................." >> ./$fileName/LSI_Products/Expander/12Gb/${i}_get_ver.txt
						./g3xu -i $i get ver $l >> ./$fileName/LSI_Products/Expander/12Gb/${i}_get_ver.txt  2>&1
					done
					for l in 0 1 2 ; do
						#cho ".................................................||................................................."
						./g3xu -i $i up fw ./$fileName/LSI_Products/Expander/12Gb/${i}_region-${l}_up_fw.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					for l in 3 ; do
						#cho ".................................................||................................................."
						./g3xu -i $i up mfg ./$fileName/LSI_Products/Expander/12Gb/${i}_region-${l}_up_mfg.bin $l -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
					done
					if [ -f ./$fileName/script_workspace/${i}_coredump_region.txt ] ; then 
						for l in $( cat ./$fileName/script_workspace/${i}_coredump_region.txt ) ; do
							#cho ".................................................||................................................."
							./g3xu -i $i up cfg ./$fileName/LSI_Products/Expander/12Gb/${i}_region-${l}_COREDUMP.bin ${l} -y  >>./$fileName/script_workspace/lsiget_errorlog.txt 2>&1
						done
					fi
	
					
					./g3xu -i $i get exp > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_exp.txt  2>&1
					./g3xu -i $i get region > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_region.txt  2>&1
					./g3xu -i $i get attach > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_attach.txt  2>&1
					./g3xu -i $i get port > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_port.txt  2>&1
					./g3xu -i $i get phy > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_phy.txt  2>&1
					./g3xu -i $i get trace > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_trace.txt  2>&1				
					./g3xu -i $i get log > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_log.txt  2>&1
					./g3xu -i $i get zgrp > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_zgrp.txt  2>&1
					./g3xu -i $i get zperm > ./$fileName/LSI_Products/Expander/12Gb/${i}_get_zperm.txt  2>&1
	
	
					if [ ! -f ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt ] ; then 
						echo $fileName > ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt
						#cho ".................................................||................................................."
						echo ".............................................get avail.............................................." >> ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt
						cat ./$fileName/LSI_Products/Expander/12Gb/get_avail.txt >> ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt 
					fi
					
					for j in ${i}_get_exp ${i}_get_ver ${i}_get_region ${i}_get_attach ${i}_get_port ${i}_get_phy ${i}_get_trace ${i}_get_log ${i}_get_zgrp ${i}_get_zperm ; do
				
						if [ -e ./$fileName/LSI_Products/Expander/12Gb/$j.txt ] ; then
							#cho ".................................................||................................................."
							echo "...........................................$j............................................" >> ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt
							cat ./$fileName/LSI_Products/Expander/12Gb/$j.txt >> ./$fileName/LSI_Products/Expander/12Gb/${i}_g3xu.txt 
	
							if [ -e ./$fileName/LSI_Products/Expander/12Gb/$j.txt ] ; then 
								mv ./$fileName/LSI_Products/Expander/12Gb/${j}.txt ./$fileName/LSI_Products/Expander/12Gb/Details/${j}.txt
							fi
						fi
					done
				done			
			fi
		fi
	fi 
fi
	


###########################################################################################################################
# Performance Team Summary File
###########################################################################################################################

if [ -f ./$fileName/Comment.txt ] ; then

	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "........................................Comment/Config Info........................................." >> ./$fileName/Summary.txt
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat  ./$fileName/Comment.txt >> ./$fileName/Summary.txt 
fi
	
if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	#3ware
	for i in $($CLI_LOCATION$CLI_NAME show | $grep ^c | cut -b 2-3); do #Support for Controller IDs 0-99
		if [ -f ./$fileName/LSI_Products/3ware/Controller_C$i.txt ] ; then
			#cho ".................................................||................................................."
			echo "####################################################################################################" >> ./$fileName/Summary.txt
			echo ".....................................3ware Controller C$i Info......................................." >> ./$fileName/Summary.txt
			echo "####################################################################################################" >> ./$fileName/Summary.txt
			cat ./$fileName/LSI_Products/3ware/Controller_C$i.txt >> ./$fileName/Summary.txt 
		fi
	done
fi

if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	#MegaRAID
	for i in $($MCLI_LOCATION$MCLI_NAME show | sed '1,/---/d' | sed '1,/---/d' | sed '/---/q' | sed '/---/d' | cut -b 1-3); do #Support for Controller IDs 0-199
		if [ -f ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt ] ; then
			#cho ".................................................||................................................."
			echo "####################################################################################################" >> ./$fileName/Summary.txt
			echo "...................................MegaRAID Controller C$i Info......................................" >> ./$fileName/Summary.txt
			echo "####################################################################################################" >> ./$fileName/Summary.txt
			cat ./$fileName/LSI_Products/MegaRAID/storcli/Controller_C$i.txt >> ./$fileName/Summary.txt 
		fi
	done
fi

if [ "$TWGETSKIPHBA" != "YES" ] ; then
	if [ "$NO_LSI_HBAs" != "YES" ] ; then 
		#HBA
		for i in $(./$LSUT_NAME 0 2>> ./misc_output.txt | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
			if [ -f ./$fileName/LSI_Products/HBA/properties_hba$i.txt ] ; then
				#cho ".................................................||................................................."
				echo "####################################################################################################" >> ./$fileName/Summary.txt
				echo "............................................HBA $i Info.............................................." >> ./$fileName/Summary.txt
				echo "####################################################################################################" >> ./$fileName/Summary.txt
				cat ./$fileName/LSI_Products/HBA/properties_hba$i.txt >> ./$fileName/Summary.txt 
			fi
		done
	fi
fi

if [ "$TWGETSKIPEXPANDER" != "YES" ] ; then
	#Expanders
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "............................................Gen 1 xutil............................................." >> ./$fileName/Summary.txt
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	if [ -s ./$fileName/script_workspace/Gen1_EXP_SAS_Address.txt ] ; then
		#cho ".................................................||................................................."
		echo "............................................xutil output............................................" >> ./$fileName/Summary.txt
		for i in `cat ./$fileName/script_workspace/Gen1_EXP_SAS_Address.txt` ; do 
		if [ -e ./$fileName/LSI_Products/Expander/3Gb/xu_$i.txt ] ; then cat ./$fileName/LSI_Products/Expander/3Gb/xu_$i.txt >> ./$fileName/Summary.txt; fi
		done
	fi
	
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "............................................Gen 2 xutil............................................." >> ./$fileName/Summary.txt
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	if [ -s ./$fileName/script_workspace/Gen2_EXP_SAS_Address.txt ] ; then
		#cho ".................................................||................................................."
		echo "............................................xutil output............................................" >> ./$fileName/Summary.txt
		for i in `cat ./$fileName/script_workspace/Gen2_EXP_SAS_Address.txt` ; do 
		if [ -e ./$fileName/LSI_Products/Expander/6Gb/xu_$i.txt ] ; then cat ./$fileName/LSI_Products/Expander/6Gb/xu_$i.txt >> ./$fileName/Summary.txt; fi
		done
	fi
	
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "............................................Gen 3 xutil............................................." >> ./$fileName/Summary.txt
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	if [ -s ./$fileName/script_workspace/Gen3_EXP_SAS_Address.txt ] ; then
		#cho ".................................................||................................................."
		echo "...........................................g3xutil output..........................................." >> ./$fileName/Summary.txt
		for i in `cat ./$fileName/script_workspace/Gen3_EXP_SAS_Address.txt` ; do 
		if [ -e ./$fileName/LSI_Products/Expander/12Gb/g3xu_$i.txt ] ; then cat ./$fileName/LSI_Products/Expander/12Gb/g3xu_$i.txt >> ./$fileName/Summary.txt; fi
		done
	fi
	
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".................................Expander cli debuginfo - 3/6/12Gb.................................." >> ./$fileName/Summary.txt
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	
	if [ "$NO_LSI_HBAs" != "YES" ] ; then 
	
		for i in $(./$LSUT_NAME 0 2>> ./misc_output.txt | awk 'BEGIN{prt=0}{if (prt==1) print $0; else if ($3=="Chip") prt=1}' | $grep LSI | cut -d. -f1); do # Support for unlimited HBAs
		       	for j in $(./$LSUT_NAME -p $i -a 64,,,0,0, 2>> ./misc_output.txt | grep EnclServ | grep LSI | cut -d. -f1); do
	
				if [ -e ./$fileName/LSI_Products/Expander/3Gb/debuginfo_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo "..................................3Gb debuginfo_hba${i}exp${j}.txt.................................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					cat ./$fileName/LSI_Products/Expander/3Gb/debuginfo_hba${i}exp${j}.txt >> ./$fileName/Summary.txt
				fi
	
				if [ -e ./$fileName/LSI_Products/Expander/3Gb/debuginfo_manual_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo "...............................3Gb debuginfo_manual_hba${i}exp${j}.txt.............................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
				 	cat ./$fileName/LSI_Products/Expander/3Gb/debuginfo_manual_hba${i}exp${j}.txt >> ./$fileName/Summary.txt 
				fi
	
				if [ -e ./$fileName/LSI_Products/Expander/6Gb/debuginfo_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo "..................................6Gb debuginfo_hba${i}exp${j}.txt.................................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
				 	cat ./$fileName/LSI_Products/Expander/6Gb/debuginfo_hba${i}exp${j}.txt >> ./$fileName/Summary.txt
		 		fi
	
				if [ -e ./$fileName/LSI_Products/Expander/6Gb/debuginfo_manual_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo "...............................6Gb debuginfo_manual_hba${i}exp${j}.txt.............................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					cat ./$fileName/LSI_Products/Expander/6Gb/debuginfo_manual_hba${i}exp${j}.txt >> ./$fileName/Summary.txt
				fi
	
				if [ -e ./$fileName/LSI_Products/Expander/12Gb/debuginfo_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo ".................................12Gb debuginfo_hba${i}exp${j}.txt.................................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					cat ./$fileName/LSI_Products/Expander/12Gb/debuginfo_hba${i}exp${j}.txt >> ./$fileName/Summary.txt
				fi
	
				if [ -e ./$fileName/LSI_Products/Expander/12Gb/debuginfo_manual_hba${i}exp${j}.txt ] ; then
					#cho ".................................................||................................................."
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					echo "..............................12Gb debuginfo_manual_hba${i}exp${j}.txt.............................." >> ./$fileName/Summary.txt
					echo "####################################################################################################" >> ./$fileName/Summary.txt
					cat ./$fileName/LSI_Products/Expander/12Gb/debuginfo_manual_hba${i}exp${j}.txt >> ./$fileName/Summary.txt
				fi
			done
		done	
	fi
fi



if [ -f ./$fileName/uname-a.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................uname -a..............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/uname-a.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/uptime.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...............................................uptime..............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/uptime.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lsmod.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...............................................lsmod................................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lsmod.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lspci/lspci-vvv.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................lspci -vvv............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lspci/lspci-vvv.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/scanpci-v.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................scanpci -v............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/scanpci-v.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/rpm-q-a-i.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...........................................rpm -q -a -i............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/rpm-q-a-i.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/df-hat.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "..............................................df -hat..............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/df-hat.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/top-b-n1.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................top -b -n1............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/top-b-n1.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/ps-ea.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...............................................ps -ea..............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/ps-ea.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/fdisk-l.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "..............................................fdisk -l.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/fdisk-l.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/fdisk-lu.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................fdisk -lu.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/fdisk-lu.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/vgs.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".................................................vgs................................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/vgs.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/lvs.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".................................................lvs................................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/lvs.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/pvs.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".................................................pvs................................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/pvs.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/lvdisplay.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "..............................................lvdisplay............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/lvdisplay.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/pvdisplay.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "..............................................pvdisplay............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/pvdisplay.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lvm/lvm.conf ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "..............................................lvm.conf............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lvm/lvm.conf >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/sysctl-a.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................sysctl -a............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/sysctl-a.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/vmstat.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...............................................vmstat..............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/vmstat.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/dmidecode.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................dmidecode.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/dmidecode.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/biosdecode.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo ".............................................biosdecode............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/biosdecode.txt >> ./$fileName/Summary.txt 
fi


if [ -f ./$fileName/lsscsi-all.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "....................................lsscsi -c -d -g -H -k -l -v....................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lsscsi-all.txt >> ./$fileName/Summary.txt 
fi


if [ -f ./$fileName/showsel.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...............................................showsel.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/showsel.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/lshw.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "................................................lshw.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/lshw.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/esxcli_output.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...........................................esxcli output............................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/esxcli_output.txt >> ./$fileName/Summary.txt 
fi

#VMWare
if [ -f ./$fileName/esxcli_output.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...........................................esxcli output............................................" >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/esxcli_output.txt >> ./$fileName/Summary.txt 
fi

#FreeBSD
if [ -f ./$fileName/devinfo-ruv.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "...........................................devinfo -ruv............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/devinfo-ruv.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/script_workspace/df.out ] ; then
	for i in $( cat ./$fileName/script_workspace/df.out ); do
		#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
		echo "...........................................dumpfs-m_$i.txt..........................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
		cat ./$fileName/dumpfs-m_$i.txt >> ./$fileName/Summary.txt 
	done
fi

if [ -f ./$fileName/sysctl-ad.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "............................................sysctl -ad.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/sysctl-ad.txt >> ./$fileName/Summary.txt 
fi

if [ -f ./$fileName/sysctl-ad.txt ] ; then
	#cho ".................................................||................................................."
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	echo "............................................sysctl -ad.............................................." >> ./$fileName/Summary.txt 
	echo "####################################################################################################" >> ./$fileName/Summary.txt
	cat ./$fileName/sysctl-ad.txt >> ./$fileName/Summary.txt 
fi

if [ "$TWGETSKIP3WARE" != "YES" ] ; then
	###########################################################################################################################
	###########################################################################################################################
	# Monitor Mode - 
	###########################################################################################################################
	###########################################################################################################################
	if [ "$tw_cli_Functional" != "NO" ] ; then
		if [ "$TWGETMONITORMODE" = "MONITOR" ] ; then
			if [ ! -d ./$fileName/LSI_Products/3ware/MONITOR ] ; then 
				mkdir ./$fileName/LSI_Products/3ware/MONITOR 
				mkdir ./$fileName/LSI_Products/3ware/MONITOR/work_space
			fi
	
			# No Carriage Return embedded on end of line!
			echo  ------------------------------------------------------------------------------ > ./$fileName/LSI_Products/3ware/MONITOR/work_space/no_alarms_check.txt
			###########################################################################################################################
			# Monitor Mode - Blank added to EOL in monitor_point_C$i.txt
			# Getting latest AEN "monitor_point" to compare new AEN's against
			# Verify monitor_point_C$i.txt is valid
			###########################################################################################################################
			
			
			for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
				$CLI_LOCATION$CLI_NAME /c$i show alarms | sed '/^$/d' | tail -n 1  > ./$fileName/LSI_Products/3ware/MONITOR/work_space/monitor_point_C$i.txt
				cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/monitor_point_C$i.txt | while read j ; do
					cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/no_alarms_check.txt | while read k ; do
						if [ "$j" = "$k" ] ; then echo "There were no alarms" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/monitor_point_C$i.txt ; fi
					done
				done
			done
			
			# NOT_FIRST_RUN
			while [ "$GOTO_STOP_MONITOR" != "YES" ] ;  do
			
				# Default 60 Second wait for checking Alarms/AENs with "show diag"
				sleep 60
				echo .
				echo "lsigetlin.sh Monitor Mode - Command line \"$TWCMDLINE\"   `if [ "$VMWARE_SUPPORTED" != "YES" ] ; then 
												 	date '+%H:%M:%S.%N' 
												 fi	
												 if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
													date '+%H:%M:%S' 
												 fi`"
				echo ""
				echo "Saves Internal Controller Log daily and on specific errors."
				echo "Exits after Controller Reset/Degraded Array/Rebuild Started AENs"
				echo ""
				echo "To terminate hit CTRL-C"
				echo ""
				echo 'If manually terminated and you still need to send the output to Support/your FAE, run;'
				echo ""
				echo "tar cf $fileName.tar ./$fileName"
				echo "gzip -9 $fileName.tar"
				echo ""
				echo "Then send just the file $fileName.tar.gz to Support/your FAE."
				echo ""
				echo "If Monitor Mode completes, run a standard capture and include that as well."
				echo .
				echo .
				echo .
				
				###########################################################################################################################
				# Monitor Mode - Blank added to EOL in alarms_C$i.txt
				# Getting latest AENs to compare "monitor_point" against, changed order to get latest AENs on top
				###########################################################################################################################
				for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
					$CLI_LOCATION$CLI_NAME /c$i show alarms reverse > ./$fileName/LSI_Products/3ware/MONITOR/work_space/alarms_C$i.txt
				done
				###########################################################################################################################
				# Monitor Mode - Blank match j & k
				# Comparing "monitor_point" and AENs, printing new AENs to new_alarms_C$i.txt 2>&1 
				###########################################################################################################################
				for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
					if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt ; fi
					cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/monitor_point_C$i.txt | while read j ; do
						cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/alarms_C$i.txt | while read k ; do
							if [ "$j" = "$k" ] ; then break ; else echo ${k} >> ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt ;	fi
						done
					done
				done
				###########################################################################################################################
				# Monitor Mode - Blank match j & k
				# Comparing "monitor_point" and AENs, printing new AENs to AnyError_new_alarms_C$i.txt 2>&1 
				# "Couldn't strip blank spaces on the end of the AEN strings to use one set of AEN files."
				###########################################################################################################################
				for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
					if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt ; fi
				done
				
				for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
					if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_monitor_point_C$i.txt ] ; then 
						cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_monitor_point_C$i.txt | while read j ; do
							cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/alarms_C$i.txt | while read k ; do
								if [ "$j" = "$k" ] ; then break ; else echo ${k} >> ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt ; fi
							done
						done
					fi
				done
					
					
				###########################################################################################################################
				# Monitor Mode - 
				# Searching for 3 specific AENs to stop test also just do a "show diag" on any "ERROR" and continue.
				###########################################################################################################################
				for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
					grep "Controller reset occurred: resets=" ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt >> ./misc_output.txt 2>&1 
					if [ "$?" -eq "0" ] ; then
						echo "Controller reset occurred: resets=" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/ResetDegradeRebuild_C$i.txt
						GOTO_STOP_MONITOR=YES
					fi
					
					if [ "$GOTO_STOP_MONITOR" != "YES" ] ; then
						grep "Degraded unit: unit=" ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt >> ./misc_output.txt 2>&1
						if [ "$?" -eq "0" ] ; then
							echo "Degraded unit: unit=" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/ResetDegradeRebuild_C$i.txt
							GOTO_STOP_MONITOR=YES
						fi
					fi
					
					if [ "$GOTO_STOP_MONITOR" != "YES" ] ; then
						grep "Rebuild started:" ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt >> ./misc_output.txt 2>&1
						if [ "$?" -eq "0" ] ; then
							echo "Rebuild started:" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/ResetDegradeRebuild_C$i.txt
							GOTO_STOP_MONITOR=YES
						fi
					fi
				# return for controller #
				done
				
				if [ "$GOTO_STOP_MONITOR" != "YES" ] ; then
				
					for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
						if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ] ; then rm ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ; fi
						if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt ] ; then
							grep "]  ERROR" ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt >> ./misc_output.txt 2>&1
							if [ "$?" -eq "0" ] ; then echo "Dummy File - AnyError_new_alarms" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ; fi
						fi
						
						if [ ! -e ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_new_alarms_C$i.txt ] ; then
							grep "]  ERROR" ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt >> ./misc_output.txt 2>&1
							if [ "$?" -eq "0" ] ; then echo "Dummy File - new_alarms" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ; fi
						fi
					# return for controller #
					done
				
					###########################################################################################################################
					# Monitor Mode - Blank added to EOL in monitor_point_C$i.txt
					# Getting latest AEN "AnyError_monitor_point" to compare new AEN's against
					###########################################################################################################################
					for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
						if [ ! -e ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ] ; then
							$CLI_LOCATION$CLI_NAME /c$i show alarms | sed '/^$/d' | tail -n 1  > ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_monitor_point_C$i.txt
						fi
					done
					###########################################################################################################################
					# Verify AnyError_monitor_point_C$i.txt is valid
					###########################################################################################################################
					for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
						if [ ! -e ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ] ; then
							cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_monitor_point_C$i.txt | while read j ; do
								cat ./$fileName/LSI_Products/3ware/MONITOR/work_space/no_alarms_check.txt | while read k ; do
									if [ "$j" = "$k" ] ; then echo "There were no alarms" > ./$fileName/LSI_Products/3ware/MONITOR/work_space/AnyError_monitor_point_C$i.txt ; fi
								done
							done
						fi
					done
					###########################################################################################################################
					# Monitor Mode - 
					# log time/date stamps
					###########################################################################################################################
					mtodayDate=`date '+DATE:%m%d%y' | cut -d: -f2`
					mcurrentTime=`date '+TIME:%H%M%S' | cut -d: -f2`
					###########################################################################################################################
					# Monitor Mode - 
					# Default 60 Second Alarm Check wait * 60 Iterations = approx. 1 hour * 24 = approx. 1 day standard "show diag"
					# 60 * 24 = 1440 lines 
					###########################################################################################################################
					for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
						echo 1 >> ./$fileName/LSI_Products/3ware/MONITOR/work_space/Count_C$i.txt
						grep -n "1" ./$fileName/LSI_Products/3ware/MONITOR/work_space/Count_C$i.txt | $grep "1440:" >> ./misc_output.txt 2>&1
						if [ "$?" -eq "0" ] ; then 
							$CLI_LOCATION$CLI_NAME /c$i show diag > ./"$fileName"/MONITOR/daily_"$mtodayDate"_"$mcurrentTime"_diag_C${i}.txt 
							rm ./$fileName/LSI_Products/3ware/MONITOR/work_space/Count_C$i.txt
						fi
					done
					###########################################################################################################################
					# Monitor Mode - 
					# The "show diag" on any error, not including the main three.
					###########################################################################################################################
					for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
						if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/CREATEAnyErrorMONITORPOINT_C$i.txt ] ; then
							$CLI_LOCATION$CLI_NAME /c$i show diag > ./"$fileName"/MONITOR/AnyError_"$mtodayDate"_"$mcurrentTime"_diag_C${i}.txt 
							rm ./$fileName/LSI_Products/3ware/MONITOR/work_space/new_alarms_C$i.txt
						fi
					done
					
				# Return for "if [ "$GOTO_STOP_MONITOR" != "YES" ] ; then"
				fi
				
			# Return for "while "$GOTO_STOP_MONITOR" != "YES" ;  do"
			done
		
	
					
			###########################################################################################################################
			# Monitor Mode - 
			# The "show diag" on the main three AENs.
			###########################################################################################################################
			for i in `cat ./$fileName/script_workspace/controller_numbers.txt` ; do #Support for Controller IDs 0-99
				if [ -f ./$fileName/LSI_Products/3ware/MONITOR/work_space/ResetDegradeRebuild_C$i.txt ] ; then
					$CLI_LOCATION$CLI_NAME /c$i show diag > ./"$fileName"/MONITOR/ResetDegradeRebuild_"$mtodayDate"_"$mcurrentTime"_diag_C${i}.txt
					sleep 300
					$CLI_LOCATION$CLI_NAME /c$i show diag > ./"$fileName"/MONITOR/ResetDegradeRebuild_5min_"$mtodayDate"_"$mcurrentTime"_diag_C${i}.txt
				fi
			done
		
		# Return from "if [ "$TWGETMONITORMODE" = "MONITOR" ] ; then"
		fi
	# Return from "if [ "$tw_cli_Functional" != "NO" ] ; then"
	fi
fi

###########################################################################################################################
# Move unused utilities to Utils
###########################################################################################################################

if [ "$NO_lsut_LSI_HBAs" = "YES" ] ; then 
	if [ -f lsiutil ] ; then mv -f lsiutil Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_scl_EXP-IOC" = "YES" ] ; then 
	if [ -f scli ] ; then mv -f scli Utils >> ./misc_output.txt 2>&1 ; fi
fi

./sasflash -listall >> ./misc_output.txt 2>&1
if [ "$?" -ne "0" ] ; then mv -f sasflash Utils >> ./misc_output.txt 2>&1 ; fi
./sas2flash -listall >> ./misc_output.txt 2>&1
if [ "$?" -ne "0" ] ; then mv -f sas2flash Utils >> ./misc_output.txt 2>&1 ; fi
./sas3flash -listall >> ./misc_output.txt 2>&1
if [ "$?" -ne "0" ] ; then mv -f sas3flash Utils >> ./misc_output.txt 2>&1 ; fi
./flashoem -listall >> ./misc_output.txt 2>&1
if [ "$?" -ne "0" ] ; then
	./flashoem -ctype sas3 -listall >> ./misc_output.txt 2>&1
	if [ "$?" -ne "0" ] ; then mv -f flashoem Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_2ircu_HBAs" = "YES" ] ; then 
	if [ -f sas2ircu ] ; then mv -f sas2ircu Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_3ircu_HBAs" = "YES" ] ; then 
	if [ -f sas3ircu ] ; then mv -f sas3ircu Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_MR_Adps" = "YES" ] ; then 
	if [ -f $MCLI_NAME ] ; then mv -f $MCLI_NAME Utils  >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_MR_Adps" = "YES" ] ; then 
	if [ -f libstorelibir-2.so.14.07-0 ] ; then mv -f libstorelibir-2.so.14.07-0 Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_3Ware_Ctrls" = "YES" ] ; then 
	if [ -f tw_cli ] ; then mv -f tw_cli Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_xu_GEN1_or_2_EXPs" = "YES" ] ; then 
	if [ -f xutil ] ; then mv -f xutil Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_g3xu_GEN3_EXPs" = "YES" ] ; then 
	if [ -f g3xutil ] ; then mv -f g3xutil Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ "$NO_g3xu_GEN3_EXPs" = "YES" ] ; then 
	if [ -f bootstrapCobra.bin ] ; then mv -f bootstrapCobra.bin Utils >> ./misc_output.txt 2>&1 ; fi
fi
if [ -d ./$fileName/LSI_Products/MegaRAID ] ; then
	if [ ! -d ./$fileName/LSI_Products/MegaRAID/Notes ] ; then mkdir ./$fileName/LSI_Products/MegaRAID/Notes ; fi
	if [ -f MegaRAID_Terminology.txt ] ; then mv -f MegaRAID_Terminology.txt ./$fileName/LSI_Products/MegaRAID/Notes >> ./misc_output.txt 2>&1 ; fi
fi




###########################################################################################################################
# Script Start/Stop times
###########################################################################################################################
TWGETLUNIXSTOPutc=`date -u`
TWGETLUNIXSTOP=`date`
echo "START Universal Time" > ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo $TWGETLUNIXSTARTutc >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo "START Local Time" >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo $TWGETLUNIXSTART >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo "STOP Universal Time" >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo $TWGETLUNIXSTOPutc >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo "STOP Local Time" >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt
echo $TWGETLUNIXSTOP >> ./$fileName/script_workspace/Script_Start_Stop_Time.txt

if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	###########################################################################################################################
	# Capture created logs in the working directory
	###########################################################################################################################
	
	if [ -f ./CtDbg.log ] ; then
		mv ./CtDbg.log ./$fileName/LSI_Products/MegaRAID/
	fi
	if [ -f ./MegaSAS.log ] ; then
		mv ./MegaSAS.log ./$fileName/LSI_Products/MegaRAID/
	fi
	if [ -f ./CmdTool.log ] ; then
		mv ./CmdTool.log ./$fileName/LSI_Products/MegaRAID/
	fi
fi
###########################################################################################################################
# Clean up
###########################################################################################################################

for i in $( cat cleanup.txt ); do

	if [ -f ./$i ] ; then
		rm -f ./$i
	fi
done


rm -f cleanup.txt
CLEANED_UP=YES

###########################################################################################################################
# Compressing the file output 
###########################################################################################################################

if [ "$TWGETSKIPMEGARAID" != "YES" ] ; then
	if [ -f ./$fileName/LSI_Products/MegaRAID/CmdTool.log ] ; then rm ./$fileName/LSI_Products/MegaRAID/CmdTool.log >> ./misc_output.txt 2>&1 ; fi 
	if [ -f ./$fileName/LSI_Products/MegaRAID/MegaSAS.log ] ; then rm ./$fileName/LSI_Products/MegaRAID/MegaSAS.log >> ./misc_output.txt 2>&1 ; fi 
fi
if [ -f ./misc_output.txt ] ; then mv ./misc_output.txt ./$fileName/script_workspace > /dev/null 2>&1 ; fi

if [ "$TWGETSKIPCOMPRESSION" != "YES" ] ; then
	tar cf $fileName.tar ./$fileName 
	gzip -9 $fileName.tar
fi

 
#Keep subdir unless variable set
if [ "$TWGETDIRECTORYKEEP" != "YES" ] ; then rm -rf $fileName > /dev/null 2>&1 ; fi 


#cho ".................................................||................................................."
echo ""
echo "Script done. The file name is;"
echo ""
echo "$fileName.tar.gz"
echo ""
echo "Send just this file as is to your support rep."
echo ""
echo "\"$BASECMD -H\" provides a help screen."
echo ""

if [ "$TWGETDIRECTORYKEEP" != "YES" ] ; then
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N' 
	fi	
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S' 
	fi 
fi

if [ "$TWGETDIRECTORYKEEP" = "YES" ] ; then 
	echo "The following subdir was left for your use;"
	echo ""
	echo "$fileName"
	echo ""
	if [ "$VMWARE_SUPPORTED" != "YES" ] ; then
		date '+%H:%M:%S.%N' 
	fi	
	if [ "$VMWARE_SUPPORTED" = "YES" ] ; then
		date '+%H:%M:%S' 
	fi
fi


if [ "$TWGETBATCHMODE" != "BATCH" ] ; then 
	if [ "$TWGETBATCHMODE" != "QUIET" ] ; then 
	   WaitQuit
	fi
fi

